if (AjxPackage.define("Mail")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */
/*
 * Package: Mail
 * 
 * Supports: The Mail application
 * 
 * Loaded:
 * 	- When composing a message
 *  - To attach a file
 *  - When viewing a single msg or conv
 */
if (AjxPackage.define("ajax.dwt.events.DwtIdleTimer")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2007, 2009, 2010, 2011, 2012, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * @class
 * Simple manager for "idle" events. Add a handler like this:
 *
 * <pre>
 *    var idleTimer = new DwtIdleTimer(10000, new AjxCallback(obj, obj.handler));
 *
 *    obj.handler = function(idle) {
 *       if (idle) {
 *          // do idle stuff here
 *       } else {
 *          // user is back
 *       }
 *    }
 * </pre>
 * 
 * With this code, when the user is idle for 10 seconds obj.handler(true) will
 * be called.  When the user gets back from idle, obj.handler(false) will be
 * called and the timer restarted.
 * </p>
 * <p>
 * To cancel a timer, call <code>idleTimer.kill()</code>. To restart it later, you can
 * <code>idleTimer.resurrect(timeout)</code>. The timeout parameter is optional, pass it only if you
 * want to modify it.
 * </p>
 * <p>
 * You can create multiple handlers, each with its own callback and timeout.  A
 * new {@link DwtIdleTimer} will start running right away and will continue to do so
 * until you <code>kill()</code> it.
 * </p>
 * 
 * @param	{number}	[timeout]		the timeout 
 * @param	{AjxCallback}	handler		the callback
 * 
 * @private
 */
DwtIdleTimer = function(timeout, handler) {
	DwtIdleTimer._initEvents();
	this.timeout = timeout;
	this.handler = handler;
	this.idle = false;
	this._onIdle = AjxCallback.simpleClosure(this.setIdle, this);
	this._startTimer();
	DwtIdleTimer.getHandlers().add(this);
};

DwtIdleTimer.idleHandlers = 0;

DwtIdleTimer.prototype.toString =
function() {
	return "DwtIdleTimer";
};

DwtIdleTimer.prototype.kill =
function() {
	this._stopTimer();
	this.idle = false;
	DwtIdleTimer.getHandlers().remove(this);
};

DwtIdleTimer.prototype.resurrect =
function(timeout) {
	this.idle = false; // make sure we start "unidle"
	DwtIdleTimer.getHandlers().add(this, null, true);
	if (timeout != null) {
		this.timeout = timeout;
	}
	this._startTimer();
};

DwtIdleTimer.prototype.setIdle =
function() {
	if (!this.idle) {
		DwtIdleTimer.idleHandlers++;
		this.idle = true;
		this.handler.run(true);
	}
};

DwtIdleTimer.prototype.resume =
function() {
	if (this.idle) {
		this.idle = false;
		this.handler.run(false);
		DwtIdleTimer.idleHandlers--;
	}
};

DwtIdleTimer.prototype._startTimer =
function() {
	this._stopTimer();
	this._timer = setTimeout(this._onIdle, this.timeout);
};

DwtIdleTimer.prototype._stopTimer =
function() {
	if (this._timer) {
		clearTimeout(this._timer);
		this._timer = null;
	}
};

DwtIdleTimer._initEvents =
function() {
	// execute only once per session
	if (!DwtIdleTimer._initialized) {
		if (!AjxEnv.isIE) {
			window.addEventListener("keydown", DwtIdleTimer.resetIdle, true);
			window.addEventListener("mousemove", DwtIdleTimer.resetIdle, true);
			window.addEventListener("mousedown", DwtIdleTimer.resetIdle, true);
			window.addEventListener("focus", DwtIdleTimer.resetIdle, true);
		} else {
			document.body.attachEvent("onkeydown", DwtIdleTimer.resetIdle);
			document.body.attachEvent("onkeyup", DwtIdleTimer.resetIdle);
			document.body.attachEvent("onmousedown", DwtIdleTimer.resetIdle);
			document.body.attachEvent("onmousemove", DwtIdleTimer.resetIdle);
			document.body.attachEvent("onmouseover", DwtIdleTimer.resetIdle);
			document.body.attachEvent("onmouseout", DwtIdleTimer.resetIdle);
			window.attachEvent("onfocus", DwtIdleTimer.resetIdle);
		}
		DwtIdleTimer._initialized = true;
	}
};

DwtIdleTimer.getHandlers =
function() {
	var a = DwtIdleTimer.HANDLERS;
	if (!a) {
		a = DwtIdleTimer.HANDLERS = new AjxVector();
	}
	return a;
};

DwtIdleTimer.resetIdle =
function() {
	var a = DwtIdleTimer.getHandlers();
	a.foreach("_startTimer"); // we need to restart timers anyway...
	if (DwtIdleTimer.idleHandlers > 0) {
		a.foreach("resume");
	}
};
}

if (AjxPackage.define("zimbraMail.mail.view.ZmComposeView")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * Creates a new compose view. The view does not display itself on construction.
 * @constructor
 * @class
 * This class provides a form for composing a message.
 *
 * @author Conrad Damon
 * 
 * @param {DwtControl}	parent		the element that created this view
 * @param {ZmController}	controller	the controller managing this view
 * @param {constant}	composeMode 	passed in so detached window knows which mode to be in on startup
 * 
 * @extends		DwtComposite
 * 
 * @private
 */
ZmComposeView = function(parent, controller, composeMode) {

	this.TEMPLATE = "mail.Message#Compose";
	this._view = ZmId.VIEW_COMPOSE + controller.sessionId;
	this._sessionId = controller.sessionId;

	DwtComposite.call(this, {parent:parent, className:"ZmComposeView", posStyle:Dwt.ABSOLUTE_STYLE,
							 id:ZmId.getViewId(this._view)});

	ZmComposeView.ADDR_SETTING[AjxEmailAddress.BCC]	= ZmSetting.SHOW_BCC;

	ZmComposeView.NOTIFY_ACTION_MAP = {};
	ZmComposeView.NOTIFY_ACTION_MAP[ZmOperation.REPLY_ACCEPT]		= ZmOperation.REPLY_ACCEPT_NOTIFY;
	ZmComposeView.NOTIFY_ACTION_MAP[ZmOperation.REPLY_DECLINE]		= ZmOperation.REPLY_DECLINE_NOTIFY;
	ZmComposeView.NOTIFY_ACTION_MAP[ZmOperation.REPLY_TENTATIVE]	= ZmOperation.REPLY_TENTATIVE_NOTIFY;

	ZmComposeView.MOVE_TO_FIELD = {};
	ZmComposeView.MOVE_TO_FIELD[ZmOperation.MOVE_TO_TO]		= AjxEmailAddress.TO;
	ZmComposeView.MOVE_TO_FIELD[ZmOperation.MOVE_TO_CC]		= AjxEmailAddress.CC;
	ZmComposeView.MOVE_TO_FIELD[ZmOperation.MOVE_TO_BCC]	= AjxEmailAddress.BCC;
	
	this._onMsgDataChange = new AjxCallback(this, this._onMsgDataChange);
	this._useAcAddrBubbles = appCtxt.get(ZmSetting.USE_ADDR_BUBBLES);

	this._controller = controller;
	this._initialize(composeMode);

	// make sure no unnecessary scrollbars show up
	this.getHtmlElement().style.overflow = "hidden";
};

ZmComposeView.prototype = new DwtComposite;
ZmComposeView.prototype.constructor = ZmComposeView;

ZmComposeView.prototype.toString =
function() {
	return "ZmComposeView";
};

//
// Constants
//

// Consts related to compose fields
ZmComposeView.QUOTED_HDRS = [
		ZmMailMsg.HDR_FROM,
		ZmMailMsg.HDR_TO,
		ZmMailMsg.HDR_CC,
		ZmMailMsg.HDR_DATE,
		ZmMailMsg.HDR_SUBJECT
];

ZmComposeView.BAD						= "_bad_addrs_";

// Message dialog placement
ZmComposeView.DIALOG_X 					= 50;
ZmComposeView.DIALOG_Y 					= 100;

// Attachment related
ZmComposeView.UPLOAD_FIELD_NAME			= "attUpload";
ZmComposeView.FORWARD_ATT_NAME			= "ZmComposeView_forAttName";
ZmComposeView.FORWARD_MSG_NAME			= "ZmComposeView_forMsgName";

// max # of attachments to show
ZmComposeView.SHOW_MAX_ATTACHMENTS		= AjxEnv.is800x600orLower ? 2 : 3;
ZmComposeView.MAX_ATTACHMENT_HEIGHT 	= (ZmComposeView.SHOW_MAX_ATTACHMENTS * 23) + "px";

// Reply/forward stuff
ZmComposeView.EMPTY_FORM_RE				= /^[\s\|]*$/;
ZmComposeView.HTML_TAG_RE				= /(<[^>]+>)/g;
ZmComposeView.SUBJ_PREFIX_RE			= new RegExp("^\\s*(Re|Fw|Fwd|" + ZmMsg.re + "|" + ZmMsg.fwd + "|" + ZmMsg.fw + "):" + "\\s*", "i");
ZmComposeView.QUOTED_CONTENT_RE			= new RegExp("^----- ", "m");
ZmComposeView.HTML_QUOTED_CONTENT_RE	= new RegExp("<br>----- ", "i");
ZmComposeView.ADDR_SETTING				= {}; // XXX: may not be necessary anymore?

ZmComposeView.OP = {};
ZmComposeView.OP[AjxEmailAddress.TO]	= ZmId.CMP_TO;
ZmComposeView.OP[AjxEmailAddress.CC]	= ZmId.CMP_CC;
ZmComposeView.OP[AjxEmailAddress.BCC]	= ZmId.CMP_BCC;


// Public methods

/**
 * Sets the current view, based on the given action. The compose form is
 * created and laid out and everything is set up for interaction with the user.
 *
 * @param	{Hash}	params		a hash of parameters
 * @param {constant}	params.action				new message, reply, forward, or an invite action
 * @param {ZmIdentity}	params.identity			the identity sending the message
 * @param {ZmMailMsg}	params.msg				the original message (reply/forward), or address (new message)
 * @param {String}	params.toOverride			initial value for To: field
 * @param {String}	params.subjOverride		initial value for Subject: field
 * @param {String}	params.extraBodyText		canned text to prepend to body (invites)
 * @param {Array}	params.msgIds				list of msg Id's to be added as attachments
 * @param {ZmIdentity}	params.identity			identity to use for this compose
 * @param {String}	params.accountName		on-behalf-of From address
 */
ZmComposeView.prototype.set =
function(params) {
	var action = this._action = params.action;
	if (this._msg) {
		this._msg.onChange = null;
	}
	this._acceptFolderId = params.acceptFolderId;
	var obo = params.accountName;
	var msg = this._msg = this._addressesMsg = params.msg;
	if (msg) {
		msg.onChange = this._onMsgDataChange;
		var folder = (!obo) ? appCtxt.getById(msg.folderId) : null;
		obo = (folder && folder.isRemote()) ? folder.getOwner() : null;

		// check if this is a draft that was originally composed obo
		var isFromDataSource = msg.identity && msg.identity.isFromDataSource;
		if (!obo && msg.isDraft && !appCtxt.multiAccounts && !isFromDataSource && !appCtxt.get(ZmSetting.ALLOW_ANY_FROM_ADDRESS)) {
			var ac = window.parentAppCtxt || window.appCtxt;
			var from = msg.getAddresses(AjxEmailAddress.FROM).get(0);
			if (from && from.address.toLowerCase() != ac.accountList.mainAccount.getEmail().toLowerCase() && !appCtxt.isMyAddress(from.address.toLowerCase())) {
				obo = from.address;
			}
		}
	}

	// list of msg Id's to add as attachments
	this._msgIds = params.msgIds;

	AjxDebug.println(AjxDebug.REPLY, "ZmComposeView::set - Reset compose view");
	this.reset(true);

	this._setFromSelect(msg);

	if (params.identity) {
		if (this.identitySelect) {
			this.identitySelect.setSelectedValue(params.identity.id);
		}
		if (appCtxt.get(ZmSetting.SIGNATURES_ENABLED) || appCtxt.multiAccounts) {
			var selected = this._getSignatureIdForAction(params.identity) || "";
			var account = appCtxt.multiAccounts && this.getFromAccount();
			this._controller.resetSignatureToolbar(selected, account);
		}
	}

	// reset To/Cc/Bcc fields
	this._showAddressField(AjxEmailAddress.TO, true, true, true);
	this._showAddressField(AjxEmailAddress.CC, true, true, true);
	//Set BCC Field to Default
	this._toggleBccField(null, appCtxt.get(ZmSetting.SHOW_BCC));

	// populate fields based on the action and user prefs
	this._setAddresses(action, AjxEmailAddress.TO, params.toOverrideObj || params.toOverride);
	if (params.ccOverride) {
		this._setAddresses(action, AjxEmailAddress.CC, params.ccOverride);
	}
	if (params.bccOverride) {
		this._setAddresses(action, AjxEmailAddress.BCC, params.bccOverride);
	}
	if (obo) {
		this._setObo(obo);
	}
	this._setSubject(action, msg, params.subjOverride);
	this._setBody(action, msg, params.extraBodyText);

	if (appCtxt.get(ZmSetting.MAIL_PRIORITY_ENABLED)) {
		var priority = "";
		if (msg && (action == ZmOperation.DRAFT)) {
			if (msg.isHighPriority) {
				priority = ZmItem.FLAG_HIGH_PRIORITY;
			} else if (msg.isLowPriority) {
				priority = ZmItem.FLAG_LOW_PRIORITY;
			}
		}
		this._setPriority(priority);
	}

	this.getHtmlEditor().moveCaretToTop();

	if (action != ZmOperation.FORWARD_ATT) {
		// save extra mime parts
		var bodyParts = msg ? msg.getBodyParts() : [];
		for (var i = 0; i < bodyParts.length; i++) {
			var bodyPart = bodyParts[i];
			var contentType = bodyPart.ct;

			if (contentType == ZmMimeTable.TEXT_PLAIN) continue;
			if (contentType == ZmMimeTable.TEXT_HTML) continue;
			if (ZmMimeTable.isRenderableImage(contentType) && bodyPart.cd == "inline") continue; // bug: 28741

			var mimePart = new ZmMimePart();
			mimePart.setContentType(contentType);
			mimePart.setContent(bodyPart.content);
			this.addMimePart(mimePart);
		}
	}

	// save form state (to check for change later)
	if (this._composeMode == DwtHtmlEditor.HTML) {
		var ta = new AjxTimedAction(this, this._setFormValue);
		AjxTimedAction.scheduleAction(ta, 10);
	} else {
		this._setFormValue();
	}
	// Force focus on the TO field
	if (!this._isReply()) {
		appCtxt.getKeyboardMgr().grabFocus(this._field[AjxEmailAddress.TO]);
	}
};

/**
 * Called automatically by the attached ZmMailMsg object when data is
 * changed, in order to support Zimlets modify subject or other values
 * (bug: 10540)
 * 
 * @private
 */
ZmComposeView.prototype._onMsgDataChange =
function(what, val) {
	if (what == "subject") {
		this._subjectField.value = val;
		this.updateTabTitle();
	}
};

ZmComposeView.prototype.getComposeMode =
function() {
	return this._composeMode;
};

ZmComposeView.prototype.getController =
function() {
	return this._controller;
};

ZmComposeView.prototype.getHtmlEditor =
function() {
	return this._htmlEditor;
};

/**
 * Gets the title.
 * 
 * @return	{String}	the title
 */
ZmComposeView.prototype.getTitle =
function() {
	var text;
	if (this._isReply()) {
		text = ZmMsg.reply;
	} else if (this._isForward()) {
		text = ZmMsg.forward;
	} else {
		text = ZmMsg.compose;
	}
	return [ZmMsg.zimbraTitle, text].join(": ");
};

/**
 * Gets the field values for each of the addr fields.
 * 
 * @return	{Array}	an array of addresses
 */
ZmComposeView.prototype.getRawAddrFields =
function() {
	var addrs = {};
	for (var i = 0; i < ZmMailMsg.COMPOSE_ADDRS.length; i++) {
		var type = ZmMailMsg.COMPOSE_ADDRS[i];
		if (this._using[type]) {
			addrs[type] = this._getAddrFieldValue(type);
		}
	}
	return addrs;
};

// returns address fields that are currently visible
ZmComposeView.prototype.getAddrFields =
function() {
	var addrs = [];
	for (var i = 0; i < ZmMailMsg.COMPOSE_ADDRS.length; i++) {
		var type = ZmMailMsg.COMPOSE_ADDRS[i];
		if (this._using[type]) {
			addrs.push(this._field[type]);
		}
	}
	return addrs;
};

// returns list of attachment field values (used by detachCompose)
ZmComposeView.prototype.getAttFieldValues =
function() {
	var attList = [];
	var atts = document.getElementsByName(ZmComposeView.UPLOAD_FIELD_NAME);

	for (var i = 0; i < atts.length; i++)
		attList.push(atts[i].value);

	return attList;
};

ZmComposeView.prototype.setBackupForm =
function() {
	this.backupForm = this._backupForm();
};

/**
* Saves *ALL* form value data to test against whether user made any changes
* since canceling SendMsgRequest. If user attempts to send again, we compare
* form data with this value and if not equal, send a new UID otherwise, re-use.
* 
* @private
*/
ZmComposeView.prototype._backupForm =
function() {
	var val = this._formValue(true, true);

	// keep track of attachments as well
	var atts = document.getElementsByName(ZmComposeView.UPLOAD_FIELD_NAME);
	for (var i = 0; i < atts.length; i++) {
		if (atts[i].value.length) {
			val += atts[i].value;
		}
	}

	// keep track of "uploaded" attachments as well :/
	val += this._getForwardAttIds(ZmComposeView.FORWARD_ATT_NAME + this._sessionId).join("");
	val += this._getForwardAttIds(ZmComposeView.FORWARD_MSG_NAME + this._sessionId).join("");

	return val;
};

ZmComposeView.prototype._isInline =
function(msg) {

	if (this._attachDialog) {
		return this._attachDialog.isInline();
	}

	msg = msg || this._msg;

	if (msg && this._msgAttId && msg.id == this._msgAttId) {
		return false;
	}

	if (msg && msg.attachments) {
		var atts = msg.attachments;
		for (var i = 0; i < atts.length; i++) {
			if (atts[i].ci) {
				return true;
			}
		}
	}

	return false;
};

ZmComposeView.prototype._handleInlineAtts =
function(msg, handleInlineDocs){

	var handled = false, ci, cid, dfsrc, inlineAtt, attached = {};

	var idoc = this._htmlEditor._getIframeDoc();
	var images = idoc.getElementsByTagName("img");
	for (var i = 0; i < images.length; i++) {
		dfsrc = images[i].getAttribute("dfsrc") || images[i].getAttribute("mce_src") || images[i].src;
		if (dfsrc) {
			if (dfsrc.substring(0,4) == "cid:") {
				cid = dfsrc.substring(4).replace("%40","@");
				var docpath = images[i].getAttribute("doc");
				if (docpath){
					msg.addInlineDocAttachment(cid, null, docpath);
					handled = true;
				} else {
					ci = "<" + cid + ">";
					inlineAtt = msg.findInlineAtt(ci);
					if (!inlineAtt && this._msg) {
						inlineAtt = this._msg.findInlineAtt(ci);
					}
                    if (inlineAtt) {
                        if(!attached[(cid+"_"+inlineAtt.part)]){
                            msg.addInlineAttachmentId(cid, null, inlineAtt.part);
                            handled = true;
                            attached[(cid+"_"+inlineAtt.part)] = true;
                        }
					}
				}
			}
		}
	}

	return handled;
};

ZmComposeView.prototype._mergeInlineAndForwardAtts =
function(msg, forwardAttIds) {

	var newFwdAttIds = [];
	var atts = this._msg.attachments;

	function checkFwdAttExists(part) {
		for (var j = 0; j < forwardAttIds.length; j++) {
			if(forwardAttIds[j] == part){
				return true;
			}
		}
		return false;
	}

	for (var i = 0; i < atts.length; i++) {
		var att = atts[i];
		if (att.ci && !checkFwdAttExists(att.part)) {
			newFwdAttIds.push(att.part);
		}
	}

	return [].concat(forwardAttIds, newFwdAttIds);
};

ZmComposeView.prototype._generateCid =
function() {
	var timeStr = ""+new Date().getTime();
	var hash = AjxSHA1.hex_sha1(timeStr + Dwt.getNextId());
	return hash + "@zimbra";
}

/**
* Returns the message from the form, after some basic input validation.
*/
ZmComposeView.prototype.getMsg =
function(attId, isDraft, dummyMsg, forceBail, contactId) {
	// Check destination addresses.
	var addrs = this._collectAddrs();

	// Any addresses at all provided? If not, bail.
	if ((!isDraft || forceBail) && !addrs.gotAddress) {
		this.enableInputs(false);
		var msgDialog = appCtxt.getMsgDialog();
		msgDialog.setMessage(ZmMsg.noAddresses, DwtMessageDialog.CRITICAL_STYLE);
		msgDialog.popup(this._getDialogXY());
		msgDialog.registerCallback(DwtDialog.OK_BUTTON, this._okCallback, this);
		this.enableInputs(true);
		return;
	}

	var cd = appCtxt.getOkCancelMsgDialog();
	cd.reset();

	// Is there a subject? If not, ask the user if they want to send anyway.
	var subject = AjxStringUtil.trim(this._subjectField.value);
	if ((!isDraft || forceBail) && subject.length == 0 && !this._noSubjectOkay) {
		this.enableInputs(false);
		cd.setMessage(ZmMsg.compSubjectMissing, DwtMessageDialog.WARNING_STYLE);
		cd.registerCallback(DwtDialog.OK_BUTTON, this._noSubjectOkCallback, this, cd);
		cd.registerCallback(DwtDialog.CANCEL_BUTTON, this._noSubjectCancelCallback, this, cd);
		cd.popup(this._getDialogXY());
		return;
	}

	// Any bad addresses?  If there are bad ones, ask the user if they want to send anyway.
	if ((!isDraft || forceBail) && addrs[ZmComposeView.BAD].size() && !this._badAddrsOkay) {
		this.enableInputs(false);
		var bad = AjxStringUtil.htmlEncode(addrs[ZmComposeView.BAD].toString(AjxEmailAddress.SEPARATOR));
		var msg = AjxMessageFormat.format(ZmMsg.compBadAddresses, bad);
		cd.setMessage(msg, DwtMessageDialog.WARNING_STYLE);
		cd.registerCallback(DwtDialog.OK_BUTTON, this._badAddrsOkCallback, this, cd);
		cd.registerCallback(DwtDialog.CANCEL_BUTTON, this._badAddrsCancelCallback, this, [addrs.badType, cd]);
		cd.setVisible(true); // per fix for bug 3209
		cd.popup(this._getDialogXY());
		return;
	} else {
		this._badAddrsOkay = false;
	}

	// Mandatory Spell Check
	if ((!isDraft || forceBail) && appCtxt.get(ZmSetting.SPELL_CHECK_ENABLED) && 
		appCtxt.get(ZmSetting.MAIL_MANDATORY_SPELLCHECK) && !this._spellCheckOkay) {
		if (this._htmlEditor.checkMisspelledWords(new AjxCallback(this, this._spellCheckShield), null, new AjxCallback(this, this._spellCheckErrorShield))) {
			return;
		}
	} else {
		this._spellCheckOkay = false;
	}

	// Create Msg Object - use dummy if provided
	var msg = dummyMsg || (new ZmMailMsg());
	msg.setSubject(subject);

	var zeroSizedAttachments = false;
	// handle Inline Attachments
    if (attId && ( (this._attachDialog && this._attachDialog.isInline()) || attId.clipboardPaste ) ){
		for (var i = 0; i < attId.length; i++) {
			var att = attId[i];
			if (att.s == 0) {
				zeroSizedAttachments = true;
				continue;
			}
			var contentType = att.ct;
			if (contentType && contentType.indexOf("image") != -1) {
				var cid = this._generateCid();
				 if( att.hasOwnProperty("id") ){
                    this._htmlEditor.replaceImage(att.id, "cid:" + cid);
                }
                else{
                    this._htmlEditor.insertImage("cid:" + cid, AjxEnv.isIE);
                }
				msg.addInlineAttachmentId(cid, att.aid);
			} else {
				msg.addAttachmentId(att.aid);
			}
		}
	} else if (attId && typeof attId != "string") {
		for (var i = 0; i < attId.length; i++) {
			if (attId[i].s == 0) {
				zeroSizedAttachments = true;
				continue;
			}
			msg.addAttachmentId(attId[i].aid);
		}
	} else if (attId) {
		msg.addAttachmentId(attId);
	}

	if (zeroSizedAttachments) {
		appCtxt.setStatusMsg(ZmMsg.zeroSizedAtts);
	}

	// check if this is a resend
	if (this.sendUID && this.backupForm) {
		// if so, check if user changed anything since canceling the send
		if (isDraft || this._backupForm() != this.backupForm) {
			this.sendUID = (new Date()).getTime();
		}
	} else {
		this.sendUID = (new Date()).getTime();
	}

	// get list of message part id's for any forwarded attachements
	var forwardAttIds = this._getForwardAttIds(ZmComposeView.FORWARD_ATT_NAME + this._sessionId);
	var forwardMsgIds = this._getForwardAttIds(ZmComposeView.FORWARD_MSG_NAME + this._sessionId);

	// --------------------------------------------
	// Passed validation checks, message ok to send
	// --------------------------------------------

	// set up message parts as necessary
	var top = new ZmMimePart();

	if (this._composeMode == DwtHtmlEditor.HTML) {
		top.setContentType(ZmMimeTable.MULTI_ALT);

		// create two more mp's for text and html content types
		var textPart = new ZmMimePart();
		textPart.setContentType(ZmMimeTable.TEXT_PLAIN);
		var self = this;
		var convertor = {
			"hr": function(el) {
				return ZmComposeView._convertHtmlPreface(self, el);
			},
			"blockquote": function(el) {
				return "\n<blockquote>\n";
			},
			"/blockquote": function(el) {
				return "\n</blockquote>\n";
			},
			"_after": AjxCallback.simpleClosure(this._applyHtmlPrefix, this, "<blockquote>", "</blockquote>")
		}
		var text = this._htmlEditor.getTextVersion(convertor);

		textPart.setContent(text);
		top.children.add(textPart);

		var htmlPart = new ZmMimePart();
		htmlPart.setContentType(ZmMimeTable.TEXT_HTML);		

		var idoc = this._htmlEditor._getIframeDoc();
		this._cleanupFileRefImages(idoc);
		this._restoreMultipartRelatedImages(idoc);
		if (!isDraft) {
			this._cleanupSignatureIds(idoc);
		}
		var defangedContent = this._htmlEditor.getContent(!isDraft);

		// Bug 27422 - Firefox and Safari implementation of execCommand("bold")
		// etc use styles, and some email clients (Entourage) don't process the
		// styles and the text remains plain. So we post-process and convert
		// those to the tags (which are what the IE version of execCommand() does).
		if (AjxEnv.isFirefox) {
			defangedContent = defangedContent.replace(/<span style="font-weight: bold;">(.+?)<\/span>/, "<strong>$1</strong>");
			defangedContent = defangedContent.replace(/<span style="font-style: italic;">(.+?)<\/span>/, "<em>$1</em>");
			defangedContent = defangedContent.replace(/<span style="text-decoration: underline;">(.+?)<\/span>/, "<u>$1</u>");
			defangedContent = defangedContent.replace(/<span style="text-decoration: line-through;">(.+?)<\/span>/, "<strike>$1</strike>");
		} else if (AjxEnv.isSafari) {
			defangedContent = defangedContent.replace(/<span class="Apple-style-span" style="font-weight: bold;">(.+?)<\/span>/, "<strong>$1</strong>");
			defangedContent = defangedContent.replace(/<span class="Apple-style-span" style="font-style: italic;">(.+?)<\/span>/, "<em>$1</em>");
			defangedContent = defangedContent.replace(/<span class="Apple-style-span" style="text-decoration: underline;">(.+?)<\/span>/, "<u>$1</u>");
			defangedContent = defangedContent.replace(/<span class="Apple-style-span" style="text-decoration: line-through;">(.+?)<\/span>/, "<strike>$1</strike>");
		}

		htmlPart.setContent(defangedContent);
        
        //set img src to cid for inline or dfsrc if external image and remove dfsrc before sending
        var content = htmlPart.getContent();
        var imgContent = content.split(/<img/i);
        for(var i=0; i<imgContent.length; i++){
            var externalImage = false;
            var dfsrc = imgContent[i].match(/dfsrc=[\"|\'](cid:[^\"\']+)/); //look for CID assignment in image
			if (dfsrc && dfsrc.length > 1) {
				dfsrc = [dfsrc[1]]; //the cid is the 2nd element, but next lines expect it as first 
			}
            if (!dfsrc){
                dfsrc = imgContent[i].match(/\s+dfsrc=[\"\'][^\"\']+[\"\']+/); //look for dfsrc="" in image
                externalImage = dfsrc ? true : false;
            }
            if (dfsrc && dfsrc.length > 0 && !externalImage){
                var tempStr = imgContent[i].replace(/\s+src=[\"\'][^\"\']+[\"\']/," src=\""+dfsrc[0]+"\""); //set src to cid
                tempStr = tempStr.replace(/\s+dfsrc=[\"\'][^\"\']+[\"\']+/,"");
                content = content.replace(imgContent[i], tempStr);
            }
            else if (dfsrc && dfsrc.length > 0 && externalImage){
                var tempArr = imgContent[i].match(/\s+dfsrc=[\"\']([^\"\']+)[\"\']/); //match dfsrc
                if (tempArr && tempArr.length > 1) {
                   var tempStr = imgContent[i].replace(/\s+dfsrc=[\"\'][^\"\']+[\"\']/," src=\""+tempArr[1]+"\"");
                   content = content.replace(imgContent[i], tempStr);
                }
            }
        }

        htmlPart.setContent(content);

		this._handleInlineAtts(msg, true); // Better Code
		var inlineAtts = msg.getInlineAttachments();
		var inlineDocAtts = msg.getInlineDocAttachments();
		var iAtts = [].concat(inlineAtts, inlineDocAtts);
		if ( iAtts &&  iAtts.length > 0 ) {
			var relatedPart = new ZmMimePart();
			relatedPart.setContentType(ZmMimeTable.MULTI_RELATED);
			relatedPart.children.add(htmlPart);
			top.children.add(relatedPart);
		} else {
			top.children.add(htmlPart);
		}
	}
	else {
		var inline = this._isInline();

		var textPart = (this._extraParts || inline) ? new ZmMimePart() : top;
		textPart.setContentType(ZmMimeTable.TEXT_PLAIN);
		textPart.setContent(this._htmlEditor.getContent());

		if (inline) {
			top.setContentType(ZmMimeTable.MULTI_ALT);
			var relatedPart = new ZmMimePart();
			relatedPart.setContentType(ZmMimeTable.MULTI_RELATED);
			relatedPart.children.add(textPart);
			top.children.add(relatedPart);
            // bug: 43156 
            // Commented as we now show inline attachments as part of forward attachments.
			//forwardAttIds = this._mergeInlineAndForwardAtts(msg, forwardAttIds);
		} else {
			if (this._extraParts) {
				top.setContentType(ZmMimeTable.MULTI_ALT);
				top.children.add(textPart);
			}
		}
	}

	// add extra message parts
	if (this._extraParts) {
		for (var i = 0; i < this._extraParts.length; i++) {
			var mimePart = this._extraParts[i];
			top.children.add(mimePart);
		}
	}

	// store text-content of the current email
	msg.textBodyContent = (this._composeMode == DwtHtmlEditor.HTML)
		? this._htmlEditor.getTextVersion()
		: this._htmlEditor.getContent();

	msg.setTopPart(top);
	msg.setSubject(subject);
	msg.setForwardAttIds(forwardAttIds);
    if (!contactId) {
    //contactId not passed in, but vcard signature may be set
        if (this._msg && this._msg._contactAttIds){
            contactId = this._msg._contactAttIds;
            this._msg.setContactAttIds([]);
        }
    }
	msg.setContactAttIds(contactId);
	for (var i = 0; i < ZmMailMsg.COMPOSE_ADDRS.length; i++) {
		var type = ZmMailMsg.COMPOSE_ADDRS[i];
		if (addrs[type] && addrs[type].all.size() > 0) {
			msg.setAddresses(type, addrs[type].all);
		}
	}
	msg.identity = this.getIdentity();
	msg.sendUID = this.sendUID;

	// save a reference to the original message
	msg._origMsg = this._msg;
	if (this._msg && this._msg._instanceDate) {
		msg._instanceDate = this._msg._instanceDate;
	}

	if (this._action != ZmOperation.NEW_MESSAGE && this._msg && !this._msgIds) {
		var isInviteReply = this._isInviteReply(this._action);
		if (this._action == ZmOperation.DRAFT) {
			msg.isReplied = (this._msg.rt == "r");
			msg.isForwarded = (this._msg.rt == "w");
			msg.isDraft = this._msg.isDraft;
			// check if we're resaving a draft that was originally a reply/forward
			if (msg.isDraft) {
				// if so, set both origId and the draft id
				msg.origId = msg.isReplied || msg.isForwarded ? this._msg.origId : null;
				msg.id = this._msg.id;
				msg.nId = this._msg.nId;
			}
		} else {
			msg.isReplied = this._isReply();
			msg.isForwarded = this._isForward();
			msg.origId = this._msg.id;
		}
		msg.isInviteReply = isInviteReply;
		msg.acceptFolderId = this._acceptFolderId;
		var inviteMode = ZmComposeView.NOTIFY_ACTION_MAP[this._action] ? ZmComposeView.NOTIFY_ACTION_MAP[this._action] : this._action;
		msg.inviteMode = isInviteReply ? inviteMode : null;
		msg.irtMessageId = this._msg.irtMessageId || this._msg.messageId;
		msg.folderId = this._msg.folderId;
	}

	if (this._action == ZmOperation.DRAFT && this._origAcctMsgId) {
		msg.origAcctMsgId = this._origAcctMsgId;
	}

	// replied/forw msg or draft shouldn't have att ID (a repl/forw voicemail mail msg may)
	if (this._msg && this._msg.attId) {
		msg.addAttachmentId(this._msg.attId);
	}

	if (this._msgAttId) {
		if (forwardMsgIds.length > 0) {
			// Check if the MsgId is already present in the fwdMsgIds list.
			var i = 0;
			while (forwardMsgIds[i] && forwardMsgIds[i] != this._msgAttId) {
				i++;
			}
			if (i == forwardMsgIds.length) {
				forwardMsgIds.push(this._msgAttId);
			}
			delete i;
		} else {
			forwardMsgIds.push(this._msgAttId);
		}
	}

	msg.setMessageAttachmentId(forwardMsgIds);

	var priority = this._getPriority();
	if (priority) {
		msg.flagLocal(priority, true);
	}

	if (this._fromSelect) {
		msg.fromSelectValue = this._fromSelect.getSelectedOption();
	}

	/*
	* finally, check for any errors via zimlets..
	* A Zimlet can listen to emailErrorCheck action to perform further check and
	* alert user about the error just before sending email. We will be showing
	* yes/no dialog. This zimlet must return an object {hasError:<true or false>,
	* errorMsg:<Some Error msg>, zimletName:<zimletName>} e.g: {hasError:true,
	* errorMsg:"you might have forgotten attaching an attachment, do you want to
	* continue?", zimletName:"com_zimbra_attachmentAlert"}
	**/
	if ((!isDraft || forceBail) && appCtxt.areZimletsLoaded()) {
		var boolAndErrorMsgArray = [];
		var showErrorDlg = false;
		var errorMsg = "";
		var zimletName = "";
		appCtxt.notifyZimlets("emailErrorCheck", [msg, boolAndErrorMsgArray]);
		var blen =  boolAndErrorMsgArray.length;
		for (var k = 0; k < blen; k++) {
			var obj = boolAndErrorMsgArray[k];
			if (obj == null || obj == undefined) { continue; }

			var hasError = obj.hasError;
			zimletName = obj.zimletName;
			if (Boolean(hasError)) {
				if (this._ignoredZimlets) {
					if (this._ignoredZimlets[zimletName]) { // if we should ignore this zimlet
						delete this._ignoredZimlets[zimletName];
						continue; // skip
					}
				}
				showErrorDlg = true;
				errorMsg = obj.errorMsg;
				break;
			}
		}
		if (showErrorDlg) {
			this.enableInputs(false);
			cd.setMessage(errorMsg, DwtMessageDialog.WARNING_STYLE);
			var params = {errDialog:cd, zimletName:zimletName};
			cd.registerCallback(DwtDialog.OK_BUTTON, this._errViaZimletOkCallback, this, params);
			cd.registerCallback(DwtDialog.CANCEL_BUTTON, this._errViaZimletCancelCallback, this, params);
			cd.popup(this._getDialogXY());
			return;
		}
	}

	return msg;
};

ZmComposeView.prototype.setDocAttachments =
function(msg, docIds) {
	if (!docIds) { return; }

	var zeroSizedAttachments = false;
	var inline = this._isInline();
	for (var i = 0; i < docIds.length; i++) {
		var docAtt = docIds[i];
		var contentType = docAtt.ct;
		if (docAtt.s == 0) {
			zeroSizedAttachments = true;
			continue;
		}
		if (this._attachDialog && inline) {
			if (contentType && contentType.indexOf("image") != -1) {
				var cid = this._generateCid();
				this._htmlEditor.insertImage("cid:" + cid, AjxEnv.isIE);
				msg.addInlineDocAttachment(cid, docAtt.id);
			} else {
				msg.addDocumentAttachment(docAtt);
			}
		}else {
			msg.addDocumentAttachment(docAtt);
		}
	}
	if (zeroSizedAttachments){
		appCtxt.setStatusMsg(ZmMsg.zeroSizedAtts);
	}
};

/**
 * Sets an address field.
 *
 * @param type	the address type
 * @param addr	the address string
 *
 * XXX: if addr empty, check if should hide field
 * 
 * @private
 */
ZmComposeView.prototype.setAddress =
function(type, addr) {

	addr = addr || "";

	var addrStr = addr.isAjxEmailAddress ? addr.toString() : addr;

	//show first, so focus works on IE.
	if (addrStr.length && !this._using[type]) {
		this._using[type] = true;
		this._showAddressField(type, true);
	}

	if (this._useAcAddrBubbles) {
		var addrInput = this._addrInputField[type];
		if (!addrStr) {
			addrInput.clear();
		}
		else {
			if (addr.isAjxEmailAddress) {
				var match = {isDL: addr.isGroup && addr.canExpand, email: addrStr};
				addrInput.addBubble({address:addrStr, match:match, skipNotify:true});
			}
			else {
				this._setAddrFieldValue(type, addrStr);
			}
		}
	}
	else {
		this._setAddrFieldValue(type, addrStr);
	}

	// Use a timed action so that first time through, addr textarea
	// has been sized by browser based on content before we try to
	// adjust it (bug 20926)
	AjxTimedAction.scheduleAction(new AjxTimedAction(this,
		function() {
			this._adjustAddrHeight(this._field[type]);
		}), 0);
};

// Sets the mode ZmHtmlEditor should be in.
ZmComposeView.prototype.setComposeMode =
function(composeMode, switchPreface, dontReplaceContent) {

	if (composeMode == this._composeMode) { return; }
	
	AjxDebug.println(AjxDebug.REPLY, "ZmComposeView::setComposeMode (new mode) - " + composeMode);
	var htmlMode = (composeMode == DwtHtmlEditor.HTML);
	if (!htmlMode || appCtxt.get(ZmSetting.HTML_COMPOSE_ENABLED)) {

		var curMember = (this._composeMode == DwtHtmlEditor.TEXT) ? this._bodyField : this._htmlEditor;
		// get these before we change mode so we can find them in current body
		var sig = this.getSignatureContent(sigId);
		var sigSep = this._getSignatureSeparator();
		this._composeMode = composeMode;
		if (!htmlMode && switchPreface) {
			this._switchPreface();
		}

		var content = this._htmlEditor.getContent();
		var sigId = this._controller._currentSignatureId;

		if (htmlMode) {

			var anyChar = "[\\s\\S]"; // Includes ALL chars, even newlines
			var baseContent = content || "";

			// Strip away included message
			if (this._msg) {
				var preface = this._getPreface(DwtHtmlEditor.TEXT);
				if (preface) {
					var incMsgRe = new RegExp(AjxStringUtil.regExEscape(preface)+anyChar+"*$");
					baseContent = content.replace(incMsgRe, "");
				}
			}

			// Strip away signature
			if (sig) {
				if (sig) {
					var sigSepRE = AjxStringUtil.regExEscape(sigSep);
					var sigRE = AjxStringUtil.regExEscape(sig.replace(/[\n\r]*$/,""));
					var sigRe = new RegExp(sigSepRE + sigRE.replace(/\\n/g,"\\s?\\n") + anyChar + "*$");
					baseContent = baseContent.replace(sigRe, "");
				}
			}

			if (this._action == ZmOperation.DRAFT) { //see below why this is only in case of draft
				baseContent = baseContent.replace(/\n/g,"<br/>");
			}

			// Do the mode switch
			this._htmlEditor.setMode(composeMode, true);
			
			if (this._action != ZmOperation.DRAFT && !dontReplaceContent) {
				baseContent = AjxStringUtil.convertToHtml(baseContent, true);
				baseContent = baseContent.replace(/\n/g,"<br/>");
				// Re-set the whole body, with optional replied/forwarded msg and signature automatically added.
				// baseContent is the text that the user may have written before switching
				this._setBody(this._action, this._msg || null, baseContent || "\n", null, true);
			}
		} else {

			var self = this;
			var convertor = {
				"hr": function(el) {
					return ZmComposeView._convertHtmlPreface(self, el);
				},
				"blockquote": function(el) {
					return "\n<blockquote>\n";
				},
				"/blockquote": function(el) {
					return "\n</blockquote>\n";
				},
				"_after": AjxCallback.simpleClosure(this._applyHtmlPrefix, this, "<blockquote>", "</blockquote>")
			};

			var account = appCtxt.multiAccounts && this.getFromAccount();
			this.applySignature(content, sigId, account, null, true); // Remove the signature before switching
			this._htmlEditor.setMode(composeMode, true, convertor); // Do the mode switch
			content = this._htmlEditor.getContent(); // Get the content in the new mode
			this.applySignature(content, null, account, sigId, false); // Reapply the signature after switching

		}

		if (htmlMode && switchPreface) {
			this._switchPreface();
		}

		// reset the body field Id and object ref
		this._bodyFieldId = this._htmlEditor.getBodyFieldId();
		this._bodyField = Dwt.byId(this._bodyFieldId);
		if (this._bodyField.disabled) {
			this._bodyField.disabled = false;
		}

		// for now, always reset message body size
		this._resetBodySize();

		// recalculate form value since HTML mode inserts HTML tags
		this._setFormValue();

		// swap new body field into tab group
		var newMember = (composeMode == DwtHtmlEditor.TEXT) ? this._bodyField : this._htmlEditor;
        //        if (window.isTinyMCE) {
        //            curMember = newMember = this._htmlEditor.getEditorContainer();
        //            this._retryHtmlEditorFocus();
        //        }
		if (curMember && newMember && (curMember != newMember) && this._controller._tabGroup) {
			this._controller._tabGroup.replaceMember(curMember, newMember);
			// focus via replaceMember() doesn't take, try again
			if (composeMode == DwtHtmlEditor.HTML) {
				this._retryHtmlEditorFocus();
			}
		}
		if (!htmlMode) {
			AjxTimedAction.scheduleAction(new AjxTimedAction(this, function() { this.getHtmlEditor().moveCaretToTop(); }), 200);
		}
	}

	if (this._msg && this._isInline() && composeMode == DwtHtmlEditor.TEXT) {
		this._showForwardField(this._msg, this._action, null, true);
	}
};

ZmComposeView.prototype._retryHtmlEditorFocus =
function() {
	if (this._htmlEditor.hasFocus()) {
		var ta = new AjxTimedAction(this, this._focusHtmlEditor);
		AjxTimedAction.scheduleAction(ta, 10);
	}
};

ZmComposeView.prototype.setDetach =
function(params) {

	this._action = params.action;
	this._msg = params.msg;

	// set the addr fields as populated
	for (var type in params.addrs) {
		this.setAddress(type, "");
		var addrs = AjxUtil.toArray(params.addrs[type]);
		this._addAddresses(type, AjxVector.fromArray(addrs));
	}

	this._subjectField.value = params.subj || "";
	this._setPriority(params.priority);
	this.updateTabTitle();

	var content = params.body || "";
	if((content == "") && (this._htmlEditor.getMode() == DwtHtmlEditor.HTML)) {
		content	= "<br>";
	}
	this._htmlEditor.setContent(content);


	if (params.forwardHtml) {
		this._attcDiv.innerHTML = params.forwardHtml;
		this._msgAttId = params.msgAttId;
	}
	if (params.identityId && this.identitySelect) {
		this.identitySelect.setSelectedValue(params.identityId);
	}

	this.backupForm = params.backupForm;
	this.sendUID = params.sendUID;

	// bug 14322 -- in Windows Firefox, DEL/BACKSPACE don't work
	// when composing in new window until we (1) enter some text
	// or (2) resize the window (!).  I chose the latter.
	if (AjxEnv.isGeckoBased && AjxEnv.isWindows) {
		window.resizeBy(1, 1);
	}
};

ZmComposeView.prototype.reEnableDesignMode =
function() {
	if (this._composeMode == DwtHtmlEditor.HTML) {
		this._htmlEditor.reEnableDesignMode();
	}
};

// user just saved draft, update compose view as necessary
ZmComposeView.prototype.processMsgDraft =
function(msgDraft) {
	if (this._isInline(msgDraft)) {
		this._handleInline(msgDraft);
	}
	this.reEnableDesignMode();
	this._action = ZmOperation.DRAFT;
	this._msg = msgDraft;
	this._msgAttId = null;
	// always redo att links since user couldve removed att before saving draft
	this.cleanupAttachments(true);
	this._showForwardField(msgDraft, ZmOperation.DRAFT);
	this._resetBodySize();
	// save form state (to check for change later)
	this._setFormValue();
};

ZmComposeView.prototype._handleInline =
function(msgObj) {
	var msg = (msgObj) ? msgObj : this._msg;
	var iDoc = this._htmlEditor._getIframeDoc();
	return (this._fixMultipartRelatedImages(msg,iDoc));
};

ZmComposeView.prototype._fixMultipartRelatedImages_onTimer =
function(msg, account) {
	// first time the editor is initialized, idoc.getElementsByTagName("img") is empty
	// Instead of waiting for 500ms, trying to add this callback. Risky but works.
	if (!this._firstTimeFixImages) {
		this._htmlEditor.addOnContentInitializedListener(new AjxCallback(this, this._fixMultipartRelatedImages, [msg, this._htmlEditor._getIframeDoc(), account]));
	} else {
		this._fixMultipartRelatedImages(msg, this._htmlEditor._getIframeDoc(), account);
	}
};

/**
 * Twiddle the img tags so that the HTML editor can display the images. Instead of
 * a cid (which is relevant only within the MIME msg), point to the img with a URL.
 * 
 * @private
 */
ZmComposeView.prototype._fixMultipartRelatedImages =
function(msg, idoc, account) {
	if (!this._firstTimeFixImages) {
		this._htmlEditor.removeOnContentInitializedListener();
		var self = this; // Fix possible hiccups during compose in new window
		setTimeout(function() {
				self._fixMultipartRelatedImages(msg, self._htmlEditor._getIframeDoc(), account);
		}, 10);
		this._firstTimeFixImages = true;
		return;
	}

	idoc = idoc || this._htmlEditor._getIframeDoc();
	if (!idoc) { return; }

    var showImages = false;
    if(msg) {
        var addr = msg.getAddress(AjxEmailAddress.FROM) || ZmMsg.unknown;
        var sender = msg.getAddress(AjxEmailAddress.SENDER); // bug fix #10652 - check invite if sentBy is set (means on-behalf-of)
        var sentBy = (sender && sender.address) ? sender : addr;
        var sentByAddr = sentBy && sentBy != ZmMsg.unknown ? sentBy.getAddress() : null;
        if (sentByAddr) {
            msg.sentByAddr = sentByAddr;
            msg.sentByDomain = sentByAddr.substr(sentByAddr.indexOf("@")+1);
            showImages = this._isTrustedSender(msg);
        }
    }
    
	var images = idoc.getElementsByTagName("img");
	var num = 0;
	for (var i = 0; i < images.length; i++) {
		var dfsrc = images[i].getAttribute("dfsrc") || images[i].getAttribute("mce_src") || images[i].src;
		if (dfsrc) {
			if (dfsrc.substring(0,4) == "cid:") {
				num++;
				var cid = "<" + dfsrc.substring(4).replace("%40","@") + ">";
				var src = msg && msg.getContentPartAttachUrl(ZmMailMsg.CONTENT_PART_ID, cid);
				if (src) {
                    //Cache cleared, becoz part id's may change.
				    src = src + "&t=" + (new Date()).getTime();
					images[i].src = src;
					images[i].setAttribute("dfsrc", dfsrc);
				}
			} else if (dfsrc.substring(0,4) == "doc:") {
				images[i].src = [appCtxt.get(ZmSetting.REST_URL, null, account), ZmFolder.SEP, dfsrc.substring(4)].join('');
			} else if (dfsrc.indexOf("//") == -1) { // check for content-location verison
				var src = msg && msg.getContentPartAttachUrl(ZmMailMsg.CONTENT_PART_LOCATION, dfsrc);
				if (src) {
                    //Cache cleared, becoz part id's may change.
					src = src + "&t=" + (new Date()).getTime();
					num++;
					images[i].src = src;
					images[i].setAttribute("dfsrc", dfsrc);
				}
			}
            else if(showImages) {
                var src = dfsrc;//x + "&t=" + (new Date()).getTime();
                num++;
                images[i].src = src;
                images[i].setAttribute("dfsrc", dfsrc);
            }
		}
	}
	AjxDebug.println(AjxDebug.REPLY, "ZmComposeView::_fixMultipartRelatedImages - num images: " + images.length);
	return (num == images.length);
};

ZmComposeView.prototype._isTrustedSender =
function(msg) {
    var trustedList = this.getTrustedSendersList();
    if (trustedList.contains(msg.sentByAddr) || trustedList.contains(msg.sentByDomain)){
        return true;
    }
    return false;
};

ZmComposeView.prototype.getTrustedSendersList =
function() {
    return this._controller.getApp().getTrustedSendersList();
};

ZmComposeView.prototype._cleanupFileRefImages =
function(idoc){

    function removeImg(img){
        var parent = img.parentNode;
        parent.removeChild(img);
    }

    if (idoc) {
		var images = idoc.getElementsByTagName("img");
		var len = images.length, fileImgs=[], img, src;
        for (var i = 0; i < images.length; i++) {
            img = images[i];
            src = img.src;
            if(img && src.indexOf('file://') == 0){
                removeImg(img);
                i--; //removeImg reduces the images.length by 1.
            }

        }
    }
};

/**
 * Change the src tags on inline img's to point to cid's, which is what we
 * want for an outbound MIME msg.
 */
ZmComposeView.prototype._restoreMultipartRelatedImages =
function(idoc) {
	if (idoc) {
		var images = idoc.getElementsByTagName("img");
		var num = 0;
		for (var i = 0; i < images.length; i++) {
			var img = images[i];
			var cid = "";
			var dfsrc = img.getAttribute("dfsrc") || img.getAttribute("mce_src");
			if (dfsrc && dfsrc.indexOf("cid:") == 0) {
				cid = dfsrc;
				img.removeAttribute("dfsrc");
			} else if (img.src && img.src.indexOf("cid:") == 0) {
				cid = img.src;
			} else if ( dfsrc && dfsrc.substring(0,4) == "doc:"){
				cid = "cid:"+this._generateCid();
				img.removeAttribute("dfsrc");
				img.setAttribute("doc", dfsrc.substring(4, dfsrc.length));
			} else {
				// If "Display External Images" is false then handle Reply/Forward
				if (dfsrc && (!this._msg || this._msg.showImages))
					//IE: Over HTTPS, http src urls for images might cause an issue.
					try{ img.src = dfsrc; }catch(ex){};
				}
			if (cid) {
				img.setAttribute("dfsrc", cid);
			}
		}
	}
};

ZmComposeView.prototype._cleanupSignatureIds =
function(idoc){
	var signatureId = this._controller._currentSignatureId;
	var signatureEl = idoc.getElementById(signatureId);
	if (signatureEl) {
		signatureEl.removeAttribute("id");
	}
};

ZmComposeView.prototype.showAttachmentDialog =
function() {
	var attachDialog = this._attachDialog = appCtxt.getAttachDialog();
	var callback = new AjxCallback(this, this._attsDoneCallback, [true]);
	attachDialog.setUploadCallback(callback);
	attachDialog.popup();
	attachDialog.enableInlineOption(this._composeMode == DwtHtmlEditor.HTML);
};

/**
 * Revert compose view to a clean state (usually called before popping compose view).
 * 
 * @param	{Boolean}	bEnableInputs		if <code>true</code>, enable the input fields
 */
ZmComposeView.prototype.reset =
function(bEnableInputs) {

	this.backupForm = null;
	this.sendUID = null;

	// reset autocomplete list
	if (this._acAddrSelectList) {
		this._acAddrSelectList.reset();
		this._acAddrSelectList.show(false);
	}

	// reset To/CC/BCC fields
	for (var i = 0; i < ZmMailMsg.COMPOSE_ADDRS.length; i++) {
		var type = ZmMailMsg.COMPOSE_ADDRS[i];
		var textarea = this._field[type];
		textarea.value = "";
		this._adjustAddrHeight(textarea, true);
		if (this._useAcAddrBubbles) {
			var addrInput = this._addrInputField[type];
			if (addrInput) {
				addrInput.clear();
			}
		}
	}

	// reset subject / body fields
	this._subjectField.value = "";
	this.updateTabTitle();

	this._htmlEditor.resetSpellCheck();
	this._htmlEditor.clear();

	// the div that holds the attc.table and null out innerHTML
	this.cleanupAttachments(true);

	this._resetBodySize();
	this._controller._curIncOptions = null;
	this._msgAttId = null;
	this._clearFormValue();

	// reset dirty shields
	this._noSubjectOkay = this._badAddrsOkay = this._spellCheckOkay = false;

	Dwt.setVisible(this._oboRow, false);

	// remove extra mime parts
	this._extraParts = null;

	// enable/disable input fields
	this.enableInputs(bEnableInputs);

	// reset state of the spell check button
	this._controller.toggleSpellCheckButton(false);

	//reset state of previous Signature cache variable.
	this._previousSignature = null;
	this._previousSignatureMode = null;

	// used by drafts handling in multi-account
	this._origAcctMsgId = null;
};

ZmComposeView.prototype.enableInputs =
function(bEnable) {
	// disable input elements so they dont bleed into top zindex'd view
	for (var i = 0; i < ZmMailMsg.COMPOSE_ADDRS.length; i++) {
		this._field[ZmMailMsg.COMPOSE_ADDRS[i]].disabled = !bEnable;
	}
	this._subjectField.disabled = this._bodyField.disabled = !bEnable;
};

/**
 * Adds an extra MIME part to the message. The extra parts will be
 * added, in order, to the end of the parts after the primary message
 * part.
 * 
 * @private
 */
ZmComposeView.prototype.addMimePart =
function(mimePart) {
	if (!this._extraParts) {
		this._extraParts = [];
	}
	this._extraParts.push(mimePart);
};

ZmComposeView.prototype.getSignatureContentSpan =
function(signature, sigContent, account) {
	signature = signature || this.getSignatureById(this._controller.getSelectedSignature(), account);
	if (!signature) { return ""; }

	var signatureId = signature.id;
	sigContent = sigContent || this.getSignatureContent(signatureId);
	if (this.getHtmlEditor().getMode() == DwtHtmlEditor.HTML) {
		sigContent = ["<div id=\"", signatureId, "\">", sigContent, "</div>"].join('');
	}

	return this._getSignatureSeparator() + sigContent;
};

ZmComposeView.prototype._attachSignatureVcard =
function(signatureId) {

	var signature = this.getSignatureById(signatureId);
	if (signature && signature.contactId) {
        if (!this._msg){
            this._msg = new ZmMailMsg();
        }
        if (this._msg._contactAttIds)
            this._msg._contactAttIds.push(signature.contactId);
        else
            this._msg.setContactAttIds(signature.contactId);
        
        //come back later and see if we need to save the draft
        AjxTimedAction.scheduleAction(new AjxTimedAction(this, this._checkSaveDraft), 500);
	}
};

ZmComposeView.prototype._checkSaveDraft =
function(){
    if (this._msg && this._msg._contactAttIds && this._msg._contactAttIds.length > 0) {
        this._controller.saveDraft(ZmComposeController.DRAFT_TYPE_MANUAL, null, null, null, this._msg._contactAttIds);
    }
};

/**
 * Called when the user selects something from the Signature menu.
 *
 * @param {String}	content				the content
 * @param {String}	replaceSignatureId	the signature id
 * @param {ZmZimbraAccount}	account				the account
 * 
 * @private
 */
ZmComposeView.prototype.applySignature =
function(content, oldSignatureId, account, newSignatureId, skipSave) {

	content = content || "";
	var ac = window.parentAppCtxt || window.appCtxt;
	var acct = account || (appCtxt.multiAccounts && this.getFromAccount());
	var signature = (newSignatureId !== null) ? this.getSignatureById(newSignatureId || this._controller.getSelectedSignature(), acct) : null;
	var isHtml = this.getHtmlEditor().getMode() == DwtHtmlEditor.HTML;
	var newLine = this._getSignatureNewLine();
	var isAbove = ac.get(ZmSetting.SIGNATURE_STYLE, null, acct) == ZmSetting.SIG_OUTLOOK;
	var done = false, donotsetcontent = false;
	var noSignature = !signature;

	var sigContent, replaceSignature;
	var newSig = signature ? this.getSignatureContent(signature.id) : "";

	if (oldSignatureId) {
		if (isHtml) {
			var idoc = this.getHtmlEditor()._getIframeDoc();
			var sigEl = idoc.getElementById(oldSignatureId);
			if (sigEl) {
				var newSigContent = this._replaceSignature(sigEl.innerHTML, newSig || "");
				if (newSigContent) {
					sigEl.innerHTML = newSigContent;

					if (signature) {
						sigEl.id = signature.id;
					} else {
						sigEl.removeAttribute("id");
					}
					done = true;
					donotsetcontent = true;
				}
			}
		} else {
			var sigContent = this.getSignatureContent(oldSignatureId);
			var oldSignature = this.getSignatureById(oldSignatureId);
			replaceSignature = (oldSignature && (oldSignature.getContentType() == ZmMimeTable.TEXT_HTML)) ?
				AjxStringUtil.convertHtml2Text(sigContent, {"#text": ZmComposeView._convertTextNode}) : sigContent;
			var sigIndex = content.indexOf(replaceSignature);
			var sigLength = replaceSignature && replaceSignature.length || 0;

			if (sigIndex != -1) {
				var contentBefore = content.substring(0, sigIndex).replace(/\s+$/,""); // Get the message content before the signature and cut off any trailing whitespace
				var contentAfter = content.substring(sigIndex + sigLength).replace(/^\s+/,""); // Get the message content after the signature and cut off any leading whitespace

				if (contentAfter) {
					newSig += "\n";
				}
				content = contentBefore + this._getSignatureSeparator() + (newSig || "\n") + contentAfter;
				done = true;
			}
		}
	}
	if (!done) {
		sigContent = this.getSignatureContentSpan(signature);
		content = this._insertSignature(content, ac.get(ZmSetting.SIGNATURE_STYLE, null, acct), sigContent, newLine);
	}

	if (!isHtml) {
		AjxTimedAction.scheduleAction(new AjxTimedAction(this, function() { this.getHtmlEditor().moveCaretToTop(); }), 200);
	}

	if (!donotsetcontent) {
		this._htmlEditor.setContent(content);
	}
	this._fixMultipartRelatedImages_onTimer(this._msg, account);

	//Caching previous Signature state.
	this._previousSignature = signature;
	this._previousSignatureMode = this._htmlEditor.getMode();

	var hadVcard = false;
	if (oldSignatureId) {
		// uncheck box for previous vcard att so it gets removed
		var oldSig = this.getSignatureById(oldSignatureId);
		if (oldSig && oldSig.contactId) {
			var vcardPart;
			var atts = this._msg && this._msg.attachments;
			if (atts && atts.length) {
               //we need to figure out what input to uncheck
               if (appCtxt.cacheGet(oldSig.contactId) && appCtxt.cacheGet(oldSig.contactId) instanceof ZmContact) {
                   var sigContact = appCtxt.cacheGet(oldSig.contactId);

                }
				for (var i = 0; i < atts.length && !vcardPart; i++) {
					if (atts[i].ct == ZmMimeTable.TEXT_VCARD) {
                        //we may have multiple vcards, determine which one to remove based on signature in cache
                        if (sigContact) {
                            var name = atts[i].filename.substring(0, atts[i].filename.length -4)
                            if (name == sigContact._fileAs)
                                vcardPart = atts[i].part
                        }
                        else {
						    vcardPart = atts[i].part;
                        }
					}
				}
			}
			var inputs = document.getElementsByName(ZmComposeView.FORWARD_ATT_NAME + this._sessionId);
			if (inputs && inputs.length) {
				for (var i = 0; i < inputs.length; i++) {
					if (inputs[i].value == vcardPart) {
						inputs[i].checked = false;
						hadVcard = true;
					}
				}
			}
		}
	}
	if (signature && signature.contactId) {
		this._attachSignatureVcard(signature.id);
	}
	else if (hadVcard && !skipSave) {
		this._controller.saveDraft(ZmComposeController.DRAFT_TYPE_MANUAL);
	}
};

/*
 * Replaces the contents of a HTML signature block with the contents of another
*/
ZmComposeView.prototype._replaceSignature = 
function(oldSigContent, newSigContent) {
	// find old sig via delimiters, so we preserve any user content that made it into sig span
	var idx = oldSigContent.indexOf(ZmComposeView.SIG_KEY);
	var lastIdx = oldSigContent.lastIndexOf(ZmComposeView.SIG_KEY);
	if (idx == -1 || lastIdx == -1) {
		idx = oldSigContent.indexOf(ZmComposeView.SIG_KEY_LC);
		lastIdx = oldSigContent.lastIndexOf(ZmComposeView.SIG_KEY_LC);
	}
	if (idx != -1 && lastIdx != -1) {
		var nIdx = newSigContent.indexOf(ZmComposeView.SIG_KEY);
		if (nIdx != -1)
			newSigContent = newSigContent.substring(nIdx);
		var nLastIdx = newSigContent.lastIndexOf(ZmComposeView.SIG_KEY);
		if (nLastIdx != -1)
			newSigContent = newSigContent.substring(0, nLastIdx + ZmComposeView.SIG_KEY.length);
		
		newSigContent = oldSigContent.substring(0, idx) + newSigContent +
						oldSigContent.substring(lastIdx + ZmComposeView.SIG_KEY.length);
	}
	return newSigContent;
};

/*
 * Convertor for text nodes that, unlike the one in AjxStringUtil._traverse, doesn't append spaces to the results
*/
ZmComposeView._convertTextNode =
function(el, ctxt) {
	if (el.nodeValue.search(AjxStringUtil._NON_WHITESPACE) != -1) {
		if (ctxt.lastNode == "ol" || ctxt.lastNode == "ul") {
			return "\n";
		}
		if (ctxt.isPreformatted) {
			return AjxStringUtil.trim(el.nodeValue);
		} else {
			return AjxStringUtil.trim(el.nodeValue.replace(AjxStringUtil._LF, ""));
		}
	}
};

ZmComposeView.prototype.getSignatureContent =
function(signatureId) {
	return this._getSignature(signatureId) || "";
};

/**
 * Adds the user's signature to the message body. An "internet" style signature
 * is prefixed by a special line and added to the bottom. An "outlook" style
 * signature is added before quoted content.
 *
 * This method is only used to add an
 *
 * @content 			optional content to use
 * 
 * @private
 */
ZmComposeView.prototype.addSignature =
function(content) {
	// bug fix #6821 - we need to pass in "content" param
	// since HTML composing in new window doesnt guarantee the html editor
	// widget will be initialized when this code is running.
	content = content || "";
	var ac = window.parentAppCtxt || window.appCtxt;
	var sigContent = this.getSignatureContentSpan();
	var account = appCtxt.multiAccounts && this.getFromAccount();
	content = this._insertSignature(content, ac.get(ZmSetting.SIGNATURE_STYLE, null, account),
									sigContent,
									this._getSignatureNewLine());

	this._htmlEditor.setContent(content);

	this._previousSignature = sigContent;
	this._previousSignatureMode = this._htmlEditor.getMode();
};

ZmComposeView.prototype._insertSignature =
function(content, sigStyle, sig, newLine) {

	var re_newlines = "(" + AjxStringUtil.regExEscape(newLine) + ")*";
	// get rid of all trailing newlines
	var re = re_newlines;
	if (this.getHtmlEditor().getMode() == DwtHtmlEditor.HTML) {
		re += "</body></html>";
	}
	re += "$";
	re = new RegExp(re, "i");
	content = content.replace(re, '');

	var what = this._controller._curIncOptions && this._controller._curIncOptions.what;
	var hasQuotedContent = (what != ZmSetting.INC_ATTACH && what != ZmSetting.INC_NONE);

	if (sigStyle == ZmSetting.SIG_OUTLOOK && hasQuotedContent) {
		var preface = this._getPreface();
		var re_preface = AjxStringUtil.regExEscape(preface).replace(/\\\"/g,"\\\"?"); // IE has a funny idea of forgetting to put quotes around html attributes, so we make the quotes optional in the regex
		var regexp = new RegExp(re_newlines + re_preface, "i");
		if (content.match(regexp)) {
			content = content.replace(regexp, [sig, newLine, preface].join(""));
		} else {
			// new message
			content = [content, sig].join("");
		}
	} else {
		content = [content, sig].join("");
	}

	return content;
};

ZmComposeView.prototype.dispose =
function() {
	if (this._identityChangeListenerObj) {
		var collection = appCtxt.getIdentityCollection();
		collection.removeChangeListener(this._identityChangeListenerObj);
	}
	DwtComposite.prototype.dispose.call(this);
};

ZmComposeView.prototype.getSignatureById =
function(signatureId, account) {
	signatureId = signatureId || this._controller.getSelectedSignature();
	return appCtxt.getSignatureCollection(account).getById(signatureId);
};

// So we can delimit the actual sig content when switching sigs (bug 46871)
ZmComposeView.SIG_KEY = '<SPAN name="x"></SPAN>';
ZmComposeView.SIG_KEY_LC = ZmComposeView.SIG_KEY.toLowerCase();

ZmComposeView.prototype._getSignature =
function(signatureId) {
	var extraSignature = this._getExtraSignature();
	signatureId = signatureId || this._controller.getSelectedSignature();

	if (!signatureId && extraSignature == "") { return; }

	var signature;

	// for multi-account, search all accounts for this signature ID
	if (appCtxt.multiAccounts) {
		var ac = window.parentAppCtxt || window.appCtxt;
		var list = ac.accountList.visibleAccounts;
		for (var i = 0; i < list.length; i++) {
			var collection = appCtxt.getSignatureCollection(list[i]);
			if (collection) {
				signature = collection.getById(signatureId);
				if (signature) {
					break;
				}
			}
		}
	} else {
		signature = appCtxt.getSignatureCollection().getById(signatureId);
	}

	if (!signature && extraSignature == "") { return; }

	var sigString = "";
	if (signature) {
		var htmlMode = (this._composeMode == DwtHtmlEditor.HTML);
		var mode = htmlMode ? ZmMimeTable.TEXT_HTML : ZmMimeTable.TEXT_PLAIN;
		var sig = signature.getValue(mode);
		var sig1 = htmlMode ? ZmComposeView.SIG_KEY + sig + ZmComposeView.SIG_KEY : sig;
		sigString = sig1 + this._getSignatureNewLine();
	}
	return (sigString + extraSignature);
};

/**
 * Returns "" or extra signature(like a quote or legal disclaimer) via zimlet
 */
ZmComposeView.prototype._getExtraSignature =
function() {
	var extraSignature = "";
	if (appCtxt.zimletsPresent()) {
		var buffer = [];
		appCtxt.notifyZimlets("appendExtraSignature", [buffer]);
		extraSignature = buffer.join(this._getSignatureNewLine());
		if (extraSignature != "") {
			extraSignature = this._getSignatureNewLine() + extraSignature;
		}
	}
	return extraSignature;
};

ZmComposeView.prototype._getSignatureSeparator =
function() {
	var ac = window.parentAppCtxt || window.appCtxt;
	var newLine = this._getSignatureNewLine();
	var sep = newLine + newLine;
	var account = appCtxt.multiAccounts && this.getFromAccount();
	if (ac.get(ZmSetting.SIGNATURE_STYLE, null, account) == ZmSetting.SIG_INTERNET) {
		sep += "-- " + newLine;
	}
	return sep;
};

ZmComposeView.prototype._getSignatureNewLine =
function() {
	return ((this._composeMode == DwtHtmlEditor.HTML) ? "<br>" : "\n");
};

ZmComposeView.prototype._getSignatureIdForAction =
function(identity, action) {

	identity = identity || this.getIdentity();
	action = action || this._action;
	var field = (this._isReply(action) || this._isForward(action)) ? ZmIdentity.REPLY_SIGNATURE : ZmIdentity.SIGNATURE;
	return identity && identity.getField(field);
};

/**
* Returns true if form contents have changed, or if they are empty.
*
* @param incAddrs		takes addresses into consideration
* @param incSubject		takes subject into consideration
* 
* @private
*/
ZmComposeView.prototype.isDirty =
function(incAddrs, incSubject) {
	// reply/forward and empty body => not dirty
	if ((this._action != ZmOperation.NEW_MESSAGE) && (this._htmlEditor.getContent().match(ZmComposeView.EMPTY_FORM_RE))) {
		return false;
	}

	if (this._isDirty) { return true; }

	var curFormValue = this._formValue(incAddrs, incSubject);

	// empty subject and body => not dirty
	if (curFormValue.match(ZmComposeView.EMPTY_FORM_RE) ||
		(this._composeMode == DwtHtmlEditor.HTML &&
		 (curFormValue == "<html><body></body></html>" ||
		  curFormValue == "<html><body><br></body></html>" ||
		  curFormValue == '<html><body><br mce_bogus="1"></body></html>')))
	{
		return false;
	}

	if (this._composeMode == DwtHtmlEditor.HTML) {
		var lower = function(match) {
			return match.toLowerCase();
		};

		var origFormValue = AjxStringUtil.trim(this._origFormValue.replace(ZmComposeView.HTML_TAG_RE, lower)); // Make sure HTML tag names & parameters are lowercase for both values in the comparison
		curFormValue = AjxStringUtil.trim(curFormValue.replace(ZmComposeView.HTML_TAG_RE, lower)); // so that <SPAN> and <span> are still equal, but <span>FOO</span> and <span>foo</span> are not.

		return (curFormValue != origFormValue);
	}
	return (curFormValue != this._origFormValue);
};

ZmComposeView.prototype.cleanupAttachments =
function(all) {
	var attachDialog = this._attachDialog;
	if (attachDialog && attachDialog.isPoppedUp()) {
		attachDialog.popdown();
	}

	if (all) {
		this._attcDiv.innerHTML = "";
		this._attcDiv.style.height = "";
		this._attachCount = 0;
	}

	// make sure att IDs don't get reused
	if (this._msg) {
		this._msg.attId = null;
        this._msg._contactAttIds = [];
	}
};

ZmComposeView.prototype.sendMsgOboIsOK =
function() {
	return (Dwt.getVisible(this._oboRow)) ? this._oboCheckbox.checked : false;
};

ZmComposeView.prototype.updateTabTitle =
function() {
	var buttonText = this._subjectField.value
		? this._subjectField.value.substr(0, ZmAppViewMgr.TAB_BUTTON_MAX_TEXT)
		: ZmComposeController.DEFAULT_TAB_TEXT;
	appCtxt.getAppViewMgr().setTabTitle(this._controller.viewId, buttonText);
};

/**
 * Used in multi-account mode to determine which account this composer is
 * belongs to.
 */
ZmComposeView.prototype.getFromAccount =
function() {
	var ac = window.parentAppCtxt || window.appCtxt;
	return this._fromSelect
		? (ac.accountList.getAccount(this._fromSelect.getSelectedOption().accountId))
		: (ac.accountList.defaultAccount || ac.accountList.activeAccount || ac.accountList.mainAccount);
};

// Private / protected methods

ZmComposeView.prototype._isInviteReply =
function(action) {
	action = action || this._action;
	return (action == ZmOperation.REPLY_ACCEPT ||
			action == ZmOperation.REPLY_CANCEL ||
			action == ZmOperation.REPLY_DECLINE ||
			action == ZmOperation.REPLY_TENTATIVE ||
			action == ZmOperation.REPLY_MODIFY ||
			action == ZmOperation.REPLY_NEW_TIME);
};

ZmComposeView.prototype._isReply =
function(action) {
	action = action || this._action;
	return (action == ZmOperation.REPLY ||
			action == ZmOperation.REPLY_ALL ||
			this._isCalReply(action) ||
			this._isInviteReply(action));
};

ZmComposeView.prototype._isCalReply =
function(action) {
	action = action || this._action;
	return (action == ZmOperation.CAL_REPLY ||
			action == ZmOperation.CAL_REPLY_ALL ||
			this._isInviteReply(action));
};

ZmComposeView.prototype._isForward =
function(action) {
	action = action || this._action;
	return (action == ZmOperation.FORWARD_INLINE ||
			action == ZmOperation.FORWARD_ATT);
};

// returns the text part given a body part (if body part is HTML, converts it to text)
ZmComposeView.prototype._getTextPart =
function(bodyPart, encodeSpace) {
	var text = "";
	// if the only content type returned is html, convert to text
	if (bodyPart.ct == ZmMimeTable.TEXT_HTML) {
		// create a temp iframe to create a proper DOM tree
		var params = {parent: this, hidden: true, html: bodyPart.content};
		var dwtIframe = new DwtIframe(params);
		if (dwtIframe) {
			var self = this;
			var convertor = {"hr":
				function(el) {
					return ZmComposeView._convertHtmlPreface(self, el);
				}
			}
			text = AjxStringUtil.convertHtml2Text(dwtIframe.getDocument().body, convertor);
			var dwtEl = this.getHtmlElement().removeChild(dwtIframe.getHtmlElement());
			delete dwtEl;
			delete dwtIframe;
		}
	} else {
		text = encodeSpace
			? AjxStringUtil.convertToHtml(bodyPart.content)
			: bodyPart.content;
	}

	return text;
};

// Consistent spot to locate various dialogs
ZmComposeView.prototype._getDialogXY =
function() {
	var loc = Dwt.toWindow(this.getHtmlElement(), 0, 0);
	return new DwtPoint(loc.x + ZmComposeView.DIALOG_X, loc.y + ZmComposeView.DIALOG_Y);
};

ZmComposeView.prototype._getForwardAttIds =
function(name) {
	var forAttIds = [];
	var forAttList = document.getElementsByName(name);

	// walk collection of input elements
	for (var i = 0; i < forAttList.length; i++) {
		if (forAttList[i].checked) {
			forAttIds.push(forAttList[i].value);
		}
	}

	return forAttIds;
};

/**
 * a callback that's called when bubbles are added or removed, since we need to resize the msg body in those cases.
 */
ZmComposeView.prototype._bubblesChangedCallback =
function() {
	if (!this._useAcAddrBubbles) { return; }
	this._resetBodySize(); // body size might change due to change in size of address field (due to new bubbles).
};

ZmComposeView.prototype._bubbleMenuCreated =
function(addrInput, menu) {

	if (!this._useAcAddrBubbles) { return; }

	this._bubbleActionMenu = menu;

	menu.addOp(ZmOperation.SEP);
	var ops = [ZmOperation.MOVE_TO_TO, ZmOperation.MOVE_TO_CC, ZmOperation.MOVE_TO_BCC];
	var listener = new AjxListener(this, this._bubbleMove);
	for (var i = 0; i < ops.length; i++) {
		menu.addOp(ops[i]);
		menu.addSelectionListener(ops[i], listener);
	}
};

ZmComposeView.prototype._bubbleMenuResetOperations =
function(addrInput, menu) {
	var sel = addrInput.getSelection();
	var ops = [ZmOperation.MOVE_TO_TO, ZmOperation.MOVE_TO_CC, ZmOperation.MOVE_TO_BCC];
	for (var i = 0; i < ops.length; i++) {
		var op = ops[i];
		var type = ZmComposeView.MOVE_TO_FIELD[op];
		menu.enable(op, sel.length > 0 && (type != addrInput.type));
	}
};

ZmComposeView.prototype._bubbleMove =
function(ev) {

	var sourceInput = ZmAddressInputField.menuContext.addrInput;
	var op = ev && ev.item && ev.item.getData(ZmOperation.KEY_ID);
	var type = ZmComposeView.MOVE_TO_FIELD[op];
	var targetInput = this._addrInputField[type];
	if (sourceInput && targetInput) {
		var sel = sourceInput.getSelection();
		if (sel.length) {
			for (var i = 0; i < sel.length; i++) {
				var bubble = sel[i];
				this._showAddressField(type, true);
				targetInput.addBubble({bubble:bubble});
				sourceInput.removeBubble(bubble.id);
			}
		}
	}
};

ZmComposeView.prototype._acCompHandler =
function(text, el, match) {
	if (this._useAcAddrBubbles) { return; }
	this._adjustAddrHeight(el);
};

ZmComposeView.prototype._acKeyupHandler =
function(ev, acListView, result, element) {
	var key = DwtKeyEvent.getCharCode(ev);
	// process any printable character or enter/backspace/delete keys
	if (result && element && (ev.inputLengthChanged ||
		(key == 3 || key == 13 || key == 8 || key == 46 ||
		(AjxEnv.isMac && key == 224)))) // bug fix #24670
	{
		this._adjustAddrHeight(element);
	}
};

ZmComposeView.prototype._adjustAddrHeight =
function(textarea, skipResetBodySize) {

	if (this._useAcAddrBubbles || !textarea) { return; }

	if (textarea.value.length == 0) {
		textarea.style.height = "21px";

		if (AjxEnv.isIE) {
			// for IE use overflow-y
			textarea.style.overflowY = "hidden";
		}
		else {
			textarea.style.overflow = "hidden";
		}

		if (!skipResetBodySize) {
			this._resetBodySize();
		}

		return;
	}

	var sh = textarea.scrollHeight;
	if (sh > textarea.clientHeight) {
		var taHeight = parseInt(textarea.style.height) || 0;
		if (taHeight <= 65) {
			if (sh >= 65) {
				sh = 65;
				if (AjxEnv.isIE)
					textarea.style.overflowY = "scroll";
				else
					textarea.style.overflow = "auto";
			}
			textarea.style.height = sh + 13;
			this._resetBodySize();
		} else {
			if (AjxEnv.isIE) {
				// for IE use overflow-y
				textarea.style.overflowY = "scroll";
			}
			else {
				textarea.style.overflow = "auto";
			}

			textarea.scrollTop = sh;
		}
	}
};

/*
* Set various address headers based on the original message and the mode we're in.
* Make sure not to duplicate any addresses, even across fields.
*/
ZmComposeView.prototype._setAddresses =
function(action, type, override) {
	this._action = action;

	if (action == ZmOperation.NEW_MESSAGE && override) {
		this.setAddress(type, override);
	} else if (this._isReply(action)) {
		var ac = window.parentAppCtxt || window.appCtxt;

		// Prevent user's login name and aliases from going into To: or Cc:
		var used = {};
		var account = appCtxt.multiAccounts && this._msg.getAccount();
		var uname = ac.get(ZmSetting.USERNAME, null, account);
		if (uname) {
			used[uname.toLowerCase()] = true;
		}
		var aliases = ac.get(ZmSetting.MAIL_ALIASES, null, account);
		for (var i = 0, count = aliases.length; i < count; i++) {
			used[aliases[i].toLowerCase()] = true;
		}

		// Check for Canonical Address's
		var defaultIdentity = ac.getIdentityCollection(account).defaultIdentity;
		if (defaultIdentity && defaultIdentity.sendFromAddress) {
			// Note: sendFromAddress is same as appCtxt.get(ZmSetting.USERNAME)
			// if the account does not have any Canonical Address assigned.
			used[defaultIdentity.sendFromAddress.toLowerCase()] = true;
		}

		// When updating address lists, use this._addressesMsg instead of this._msg, because
		// this._msg changes after a draft is saved.
		var addrAdded, addrVec;
		if (!this._addressesMsg.isSent) {
			var isDefaultIdentity = true;
			if(this.identitySelect) {
				var isDefaultIdentity = defaultIdentity.id == this.identitySelect.getValue(); 
			}
			addrVec = this._addressesMsg.getReplyAddresses(action, used, isDefaultIdentity);
			addrAdded = this._addAddresses(AjxEmailAddress.TO, addrVec, used);
			if (action == ZmOperation.REPLY_ALL) {
				for (var i = 0, len = addrVec.size(); i < len; i++) {
					var a = addrVec.get(i).address;
					used[a] = true;
				}
			}
		} else if (action == ZmOperation.REPLY) {
			addrVec = this._addressesMsg.getAddresses(AjxEmailAddress.TO);
			addrAdded = this._addAddresses(AjxEmailAddress.TO, addrVec);
		}
		if (!addrAdded && addrVec && addrVec.size()) {
			// make sure we have at least one TO address if possible
			this._addAddresses(AjxEmailAddress.TO, addrVec.slice(0, 1));
		}

		// reply to all senders if reply all (includes To: and Cc:)
		if (action == ZmOperation.REPLY) {
			this.setAddress(AjxEmailAddress.CC, "");
		} else if (action == ZmOperation.REPLY_ALL) {
			var addrs = new AjxVector();
			addrs.addList(this._addressesMsg.getAddresses(AjxEmailAddress.CC));
			var toAddrs = this._addressesMsg.getAddresses(AjxEmailAddress.TO);
			if (this._addressesMsg.isSent) {
				// sent msg replicates To: and Cc: (minus duplicates)
				this._addAddresses(AjxEmailAddress.TO, toAddrs, used);
			} else {
				addrs.addList(toAddrs);
			}
			this._addAddresses(AjxEmailAddress.CC, addrs, used);
		}
	} else if (action == ZmOperation.DRAFT || action == ZmOperation.SHARE) {
		for (var i = 0; i < ZmMailMsg.COMPOSE_ADDRS.length; i++) {
			var addrs = this._msg.getAddresses(ZmMailMsg.COMPOSE_ADDRS[i]);
			this._addAddresses(ZmMailMsg.COMPOSE_ADDRS[i], addrs);
		}
	} else if(action == ZmOperation.DECLINE_PROPOSAL) {
        var toAddrs = this._addressesMsg.getAddresses(AjxEmailAddress.FROM);
		this._addAddresses(AjxEmailAddress.TO, toAddrs);
    }
};

// Adds the given addresses to the form. If we're using address bubbles, we need to add each
// address separately in case it's a DL.
ZmComposeView.prototype._addAddresses =
function(type, addrVec, used) {

	var addrAdded = false;
	used = used || {};
	var addrList = [];
	var addrs = addrVec && addrVec.getArray();
	if (addrs && addrs.length) {
		for (var i = 0, len = addrs.length; i < len; i++) {
			var addr = addrs[i];
			var email = addr.isAjxEmailAddress ? addr && addr.getAddress() : addr;
			if (!email) { continue; }
			email = email.toLowerCase();
			if (!used[email]) {
				if (this._useAcAddrBubbles) {
					this.setAddress(type, addr);	// add the bubble now
				}
				else {
					addrList.push(addr);
				}
				used[email] = true;
				addrAdded = true;
			}
		}
		if (!this._useAcAddrBubbles) {
			// calls implicit toString() on each addr object
			var addrStr = addrList.join(AjxEmailAddress.SEPARATOR);
			this.setAddress(type, addrStr);
		}
	}
	return addrAdded;
};

ZmComposeView.prototype._setObo =
function(obo) {
	Dwt.setVisible(this._oboRow, true);
	this._oboCheckbox.checked = true;
	this._oboLabel.innerHTML = AjxMessageFormat.format(ZmMsg.sendObo, obo);
};

ZmComposeView.prototype._setSubject =
function(action, msg, subjOverride) {
	if ((action == ZmOperation.NEW_MESSAGE && subjOverride == null)) {
		return;
	}

	var subj = subjOverride || ( (msg) ? msg.subject : "" );

	if (action == ZmOperation.REPLY_CANCEL && !subj) {
		var inv = (msg) ? msg.invite : null;
		if (inv) {
			subj = inv.getName();
		}
	}

	if (action != ZmOperation.DRAFT && subj) {
		var regex = ZmComposeView.SUBJ_PREFIX_RE;
		while (regex.test(subj))
			subj = subj.replace(regex, "");
	}

	var prefix = "";
	switch (action) {
		case ZmOperation.REPLY:
		case ZmOperation.REPLY_ALL: 		prefix = "Re: "; break;
		case ZmOperation.REPLY_CANCEL: 		prefix = ZmMsg.cancelled + ": "; break;
		case ZmOperation.FORWARD_INLINE:
		case ZmOperation.FORWARD_ATT: 		prefix = "Fwd: "; break;
		case ZmOperation.REPLY_ACCEPT:		prefix = ZmMsg.subjectAccept + ": "; break;
		case ZmOperation.REPLY_DECLINE:		prefix = ZmMsg.subjectDecline + ": "; break;
		case ZmOperation.REPLY_TENTATIVE:	prefix = ZmMsg.subjectTentative + ": "; break;
		case ZmOperation.REPLY_NEW_TIME:	prefix = ZmMsg.subjectNewTime + ": "; break;
	}
	this._subjectField.value = prefix + (subj || "");
	this.updateTabTitle();
};

ZmComposeView.prototype._setBody =
function(action, msg, extraBodyText) {
	
	AjxDebug.println(AjxDebug.REPLY, "ZmComposeView::_setBody");
	var htmlMode = (this._composeMode == DwtHtmlEditor.HTML);

	var isDraft = (action == ZmOperation.DRAFT);
	if (msg && msg.isInvite() && this._isForward(action)) {
		action = this._action = ZmOperation.FORWARD_INLINE;
	}

	// get reply/forward prefs as necessary
	var incOptions = this._controller._curIncOptions;
    var ac = window.parentAppCtxt || window.appCtxt;
	if (!incOptions) {
		if (this._isReply(action)) {
			incOptions = {what:		ac.get(ZmSetting.REPLY_INCLUDE_WHAT),
						  prefix:	ac.get(ZmSetting.REPLY_USE_PREFIX),
						  headers:	ac.get(ZmSetting.REPLY_INCLUDE_HEADERS)};
		} else if (isDraft) {
			incOptions = {what:			ZmSetting.INC_BODY};
		} else if (action == ZmOperation.FORWARD_INLINE) {
			incOptions = {what:		ZmSetting.INC_BODY,
						  prefix:	ac.get(ZmSetting.FORWARD_USE_PREFIX),
						  headers:	ac.get(ZmSetting.FORWARD_INCLUDE_HEADERS)};
		} else if (action == ZmOperation.FORWARD_ATT) {
			incOptions = {what:		ZmSetting.INC_ATTACH};
		} else if (action == ZmOperation.DECLINE_PROPOSAL) {
			incOptions = {what:		ZmSetting.INC_BODY};
		} else if (action == ZmOperation.NEW_MESSAGE) {
			incOptions = {what:		ZmSetting.INC_NONE};
		} else {
			incOptions = {};
		}
		this._controller._curIncOptions = incOptions;	// pointer, not a copy
	}
	if (incOptions.what == ZmSetting.INC_ATTACH && !this._msg) {
		incOptions.what = ZmSetting.INC_NONE;
	}
	AjxDebug.println(AjxDebug.REPLY, "Inc options: " + [incOptions.what, incOptions.prefix, incOptions.headers].join(" / ")); 
	
	var crlf = htmlMode ? "<br>" : ZmMsg.CRLF;
	var crlf2 = htmlMode ? "<br><br>" : ZmMsg.CRLF2;

	var sigPre = "", body = "", headers = [], preface = "", value = "";

	var bodyInfo = {};
	var what = incOptions.what;
	if (msg && (what == ZmSetting.INC_BODY || what == ZmSetting.INC_SMART)) {
		bodyInfo = this._getBodyContent(msg, htmlMode);
		body = bodyInfo.body;
		AjxDebug.println(AjxDebug.REPLY, "Body length: " + body.length);
		// Bug 7160: Strip off the ~*~*~*~ from invite replies.
		if (this._isInviteReply(action)) {
			body = body.replace(ZmItem.NOTES_SEPARATOR, "");
		}
		if (what == ZmSetting.INC_SMART) {
			if (htmlMode) {
				body = body.replace(this._preface,""); // Remove preface and anything inside <blockquote> tags
				var lastTag = "</blockquote>";
				var fidx = body.indexOf("<blockquote");
				var lidx = body.lastIndexOf(lastTag);
				if (fidx!=-1 && lidx!=-1) {
					body = body.substring(0, fidx) + body.substring(lidx + lastTag.length);
				}
			} else {
				if (this._preface) {
					var idx = body.indexOf(this._preface); // Remove everything after preface
					if (idx > 0) {
						body = body.substr(0, idx);
					}
				}
			}
			AjxDebug.println(AjxDebug.REPLY, "Body length in smart mode: " + body.length);
		}
	}

	var sigStyle, sig, sigId, sigFormat;
	var account = ac.multiAccounts && this.getFromAccount();
	if (ac.get(ZmSetting.SIGNATURES_ENABLED, null, account)) {
		sig = this.getSignatureContentSpan(null, null, account);
		sigStyle = sig && ac.get(ZmSetting.SIGNATURE_STYLE, null, account);
		AjxDebug.println(AjxDebug.REPLY, "Sig style: " + sigStyle);
		sigId = this._controller.getSelectedSignature();
		var signature = this.getSignatureById(sigId);
		sigFormat = signature && signature.getContentType();
	}
	if (sigStyle == ZmSetting.SIG_OUTLOOK) {
		sigPre = (this._composeMode == DwtHtmlEditor.TEXT || sigFormat == ZmMimeTable.TEXT_PLAIN) ? sig + crlf : sig;
	}

	extraBodyText = extraBodyText || "";
	if (htmlMode) {
		extraBodyText = extraBodyText.replace(/\n/g, "<br>");
	}

	var preText;
	if (sigPre) {
		if (extraBodyText) {
			if (htmlMode) {
				var fragment = document.createElement("div");
				fragment.innerHTML = extraBodyText;
				var sigEl = Dwt.byId(sigId, fragment);
				if (sigEl) { // Signature found in extraBodyText. Replace signature content
					sigEl.innerHTML = this._replaceSignature(sigEl.innerHTML, sigPre);
					preText = fragment.innerHTML;
				} else { // Signature not found in extraBodyText. Append signature content
					preText = extraBodyText + sigPre;
				}
			} else { // Remove existing signature from text message
				extraBodyText = extraBodyText.replace(new RegExp(AjxStringUtil.regExEscape(sigPre)+"[\\s\\S]*","i"),"");
				preText = extraBodyText + sigPre;
				preText += crlf;
			}
			this.applySignature(extraBodyText, sigId, account, null, true); // Put in new signature
		} else { // There is no pre-existing text. Just append the signature
			preText = sigPre;
		}
	} else { // No signature, just take the extraBodyText
		preText = extraBodyText;
	}
	AjxDebug.println(AjxDebug.REPLY, "preText: " + AjxStringUtil.htmlEncode(preText));

	if (incOptions.headers && msg) {
		for (var i = 0; i < ZmComposeView.QUOTED_HDRS.length; i++) {
			var hdr = msg.getHeaderStr(ZmComposeView.QUOTED_HDRS[i], htmlMode);
			if (hdr) {
				headers.push(hdr);
			}
		}
	}

	this._msgAttId = null; //clear it in case of switching from "as attachment" back to "include original message" or to "don't include original"

	if (action == ZmOperation.REPLY_CANCEL) {
	  	value += crlf + sigPre;
	} else if (incOptions.what == ZmSetting.INC_NONE) {
		value = preText;
	} else if (incOptions.what == ZmSetting.INC_ATTACH && this._msg) {
		value = preText;
		this._msgAttId = this._msg.id;
	} else {
		var preface = this._preface = this._getPreface();
		var divider = (!body && !headers) ? "" : htmlMode ? preface : preface + crlf;
		AjxDebug.println(AjxDebug.REPLY, "divider: " + AjxStringUtil.htmlEncode(divider));
		var leadingSpace = preText ? "" : crlf2;
		var wrapParams = ZmHtmlEditor.getWrapParams(htmlMode, incOptions);
		wrapParams.preserveReturns = true;
		if (incOptions.what == ZmSetting.INC_BODY) {
			if (isDraft) {
				value = body;
			} else if (htmlMode) {
				var headerText = headers.length ? headers.join(crlf) + crlf2 : "";
				wrapParams.text = isDraft ? body : headerText + body;
				var bodyText = AjxStringUtil.wordWrap(wrapParams);
				AjxDebug.println(AjxDebug.REPLY, "bodyText length: " + bodyText.length);
				value = leadingSpace + preText + divider + bodyText;
				AjxDebug.println(AjxDebug.REPLY, "value length A: " + value.length);
			} else {
				var headerText = "";
				if (headers.length) {
					var text = wrapParams.text = headers.join(crlf) + (incOptions.prefix ? crlf : crlf2);
					wrapParams.len = 120; // headers tend to be longer
					headerText = incOptions.prefix ? AjxStringUtil.wordWrap(wrapParams) : text;
				}
				wrapParams.text = body;
				wrapParams.len = ZmHtmlEditor.WRAP_LENGTH;
				var bodyText = incOptions.prefix ? AjxStringUtil.wordWrap(wrapParams) : body;
				value = leadingSpace + preText + divider + headerText + bodyText;
			}
		} else if (incOptions.what == ZmSetting.INC_SMART) {
			var chunks = AjxStringUtil.getTopLevel(body);
			if (chunks.length) {
				body = chunks.join(crlf2);
			}
			if (htmlMode) {
				var headerText = headers.length ? headers.join(crlf) + crlf2 : "";
				wrapParams.text = isDraft ? body : headerText + body;
				var bodyText = AjxStringUtil.wordWrap(wrapParams);
				value = leadingSpace + preText + divider + bodyText;
			} else {
				var headerText = "";
				if (headers.length) {
					var text = wrapParams.text = headers.join(crlf) + (incOptions.prefix ? crlf : crlf2);
					wrapParams.len = 120; // headers tend to be longer
					headerText = incOptions.prefix ? AjxStringUtil.wordWrap(wrapParams) : text;
				}
				wrapParams.text = body;
				wrapParams.len = ZmHtmlEditor.WRAP_LENGTH;
				var bodyText = incOptions.prefix ? AjxStringUtil.wordWrap(wrapParams) : body;
				value = leadingSpace + preText + divider + headerText + bodyText;
			}
		}
	}

	var isHtmlEditorInitd = this._htmlEditor.isHtmlModeInited();
	if (!isHtmlEditorInitd) {
		this._fixMultipartRelatedImages_onTimer(msg);
	}

	var vLen = value ? value.length : 0;
	AjxDebug.println(AjxDebug.REPLY, "value length B: " + vLen);
	if (!isDraft && sigStyle == ZmSetting.SIG_INTERNET) {
		AjxDebug.println(AjxDebug.REPLY, "internet style sig, call addSignature()");
		this.addSignature(value);
	} else {
		value = value || (htmlMode ? "<br>" : "");
		AjxDebug.println(AjxDebug.REPLY, "value snippet: " + AjxStringUtil.htmlEncode(value.substr(0, 200)));
		this._htmlEditor.setContent(value);
	}

	if (isHtmlEditorInitd) {
		this._fixMultipartRelatedImages_onTimer(msg);
	}

	var hasInlineImages = (bodyInfo && bodyInfo.hasInlineImages) || !ac.get(ZmSetting.VIEW_AS_HTML);
	this._showForwardField(msg || this._msg, action, incOptions, hasInlineImages, bodyInfo && bodyInfo.hasInlineAtts);

	if (sigId && !isDraft) {
        this._attachSignatureVcard(sigId);
	}
};

ZmComposeView.prototype.getUnQuotedContent =
function() {
	var body = this._htmlEditor.getContent();
	var prefaceIndex = body.indexOf(this._getPreface());
	if (prefaceIndex != -1) {
		body = body.substring(0, prefaceIndex);
	}
	if (this.getHtmlEditor().getMode() == DwtHtmlEditor.HTML) {
		body = AjxStringUtil.htmlPlatformIndependent(body);
	}
	return body;
};

ZmComposeView.prototype._getBodyContent =
function(msg, htmlMode) {

	var body, bodyPart, hasInlineImages, hasInlineAtts;
	var crlf = htmlMode ? "<br>" : ZmMsg.CRLF;
	var crlf2 = htmlMode ? "<br><br>" : ZmMsg.CRLF2;

	// bug fix #7271 - if we have multiple body parts, append them all first
	var parts = msg.getBodyParts();
	if (parts && parts.length > 1) {
		var bodyArr = [];
		for (var k = 0; k < parts.length; k++) {
			var part = parts[k];
			// bug: 28741
			if (ZmMimeTable.isRenderableImage(part.ct)) {
				bodyArr.push([crlf, "[", part.ct, ":", (part.filename || "..."), "]", crlf].join(""));
				hasInlineImages = true;
			} else if (part.filename && part.cd == "inline") {   //Inline attachments
				var attInfo = ZmMimeTable.getInfo(part.ct);
				attInfo = attInfo ? attInfo.desc : part.ct;
				bodyArr.push([crlf, "[", attInfo, ":", (part.filename||"..."), "]", crlf].join(""));
				hasInlineAtts = true;
			} else if (part.ct == ZmMimeTable.TEXT_PLAIN || (part.body && ZmMimeTable.isTextType(part.ct))) {
				bodyArr.push( htmlMode ? AjxStringUtil.convertToHtml(part.content) : part.content );
			} else if (part.ct == ZmMimeTable.TEXT_HTML) {
				if (htmlMode){
					bodyArr.push(part.content);
				} else {
					var div = document.createElement("div");
					div.innerHTML = part.content;
					bodyArr.push(AjxStringUtil.convertHtml2Text(div));
				}
			}
		}
		body = bodyArr.join(crlf);
	} else {
		if (htmlMode) {
			body = msg.getBodyPart(ZmMimeTable.TEXT_HTML);
			if (body) {
				body = AjxUtil.isString(body) ? body : body.content;
			} else {
				// if no html part exists, just grab the text
				bodyPart = msg.getBodyPart();
				body = bodyPart ? this._getTextPart(bodyPart, true) : null;
			}
		} else {
			hasInlineImages = msg.hasInlineImagesInMsgBody();
			// grab text part out of the body part
			bodyPart = msg.getBodyPart(ZmMimeTable.TEXT_PLAIN) || msg.getBodyPart(ZmMimeTable.TEXT_HTML, true) || msg.getTextBodyPart();            
			body = bodyPart ? this._getTextPart(bodyPart) : null;
		}
	}

	if (bodyPart && AjxUtil.isObject(bodyPart) && bodyPart.truncated) {
		body += crlf2 + ZmMsg.messageTruncated + crlf2;
	}

	return {body:body || "", bodyPart:bodyPart, hasInlineImages:hasInlineImages, hasInlineAtts:hasInlineAtts};
};

ZmComposeView.prototype._getPreface =
function(mode, action) {

	mode = mode || this._composeMode;
	action = action || this._action;
	var preface;
	if (mode == DwtHtmlEditor.HTML) {
		preface = '<hr id="zwchr">';
	} else {
		var msgText = (action == ZmOperation.FORWARD_INLINE) ? AjxMsg.forwardedMessage : AjxMsg.origMsg;
		preface = [ZmMsg.DASHES, " ", msgText, " ", ZmMsg.DASHES].join("");
	}
	return preface;
};

// if mode isn't given, assumes mode has already been switched internally
ZmComposeView.prototype._switchPreface =
function(mode) {

	mode = mode || this._composeMode;
	var htmlMode = (mode == DwtHtmlEditor.HTML);
	var otherMode = htmlMode ? DwtHtmlEditor.TEXT : DwtHtmlEditor.HTML;
	var curPreface = this._getPreface(otherMode);
	var newPreface = this._getPreface(mode);
	if (!htmlMode) {
		newPreface = newPreface + "<br>";	// so that text preface is followed by a return after conversion
		this._switchedToText = true;
	} else if (this._switchedToText) {
		curPreface = new RegExp(curPreface + "\\s*<br>");
	}
	var content = this.getHtmlEditor().getContent();
	content = content.replace(curPreface, newPreface);
	this._htmlEditor.setContent(content);
	this._preface = newPreface;
};

// for getting text version of HTML part when sending, used by AjxStringUtil._traverse
ZmComposeView._convertHtmlPreface =
function(self, el) {
	return (el && el.id == "zwchr") ? self._getPreface(DwtHtmlEditor.TEXT) + ZmMsg.CRLF : null;
};

ZmComposeView.prototype._applyHtmlPrefix =
function(tagStart, tagEnd, text) {
	var incOptions = this._controller._curIncOptions;
	if (incOptions && incOptions.prefix) {
		var wrapParams = ZmHtmlEditor.getWrapParams(false, incOptions);
		wrapParams.preserveReturns = true;

		var lines = text.split("\n");
		var level = 0;
		var out = [];
		var k = 0;
		for (var i=0; i<lines.length; i++) {
			var line = lines[i];
			if (line==tagStart) {
				level++;
			} else if (line==tagEnd) {
				level--;
			} else {
				if (line=="") {
					var lastLine = lines[i-1];
					if (lastLine && (lastLine != tagStart && lastLine != tagEnd)) {
						out[k++] = line;
					}
				} else {
					wrapParams.len = ZmHtmlEditor.WRAP_LENGTH;
					for (var j=0; j<level; j++) {
						wrapParams.text = line;
						line = AjxStringUtil.wordWrap(wrapParams);
					}
					line = line.replace(/^\n|\n$/,"");
					out[k++] = line;
				}
			}
		}
		return out.join("\n");
	} else {
		return text.replace(tagStart,"").replace(tagEnd,"");
	}
}



ZmComposeView.prototype.resetBody =
function(action, msg, extraBodyText, incOptions) {
	this.cleanupAttachments(true);
	this._isDirty = this._isDirty || this.isDirty();
	this._setBody(action, msg, extraBodyText, incOptions);
	this._setFormValue();
	this._resetBodySize();
};

// Generic routine for attaching an event handler to a field. Since "this" for the handlers is
// the incoming event, we need a way to get at ZmComposeView, so it's added to the event target.
ZmComposeView.prototype._setEventHandler =
function(id, event, addrType) {
	var field = document.getElementById(id);
	field._composeView = this._internalId;
	if (addrType) {
		field._addrType = addrType;
	}
	var lcEvent = event.toLowerCase();
	field[lcEvent] = ZmComposeView["_" + event];
};

ZmComposeView.prototype._setBodyFieldCursor =
function(extraBodyText) {
	if (this._composeMode == DwtHtmlEditor.HTML) { return; }

	// this code moves the cursor to the beginning of the body
	if (AjxEnv.isIE) {
		var tr = this._bodyField.createTextRange();
		if (extraBodyText) {
			tr.move('character', extraBodyText.length + 1);
		} else {
			tr.collapse(true);
		}
		tr.select();
	} else {
		var index = extraBodyText ? (extraBodyText.length + 1) : 0;
		Dwt.setSelectionRange(this._bodyField, index, index);
	}
};

ZmComposeView.prototype.isTinyMCEEnabled =
function() {
    // Bug 54805
    // disable tinymce
    //return window.isTinyMCE;
    return false;
};

/**
 * This should be called only once for when compose view loads first time around
 * 
 * @private
 */
ZmComposeView.prototype._initialize =
function(composeMode) {

	// init address field objects
	this._divId = {};
	this._buttonTdId = {};
	this._fieldId = {};
	this._using = {};
	this._button = {};
	this._field = {};
	this._divEl = {};
	if (this._useAcAddrBubbles) {
		this._addrInputField = {};
	}
	this._internalId = AjxCore.assignId(this);

	// init html
	this._createHtml();

	// init compose view w/ based on user prefs
	var bComposeEnabled = appCtxt.get(ZmSetting.HTML_COMPOSE_ENABLED);
	var composeFormat = appCtxt.get(ZmSetting.COMPOSE_AS_FORMAT);
	var defaultCompMode = bComposeEnabled && composeFormat == ZmSetting.COMPOSE_HTML
		? DwtHtmlEditor.HTML : DwtHtmlEditor.TEXT;
	this._composeMode = composeMode || defaultCompMode;
	this._clearFormValue();

	// init html editor
	if (this.isTinyMCEEnabled()) {
		this._htmlEditor = new ZmAdvancedHtmlEditor(this, DwtControl.RELATIVE_STYLE, null, this._composeMode);
		this._bodyFieldId = this._htmlEditor.getBodyFieldId();
		this._bodyField = document.getElementById(this._bodyFieldId);
	} else {
        if( AjxEnv.isChrome ){
            this._isPasteEnabled = true;
        }
		this._htmlEditor = new ZmHtmlEditor(this, DwtControl.RELATIVE_STYLE, null, this._composeMode, null, this._isPasteEnabled);
		this._bodyFieldId = this._htmlEditor.getBodyFieldId();
		this._bodyField = document.getElementById(this._bodyFieldId);
        if( this._isPasteEnabled ){
            this._htmlEditor.addEventCallback( new AjxListener(this, this._handleEditorEvent) );
        }
	}
	this._includedPreface = "";

	// misc. inits
	this.setScrollStyle(DwtControl.SCROLL);
	this._attachCount = 0;

	// init listeners
	this.addControlListener(new AjxListener(this, this._controlListener));
};

ZmComposeView.prototype._createHtml =
function(templateId) {

	var data = {
		id:					this._htmlElId,
		headerId:			ZmId.getViewId(this._view, ZmId.CMP_HEADER),
		fromSelectId:		ZmId.getViewId(this._view, ZmId.CMP_FROM_SELECT),
		toRowId:			ZmId.getViewId(this._view, ZmId.CMP_TO_ROW),
		toPickerId:			ZmId.getViewId(this._view, ZmId.CMP_TO_PICKER),
		toInputId:			ZmId.getViewId(this._view, ZmId.CMP_TO_INPUT),
		toCellId:			ZmId.getViewId(this._view, ZmId.CMP_TO_CELL),
		ccRowId:			ZmId.getViewId(this._view, ZmId.CMP_CC_ROW),
		ccPickerId:			ZmId.getViewId(this._view, ZmId.CMP_CC_PICKER),
		ccInputId:			ZmId.getViewId(this._view, ZmId.CMP_CC_INPUT),
		ccCellId:			ZmId.getViewId(this._view, ZmId.CMP_CC_CELL),
		bccRowId:			ZmId.getViewId(this._view, ZmId.CMP_BCC_ROW),
		bccPickerId:		ZmId.getViewId(this._view, ZmId.CMP_BCC_PICKER),
		bccInputId:			ZmId.getViewId(this._view, ZmId.CMP_BCC_INPUT),
		bccToggleId:		ZmId.getViewId(this._view, ZmId.CMP_BCC_TOGGLE),
		bccCellId:			ZmId.getViewId(this._view, ZmId.CMP_BCC_CELL),
		subjectRowId:		ZmId.getViewId(this._view, ZmId.CMP_SUBJECT_ROW),
		subjectInputId:		ZmId.getViewId(this._view, ZmId.CMP_SUBJECT_INPUT),
		oboRowId:			ZmId.getViewId(this._view, ZmId.CMP_OBO_ROW),
		oboCheckboxId:		ZmId.getViewId(this._view, ZmId.CMP_OBO_CHECKBOX),
		oboLabelId:			ZmId.getViewId(this._view, ZmId.CMP_OBO_LABEL),
		identityRowId:		ZmId.getViewId(this._view, ZmId.CMP_IDENTITY_ROW),
		identitySelectId:	ZmId.getViewId(this._view, ZmId.CMP_IDENTITY_SELECT),
		priorityId:			ZmId.getViewId(this._view, ZmId.CMP_PRIORITY),
		attRowId:			ZmId.getViewId(this._view, ZmId.CMP_ATT_ROW),
		attDivId:			ZmId.getViewId(this._view, ZmId.CMP_ATT_DIV),
		zdndToolTipId:		ZmId.getViewId(this._view, ZmId.CMP_DND_TOOLTIP),
		acAddrBubbles:		this._useAcAddrBubbles
	};

	this._createHtmlFromTemplate(templateId || this.TEMPLATE, data);
};

ZmComposeView.prototype._createHtmlFromTemplate =
function(templateId, data) {

	DwtComposite.prototype._createHtmlFromTemplate.call(this, templateId, data);

	// global identifiers
	this._identityDivId = data.identityRowId;

	// init autocomplete list
	if (appCtxt.get(ZmSetting.CONTACTS_ENABLED) || appCtxt.get(ZmSetting.GAL_ENABLED) || appCtxt.isOffline) {
		var params = {
			dataClass:		appCtxt.getAutocompleter(),
			matchValue:		ZmAutocomplete.AC_VALUE_FULL,
			compCallback:	(new AjxCallback(this, this._acCompHandler)),
			keyUpCallback:	(new AjxCallback(this, this._acKeyupHandler)),
			options:		{addrBubbles:this._useAcAddrBubbles}
		};
		this._acAddrSelectList = new ZmAutocompleteListView(params);
	}

	var isPickerEnabled = (appCtxt.get(ZmSetting.CONTACTS_ENABLED) ||
						   appCtxt.get(ZmSetting.GAL_ENABLED) ||
						   appCtxt.multiAccounts);
	this._pickerButton = {};

	// process compose fields
	for (var i = 0; i < ZmMailMsg.COMPOSE_ADDRS.length; i++) {
		var type = ZmMailMsg.COMPOSE_ADDRS[i];
		var typeStr = AjxEmailAddress.TYPE_STRING[type];

		// save identifiers
		this._divId[type] = [data.id, typeStr, "row"].join("_");
		this._buttonTdId[type] = [data.id, typeStr, "picker"].join("_");
		var inputId = this._fieldId[type] = [data.id, typeStr, "control"].join("_");

		// save field elements
		this._divEl[type] = document.getElementById(this._divId[type]);
		var aifId;
		if (this._useAcAddrBubbles) {
			var aifParams = {
				parent:								this,
				autocompleteListView:				this._acAddrSelectList,
				bubbleAddedCallback:				(new AjxCallback(this, this._bubblesChangedCallback)),
				bubbleRemovedCallback:				(new AjxCallback(this, this._bubblesChangedCallback)),
				bubbleMenuCreatedCallback:			(new AjxCallback(this, this._bubbleMenuCreated)),
				bubbleMenuResetOperationsCallback:	(new AjxCallback(this, this._bubbleMenuResetOperations)),
				inputId:							inputId,
				type:								type
			}
			var aif = this._addrInputField[type] = new ZmAddressInputField(aifParams);
			aifId = aif._htmlElId;
			var cellId = [data.id, typeStr, "cell"].join("_");
			aif.reparentHtmlElement(cellId);
		}

		// save field control
		this._field[type] = document.getElementById(this._fieldId[type]);
		if (this._field[type]) {
			this._field[type].addrType = type;
			if (!this._useAcAddrBubbles) {
				this._setEventHandler(this._fieldId[type], "onFocus");
			}
		}

		// create picker
		if (isPickerEnabled) {
			var pickerId = this._buttonTdId[type];
			var pickerEl = document.getElementById(pickerId);
			if (pickerEl) {
				var buttonId = ZmId.getButtonId(this._view, ZmComposeView.OP[type]);
				var button = this._pickerButton[type] = new DwtButton({parent:this, id:buttonId});
				button.setText(pickerEl.innerHTML);
				button.replaceElement(pickerEl);

				button.addSelectionListener(new AjxListener(this, this._addressButtonListener));
				button.addrType = type;

				// autocomplete-related handlers
				if (appCtxt.get(ZmSetting.CONTACTS_ENABLED) || appCtxt.isOffline) {
					this._acAddrSelectList.handle(this._field[type], aifId);
				} else {
					this._setEventHandler(this._fieldId[type], "onKeyUp");
				}

				this._button[type] = button;
			}
		}
	}

	// save reference to DOM objects per ID's
	this._headerEl = document.getElementById(data.headerId);
	this._subjectField = document.getElementById(data.subjectInputId);
	this._oboRow = document.getElementById(data.oboRowId);
	this._oboCheckbox = document.getElementById(data.oboCheckboxId);
	this._oboLabel = document.getElementById(data.oboLabelId);
	this._attcDiv = document.getElementById(data.attDivId);

	this._setEventHandler(data.subjectInputId, "onKeyUp");
	this._setEventHandler(data.subjectInputId, "onBlur");
	this._setEventHandler(data.subjectInputId, "onFocus");

	if (appCtxt.multiAccounts) {
		if (!this._fromSelect) {
			this._fromSelect = new DwtSelect({parent:this, parentElement:data.fromSelectId});
			this._fromSelect.addChangeListener(new AjxListener(this, this._handleFromListener));
		}
	} else {
		// initialize identity select
		var identityOptions = this._getIdentityOptions();
		this.identitySelect = new DwtSelect({parent:this, options:identityOptions});
		this.identitySelect.setToolTipContent(ZmMsg.chooseIdentity);

		if (!this._identityChangeListenerObj) {
			this._identityChangeListenerObj = new AjxListener(this, this._identityChangeListener);
		}
		var ac = window.parentAppCtxt || window.appCtxt;
		var accounts = ac.accountList.visibleAccounts;
		for (var i = 0; i < accounts.length; i++) {
			var identityCollection = ac.getIdentityCollection(accounts[i]);
			identityCollection.addChangeListener(this._identityChangeListenerObj);
		}

		this.identitySelect.replaceElement(data.identitySelectId);
		this._setIdentityVisible();
	}

	if (appCtxt.get(ZmSetting.MAIL_PRIORITY_ENABLED)) {
		var buttonId = ZmId.getButtonId(this._view, ZmId.CMP_PRIORITY);
		this._priorityButton = new DwtButton({parent:this, id:buttonId});
		this._priorityButton.setMenu(new AjxCallback(this, this._priorityButtonMenuCallback));
		this._priorityButton.reparentHtmlElement(data.priorityId);
		this._priorityButton.setToolTipContent(ZmMsg.setPriority);
	}

	// Toggle BCC
	this._toggleBccEl = document.getElementById(data.bccToggleId);
	if (this._toggleBccEl) {
		Dwt.setHandler(this._toggleBccEl, DwtEvent.ONCLICK, AjxCallback.simpleClosure(this._toggleBccField, this));
	}
};

ZmComposeView.prototype._handleFromListener =
function(ev) {
	var newVal = ev._args.newValue;
	var oldVal = ev._args.oldValue;
	if (oldVal == newVal) { return; }

	var ac = window.parentAppCtxt || window.appCtxt;
	var newOption = this._fromSelect.getOptionWithValue(newVal);
	var newAccount = ac.accountList.getAccount(newOption.accountId);
	var collection = ac.getIdentityCollection(newAccount);
	var identity = collection && collection.getById(newVal);

	var sigId = this._getSignatureIdForAction(identity || collection.defaultIdentity) || "";

	this._controller._accountName = newAccount.name;
	this._controller.resetSignatureToolbar(sigId, newAccount);
	this._controller.resetSignature(sigId, newAccount);
	this._controller._resetReadReceipt(newAccount);

	// reset account for autocomplete to use
	if (this._acAddrSelectList) {
		this._acAddrSelectList.setActiveAccount(newAccount);
	}

	// if this message is a saved draft, check whether it needs to be moved
	// based on newly selected value.
	if (this._msg && this._msg.isDraft) {
		var oldOption = this._fromSelect.getOptionWithValue(oldVal);
		var oldAccount = ac.accountList.getAccount(oldOption.accountId);

		// cache old info so we know what to delete after new save
		var msgId = this._origAcctMsgId = this._msg.id;

		this._msg = this._addressesMsg = null;
		var callback = new AjxCallback(this, this._handleMoveDraft, [oldAccount.name, msgId]);
		this._controller.saveDraft(this._controller._draftType, null, null, callback);
	}

	this._resetPickerButtons(newAccount);
};

ZmComposeView.prototype._handleMoveDraft =
function(accountName, msgId) {
	var jsonObj = {
		ItemActionRequest: {
			_jsns:  "urn:zimbraMail",
			action: { id:msgId, op:"delete" }
		}
	};
	var params = {
		jsonObj: jsonObj,
		asyncMode: true,
		accountName: accountName
	};
	appCtxt.getAppController().sendRequest(params);
};

ZmComposeView.prototype._toggleBccField =
function(ev, force) {
	var isBccFieldVisible = Dwt.getVisible(this._divEl[AjxEmailAddress.BCC]);
	if (typeof force != "undefined") {
		isBccFieldVisible = !force;
	}
	this._showAddressField(AjxEmailAddress.BCC, !isBccFieldVisible);
};

ZmComposeView.prototype._createPriorityMenuItem =
function(menu, text, flag) {
	var item = DwtMenuItem.create({parent:menu, imageInfo:this._getPriorityImage(flag), text:text});
	item._priorityFlag = flag;
	item.addSelectionListener(this._priorityMenuListnerObj);
};

ZmComposeView.prototype._priorityButtonMenuCallback =
function() {
	var menu = new DwtMenu({parent:this._priorityButton});
	this._priorityMenuListnerObj = new AjxListener(this, this._priorityMenuListner);
	this._createPriorityMenuItem(menu, ZmMsg.high, ZmItem.FLAG_HIGH_PRIORITY);
	this._createPriorityMenuItem(menu, ZmMsg.normal, "");
	this._createPriorityMenuItem(menu, ZmMsg.low, ZmItem.FLAG_LOW_PRIORITY);
	return menu;
};

ZmComposeView.prototype._getPriorityImage =
function(flag) {
	if (flag == ZmItem.FLAG_HIGH_PRIORITY)	{ return "PriorityHigh_list"; }
	if (flag == ZmItem.FLAG_LOW_PRIORITY)	{ return "PriorityLow_list"; }
	return "PriorityNormal_list";
};

ZmComposeView.prototype._priorityMenuListner =
function(ev) {
	this._setPriority(ev.dwtObj._priorityFlag);
};

ZmComposeView.prototype._getPriority =
function() {
	return (this._priorityButton)
		? (this._priorityButton._priorityFlag || "") : "";
};

ZmComposeView.prototype._setPriority =
function(flag) {
	if (this._priorityButton) {
		flag = flag || "";
		this._priorityButton.setImage(this._getPriorityImage(flag));
		this._priorityButton._priorityFlag = flag;
	}
};

ZmComposeView.prototype._getIdentityOptions =
function() {
	var options = [];
	var identityCollection = appCtxt.getIdentityCollection();
	var identities = identityCollection.getIdentities(true);
	for (var i = 0, count = identities.length; i < count; i++) {
		var identity = identities[i];
		options.push(new DwtSelectOptionData(identity.id, this._getIdentityText(identity)));
	}
	return options;
};

ZmComposeView.prototype._getIdentityText =
function(identity, account) {
	var name = identity.name;
	if (identity.isDefault && name == ZmIdentity.DEFAULT_NAME) {
		name = account ? account.getDisplayName() : ZmMsg.accountDefault;
	}

	// default replacement parameters
	var defaultIdentity = appCtxt.getIdentityCollection().defaultIdentity;
	var params = [
		name,
		(identity.sendFromDisplay || ""),
		identity.sendFromAddress,
		ZmMsg.accountDefault,
		appCtxt.get(ZmSetting.DISPLAY_NAME),
		defaultIdentity.sendFromAddress
	];

	// get appropriate pattern
	var pattern;
	if (identity.isDefault) {
		pattern = ZmMsg.identityTextPrimary;
	}
	else if (identity.isFromDataSource) {
		var ds = appCtxt.getDataSourceCollection().getById(identity.id);
		if (params[1] == "") {
			params[1] = ds.userName || "";
		}
		params[2] = ds.getEmail();
		var provider = ZmDataSource.getProviderForAccount(ds);
		if (provider) {
			pattern = ZmMsg["identityText-"+provider.id];
		}
		else if (params[0] && params[1] && params[2] &&
				(params[0] != params[1] != params[2]))
		{
			pattern = ZmMsg.identityTextPersona;
		}
		else {
			pattern = ZmMsg.identityTextExternal;
		}
	}
	else {
		pattern = ZmMsg.identityTextPersona;
	}

	// format text
	return AjxMessageFormat.format(pattern, params);
};

ZmComposeView.prototype._identityChangeListener =
function(ev) {

	if (!this.identitySelect) { return; }

	var identity = ev.getDetail("item");
	if (!identity) { return; }
	if (ev.event == ZmEvent.E_CREATE) {
		// TODO: add identity in sort position
		this._setIdentityVisible();
		var text = this._getIdentityText(identity);
		var option = new DwtSelectOptionData(identity.id, text);
		this.identitySelect.addOption(option);
	} else if (ev.event == ZmEvent.E_DELETE) {
		this.identitySelect.removeOptionWithValue(identity.id);
		this._setIdentityVisible();
	} else if (ev.event == ZmEvent.E_MODIFY) {
		// TODO: see if it was actually name that changed
		// TODO: re-sort list
		var text = this._getIdentityText(identity);
		this.identitySelect.rename(identity.id, text);
	}
};

ZmComposeView.prototype._setIdentityVisible =
function() {
	if (!appCtxt.get(ZmSetting.IDENTITIES_ENABLED)) { return; }
	var div = document.getElementById(this._identityDivId);
	if (!div) return;

	var visible = appCtxt.getIdentityCollection().getSize() > 1;
	Dwt.setVisible(div, visible);
};

ZmComposeView.prototype.getIdentity =
function() {
	var ac = window.parentAppCtxt || window.appCtxt;

	if (appCtxt.multiAccounts) {
		var newVal = this._fromSelect.getValue();
		var newOption = this._fromSelect.getOptionWithValue(newVal);
		var newAccount = ac.accountList.getAccount(newOption.accountId);
		var collection = ac.getIdentityCollection(newAccount);
		return collection && collection.getById(newVal);
	}

	if (this.identitySelect) {
		var collection = ac.getIdentityCollection();
		var val = this.identitySelect.getValue();
		var identity = collection.getById(val);
		return identity ? identity : collection.defaultIdentity;
	}
};

ZmComposeView.prototype._showForwardField =
function(msg, action, incOptions, includeInlineImages, includeInlineAtts) {

	var html = "";
	if (!(this._msgIds && this._msgIds.length) &&
		((incOptions && incOptions.what == ZmSetting.INC_ATTACH) || action == ZmOperation.FORWARD_ATT))
	{
		html = AjxTemplate.expand("mail.Message#ForwardOneMessage", {message:msg});
		this._attachCount = 1;
	}
	else if (msg && (msg.hasAttach || includeInlineImages || includeInlineAtts))
	{
		var attLinks = msg.getAttachmentLinks(false, includeInlineImages, includeInlineAtts);
		if (attLinks.length > 0) {
			var data = {
				attachments: attLinks,
				isNew: action == ZmOperation.NEW_MESSAGE,
				isForward: action == ZmOperation.FORWARD,
				isForwardInline: action == ZmOperation.FORWARD_INLINE,
				isDraft: action == ZmOperation.DRAFT,
				fwdFieldName:(ZmComposeView.FORWARD_ATT_NAME + this._sessionId)
			};
			html = AjxTemplate.expand("mail.Message#ForwardAttachments", data);

			if (attLinks.length >= ZmComposeView.SHOW_MAX_ATTACHMENTS) {
				this._attcDiv.style.height = ZmComposeView.MAX_ATTACHMENT_HEIGHT;
				this._attcDiv.style.overflow = "auto";
			}
			this._attachCount = attLinks.length;
		}
	} else if (this._msgIds && this._msgIds.length) {
		// use main window's appCtxt
		var appCtxt = window.parentAppCtxt || window.appCtxt;
		var messages = [];
		for (var i = 0; i < this._msgIds.length; i++) {
			var message = appCtxt.cacheGet(this._msgIds[i]);
			if (!message) continue;
			messages.push(message);
		}
		var data = {
			messages: messages,
			fwdFieldName: (ZmComposeView.FORWARD_MSG_NAME + this._sessionId)
		};
		html = AjxTemplate.expand("mail.Message#ForwardMessages", data);
		if (messages.length >= ZmComposeView.SHOW_MAX_ATTACHMENTS) {
			this._attcDiv.style.height = ZmComposeView.MAX_ATTACHMENT_HEIGHT;
			this._attcDiv.style.overflow = "auto";
		}
		this._attachCount = messages.length;
	}

	this._attcDiv.innerHTML = html;
};

// Miscellaneous methods
ZmComposeView.prototype._resetBodySize =
function() {
	var size = this.getSize();
	if (size.x <= 0 || size.y <= 0) { return; }

	var height = size.y - Dwt.getSize(this._headerEl).y;
	if (height != size.y) {
		this._htmlEditor.setSize(size.x, height);
	}
};

ZmComposeView.prototype._resetPickerButtons =
function(account) {
	var ac = window.parentAppCtxt || window.appCtxt;
	var isEnabled = ac.get(ZmSetting.CONTACTS_ENABLED, null, account) ||
					ac.get(ZmSetting.GAL_ENABLED, null, account);

	for (var i in this._pickerButton) {
		var button = this._pickerButton[i];
		button.setEnabled(isEnabled);
	}
};

ZmComposeView.prototype._setFromSelect =
function(msg) {
	if (!this._fromSelect) { return; }

	this._fromSelect.clearOptions();

	var ac = window.parentAppCtxt || window.appCtxt;
	var identity;
	var active = ac.getActiveAccount();
	var accounts = ac.accountList.visibleAccounts;

	for (var i = 0; i < accounts.length; i++) {
		var acct = accounts[i];
		if (appCtxt.isOffline && acct.isMain) { continue; }

		var identities = ac.getIdentityCollection(acct).getIdentities();
		if (ac.isFamilyMbox || ac.get(ZmSetting.OFFLINE_SMTP_ENABLED, null, acct)) {
			for (var j = 0; j < identities.length; j++) {
				identity = identities[j];

				var text = this._getIdentityText(identity, acct);
				var icon = appCtxt.isOffline ? acct.getIcon() : null;
				var option = new DwtSelectOption(identity.id, false, text, null, null, icon);
				option.addr = new AjxEmailAddress(identity.sendFromAddress, AjxEmailAddress.FROM, identity.sendFromDisplay);
				option.accountId = acct.id;

				this._fromSelect.addOption(option);
			}
		}
	}

	var selectedIdentity;
	if (msg) {
		var coll = ac.getIdentityCollection(msg.getAccount());
		selectedIdentity = (msg.isDraft)
			? coll.selectIdentity(msg, AjxEmailAddress.FROM)
			: coll.selectIdentity(msg);
		if (!selectedIdentity) {
			selectedIdentity = coll.defaultIdentity;
		}
	}

	if (!selectedIdentity) {
		selectedIdentity = ac.getIdentityCollection(active).defaultIdentity;
	}

    if(selectedIdentity && selectedIdentity.id) {
        this._fromSelect.setSelectedValue(selectedIdentity.id);
    }

	// for cross account searches, the active account isn't necessarily the
	// account of the selected conv/msg so reset it based on the selected option.
    // if active-account is local/main acct, reset it based on selected option.
	if ((appCtxt.getSearchController().searchAllAccounts && this._fromSelect) || active.isMain) {
		active = this.getFromAccount();
		this._controller._accountName = active.name;
	}

	if (this._acAddrSelectList) {
		this._acAddrSelectList.setActiveAccount(active);
	}

	this._resetPickerButtons(active);
};

// Show address field
ZmComposeView.prototype._showAddressField =
function(type, show, skipNotify, skipFocus) {
	this._using[type] = show;
	Dwt.setVisible(this._divEl[type], show);
	this._setAddrFieldValue(type, "");	 // bug fix #750 and #3680
	this._field[type].noTab = !show;
	var setting = ZmComposeView.ADDR_SETTING[type];
	if (setting) {
		appCtxt.set(setting, show, null, false, skipNotify);
	}
	if (type == AjxEmailAddress.BCC) {
		Dwt.setInnerHtml(this._toggleBccEl, show ? ZmMsg.hideBCC : ZmMsg.showBCC );
	}
	this._resetBodySize();
};

// Grab the addresses out of the form. Optionally, they can be returned broken
// out into good and bad addresses, with an aggregate list of the bad ones also
// returned. If the field is hidden, its contents are ignored.
ZmComposeView.prototype._collectAddrs =
function() {

	var addrs = {};
	addrs[ZmComposeView.BAD] = new AjxVector();
	for (var i = 0; i < ZmMailMsg.COMPOSE_ADDRS.length; i++) {
		var type = ZmMailMsg.COMPOSE_ADDRS[i];
		if (!this._using[type]) { continue; }

		var val = this._getAddrFieldValue(type);
		if (val.length == 0) { continue; }
		var result = AjxEmailAddress.parseEmailString(val, type, false);
		if (result.all.size() == 0) { continue; }
		addrs.gotAddress = true;
		addrs[type] = result;
		if (result.bad.size()) {
			addrs[ZmComposeView.BAD].addList(result.bad);
			if (!addrs.badType) {
				addrs.badType = type;
			}
		}
	}
	return addrs;
};

// Returns a string representing the form content
ZmComposeView.prototype._formValue =
function(incAddrs, incSubject) {
	var vals = [];
	if (incAddrs) {
		for (var i = 0; i < ZmMailMsg.COMPOSE_ADDRS.length; i++) {
			var type = ZmMailMsg.COMPOSE_ADDRS[i];
			if (this._using[type]) {
				vals.push(this._getAddrFieldValue(type));
			}
		}
	}
	if (incSubject) {
		vals.push(this._subjectField.value);
	}
	var content = this._htmlEditor.getContent();
	AjxDebug.println(AjxDebug.REPLY, "ZmComposeView::_formValue - content length: " + content.length);
	vals.push(content);
	var str = vals.join("|");
	str = str.replace(/\|+/, "|");
	return str;
};

// Listeners

// Address buttons invoke contact picker
ZmComposeView.prototype._addressButtonListener =
function(ev, addrType) {
	var obj = ev ? DwtControl.getTargetControl(ev) : null;
	this.enableInputs(false);

	if (!this._contactPicker) {
		AjxDispatcher.require("ContactsCore");
		var buttonInfo = [
			{ id: AjxEmailAddress.TO,	label: ZmMsg[AjxEmailAddress.TYPE_STRING[AjxEmailAddress.TO]] },
			{ id: AjxEmailAddress.CC,	label: ZmMsg[AjxEmailAddress.TYPE_STRING[AjxEmailAddress.CC]] },
			{ id: AjxEmailAddress.BCC,	label: ZmMsg[AjxEmailAddress.TYPE_STRING[AjxEmailAddress.BCC]] }
		];
		this._contactPicker = new ZmContactPicker(buttonInfo);
		this._contactPicker.registerCallback(DwtDialog.OK_BUTTON, this._contactPickerOkCallback, this);
		this._contactPicker.registerCallback(DwtDialog.CANCEL_BUTTON, this._contactPickerCancelCallback, this);
	}

	var curType = obj ? obj.addrType : addrType;
	var addrList = {};
	var addrs = !this._useAcAddrBubbles && this._collectAddrs();
	for (var i = 0; i < ZmMailMsg.COMPOSE_ADDRS.length; i++) {
		var type = ZmMailMsg.COMPOSE_ADDRS[i];
		addrList[type] = this._useAcAddrBubbles ? this._addrInputField[type].getAddresses(true) :
				   								  addrs[type] && addrs[type].good.getArray();
	}
	this._contactPicker.addPopdownListener(this._controller._dialogPopdownListener);
	var str = (this._field[curType].value && !(addrList[curType] && addrList[curType].length))
		? this._field[curType].value : "";

	var account;
	var context = appCtxt.isChildWindow ? parentAppCtxt : appCtxt;
	if (appCtxt.multiAccounts && this._fromSelect) {
		var addr = this._fromSelect.getSelectedOption().addr;
		account = context.accountList.getAccountByEmail(addr.address);
	}
	this._contactPicker.popup(curType, addrList, str, account);
};

ZmComposeView.prototype._controlListener =
function() {
	this._resetBodySize();
};


// Callbacks

// Transfers addresses from the contact picker to the compose view.
ZmComposeView.prototype._contactPickerOkCallback =
function(addrs) {

	this.enableInputs(true);
	for (var i = 0; i < ZmMailMsg.COMPOSE_ADDRS.length; i++) {
		var type = ZmMailMsg.COMPOSE_ADDRS[i];
		this.setAddress(type, "");
		var addrVec = this._expandAddrs(addrs[type]);
		this._addAddresses(type, addrVec);
	}

	//I still need this here since REMOVING stuff with the picker does not call removeBubble in the ZmAddresInputField.
	//Also - it's better to do it once than for every bubble in this case. user might add many addresses with the picker
	this._bubblesChangedCallback();

	this._contactPicker.removePopdownListener(this._controller._dialogPopdownListener);
	this._contactPicker.popdown();
	this.reEnableDesignMode();
};

// Expands any addresses that are groups
ZmComposeView.prototype._expandAddrs =
function(addrs) {
	var addrsNew = [];
	var addrsArray = (addrs instanceof AjxVector) ? addrs.getArray() : addrs;
	if (addrsArray && addrsArray.length) {
		for (var i = 0; i < addrsArray.length; i++) {
			var addr = addrsArray[i];
			if (addr) {
				if (addr.isGroup) {
					var members = AjxEmailAddress.split(addr.address);
					addrsNew = addrsNew.concat(members);
				}
				else {
					addrsNew.push(addr);
				}
			}
		}
	}
	return AjxVector.fromArray(addrsNew);
};

ZmComposeView.prototype._contactPickerCancelCallback =
function() {
	this.enableInputs(true);
	this.reEnableDesignMode();
};

// this callback is triggered when an event occurs inside the html editor (when in HTML mode)
// it is used to set focus to the To: field when user hits the TAB key
ZmComposeView.prototype._htmlEditorEventCallback =
function(args) {
	var rv = true;
	if (args.type == "keydown") {
		var key = DwtKeyEvent.getCharCode(args);
		if (key == DwtKeyEvent.KEY_TAB) {
			var toField = document.getElementById(this._fieldId[AjxEmailAddress.TO]);
			if (toField) {
				appCtxt.getKeyboardMgr().grabFocus(toField);
			}
			rv = false;
		}
	}
	return rv;
};

// needed to reset design mode when in html compose format for gecko
ZmComposeView.prototype._okCallback =
function() {
	appCtxt.getMsgDialog().popdown();
	this._controller.resetToolbarOperations();
	this.reEnableDesignMode();
};

// User has agreed to send message without a subject
ZmComposeView.prototype._noSubjectOkCallback =
function(dialog) {
	this._noSubjectOkay = true;
	this._popDownAlertAndSendMsg(dialog);
};

//this is used by several kinds of alert dialogs
ZmComposeView.prototype._popDownAlertAndSendMsg =
function(dialog) {
	// not sure why: popdown (in FF) seems to create a race condition,
	// we can't get the attachments from the document anymore.
	// W/in debugger, it looks fine, but remove the debugger and any
	// alerts, and gotAttachments will return false after the popdown call.

	if (AjxEnv.isIE) {
		dialog.popdown();
	}
	// bug fix# 3209
	// - hide the dialog instead of popdown (since window will go away anyway)
	if (AjxEnv.isNav && appCtxt.isChildWindow) {
		dialog.setVisible(false);
	}

	// dont make any calls after sendMsg if child window since window gets destroyed
	if (appCtxt.isChildWindow && !AjxEnv.isNav) {
		this._controller.sendMsg();
	} else {
		// bug fix #3251 - call popdown BEFORE sendMsg
		dialog.popdown();
		this._controller.sendMsg();
	}
};

// User has canceled sending message without a subject
ZmComposeView.prototype._noSubjectCancelCallback =
function(dialog) {
	this.enableInputs(true);
	dialog.popdown();
	appCtxt.getKeyboardMgr().grabFocus(this._subjectField);
	this._controller.resetToolbarOperations();
	this.reEnableDesignMode();
};

ZmComposeView.prototype._errViaZimletOkCallback =
function(params) {
	var dialog = params.errDialog; 
	var zimletName = params.zimletName;
	//add this zimlet to ignoreZimlet string
	this._ignoredZimlets = this._ignoredZimlets || {};
	this._ignoredZimlets[zimletName] = true;
	this._popDownAlertAndSendMsg(dialog);
};

ZmComposeView.prototype._errViaZimletCancelCallback =
function(params) {
	var dialog = params.errDialog; 
	var zimletName = params.zimletName;
	this.enableInputs(true);
	dialog.popdown();
	this._controller.resetToolbarOperations();
	this.reEnableDesignMode();
};

// User has agreed to send message with bad addresses
ZmComposeView.prototype._badAddrsOkCallback =
function(dialog) {
	this.enableInputs(true);
	this._badAddrsOkay = true;
	dialog.popdown();
	this._controller.sendMsg();
};

// User has declined to send message with bad addresses - set focus to bad field
ZmComposeView.prototype._badAddrsCancelCallback =
function(type, dialog) {
	this.enableInputs(true);
	this._badAddrsOkay = false;
	dialog.popdown();
	if (this._using[type]) {
		appCtxt.getKeyboardMgr().grabFocus(this._field[type]);
	}
	this._controller.resetToolbarOperations();
	this.reEnableDesignMode();
};

// Files have been uploaded, re-initiate the send with an attachment ID.
ZmComposeView.prototype._attsDoneCallback =
function(isDraft, status, attId, docIds) {
	DBG.println(AjxDebug.DBG1, "Attachments: isDraft = " + isDraft + ", status = " + status + ", attId = " + attId);
	if (status == AjxPost.SC_OK) {
		this._controller.saveDraft(ZmComposeController.DRAFT_TYPE_AUTO, attId, docIds);
	} else if (status == AjxPost.SC_UNAUTHORIZED) {
		// auth failed during att upload - let user relogin, continue with compose action
		var ex = new AjxException("401 response during attachment upload", ZmCsfeException.SVC_AUTH_EXPIRED);
		var callback = new AjxCallback(this._controller, isDraft ? this._controller.saveDraft : this._controller._send);
		this._controller._handleException(ex, {continueCallback:callback});
	} else {
		// bug fix #2131 - handle errors during attachment upload.
		var msg = AjxMessageFormat.format(ZmMsg.errorAttachment, (status || AjxPost.SC_NO_CONTENT));
		switch (status) {
			// add other error codes/message here as necessary
			case AjxPost.SC_REQUEST_ENTITY_TOO_LARGE: 	msg += " " + ZmMsg.errorAttachmentTooBig + "<br><br>"; break;
			default: 									msg += " "; break;
		}

		this._controller.popupErrorDialog(msg + ZmMsg.errorTryAgain, null, null, true);
		this._controller.resetToolbarOperations();
	}
};


//Mandatory Spellcheck Callback
ZmComposeView.prototype._spellCheckShield =
function(words){
	if (words && words.available && words.misspelled != null && words.misspelled.length != 0) {
		var msgDialog = appCtxt.getYesNoMsgDialog();
		msgDialog.setMessage(AjxMessageFormat.format(ZmMsg.misspellingsMessage, [words.misspelled.length]));
		msgDialog.registerCallback(DwtDialog.YES_BUTTON, this._spellCheckShieldOkListener, this, [ msgDialog, words ] );
		msgDialog.registerCallback(DwtDialog.NO_BUTTON, this._spellCheckShieldCancelListener, this, msgDialog);
		msgDialog.associateEnterWithButton(DwtDialog.NO_BUTTON);
		var composeView = this;
		msgDialog.handleKeyAction = function(actionCode, ev) { if (actionCode && actionCode==DwtKeyMap.CANCEL) { composeView._spellCheckShieldOkListener(msgDialog, words, ev); return(true); } };
		msgDialog.popup(null, DwtDialog.NO_BUTTON);
	} else {
		this._spellCheckOkay = true;
		this._controller.sendMsg();
	}
};

ZmComposeView.prototype._spellCheckShieldOkListener =
function(msgDialog, words, ev){

	this._controller._toolbar.enableAll(true);
    this.enableInputs(true);
	this._controller.toggleSpellCheckButton(true);
	this._htmlEditor.discardMisspelledWords();

	this._spellCheckOkay = false;
	msgDialog.popdown();

	this._htmlEditor.onExitSpellChecker = new AjxCallback(this._controller, this._controller.toggleSpellCheckButton, true)
	this._htmlEditor._spellCheckCallback(words);
};

ZmComposeView.prototype._spellCheckShieldCancelListener =
function(msgDialog, ev){
	this._spellCheckOkay = true;
	msgDialog.popdown();
	this._controller.sendMsg();
};

ZmComposeView.prototype._spellCheckErrorShield =
function(ex){
	var msgDialog = appCtxt.getYesNoMsgDialog();
	msgDialog.setMessage(ZmMsg.spellCheckFailed);
	msgDialog.registerCallback(DwtDialog.YES_BUTTON, this._spellCheckErrorShieldOkListener, this, msgDialog );
	msgDialog.registerCallback(DwtDialog.NO_BUTTON, this._spellCheckErrorShieldCancelListener, this, msgDialog);
	msgDialog.associateEnterWithButton(DwtDialog.NO_BUTTON);
	msgDialog.popup(null, DwtDialog.NO_BUTTON);

	return true;
};

ZmComposeView.prototype._spellCheckErrorShieldOkListener =
function(msgDialog, ev){

	this._controller._toolbar.enableAll(true);
	this._controller.toggleSpellCheckButton(false);
	this._htmlEditor.discardMisspelledWords();
	msgDialog.popdown();

	this._spellCheckOkay = true;
	this._controller.sendMsg();
	
};

ZmComposeView.prototype._spellCheckErrorShieldCancelListener =
function(msgDialog, ev){
	this._controller._toolbar.enableAll(true);
	this._controller.toggleSpellCheckButton(false);
	this._htmlEditor.discardMisspelledWords();
	msgDialog.popdown();
};

ZmComposeView.prototype._setFormValue =
function() {
	this._origFormValue = this._formValue();
};

ZmComposeView.prototype._clearFormValue =
function() {
	this._origFormValue = "";
	this._isDirty = false;
};

ZmComposeView.prototype._focusHtmlEditor =
function() {
	this._htmlEditor.focus();
};


// Static methods

ZmComposeView._onKeyUp =
function(ev) {

	ev = DwtUiEvent.getEvent(ev);
	var element = DwtUiEvent.getTargetWithProp(ev, "id");
	if (!element) { return true; }
	var cv = AjxCore.objectWithId(element._composeView);

	if (element != cv._subjectField) {
		cv._adjustAddrHeight(element);
	}
	return true;
};

// set focus within tab group to element so tabbing works
ZmComposeView._onFocus =
function(ev) {

	ev = DwtUiEvent.getEvent(ev);
	var element = DwtUiEvent.getTargetWithProp(ev, "id");
	if (!element) { return true; }

	var kbMgr = appCtxt.getKeyboardMgr();
	if (kbMgr.__currTabGroup) {
		kbMgr.__currTabGroup.setFocusMember(element);
	}
};

ZmComposeView._onBlur =
function(ev) {

	var element = DwtUiEvent.getTargetWithProp(ev, "id");
	if (!element) { return true; }
	var cv = AjxCore.objectWithId(element._composeView);

	cv.updateTabTitle();

	return true;
};

// for com.zimbra.dnd zimlet
ZmComposeView.prototype.uploadFiles =
function() {
	var attachDialog = appCtxt.getAttachDialog();
	var callback = new AjxCallback(this, this._attsDoneCallback, [true]);
	attachDialog.setUploadCallback(callback);
	attachDialog.upload(callback, document.getElementById("zdnd_form"));
};

ZmComposeView.prototype.deactivate =
function() {
	this._controller.inactive = true;
};

ZmComposeView.prototype._getAddrFieldValue =
function(type) {

	var val = "";
	if (this._useAcAddrBubbles) {
		var addrInput = this._addrInputField[type];
		if (addrInput) {
			val = addrInput.getValue();
		}
	}
	else {
		val = AjxStringUtil.trim(this._field[type].value)
	}

	return val;
};

ZmComposeView.prototype._setAddrFieldValue =
function(type, value) {

	if (this._useAcAddrBubbles) {
		var addrInput = this._addrInputField[type];
		if (addrInput) {
			addrInput.setValue(value, true);
		}
	}
	else {
		this._field[type].value = value || "";
	}
};

ZmComposeView.prototype._handleEditorEvent = function(ev){
    if( ev.type === "paste" ){
        this._controller._pasteHandler(ev);
    }
    return true;
};

ZmComposeView.prototype._getIframeDoc = function(){
    var editor = this._htmlEditor;
    if( editor ){
        return editor._getIframeDoc();
    }
};
}
if (AjxPackage.define("zimbraMail.mail.view.ZmConvView")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

ZmConvView = function(params) {

	params.className = "ZmConvView";
	params.posStyle = Dwt.ABSOLUTE_STYLE;
	params.mode = ZmId.VIEW_CONV;
	ZmDoublePaneView.call(this, params);

	this._changeListener = new AjxListener(this, this._convChangeListener);
	
	// add change listener to tree view to catch empty trash action
	var folderTree = appCtxt.getFolderTree();
	if (folderTree) {
		folderTree.addChangeListener(new AjxListener(this, this._folderChangeListener));
	}
	
	// Add a change listener to taglist to track tag color changes
	this._tagList = appCtxt.getTagTree();
	if (this._tagList) {
		this._tagList.addChangeListener(new AjxListener(this, this._tagChangeListener));
	}
	
	this._controller = params.controller;
}

ZmConvView.prototype = new ZmDoublePaneView;
ZmConvView.prototype.constructor = ZmConvView;

ZmConvView._TAG_CLICK = "ZmConvView._TAG_CLICK";
ZmConvView._TAG_ANCHOR = "TA";
ZmConvView._TAGLIST_HEIGHT = 18;


// Public methods

ZmConvView.prototype.toString = 
function() {
	return "ZmConvView";
};

ZmConvView.prototype.addTagClickListener =
function(listener) {
	this.addListener(ZmConvView._TAG_CLICK, listener);
};

ZmConvView.prototype.setItem =
function(conv) {
	if (!(conv instanceof ZmConv)) { return; }
		
	// Remove and re-add listeners for current conversation if it exists
	if (this._conv) {
		this._conv.removeChangeListener(this._changeListener);
	}
	this._conv = conv;
	conv.addChangeListener(this._changeListener);

	this._mailListView.set(conv.msgs, ZmItem.F_DATE);
	this.isStale = false;
	this._setSubject(conv.subject);
	this._setTags(conv);
	
	// display "hot" message (or newest if no search performed)
	var offset = this._mailListView.offset;
	var hot = this._conv.getFirstHotMsg({offset:offset, limit:this._mailListView.getLimit(offset)});
	this._mailListView.setSelection(hot);
};

ZmConvView.prototype.reset = 
function() {
	this._sashMoved = false;
	ZmDoublePaneView.prototype.reset.call(this);
};

/**
 * Remove this view's listeners on the conv and its msg list now that the view has been popped.
 */
ZmConvView.prototype.deactivate = 
function() {
	if (this._conv.msgs) {
		this._conv.msgs.removeChangeListener(this._mailListView._listChangeListener);
	}
	this._conv.removeChangeListener(this._changeListener);
};


ZmConvView.prototype.getTitle =
function() {
	return [ZmMsg.zimbraTitle, this._conv.subject].join(": ");
};

// Private / protected methods

ZmConvView.prototype._createMailListView =
function(params) {
	return ZmDoublePaneView.prototype._createMailListView.apply(this, arguments);
};

// override since we need to take summary into consideration
ZmConvView.prototype._resetSize = 
function(newWidth, newHeight, force) {

	if (newWidth <= 0 || newHeight <= 0) { return; }
	if (!force && newWidth == this._lastResetWidth && newHeight == this._lastResetHeight) { return; }

	var readingPaneOnRight = this._controller.isReadingPaneOnRight();

	var summaryHeight = this._summary.getHtmlElement().offsetHeight;
	if (this.isMsgViewVisible()) {
		var sash = this.getSash();
		var sashSize = sash.getSize();
		var sashThickness = readingPaneOnRight ? sashSize.x : sashSize.y;
		if (readingPaneOnRight) {
			var listViewWidth = this._vertSashX || Math.floor(newWidth / 2.5);
			this._mailListView.setLocation(0, summaryHeight);
			this._mailListView.resetSize(listViewWidth, newHeight - summaryHeight);
			sash.setLocation(listViewWidth, 0);
			this._msgView.setBounds(listViewWidth + sashThickness, 0,
									newWidth - (listViewWidth + sashThickness), newHeight);
		} else {
			var listViewHeight = 0, upperHeight, rowsHeight = 0;
			if (this._horizSashY) {
				listViewHeight = this._horizSashY - summaryHeight;
				upperHeight = summaryHeight + listViewHeight;
			} else {
				// set height of MLV based on number of msgs in conv - since that number is usually small,
				// we can give the MV more space
				var list = this._mailListView.getList();
				if (list && list.size() > 0) {
					var threshold = Math.min(list.size() + 1, 6);
					var div = document.getElementById(this._mailListView._getItemId(list.get(0)));
					if (div) {
						var rowsHeight = Dwt.getSize(div).y * threshold;
						listViewHeight = rowsHeight + DwtListView.HEADERITEM_HEIGHT;
						upperHeight = this._minUpperHeight = summaryHeight + listViewHeight;
					}
				}
			}
			if (summaryHeight && listViewHeight && upperHeight) {
				this._mailListView.setLocation(0, summaryHeight);
				this._mailListView.resetSize(newWidth, listViewHeight);
				sash.setLocation(0, upperHeight);
				var mvHeight = Math.max((newHeight - (upperHeight + sashThickness)), 0);
				this._msgView.setBounds(0, upperHeight + sashThickness, newWidth, mvHeight);
			}
		}
	} else {
		this._mailListView.resetSize(newWidth, newHeight - summaryHeight);
	}

	if (AjxEnv.isIE && this._subjectDiv) {
		Dwt.setSize(this._subjectDiv, newWidth - 95);
	}
	
	this._mailListView._resetColWidth();
};

ZmConvView.prototype._sashCallback =
function(delta) {

	var readingPaneOnRight = this._controller.isReadingPaneOnRight();

	if (delta > 0) {
		if (readingPaneOnRight) {
			// moving sash right
			this._summary.setSize(this._summary.getSize().x + delta, Dwt.DEFAULT);
			this._mailListView.resetSize(this._mailListView.getSize().x + delta, Dwt.DEFAULT);
			this._msgView.setBounds(this._msgView.getLocation().x + delta, Dwt.DEFAULT,
									this._msgView.getSize().x - delta, Dwt.DEFAULT);
		} else {
			// moving sash down
			var newMsgViewHeight = this._msgView.getSize().y - delta;
			var minMsgViewHeight = this._msgView.getMinHeight();
			// make sure msg header remains visible
			if (newMsgViewHeight > minMsgViewHeight) {
				this._mailListView.resetSize(Dwt.DEFAULT, this._mailListView.getSize().y + delta);
				this._msgView.setSize(Dwt.DEFAULT, newMsgViewHeight);
				this._msgView.setLocation(Dwt.DEFAULT, this._msgView.getLocation().y + delta);
			} else {
				delta = 0;
			}
		}
	} else {
		var absDelta = Math.abs(delta);
		if (readingPaneOnRight) {
			// moving sash left
			this._summary.setSize(this._summary.getSize().x - absDelta, Dwt.DEFAULT);
			this._mailListView.resetSize(this._mailListView.getSize().x - absDelta, Dwt.DEFAULT);
			this._msgView.setBounds(this._msgView.getLocation().x - absDelta, Dwt.DEFAULT,
									this._msgView.getSize().x + absDelta, Dwt.DEFAULT);
		} else {
			// moving sash up
			// make sure summary and MLV remain visible
			if (this._horizSashY - absDelta > this._minUpperHeight) {
				this._mailListView.resetSize(Dwt.DEFAULT, this._mailListView.getSize().y - absDelta);
				this._msgView.setSize(Dwt.DEFAULT, this._msgView.getSize().y + absDelta);
				this._msgView.setLocation(Dwt.DEFAULT, this._msgView.getLocation().y - absDelta);
			} else {
				delta = 0;
			}
		}
	}

	if (delta) {
		this._mailListView._resetColWidth();
		if (readingPaneOnRight) {
			this._vertSashX = this._vertMsgSash.getLocation().x;
		} else {
			this._horizSashY = this._horizMsgSash.getLocation().y;
		}
	}

	return delta;
};

ZmConvView.prototype._initHeader =
function() {
	this._summary = new DwtComposite(this, "Summary", Dwt.RELATIVE_STYLE);

	var tagDivId = appCtxt.get(ZmSetting.TAGGING_ENABLED) ? (this._htmlElId + "_tagDiv") : null;
	var subs = { id: this._htmlElId, tagDivId: tagDivId };

	this._summary.getHtmlElement().innerHTML = AjxTemplate.expand("mail.Message#MessageListHeader", subs);

	// add the close button
	var closeButtonId = this._htmlElId + "_closeBtnCell";
	var closeButton = new DwtButton({parent:this, className:"DwtToolbarButton", parentElement:closeButtonId});
	closeButton.setImage("Close");
	closeButton.setText(ZmMsg.close);
	closeButton.addSelectionListener(new AjxListener(this, this._closeButtonListener));
	
	// save the necessary DOM objects we just created
	this._subjectDiv = document.getElementById(this._htmlElId + "_subjDiv");

	if (tagDivId) {
		this._tagDiv = document.getElementById(tagDivId);
		Dwt.setSize(this._tagDiv, Dwt.DEFAULT, ZmConvView._TAGLIST_HEIGHT);
		Dwt.setVisible(this._tagDiv, false);
	}
};

ZmConvView.prototype._setSubject =
function(subject) {
	this._subjectDiv.innerHTML = subject != null && subject != ""
		? AjxStringUtil.htmlEncode(subject)
		: AjxStringUtil.htmlEncode(ZmMsg.noSubject);
};

ZmConvView.prototype._setTags =
function(conv) {
	if (!appCtxt.get(ZmSetting.TAGGING_ENABLED)) return;

	var numTags = conv.tags.length;
	var origVis = Dwt.getVisible(this._tagDiv);
	var newVis = (numTags > 0);
	if (origVis != newVis) {
		Dwt.setVisible(this._tagDiv, newVis);
		var sz = this.getSize();
		this._resetSize(sz.x, sz.y, true);
		if (!newVis) {
			this._tagDiv.innerHTML = "";
			return;
		}
	}
	
	if (!numTags) return;
		
	var ta = new Array();
	for (var j = 0; j < numTags; j++)
		ta[j] = this._tagList.getById(conv.tags[j]);

	if (ta.length == 0)	return;
	ta.sort(ZmTag.sortCompare);

	var html = new Array();
	var i = 0;
	html[i++] = "<table cellspacing=0 cellpadding=0 border=0 width=100%>";
	html[i++] = "<tr><td style='overflow:hidden'>";
	
	for (var j = 0; j < ta.length; j++) {
		var tag = ta[j];
		if (!tag) continue;
		var anchorId = [this._tagDiv.id, ZmConvView._TAG_ANCHOR, tag.id].join("");
		var imageId = [this._tagDiv.id, ZmDoublePaneView._TAG_IMG, tag.id].join("");

		html[i++] = "<a href='javascript:;' onclick='ZmConvView._tagClick(\"";
		html[i++] = this._htmlElId;
		html[i++] = '","';
		html[i++] = tag.id;
		html[i++] = "\"); return false;' id='";
		html[i++] = anchorId;
		html[i++] = "'>";
		html[i++] = "<table style='display:inline; vertical-align:middle;' border=0 cellspacing=0 cellpadding=0><tr><td>";
		html[i++] = AjxImg.getImageHtml(tag.getIconWithColor(), null, ["id='", imageId, "'"].join(""));
		html[i++] = "</td></tr></table>";
		html[i++] = AjxStringUtil.htmlEncodeSpace(tag.name);
		html[i++] = "</a>";
	}
	html[i++] = "</td></tr></table>";
	this._tagDiv.innerHTML = html.join("");
};


// Listeners

ZmConvView.prototype._convChangeListener =
function(ev) {
	if (ev.type != ZmEvent.S_CONV)
		return;
	var fields = ev.getDetail("fields");
	if (ev.event == ZmEvent.E_TAGS || ev.event == ZmEvent.E_REMOVE_ALL) {
		this._setTags(this._conv);
	} else if (ev.event == ZmEvent.E_MODIFY && (fields && fields[ZmItem.F_ID])) {
		this._controller._convId = this._conv.id;
	}
};

ZmConvView.prototype._folderChangeListener = 
function(ev) {
	if (ev.event == ZmEvent.E_DELETE &&
	    ev.source instanceof ZmFolder && 
		ev.source.id == ZmFolder.ID_TRASH && 
		this._conv.msgs) 
	{
		// user emptied trash folder.. search for any msgs in trash and remove from list view
		var list = this._conv.msgs.getArray();
		var len = list.length; // save original length
		for (var i = 0; i < list.length; i++) {
			var folder = appCtxt.getById(list[i].folderId);
			if (folder.isInTrash()) {
				this._mailListView.removeItem(list[i]);
				this._conv.msgs.remove(list[i], true);
				i--;
			}
		}
		
		this._controller._resetNavToolBarButtons(this._controller._getViewType());
		
		if (len != this._conv.numMsgs) {
			// allow CLV to update its msg count if its been changed
			var fields = {};
			fields[ZmItem.F_SIZE] = true;
			this._conv.list._notify(ZmEvent.E_MODIFY, {items: [this._conv], fields: fields});
			// reset selection to first msg
			this._mailListView.setSelection(this._conv.msgs.getVector().get(0));
		}
	}
};

ZmConvView.prototype._tagChangeListener =
function(ev) {
	if (ev.type != ZmEvent.S_TAG)
		return;

	var fields = ev.getDetail("fields");
	if (ev.event == ZmEvent.E_MODIFY && (fields && fields[ZmOrganizer.F_COLOR])) {
		var tag = ev.getDetail("organizers")[0];
		var img = document.getElementById(this._tagDiv.id +  ZmDoublePaneView._TAG_IMG + tag.id);
		if (img)
			AjxImg.setImage(img, tag.getIconWithColor());
	}
	
	if (ev.event == ZmEvent.E_DELETE || ev.event == ZmEvent.MODIFY)
		this._setTags(this._conv);
};

ZmConvView.prototype._closeButtonListener = 
function(ev) {
	this._controller._app.popView();
};

// mimics handling by ZmConvListController.prototype._listSelectionListener
ZmConvView.prototype._staleHandler =
function() {

	var ctlr = this._controller._parentController || AjxDispatcher.run("GetConvListController");
	this._conv._loaded = false;	// force request to be made
	var respCallback = new AjxCallback(ctlr, ctlr._handleResponseListSelectionListener, this._conv);
	AjxDispatcher.run("GetConvController").show(ctlr._activeSearch, this._conv, ctlr, respCallback, true);
};

// Static methods

ZmConvView._tagClick =
function(myId, tagId) {
	var dwtObj = DwtControl.fromElementId(myId);
	dwtObj.notifyListeners(ZmConvView._TAG_CLICK, tagId);
};

ZmConvView.prototype.handleRemoveAttachment =
function(oldMsgId, newMsg) {
	var ac = window.parentAppCtxt || window.appCtxt;
	// cache this actioned ID so we can reset selection to it once the CREATE
	// notifications have been processed.
	ac.getApp(ZmApp.MAIL).getMailListController().actionedMsgId = newMsg.id;
	var msgView = this.getMsgView();
	if (msgView) {
		var mailListView = this.getMailListView();
		if (mailListView) {
			var mailList = mailListView.getList();
			var pos = mailList.indexOf(msgView._msg);
			mailList.replace(pos, newMsg);
			mailListView.set(mailList.clone(), mailListView._sortByString);
			mailListView.setSelection(newMsg);
		}
		if (this._conv) {
			//hack to avoid popview if only one message in conversation; delete notification will cause view to be popped otherwise
			this._conv.addMsg(newMsg);
			this._conv.numMsgs = this._conv.msgIds.length;
		}
		msgView.handleRemoveAttachment(oldMsgId, newMsg);
	}
};
}
if (AjxPackage.define("zimbraMail.mail.view.ZmMailAssistant")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

ZmMailAssistant = function() {
	ZmAssistant.call(this, ZmMsg.createNewMsg, ZmMsg.ASST_CMD_MAIL, ZmMsg.ASST_CMD_SUM_MAIL);
};

ZmMailAssistant.prototype = new ZmAssistant();
ZmMailAssistant.prototype.constructor = ZmMailAssistant;

ZmMailAssistant.prototype.okHandler =
function(dialog) {
	return true;	//override
};

ZmMailAssistant._FIELD_ORDER = [
     'subject', 'to', 'cc', 'bcc', 'body'
];

ZmMailAssistant._FIELDS = {
  	 cc: { field: ZmMsg.cc, key: 'cc' },
  	bcc: { field: ZmMsg.bcc, key: 'bcc' },
	  b: { field: ZmMsg.body, key: 'body', defaultValue: ZmMsg.ASST_MAIL_body, multiLine: true },  	
   body: { field: ZmMsg.body, key: 'body', defaultValue: ZmMsg.ASST_MAIL_body, multiLine: true },
	  s: { field: ZmMsg.subject, key: 'subject', defaultValue: ZmMsg.ASST_MAIL_subject },
subject: { field: ZmMsg.subject, key: 'subject', defaultValue: ZmMsg.ASST_MAIL_subject },
  	 to: { field: ZmMsg.to, key: 'to', defaultValue: ZmMsg.ASST_MAIL_to }
};

ZmMailAssistant._OBJECTS = { };
ZmMailAssistant._OBJECTS[ZmObjectManager.EMAIL] = { defaultType: 'to' };
ZmMailAssistant._OBJECTS[ZmAssistant._BRACKETS] = { defaultType: 'body', aliases: { b: 'body', t: 'to', c: 'cc' }};

// check address first, since we grab any fields quoted with [], objects in them won't be matched later
ZmMailAssistant._OBJECT_ORDER = [
	ZmAssistant._BRACKETS //, ZmObjectManager.EMAIL
];

ZmMailAssistant.prototype.getHelp =
function() {
	return ZmMsg.ASST_MAIL_HELP;
};

ZmMailAssistant.prototype.okHandler =
function(dialog) {
	var bad = new AjxVector();	
	var good = new AjxVector();
	var msg = this.getMessage(bad, good);

	var cd = this._confirmDialog = appCtxt.getOkCancelMsgDialog();
	cd.reset();
	
	var subject = AjxStringUtil.trim(msg.subject);
	if ((subject == null || subject == "") && !this._noSubjectOkay) {
    	cd.setMessage(ZmMsg.compSubjectMissing, DwtMessageDialog.WARNING_STYLE);
		cd.registerCallback(DwtDialog.OK_BUTTON, this._noSubjectOkCallback, this, dialog);
		cd.registerCallback(DwtDialog.CANCEL_BUTTON, this._noSubjectCancelCallback, this, dialog);
	    cd.popup();
		return false;
	}

	if (good.size() == 0) {
		dialog.messageDialog(ZmMsg.noAddresses, DwtMessageDialog.CRITICAL_STYLE);
		return false;
	}

	if (bad.size() > 0 && !this._badAddrsOkay) {
	    var badAddrs = AjxStringUtil.htmlEncode(bad.toString(AjxEmailAddress.SEPARATOR));
	    var message = AjxMessageFormat.format(ZmMsg.compBadAddresses, badAddrs);
    	cd.setMessage(message, DwtMessageDialog.WARNING_STYLE);
		cd.registerCallback(DwtDialog.OK_BUTTON, this._badAddrsOkCallback, this, dialog);
		cd.registerCallback(DwtDialog.CANCEL_BUTTON, this._badAddrsCancelCallback, this, dialog);
	    cd.popup();
		return false;
	} else {
		this._badAddrsOkay = false;
	}

	var respCallback = new AjxCallback(this, this._handleResponseSendMsg, [dialog]);
	var errorCallback = new AjxCallback(this, this._handleErrorSendMsg, [dialog]);
	
	msg.send(false, respCallback, errorCallback);
	// need to popdown in handle response instead of returning true..
	return false;
};

ZmMailAssistant.prototype._handleResponseSendMsg =
function(dialog) {
	if (!appCtxt.isOffline) { // see bug #29372
		appCtxt.setStatusMsg(ZmMsg.messageSent);
	}
	dialog.popdown();
};

ZmMailAssistant.prototype._handleErrorSendMsg =
function(dialog, ex) {
	var msg = null;
	if (ex.code == ZmCsfeException.MAIL_SEND_ABORTED_ADDRESS_FAILURE) {
		var invalid = ex.getData(ZmCsfeException.MAIL_SEND_ADDRESS_FAILURE_INVALID);
		var invalidMsg = (invalid && invalid.length) ? AjxMessageFormat.format(ZmMsg.sendErrorInvalidAddresses,
														AjxStringUtil.htmlEncode(invalid.join(", "))) : null;
		msg = ZmMsg.sendErrorAbort + "<br/>" + invalidMsg;
	} else if (ex.code == ZmCsfeException.MAIL_SEND_PARTIAL_ADDRESS_FAILURE) {
		var invalid = ex.getData(ZmCsfeException.MAIL_SEND_ADDRESS_FAILURE_INVALID);
		msg = (invalid && invalid.length) ? AjxMessageFormat.format(ZmMsg.sendErrorPartial,
											AjxStringUtil.htmlEncode(invalid.join(", "))) : ZmMsg.sendErrorAbort;
	}
	if (msg) {
		dialog.messageDialog(msg, DwtMessageDialog.CRITICAL_STYLE);
		return true;
	} else {
		return false;
	}
};

// User has agreed to send message with no subject
ZmMailAssistant.prototype._noSubjectOkCallback =
function(dialog) {
	this._noSubjectOkay = true;
	this._confirmDialog.popdown();
	this.okHandler(dialog);
};

// User has declined to send message with no subject
ZmMailAssistant.prototype._noSubjectCancelCallback =
function(dialog) {
	this._noSubjectOkay = false;
	this._confirmDialog.popdown();
};

// User has agreed to send message with bad addresses
ZmMailAssistant.prototype._badAddrsOkCallback =
function(dialog) {
	this._badAddrsOkay = true;
	this._confirmDialog.popdown();
	this.okHandler(dialog);
};

// User has declined to send message with bad addresses - set focus to bad field
ZmMailAssistant.prototype._badAddrsCancelCallback =
function(dialog) {
	this._badAddrsOkay = false;
	this._confirmDialog.popdown();
};

ZmMailAssistant.prototype._getAddrs =
function(msg, type, value, bad, good) {
	if (value == null || value == "") return;
	var result = AjxEmailAddress.parseEmailString(value, type, false)
	if (result.bad.size() > 0 && bad != null) bad.addList(result.bad);
	if (result.all.size() > 0) 	{
		msg.setAddresses(type, result.all);
		if (good != null) good.addList(result.all);
	}
};

// bad is a vector that gets filled with bad addresses
ZmMailAssistant.prototype.getMessage =
function(bad, good) { 
	var msg = new ZmMailMsg();
	var body = new ZmMimePart();
	body.setContentType(ZmMimeTable.TEXT_PLAIN);
	var bodyText = this._mailFields.body != null ? this._mailFields.body : "";
	var identity = appCtxt.getIdentityCollection().defaultIdentity;
	if (Boolean(identity.signature)) {
		var sig = identity.signature;
		if (sig != null && sig != "") {
			bodyText = bodyText + "\n" + 
				((identity.signatureStyle == ZmSetting.SIG_INTERNET) ? "--\n" : "") +
				sig + "\n";
		}
	}
	body.setContent(bodyText);
	msg.setTopPart(body);
	if (this._mailFields.subject) msg.setSubject(this._mailFields.subject);
	this._getAddrs(msg, AjxEmailAddress.TO, this._mailFields.to, bad, good);
	this._getAddrs(msg, AjxEmailAddress.CC, this._mailFields.cc, bad, good);
	this._getAddrs(msg, AjxEmailAddress.BCC, this._mailFields.bcc, bad, good);		
	return msg;
}

ZmMailAssistant.__RE_handleSubject = new RegExp([ZmAssistant.SPACES,"\"([^\"]*)\"?",ZmAssistant.SPACES].join(""));
ZmMailAssistant.__RE_handleKeyValue = new RegExp([
    "((",ZmAssistant.WORD,")(:",ZmAssistant.SPACES,")(.*?)",ZmAssistant.SPACES,")(",ZmAssistant.WORD,":|$)"
].join(""),"m");

ZmMailAssistant.prototype.handle =
function(dialog, verb, args) {
	dialog._setOkButton(AjxMsg.ok, true, true); //, true, "NewMessage");
	var match;
	this._mailFields = {};	

	if (!this._mailFields.subject) {
		match = args.match(ZmMailAssistant.__RE_handleSubject);
		if (match) {
			this._mailFields.subject = match[1];
			args = args.replace(match[0], " ");
		}
	}

	while(match = args.match(ZmMailAssistant.__RE_handleKeyValue)) {
		var k = match[2];
		var v = match[4];
		var strip = match[1];
		var field = ZmMailAssistant._FIELDS[k];
		if (field) {
			if (field.key == 'body') {
				strip = match[2] + match[3];
				v = ZmAssistant.trimLeading(args.replace(strip,""));
				args = "";
			}
			if (v == null) v = "";
			this._mailFields[field.key] = v;
		}
		if (args) args = args.replace(strip,"");
	}

	var index = -1, ri;

	for (var i=0; i < ZmMailAssistant._FIELD_ORDER.length; i++) {	
		var key = ZmMailAssistant._FIELD_ORDER[i];
		var field = ZmMailAssistant._FIELDS[key];
		var value = this._mailFields[key]; 
		if (value != null) {
			value = field.multiLine ? AjxStringUtil.convertToHtml(value) : AjxStringUtil.htmlEncode(value);
		}
		if (field.defaultValue || value != null) {
			//var useDefault = (value == null || value == "");
			var useDefault = (value == null);
			ri = this._setField(field.field, useDefault ? field.defaultValue : value, useDefault, false, index+1);
		} else {
			ri = this._setOptField(field.field, value, false, false, index+1);
		}
		index = Math.max(index, ri);
	}
};

ZmMailAssistant.prototype._getConfirmDialog = 
function(dialog) {
	if (!this._confirmDialog) {
		this._confirmDialog = new DwtMessageDialog({parent:dialog.shell, buttons:[DwtDialog.OK_BUTTON, DwtDialog.CANCEL_BUTTON]});
	}
	return this._confirmDialog;
};

}
if (AjxPackage.define("zimbraMail.mail.view.ZmAttachmentsView")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

ZmAttachmentsView = function(parent, controller) {


	DwtComposite.call(this, {parent:parent, className:"ZmAttachmentsView", posStyle:Dwt.ABSOLUTE_STYLE,
							 id:ZmId.getViewId(ZmId.VIEW_ATTACHMENTS)});

	this._controller = controller;

    this._initialize();

    // make sure no unnecessary scrollbars show up
	this.getHtmlElement().style.overflow = "hidden";

    this.addListener(DwtEvent.CONTROL, new AjxListener(this, this._controlEventListener));
};

ZmAttachmentsView.prototype = new DwtComposite;
ZmAttachmentsView.prototype.constructor = ZmAttachmentsView;

ZmAttachmentsView.prototype.toString = function() {
	return "ZmAttachmentsView";
};

ZmAttachmentsView.prototype.set = function(mailMsgList){

    this._attListView.set(mailMsgList);

    this.resize();

};

ZmAttachmentsView.prototype.resize = function(){

    var size = Dwt.getSize(this.getHtmlElement());
    var height = size.y - 10; //Correction for better look and feel

    var filterTreeSize = Dwt.getSize(this._viewContainer);
    var width  = filterTreeSize.x;

    this._attListView.setSize(width, height);
};

ZmAttachmentsView.prototype._controlEventListener = function(ev){

    this.resize();

};

ZmAttachmentsView.prototype.getFilterByTypes = function(){
    var type = this.getSelectedItem(ZmAttachmentsController.CATEGORY_BYTYPE);
    return type ? type.ct : ["any"];
};


ZmAttachmentsView.prototype.getSelectedItem = function(treeType){
    var selection = this.getSelection(treeType);
    return ( selection ) ? selection.getData(ZmAttachmentsView.ATT_KEY) : null;
};

ZmAttachmentsView.prototype.getSelection= function(treeType){
    var selection = this._tree[treeType].getSelection();
    return (selection && selection.length > 0) ? selection[0] : null;
};

ZmAttachmentsView.prototype.setQueryString = function(str){
    this._queryStringDiv.innerHTML = str;  
};

//Building UI

ZmAttachmentsView.prototype._initialize = function(){

    this._createHtml();
    this._createWidgets();

};

ZmAttachmentsView.prototype._createHtml = function(){

    var attachId = Dwt.getNextId();

    //var html = AjxTemplate.expand("mail.Message#AttachmentsView", {id:attachId});
    var html = "<table border='0' width='100%' height='100%'><tr><td width='15%' style='vertical-align:top;'><div id='"+attachId+"_filters'></div></td><td style='vertical-align:top;'><div id='"+attachId+"_results'></div></td></tr></table>";
    this.setContent(html);

    this._filtersContainer = Dwt.byId(attachId+"_filters");
    this._viewContainer = Dwt.byId(attachId+"_results");
};

ZmAttachmentsView.prototype._createWidgets = function(){

    this._createFilterTrees();

    this._createQueryStringDiv();

    this._attListView = new ZmAttachmentsListView(this, this._controller);

    this._viewContainer.appendChild(this._attListView.getHtmlElement());

};

ZmAttachmentsView.prototype._createQueryStringDiv = function(){

    var queryStringDivId = Dwt.getNextId();

    var html = "<div id='"+queryStringDivId+"' class='DwtTree ZmFilterTree' style='border-width:1px;padding:5px;'></div>";

    var div = document.createElement("div");
    div.style.paddingTop = '10px';
    div.innerHTML = html;

    this._filtersContainer.appendChild(div);

    this._queryStringDiv = document.getElementById(queryStringDivId);
    
};


ZmAttachmentsView.prototype._createFilterTrees = function(){

    this._tree = {};

    var treeListener = new AjxListener(this._controller, this._controller._filterTreeListener);

    this._createTree(ZmAttachmentsController.CATEGORY_BYTYPE, treeListener);
    this._createTree(ZmAttachmentsController.CATEGORY_BYTIME, treeListener);
    this._createOverviewTree(ZmAttachmentsController.CATEGORY_BYFOLDER, ZmOrganizer.FOLDER, treeListener);
    this._createOverviewTree(ZmAttachmentsController.CATEGORY_BYTAG, ZmOrganizer.TAG, treeListener);

};

ZmAttachmentsView.prototype._createOverviewTree = function(type, treeType, selectionListener){

    var treeIds = [];
    treeIds.push(treeType);

    var omit = {};
    if(treeType == ZmOrganizer.FOLDER){
        omit[ZmFolder.ID_ATTACHMENTS] = true;
    }

    var params = {
		overviewId: Dwt.getNextId(),
		parent: this,
		treeIds: treeIds,
        overviewClass: "DwtTree ZmFilterTree"
    };
    var overview = appCtxt.getOverviewController().createOverview(params);
	overview.set(treeIds, omit);

    var treeView = this._tree[type] = overview.getTreeView(treeType);

    var rootItem = treeView.getTreeItemById(ZmOrganizer.ID_ROOT);
    rootItem.setExpanded(true);
    rootItem.enableSelection(false);
    rootItem.setText(ZmMsg[ZmAttachmentsController.CATEGORY_MSGKEY[type]]);

    //this._newHeaderItem(treeView, {msgKey:ZmAttachmentsController.CATEGORY_MSGKEY[type]});

    this._filtersContainer.appendChild(overview.getHtmlElement());

    treeView.addSelectionListener(selectionListener);

    return treeView;
};

ZmAttachmentsView.prototype._createTree = function(type, selectionListener){

    var tree = this._tree[type]  =  new DwtTree({parent:this, className:"DwtTree ZmFilterTree"});

    var root = this._newHeaderItem(tree, {msgKey:ZmAttachmentsController.CATEGORY_MSGKEY[type]});
    tree.addSeparator();

    var categoryItems = ZmAttachmentsController.CATEGORY_ITEMS[type];
    for(var i=0; i<categoryItems.length; i++){
        this._newItem(root, categoryItems[i]);
    }
    tree.addSeparator();
    tree.addSeparator();

    this._filtersContainer.appendChild(tree.getHtmlElement());

    tree.addSelectionListener(selectionListener);

    root.setExpanded(true);

    return tree;
};

ZmAttachmentsView.ATT_KEY = "__att_key__";
ZmAttachmentsView.prototype._newItem = function(tree, attrs){
    var ti = new DwtTreeItem({parent:tree});
	ti.setImage(attrs.image);
	ti.setData(ZmAttachmentsView.ATT_KEY, attrs);
	ti.setText(attrs.msgKey ? ZmMsg[attrs.msgKey] : attrs.name);
	return ti;
};

ZmAttachmentsView.prototype._newHeaderItem = function(tree, attrs){
    var hti = new DwtHeaderTreeItem({parent:tree, className:"overviewHeader", index:0});
    hti.setData(ZmAttachmentsView.ATT_KEY, attrs);
	hti.setText(ZmMsg[attrs.msgKey]);
    hti.enableSelection(false);
    return hti;
};

ZmAttachmentsView.prototype._resetFilterTrees = function(){
   for(var treeType in this._tree){
       this._tree[treeType].deselectAll();
   }
};

//ZmAttachmentsListView

ZmAttachmentsListView = function(parent, controller) {

	ZmListView.call(this, {parent:parent, className:"ZmAttachmentsListView", posStyle:DwtControl.ABSOLUTE_STYLE,
					type:ZmItem.MSG, controller:controller});

    this._controller = controller;
    this._attachmentsView = parent;

    this._setAllowSelection();
    this.setScrollStyle(DwtControl.SCROLL);
};

ZmAttachmentsListView.prototype = new DwtListView;
ZmAttachmentsListView.prototype.constructor = ZmAttachmentsListView;

ZmAttachmentsListView.prototype.set =
function(list) {
 
    if(list instanceof ZmMailList){
       this._types = this._attachmentsView.getFilterByTypes();
       DwtListView.prototype.set.call(this, list.getVector());
    }
};

ZmAttachmentsListView.prototype.isValidType = function(attach){

    if(this._types[0] == "any") return true;

    if(this._types[0] == "image"){
        if(attach.ct.indexOf("image") == 0){
            return true;
        }
    }else{
        for(var i=0; i<this._types.length; i++){
            if(attach.ct == this._types[i]) return true;
        }
    }

    return false;
};

ZmAttachmentsListView.prototype._createItemHtml =
function(msg) {

    var hrefRoot = appCtxt.get(ZmSetting.CSFE_MSG_FETCHER_URI) + "&loc=" + AjxEnv.DEFAULT_LOCALE + "&id=" + msg.id + "&part="

    var htmlEl = [];

    var atts = msg.attachments;
    for (var i = 0; i < atts.length; i++) {

        var attach = atts[i];

        if(!this.isValidType(attach)) continue;

        var name = attach.name || attach.filename || (ZmMsg.unknown + " <" + attach.ct + ">");
        if(name.length>14){
		    name = name.substring(0,14)+"...";
	    }

        var mimeInfo = ZmMimeTable.getInfo(attach.ct);
        var icon = icon = "Img" + ( mimeInfo ? mimeInfo.imageLarge : "UnknownDoc_48");

        var div = document.createElement("div");
        div.className = "ZmBriefcaseItem";

        var div1 = document.createElement("div");
        div1.className = "ZmThumbnailItem";

        var div2 = document.createElement("div");
        div2.className = icon+" ZmThumbnailIcon";

        div1.appendChild(div2);
        div.appendChild(div1);

        var div2 = document.createElement("div");
        div2.className = "ZmThumbnailName";

        var span = document.createElement("span");
        span.innerHTML = ["<a href='",(hrefRoot + attach.part),"' target='_blank'>",name,"</a>"].join("");

        div2.appendChild(span);
        div.appendChild(div2);

        this.associateItemWithElement(msg, div);

        htmlEl.push(div);

    }

    return htmlEl;

};

ZmAttachmentsListView.prototype._addRow =
function(row, index) {
	if (!row) { return; }

	// bug fix #1894 - check for childNodes length otherwise IE barfs
	var len = this._parentEl.childNodes.length;

    if (index != null && len > 0 && index != len) {
        var childNodes = this._parentEl.childNodes;
        this._parentEl.insertBefore(row, childNodes[index]);
    } else {
		this._parentEl.appendChild(row);
	}
};

ZmAttachmentsListView.prototype._itemClicked =
function(clickedEl, ev) {

	this._selectedClass = "ZmBriefcaseItemSelected";
	this._kbFocusClass = "ZmBriefcaseItemFocused";
	this._normalClass = "ZmBriefcaseItem";
	this._disabledSelectedClass = "ZmBriefcaseItemDisabledSelect";
	this._rightClickClass = "ZmBriefcaseItemSelected";
	this._styleRe = new RegExp(
        "\\b(" +
        [   this._disabledSelectedClass,
            this._selectedClass,
            this._kbFocusClass,
            this._dndClass,
            this._rightClickClass//,
//          this._normalClass
        ].join("|") +
        ")\\b", "g"
    );

	DwtListView.prototype._itemClicked.call(this, clickedEl, ev);
};








}
if (AjxPackage.define("zimbraMail.mail.view.ZmMailConfirmView")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * Creates a new controller to show mail send confirmation.
 * @class
 * 
 * @param {DwtControl}	parent		the element that created this view
 * @param {ZmController}	controller	the controller managing this view
 * 
 * @extends		DwtComposite
 * 
 * @private
 */
ZmMailConfirmView = function(parent, controller) {

	this._view = ZmId.VIEW_MAIL_CONFIRM + controller.sessionId;
	DwtComposite.call(this, {parent:parent, className:"ZmMailConfirmView", posStyle:Dwt.ABSOLUTE_STYLE,
							 id:ZmId.getViewId(this._view)});

	this.setVisible(false);
	this._controller = controller;
	this._tabGroup = new DwtTabGroup(this._htmlElId);
	this._createHtmlFromTemplate("mail.Message#ZmMailConfirmView", { id: this._htmlElId } );
	var buttonArgs = {
		parent: this,
		parentElement: this._htmlElId + "_addButton"
	};
	this._addContactsButton = new DwtButton(buttonArgs);
	this._addContactsButton.setText(ZmMsg.ok);
	this._addContactsButton.addSelectionListener(new AjxListener(this, this._addContactsListener));

	this._summaryFormat = new AjxMessageFormat(ZmMsg.confirmSummary);
};

ZmMailConfirmView.prototype = new DwtComposite;
ZmMailConfirmView.prototype.constructor = ZmMailConfirmView;

ZmMailConfirmView.prototype.toString = function() {
	return "ZmMailConfirmView";
};

/**
 * Adds a listener for the Create Contacts button.
 * 
 * @param	{AjxListener}	listener		the listener
 */
ZmMailConfirmView.prototype.addNewContactsListener =
function(listener) {
	this.addListener(DwtEvent.ACTION, listener);
};

/**
 * Shows confirmation that the message was sent.
 *
 * @param {ZmMailMsg}	msg			the message that was sent
 */
ZmMailConfirmView.prototype.showConfirmation =
function(msg) {
	this._tabGroup.removeAllMembers();
	
	this._showLoading(true);
	var addresses = msg.getAddresses(AjxEmailAddress.TO).getArray().concat(msg.getAddresses(AjxEmailAddress.CC).getArray());

	if (!appCtxt.get(ZmSetting.CONTACTS_ENABLED) || appCtxt.get(ZmSetting.AUTO_ADD_ADDRESS)) {
		this._setView(msg, [], [], addresses);
	} else {
		var callback = new AjxCallback(this, this._handleResponseGetContacts, [msg]);
		appCtxt.getApp(ZmId.APP_CONTACTS).getContactsByEmails(addresses, callback);
	}

	appCtxt.notifyZimlets("onMailConfirm", [this, msg, Dwt.byId(this._htmlElId + "_ad")]);
};

ZmMailConfirmView.prototype.getController =
function() {
	return this._controller;
};

ZmMailConfirmView.prototype.getTitle =
function() {
	return [ZmMsg.zimbraTitle, ZmMsg.messageSent].join(": ");
};

ZmMailConfirmView.prototype.getTabGroupMember = function() {
	return this._tabGroup;
};

ZmMailConfirmView.prototype.getDefaultFocusItem =
function() {
	return this._addContactsButton;
};

ZmMailConfirmView.prototype._handleResponseGetContacts =
function(msg, contacts) {
	var newAddresses = [],
		existingContacts = [];
	for (var i = 0, count = contacts.length; i < count; i++) {
		if (contacts[i].contact) {
			existingContacts.push(contacts[i]);
		} else {
			newAddresses.push(contacts[i].address);
		}
	}
	this._setView(msg, newAddresses, existingContacts, []);
};

ZmMailConfirmView.prototype._setView =
function(msg, newAddresses, existingContacts, displayAddresses) {
	this._showLoading(false);
	Dwt.byId(this._htmlElId + "_summary").innerHTML = AjxStringUtil.htmlEncode(this._summaryFormat.format(msg.subject));
	this._showNewAddresses(newAddresses);
	this._showExistingContacts(existingContacts);
	this._showDisplayAddresses(displayAddresses);
};

ZmMailConfirmView.prototype._showLoading =
function(loading) {
	Dwt.setVisible(Dwt.byId(this._htmlElId + "_loading"), loading);
	Dwt.setVisible(Dwt.byId(this._htmlElId + "_notLoading"), !loading);
	this.setVisible(true);	
};

ZmMailConfirmView.prototype._showNewAddresses =
function(newAddresses) {
	var visible = newAddresses.length;
	Dwt.setVisible(Dwt.byId(this._htmlElId + "_newAddresses"), visible);
	this._addContactsButton.setVisible(visible);
	if (this._newAddressForms) {
		for (var i = 0, count = this._newAddressForms.length; i < count; i++) {
			this._newAddressForms[i].dispose();
		}
		this._newAddressForms = [];
	}
	this._newAddressForms = [];
	var newAddressBox = Dwt.byId(this._htmlElId + "_newAddressBox");
	for (var i = 0, count = newAddresses.length; i < count; i++) {
		var div = document.createElement("DIV");
		newAddressBox.appendChild(div);
		var address = newAddresses[i],
			first = "",
			last = "";
		var name = address.getName();
		if (name) {
			var index = name.lastIndexOf(" ");
			if (index != -1) {
				first = name.substring(0, index);
				last = name.substring(index + 1);
			} else {
				first = name;
			}
		}
		var args = { parent: this, parentElement: div };
		args.form = {
			items: [
				// default items
				{ id: "CHECKBOX", type: "DwtCheckbox", label: address.getAddress() },
				{ id: "FIRST", type: "DwtInputField", value: first, hint: ZmMsg.AB_FIELD_firstName },
				{ id: "LAST", type: "DwtInputField", value: last, hint: ZmMsg.AB_FIELD_lastName }
			],
			template: "mail.Message#ZmMailConfirmViewNewAddress"
		};
		var form = new DwtForm(args);
		this._newAddressForms.push(form);
		this._tabGroup.addMember(form.getTabGroupMember());
	}
	if (visible) {
		this._tabGroup.addMember(this._addContactsButton);
	}
};

ZmMailConfirmView.prototype._showExistingContacts =
function(existingContacts) {
	Dwt.setVisible(Dwt.byId(this._htmlElId + "_existingContacts"), existingContacts.length);
	var existingContactBox = Dwt.byId(this._htmlElId + "_existingContactBox");
	existingContactBox.innerHTML = "";
	for (var i = 0, count = existingContacts.length; i < count; i++) {
		var div = document.createElement("DIV");
		existingContactBox.appendChild(div);
		var data = existingContacts[i];
		var display;
		var fullName = data.contact.getFullName();
		if (fullName) {
			display = [fullName, " <", data.address.getAddress(), ">"].join("");
		} else {
			display = data.address.getAddress();
		}
		div.innerHTML = AjxTemplate.expand("mail.Message#ZmMailConfirmViewExistingContact", { text: AjxStringUtil.htmlEncode(display) });
	}
};

ZmMailConfirmView.prototype._showDisplayAddresses =
function(displayAddresses) {
	Dwt.setVisible(Dwt.byId(this._htmlElId + "_displayAddresses"), displayAddresses.length);
	var displayAddressBox = Dwt.byId(this._htmlElId + "_displayAddressBox");
	displayAddressBox.innerHTML = "";
	for (var i = 0, count = displayAddresses.length; i < count; i++) {
		var div = document.createElement("DIV");
		displayAddressBox.appendChild(div);
		var address = displayAddresses[i].toString();
		div.innerHTML = AjxTemplate.expand("mail.Message#ZmMailConfirmViewExistingContact", { text: AjxStringUtil.htmlEncode(address) });
	}
};

ZmMailConfirmView.prototype._addContactsListener =
function() {
	var newAddresses = [];
	for (var i = 0, count = this._newAddressForms.length; i < count; i++) {
		var form = this._newAddressForms[i];
		if (form.getValue("CHECKBOX")) {
			var data = {};
			data[ZmContact.F_email] = form.getControl("CHECKBOX").getText();
			data[ZmContact.F_firstName] = form.getValue("FIRST");
			data[ZmContact.F_lastName] = form.getValue("LAST");
			newAddresses.push(data);
		}
	}
	this.notifyListeners(DwtEvent.ACTION, newAddresses);
};

ZmMailConfirmView.prototype.deactivate =
function() {
	this._controller.inactive = true;
};
}
if (AjxPackage.define("zimbraMail.mail.view.ZmSelectAddrDialog")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * @overview
 */

/**
 * Creates a dialog to allow the user to select a recipient address to search on.
 * @class
 * Provides the user a way to search on a recipient when they click on the To:
 * column in Sent or Drafts. That is in place of actual sorting. See bug 6830.
 *
 * @author Conrad Damon
 *
 * @param {DwtComposite}	parent				the parent widget (the shell)
 *
 * @extends		DwtDialog
 */
ZmSelectAddrDialog = function(parent) {

	if (arguments.length == 0) { return; }

	DwtDialog.call(this, {parent:parent, title:ZmMsg.findEmailsSentFolderTitle, className:"ZmSelectAddrDialog"});
    var okButton = this.getButton(DwtDialog.OK_BUTTON);
    if (okButton)
	    okButton.setText(ZmMsg.find);

    // create auto-completer
    if (appCtxt.get(ZmSetting.CONTACTS_ENABLED) || appCtxt.get(ZmSetting.GAL_ENABLED)) {
        var params = {
            dataClass: appCtxt.getAutocompleter(),
            matchValue: ZmAutocomplete.AC_VALUE_EMAIL,
            compCallback: (new AjxCallback(this, this._handleCompletionData, [this])),
            keyUpCallback: (new AjxCallback(this, this._acKeyUpListener))
        };
        this._acAddrSelectList = new ZmAutocompleteListView(params);
    }

	this.setContent(this._contentHtml());
    this._createInputView();
    this._createControls();
	this.setButtonListener(DwtDialog.OK_BUTTON, new AjxListener(this, this._okButtonListener));
};

ZmSelectAddrDialog.prototype = new DwtDialog;
ZmSelectAddrDialog.prototype.constructor = ZmQuickAddDialog;

ZmSelectAddrDialog.prototype.toString =
function() {
	return "ZmSelectAddrDialog";
};

ZmSelectAddrDialog.prototype._contentHtml =
function() {
    var html = [];
	var idx = 0;
	html[idx++] = "<table cellpadding=0 cellspacing=0 border=0>";
	html[idx++] = AjxTemplate.expand("share.Dialogs#ZmSelectAddrDialog", {id:this._htmlElId});
	html[idx++] = "</table>";
	return html.join("");
};

ZmSelectAddrDialog.prototype._createControls =
function() {
 if (this._htmlElId) {
     var cboxId = this._htmlElId + "_cboxFind";
     this._findEmailsCheckbox = new DwtCheckbox({parent: this, id: cboxId, checked: true});
     this._findEmailsCheckbox.replaceElement(document.getElementById(cboxId));
     this._findEmailsCheckbox.setText(ZmMsg.findEmailsToAll);
 }
};

ZmSelectAddrDialog.prototype._createInputView =
function(){
    this._recipientInput = new DwtInputField({parent: this});
	this._recipientInput.setData(Dwt.KEY_OBJECT, this);
	this._recipientInput.setRequired(true);
	Dwt.associateElementWithObject(this._recipientInput.getInputElement(), this);
    this._recipientInput.setValidatorFunction(null, DwtInputField.validateEmail);

    var inputEl = this._recipientInput.getInputElement();
	if (this._acAddrSelectList) {
		this._acAddrSelectList.handle(inputEl);
	}

    var id = this._htmlElId + "_addrListTd";
	var td = document.getElementById(id);
	if (td) {
		this._recipientInput.reparentHtmlElement(td);
	}
};


/**
 * Populates the input field before showing the dialog.
 *
 * @param {array}		addrs			list of AjxEmailAddress to show user
 * @param {string}		folderId		ID of outbound folder
 */
ZmSelectAddrDialog.prototype.popup =
function(addrs, folderId) {

	this._addrs = [];
    var idx = folderId.indexOf(":");
    this._folderId = (idx != -1) ? (folderId.substr(idx + 1)) : folderId;
	var used = {}, id = 1;
	for (var i = 0, len = addrs.length; i < len; i++) {
		var addr = addrs[i];
		var email = addr.getAddress();
		// remove duplicates
		if (!used[email]) {
			addr.id = id++;
            addr.setAddress(AjxEmailAddress.split(email)[0]); //just show email portion
			this._addrs.push(addr);
			used[email] = true;
		}
	}
	this._addrs.sort(AjxEmailAddress.sortCompareByAddress);
    var inputValue = "";
    for (var i=0; i<this._addrs.length; i++){
        inputValue += this._addrs[i].getAddress() + "; ";
    }
	this._recipientInput.setValue(inputValue);
    var size = this.getSize();
	Dwt.setSize(this._recipientInput.getInputElement(), size.x - 87);
    DwtDialog.prototype.popup.call(this);

    if (addrs.length == 0)
        this._recipientInput.focus();
};

/**
 * @private
 */
ZmSelectAddrDialog.prototype._okButtonListener =
function() {


    var error;
    if (this._recipientInput.isValid() == null) {
	    error = this._recipientInput.getValue() ? AjxMsg.invalidEmailAddr : AjxMsg.valueIsRequired;
	}

    if (error) {
        return this._showError(error);
    }

    var emails = this._recipientInput.getValue();

	if (emails) {
        emails = AjxStringUtil.trim(emails);
        if (emails.substring(emails.length -1) == ";")
            emails = emails.substring(0, emails.length-1); //remove trailing ";"
        var query = [];
        var addr = emails.split(";");
		var folder = ZmFolder.QUERY_NAME[this._folderId];
	    for (var i = 0; i < addr.length; i++) {
            if (AjxEmailAddress.isValid(addr[i]))
		        query[i] = ["tocc:(", addr[i], ")"].join("");
            else{
                return this._showError(AjxMessageFormat.format(AjxMsg.invalidEmailAddrValue, addr[i]));
            }

        }
        var operator =  this._findEmailsCheckbox && this._findEmailsCheckbox.isSelected() ? " AND " : " OR ";
		appCtxt.getSearchController().search({query:"in:" + folder + " AND (" + query.join(operator) + ")"});
	}
	this.popdown();
};

/**
 * @private
 */
ZmSelectAddrDialog.prototype._showError =
function(errorMessage) {

    var dialog = appCtxt.getErrorDialog();
    dialog.setMessage(errorMessage);
    dialog.popup(null, true);
};

/**
 * Creates the address list view.
 * @class
 * This class represents the address list view.
 *
 * @param	{DwtComposite}	parent		the parent widget
 *
 * @extends		DwtListView
 * 
 * @private
 */
ZmAddrListView = function(parent) {

	var headerList = this._getHeaderList();
	DwtListView.call(this, {parent:parent, headerList:headerList, view:"ALV"});
};

ZmAddrListView.COL_NAME	= "na";
ZmAddrListView.COL_ADDR	= "ad";

ZmAddrListView.prototype = new DwtListView;
ZmAddrListView.prototype.constructor = ZmAddrListView;

ZmAddrListView.prototype.toString =
function() {
	return "ZmAddrListView";
};

ZmAddrListView.prototype._getHeaderList =
function() {
	return [
		(new DwtListHeaderItem({field:ZmAddrListView.COL_NAME, text:ZmMsg.name, width:ZmMsg.COLUMN_WIDTH_NAME_SD})),
		(new DwtListHeaderItem({field:ZmAddrListView.COL_ADDR, text:ZmMsg.address}))
	];
};

ZmAddrListView.prototype._getCellContents =
function(html, idx, item, field, colIdx, params) {
	if (field == ZmAddrListView.COL_NAME) {
		html[idx++] = AjxStringUtil.htmlEncode(item.getName());
	} else if (field == ZmAddrListView.COL_ADDR) {
		html[idx++] = AjxStringUtil.htmlEncode(item.getAddress());
	}

	return idx;
};
}

if (AjxPackage.define("zimbraMail.mail.controller.ZmComposeController")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * Creates a new compose controller to manage message composition.
 * @constructor
 * @class
 * This class manages message composition.
 *
 * @author Conrad Damon
 * 
 * @param {ZmComposite}	container	the containing shell
 * @param {ZmMailApp}	mailApp			the containing app
 * 
 * @extends		ZmController
 */
ZmComposeController = function(container, mailApp) {

	ZmController.call(this, container, mailApp);

	this._action = null;

	ZmComposeController._setStatics();

	this._listeners = {};
	this._listeners[ZmOperation.SEND] = new AjxListener(this, this._sendListener);
	this._listeners[ZmOperation.SEND_MENU] = new AjxListener(this, this._sendListener);
	this._listeners[ZmOperation.SEND_LATER] = new AjxListener(this, this._sendLaterListener);
	this._listeners[ZmOperation.CANCEL] = new AjxListener(this, this._cancelListener);
	this._listeners[ZmOperation.ATTACHMENT] = new AjxListener(this, this._attachmentListener);
	this._listeners[ZmOperation.DETACH_COMPOSE] = new AjxListener(this, this._detachListener);
	this._listeners[ZmOperation.SAVE_DRAFT] = new AjxListener(this, this._saveDraftListener);
	this._listeners[ZmOperation.SPELL_CHECK] = new AjxListener(this, this._spellCheckListener);
	this._listeners[ZmOperation.COMPOSE_OPTIONS] = new AjxListener(this, this._optionsListener);

	this._dialogPopdownListener = new AjxListener(this, this._dialogPopdownActionListener);

	var settings = appCtxt.getSettings();
	var scl = this._settingChangeListener = new AjxListener(this, this._handleSettingChange);
	for (var i = 0; i < ZmComposeController.SETTINGS.length; i++) {
		settings.getSetting(ZmComposeController.SETTINGS[i]).addChangeListener(scl);
	}

	this._autoSaveTimer = null;
	this._draftType = ZmComposeController.DRAFT_TYPE_NONE;
};

ZmComposeController.prototype = new ZmController();
ZmComposeController.prototype.constructor = ZmComposeController;

ZmComposeController.prototype.toString =
function() {
	return "ZmComposeController";
};

//
// Constants
//

ZmComposeController.SIGNATURE_KEY = "sigKeyId";

// Constants for defining the reason for saving a draft message.
/**
 * Defines the "none" draft type reason.
 */
ZmComposeController.DRAFT_TYPE_NONE		= "none";
/**
 * Defines the "manual" draft type reason.
 */
ZmComposeController.DRAFT_TYPE_MANUAL	= "manual";
/**
 * Defines the "auto" draft type reason.
 */
ZmComposeController.DRAFT_TYPE_AUTO		= "auto";
/**
 * Defines the "delaysend" draft type reason.
 */
ZmComposeController.DRAFT_TYPE_DELAYSEND	= "delaysend";

ZmComposeController.DEFAULT_TAB_TEXT = ZmMsg.compose;

ZmComposeController.NEW_WINDOW_WIDTH = 975;
ZmComposeController.NEW_WINDOW_HEIGHT = 475;

ZmComposeController._setStatics =
function() {

	if (ZmComposeController.SETTINGS) { return; }

	// settings whose changes affect us (so we add a listener to them)
	ZmComposeController.SETTINGS = [ZmSetting.SHOW_BCC, ZmSetting.USE_ADDR_BUBBLES];

	// radio groups for options items
	ZmComposeController.RADIO_GROUP = {};
	ZmComposeController.RADIO_GROUP[ZmOperation.REPLY]				= 1;
	ZmComposeController.RADIO_GROUP[ZmOperation.REPLY_ALL]			= 1;
    ZmComposeController.RADIO_GROUP[ZmOperation.CAL_REPLY]			= 1;
	ZmComposeController.RADIO_GROUP[ZmOperation.CAL_REPLY_ALL]		= 1;
	ZmComposeController.RADIO_GROUP[ZmOperation.FORMAT_HTML]		= 2;
	ZmComposeController.RADIO_GROUP[ZmOperation.FORMAT_TEXT]		= 2;
	ZmComposeController.RADIO_GROUP[ZmOperation.INC_ATTACHMENT]		= 3;
    ZmComposeController.RADIO_GROUP[ZmOperation.INC_BODY]	    	= 3;
	ZmComposeController.RADIO_GROUP[ZmOperation.INC_NONE]			= 3;
	ZmComposeController.RADIO_GROUP[ZmOperation.INC_SMART]			= 3;

	// translate between include settings and operations
	ZmComposeController.INC_OP = {};
	ZmComposeController.INC_OP[ZmSetting.INC_ATTACH]		= ZmOperation.INC_ATTACHMENT;
	ZmComposeController.INC_OP[ZmSetting.INC_BODY]			= ZmOperation.INC_BODY;
	ZmComposeController.INC_OP[ZmSetting.INC_NONE]			= ZmOperation.INC_NONE;
	ZmComposeController.INC_OP[ZmSetting.INC_SMART]			= ZmOperation.INC_SMART;
	ZmComposeController.INC_MAP = {};
	for (var i in ZmComposeController.INC_OP) {
		ZmComposeController.INC_MAP[ZmComposeController.INC_OP[i]] = i;
	}

	ZmComposeController.OPTIONS_TT = {};
	ZmComposeController.OPTIONS_TT[ZmOperation.NEW_MESSAGE]		= "composeOptions";
	ZmComposeController.OPTIONS_TT[ZmOperation.REPLY]			= "replyOptions";
	ZmComposeController.OPTIONS_TT[ZmOperation.REPLY_ALL]		= "replyOptions";
    ZmComposeController.OPTIONS_TT[ZmOperation.CAL_REPLY]			= "replyOptions";
	ZmComposeController.OPTIONS_TT[ZmOperation.CAL_REPLY_ALL]		= "replyOptions";
	ZmComposeController.OPTIONS_TT[ZmOperation.FORWARD_ATT]		= "forwardOptions";
	ZmComposeController.OPTIONS_TT[ZmOperation.FORWARD_INLINE]	= "forwardOptions";

	ZmComposeController.OP_CHECK = {};
	ZmComposeController.OP_CHECK[ZmOperation.REQUEST_READ_RECEIPT] 	= true;
	ZmComposeController.OP_CHECK[ZmOperation.USE_PREFIX] 			= true;
	ZmComposeController.OP_CHECK[ZmOperation.INCLUDE_HEADERS] 		= true;
};

//
// Public methods
//

/**
* Called by ZmNewWindow.unload to remove ZmSettings listeners (which reside in
* the parent window). Otherwise, after the child window is closed, the parent
* window is still referencing the child window's compose controller, which has
* been unloaded!!
* 
* @private
*/
ZmComposeController.prototype.dispose =
function() {
	var settings = appCtxt.getSettings();
	for (var i = 0; i < ZmComposeController.SETTINGS.length; i++) {
		settings.getSetting(ZmComposeController.SETTINGS[i]).removeChangeListener(this._settingChangeListener);
	}
	this._composeView.dispose();

	var app = this.getApp();
	app.disposeTreeControllers();
};

/**
 * Begins a compose session by presenting a form to the user.
 *
 * @param	{Hash}	params		a hash of parameters
 * @param {constant}	params.action		the new message, reply, forward, or an invite action
 * @param {Boolean}	params.inNewWindow		if <code>true</code>, we are in detached window
 * @param {ZmMailMsg}	params.msg			the original message (reply/forward), or address (new message)
 * @param {String}	params.toOverride 	the initial value for To: field
 * @param {String}	params.subjOverride 	the initial value for Subject: field
 * @param {String}	params.extraBodyText the canned text to prepend to body (invites)
 * @param {AjxCallback}	params.callback		the callback to run after view has been set
 * @param {String}	params.accountName	the on-behalf-of From address
 * @param {String}	accountName	[string]*		on-behalf-of From address
 */
ZmComposeController.prototype.doAction =
function(params) {

	AjxDebug.println(AjxDebug.REPLY, "-------------- " + params.action + " ----------------------------------");
	var ac = window.parentAppCtxt || window.appCtxt;
	if (params.action == ZmOperation.REPLY || params.action == ZmOperation.REPLY_ALL) {
		var str = ["Browser: " + AjxEnv.browser,
				   "Offline: " + Boolean(ac.isOffline),
				   "Multi accts: " + ac.multiAccounts,
				   "New Window: " + params.inNewWindow,
				   "Reading pane location: " + ac.get(ZmSetting.READING_PANE_LOCATION)].join(" / ");
		AjxDebug.println(AjxDebug.REPLY, str);
	}
	
	// in zdesktop, it's possible there are no accounts that support smtp
	if (ac.isOffline && !ac.get(ZmSetting.OFFLINE_COMPOSE_ENABLED)) {
		var d = ac.getMsgDialog();
		d.setMessage(ZmMsg.composeDisabled, DwtMessageDialog.CRITICAL_STYLE);
		d.popup();
		return;
	}

	this._msgSent = false;
	if (params.inNewWindow) {
        var msgId = params.msg ? params.msg.nId : (this._msg ? this._msg.nId : Dwt.getNextId());
		var newWinObj = ac.getNewWindow(false, ZmComposeController.NEW_WINDOW_WIDTH, ZmComposeController.NEW_WINDOW_HEIGHT, ZmId.VIEW_COMPOSE + "_" + msgId);

		// this is how child window knows what to do once loading:
		newWinObj.command = "compose";
		newWinObj.params = params;
        if (newWinObj.win) {
            newWinObj.win.focus();
        }
	} else {
		this._setView(params);
		this._listController = params.listController;
	}
};

/**
 * Toggles the spell check button.
 * 
 * @param	{Boolean}	selected		if <code>true</code>, toggle the spell check to "selected"
 * 
 */
ZmComposeController.prototype.toggleSpellCheckButton =
function(selected) {
	var spellCheckButton = this._toolbar.getButton(ZmOperation.SPELL_CHECK);
	if (spellCheckButton) {
		spellCheckButton.setSelected((selected || false));
	}
};

/**
 * Detaches compose view to child window.
 * 
 */
ZmComposeController.prototype.detach =
function() {
	// bug fix #7192 - disable detach toolbar button
	this._toolbar.enable(ZmOperation.DETACH_COMPOSE, false);

	var view = this._composeView;
	var msg = view._msg;
	var subj = view._subjectField.value;
	var forAttHtml = view._attcDiv.innerHTML;
	var msgAttId = view._msgAttId; //include original as attachment
	var body = this._getBodyContent();
	var composeMode = view.getComposeMode();
	var backupForm = view.backupForm;
	var sendUID = view.sendUID;
	var action = view._action || this._action;
	var identity = view.getIdentity();
    var requestReadReceipt = this.isRequestReadReceipt();

	var addrList = {};
	var addrs = !view._useAcAddrBubbles && view.getRawAddrFields();
	for (var i = 0; i < ZmMailMsg.COMPOSE_ADDRS.length; i++) {
		var type = ZmMailMsg.COMPOSE_ADDRS[i];
		addrList[type] = view._useAcAddrBubbles ? view._addrInputField[type].getAddresses(true) : addrs[type];
	}

	// this is how child window knows what to do once loading:
    var msgId = (msg && msg.nId) || Dwt.getNextId();
	var newWinObj = appCtxt.getNewWindow(false, ZmComposeController.NEW_WINDOW_WIDTH, ZmComposeController.NEW_WINDOW_HEIGHT, ZmId.VIEW_COMPOSE + "_" + msgId);
    if (newWinObj.win) {
        newWinObj.win.focus();
    }
	newWinObj.command = "composeDetach";
	newWinObj.params = {
		action: action,
		msg: msg,
		addrs: addrList,
		subj: subj,
		priority: view._getPriority(),
		forwardHtml: forAttHtml,
		msgAttId: msgAttId,
		body: body,
		composeMode: composeMode,
		identityId: (identity ? identity.id : null),
		accountName: this._accountName,
		backupForm: backupForm,
		sendUID: sendUID,
		msgIds: this._msgIds,
		forAttIds: this._forAttIds,
		sessionId: this.sessionId,
        readReceipt: requestReadReceipt
	};
};

ZmComposeController.prototype.popShield =
function() {
	var dirty = this._composeView.isDirty();
	if (!dirty && (this._draftType != ZmComposeController.DRAFT_TYPE_AUTO)) {
		return true;
	}

	var ps = this._popShield = appCtxt.getYesNoCancelMsgDialog();
	if (this._draftType == ZmComposeController.DRAFT_TYPE_AUTO) {
		// Messsage has been saved, but never explicitly by the user.
		// Ask if he wants to keep the autosaved draft.
		ps.reset();
		ps.setMessage(ZmMsg.askSaveAutosavedDraft, DwtMessageDialog.WARNING_STYLE);
		if (dirty) {
			ps.registerCallback(DwtDialog.YES_BUTTON, this._popShieldYesCallback, this);
		} else {
			ps.registerCallback(DwtDialog.YES_BUTTON, this._popShieldNoCallback, this);
		}
		ps.registerCallback(DwtDialog.NO_BUTTON, this._popShieldDiscardCallback, this);
		ps.registerCallback(DwtDialog.CANCEL_BUTTON, this._popShieldDismissCallback, this);
	} else if (this._canSaveDraft()) {
		ps.reset();
		ps.setMessage(ZmMsg.askSaveDraft, DwtMessageDialog.WARNING_STYLE);
		ps.registerCallback(DwtDialog.YES_BUTTON, this._popShieldYesCallback, this);
		ps.registerCallback(DwtDialog.NO_BUTTON, this._popShieldNoCallback, this);
		ps.registerCallback(DwtDialog.CANCEL_BUTTON, this._popShieldDismissCallback, this);
	} else {
		ps = this._popShield = appCtxt.getYesNoMsgDialog();
		ps.setMessage(ZmMsg.askLeaveCompose, DwtMessageDialog.WARNING_STYLE);
		ps.registerCallback(DwtDialog.YES_BUTTON, this._popShieldYesCallback, this);
		ps.registerCallback(DwtDialog.NO_BUTTON, this._popShieldDismissCallback, this);
	}
	ps.addPopdownListener(this._dialogPopdownListener);
	ps.popup(this._composeView._getDialogXY());

	return false;
};

// We don't call ZmController._preHideCallback here because it saves the current
// focus member, and we want to start over each time
ZmComposeController.prototype._preHideCallback =
function(view, force) {

	if (this._autoSaveTimer) {
        this._autoSaveTimer.kill();

		//the following is a bit suspicous to me. I assume maybe this method might be called with force == true
		//in a way that is not after the popShield was activated? That would be the only explanation to have this.
		//I wonder if that's the case that leaves orphan drafts
		if (force) {
			// auto-save if we leave this compose tab and the message has not yet been sent
			//note that _msgSent is true after message was saved as draft.
			if (!this._msgSent) {
				//note that it will not save if !this._composeView.isDirty(), which is the case after the
				//user said "no" to keeping the draft.
				this._autoSaveCallback(true);
			}
		}
	}
	return force ? true : this.popShield();
};

ZmComposeController.prototype._preUnloadCallback =
function(view) {
	return !this._composeView.isDirty();
};


ZmComposeController.prototype._preShowCallback =
function() {
	this._setSearchToolbarVisibilityPerSkin(false);
	return true;
};

ZmComposeController.prototype._postShowCallback =
function() {
	// always reset auto save every time this view is shown. This covers the
	// case where a compose tab is inactive and becomes active when user clicks
	// on compose tab.
	this._initAutoSave();

	if (!appCtxt.isChildWindow) {
		// no need to "restore" focus between windows
		ZmController.prototype._postShowCallback.call(this);
	}
	var composeMode = this._composeView.getComposeMode();
	if (this._action != ZmOperation.NEW_MESSAGE &&
		this._action != ZmOperation.FORWARD_INLINE &&
		this._action != ZmOperation.FORWARD_ATT)
	{
		if (composeMode == DwtHtmlEditor.HTML) {
			var ta = new AjxTimedAction(this._composeView, this._composeView._focusHtmlEditor);
			AjxTimedAction.scheduleAction(ta, 10);
		}
		this._composeView._setBodyFieldCursor();
	}
};

ZmComposeController.prototype._postHideCallback =
function() {
	// hack to kill the child window when replying to an invite
	if (appCtxt.isChildWindow &&
		(this._action == ZmOperation.REPLY_ACCEPT ||
		this._action == ZmOperation.REPLY_DECLINE ||
		this._action == ZmOperation.REPLY_TENTATIVE))
	{
		window.close();
		return;
	}

	this._setSearchToolbarVisibilityPerSkin(true);
};

/**
 * This method gets called if user clicks on mailto link while compose view is
 * already being used.
 * 
 * @private
 */
ZmComposeController.prototype.resetComposeForMailto =
function(params) {
	if (this._popShield && this._popShield.isPoppedUp()) {
		return false;
	}

	var ps = this._popShield = appCtxt.getYesNoCancelMsgDialog();
	ps.reset();
	ps.setMessage(ZmMsg.askSaveDraft, DwtMessageDialog.WARNING_STYLE);
	ps.registerCallback(DwtDialog.YES_BUTTON, this._popShieldYesCallback, this, params);
	ps.registerCallback(DwtDialog.NO_BUTTON, this._popShieldNoCallback, this, params);
	ps.registerCallback(DwtDialog.CANCEL_BUTTON, this._popShieldDismissCallback, this);
	ps.addPopdownListener(this._dialogPopdownListener);
	ps.popup(this._composeView._getDialogXY());

	return true;
};

/**
 * Sends the message represented by the content of the compose view.
 * 
 * @param	{String}	attId		the id
 * @param	{constant}	draftType		the draft type (see <code>ZmComposeController.DRAFT_TYPE_</code> constants)
 * @param	{AjxCallback}	callback		the callback
 * @param	{Boolean}	processDataURIImages
 */
ZmComposeController.prototype.sendMsg =
function(attId, draftType, callback, contactId, processDataURIImages) {
    if(typeof processDataURIImages === "undefined"){
        processDataURIImages = true;
    }
    if(processDataURIImages){
        var processDataURIImagesCallback = new AjxCallback(this, this._sendMsg, [attId, null, draftType, callback, contactId]);
        var result = this._processDataURIImages(this._composeView._getIframeDoc() ,processDataURIImagesCallback);
        if(result){
            return;
        }
    }
	return this._sendMsg(attId, null, draftType, callback, contactId);
};

/**
 * Sends the message represented by the content of the compose view with specified docIds as attachment.
 * 
 * @param	{Array}	docIds		the document Ids
 * @param	{constant}	draftType		the draft type (see <code>ZmComposeController.DRAFT_TYPE_</code> constants)
 * @param	{AjxCallback}	callback		the callback
 */
ZmComposeController.prototype.sendDocs =
function(docIds, draftType, callback, contactId) {
	return this._sendMsg(null, docIds, draftType, callback, contactId);
};

/**
 * Sends the message represented by the content of the compose view.
 * 
 * @private
 */
ZmComposeController.prototype._sendMsg =
function(attId, docIds, draftType, callback, contactId) {

	var isTimed = Boolean(this._sendTime);
	draftType = draftType || (isTimed ? ZmComposeController.DRAFT_TYPE_DELAYSEND : ZmComposeController.DRAFT_TYPE_NONE);
	var isDraft = draftType != ZmComposeController.DRAFT_TYPE_NONE;
    var isAutoSave = draftType == ZmComposeController.DRAFT_TYPE_AUTO;

	// bug fix #38408 - briefcase attachments need to be set *before* calling
	// getMsg() but we cannot do that without having a ZmMailMsg to store it in.
	// File this one under WTF.
	var tempMsg;
	if (docIds) {
		tempMsg = new ZmMailMsg();
		this._composeView.setDocAttachments(tempMsg, docIds);
	}
	var msg = this._composeView.getMsg(attId, isDraft, tempMsg, isTimed, contactId);

	if (!msg) {
		return;
	}

	if (this._autoSaveTimer) {
		//kill the timer, no save is attempted while message is pending
        this._autoSaveTimer.kill();
	}

	var origMsg = msg._origMsg;
	var isCancel = (msg.inviteMode == ZmOperation.REPLY_CANCEL);
	var isModify = (msg.inviteMode == ZmOperation.REPLY_MODIFY);

	if (isCancel || isModify) {
		var appt = origMsg._appt;
		var respCallback = new AjxCallback(this, this._handleResponseCancelOrModifyAppt);
		if (isCancel) {
			appt.cancel(origMsg._mode, msg, respCallback);
		} else {
			appt.save();
		}
		return;
	}

	var ac = window.parentAppCtxt || window.appCtxt;
	var acctName = appCtxt.multiAccounts
		? this._composeView.getFromAccount().name : this._accountName;

	if (isDraft) {
		if (appCtxt.multiAccounts) {
			// for offline, save drafts based on account owner of From: dropdown
			acctName = ac.accountList.getAccount(msg.fromSelectValue.accountId).name;
		} else {
			acctName = ac.getActiveAccount().name;
		}
		if (msg._origMsg && msg._origMsg.isDraft) {
			// if shared folder, make sure we save the draft under the owner account name
			var folder = msg.folderId ? ac.getById(msg.folderId) : null;
			if (folder && folder.isRemote()) {
				acctName = folder.getOwner();
			}
		}
	} else {
		// if shared folder, make sure we send the email on-behalf-of
		var folder = msg.folderId ? ac.getById(msg.folderId) : null;
		if (folder && folder.isRemote() && this._composeView.sendMsgOboIsOK()) {
			acctName = folder.getOwner();            
		}
	}

	if (origMsg) {
		origMsg.sendAsMe = !this._composeView.sendMsgOboIsOK();
	}

	// If this message had been saved from draft and it has a sender (meaning
	// it's a reply from someone else's account) then get the account name from
	// the from field.
	if (!acctName && !isDraft && origMsg && origMsg.isDraft) {
		if (this._composeView.sendMsgOboIsOK()) {
			if (origMsg._addrs[ZmMailMsg.HDR_FROM] &&
				origMsg._addrs[ZmMailMsg.HDR_SENDER] &&
				origMsg._addrs[ZmMailMsg.HDR_SENDER].size())
			{
				acctName = origMsg._addrs[ZmMailMsg.HDR_FROM].get(0).address;
			}
		} else {
			origMsg.sendAsMe = true; // hack.
		}
	}

	// check for read receipt
	var requestReadReceipt = this.isRequestReadReceipt();

	var respCallback = new AjxCallback(this, this._handleResponseSendMsg, [draftType, msg, callback]);
	var errorCallback = new AjxCallback(this, this._handleErrorSendMsg, [draftType, msg]);
	msg.send(isDraft, respCallback, errorCallback, acctName, null, requestReadReceipt, null, this._sendTime, isAutoSave);
	this._resetDelayTime();
};

ZmComposeController.prototype._handleResponseSendMsg =
function(draftType, msg, callback, result) {
	var resp = result.getResponse();
	// Reset the autosave interval to the default
	delete(this._autoSaveInterval);
	// Re-enable autosave
	this._initAutoSave();
	var needToPop = this._processSendMsg(draftType, msg, resp);

	this._msg = msg;

	if (callback) {
		callback.run(result);
	}

    if(this.sendMsgCallback) {
        this.sendMsgCallback.run(result);
    }

	appCtxt.notifyZimlets("onSendMsgSuccess", [this, msg]);//notify Zimlets on success

	if (needToPop) {
		this._app.popView(true, this._currentView);
	}

};

ZmComposeController.prototype._handleResponseCancelOrModifyAppt =
function() {
	AjxDebug.println(AjxDebug.REPLY, "Reset compose view: _handleResponseCancelOrModifyAppt");
	this._composeView.reset(false);
	this._app.popView(true);
    appCtxt.setStatusMsg(ZmMsg.messageSent);
};

ZmComposeController.prototype._handleErrorSendMsg =
function(draftType, msg, ex) {
    var retVal = false;
    this.resetToolbarOperations();
    this._composeView.enableInputs(true);

    appCtxt.notifyZimlets("onSendMsgFailure", [this, ex, msg]);//notify Zimlets on failure
    if (ex && ex.code) {

        var msg = null;
        var showMsg = false;
        if (ex.code == ZmCsfeException.MAIL_SEND_ABORTED_ADDRESS_FAILURE) {
            var invalid = ex.getData ? ex.getData(ZmCsfeException.MAIL_SEND_ADDRESS_FAILURE_INVALID) : null;
            var invalidMsg = (invalid && invalid.length)
                ? AjxMessageFormat.format(ZmMsg.sendErrorInvalidAddresses, AjxStringUtil.htmlEncode(invalid.join(", ")))
                : null;
            msg = ZmMsg.sendErrorAbort + "<br/>" + invalidMsg;
            this.popupErrorDialog(msg, ex, true, true);
            retVal = true;
        } else if (ex.code == ZmCsfeException.MAIL_SEND_PARTIAL_ADDRESS_FAILURE) {
            var invalid = ex.getData ? ex.getData(ZmCsfeException.MAIL_SEND_ADDRESS_FAILURE_INVALID) : null;
            msg = (invalid && invalid.length)
                ? AjxMessageFormat.format(ZmMsg.sendErrorPartial, AjxStringUtil.htmlEncode(invalid.join(", ")))
                : ZmMsg.sendErrorAbort;
            showMsg = true;
        } else if (ex.code == AjxException.CANCELED) {
            if (draftType == ZmComposeController.DRAFT_TYPE_AUTO) {
                if (!this._autoSaveInterval) {
                    // Request was cancelled due to a ZmRequestMgr send timeout.
                    // The server can either be hung or this particular message is taking
                    // too long to process. Backoff the send interval - restored to
                    // default upon first successful save
                    this._autoSaveInterval = appCtxt.get(ZmSetting.AUTO_SAVE_DRAFT_INTERVAL);
                }
                if (this._autoSaveInterval) {
                    this._autoSaveInterval *= 2;
                    if (this._autoSaveInterval > 300) {
                        // Cap the save attempt interval at 5 minutes
                        this._autoSaveInterval = 300;
                    }
                }
            }
            msg = ZmMsg.cancelSendMsgWarning;
            this._composeView.setBackupForm();
            retVal = true;
        } else if (ex.code == ZmCsfeException.MAIL_QUOTA_EXCEEDED) {
            if (this._composeView._attachDialog) {
                msg = ZmMsg.errorQuotaExceeded;
                this._composeView._attachDialog.setFooter('You have exceeded your mail quota. Please remove some attachments and try again.' );
                showMsg = true;
            }
        }
        if (msg && showMsg) {
            var msgDialog = appCtxt.getMsgDialog();
            msgDialog.setMessage(msg, DwtMessageDialog.CRITICAL_STYLE);
            msgDialog.popup();
            retVal = true;
        }
    }

    // Assume the user stays on the compose view, so we need the timer.
    // (it was canceled when send was called)
    this._initAutoSave();
    return retVal;
};

/**
 * Creates a new ZmComposeView if one does not already exist
 *
 * @param initHide	Set to true if compose view should be initially rendered
 *					off screen (used as an optimization to preload this view)
 *
 * @private
 */
ZmComposeController.prototype.initComposeView =
function(initHide, composeMode) {
	if (this._composeView) { return; }

	this._composeView = new ZmComposeView(this._container, this, composeMode);
	var callbacks = {};
	callbacks[ZmAppViewMgr.CB_PRE_HIDE] = new AjxCallback(this, this._preHideCallback);
	callbacks[ZmAppViewMgr.CB_PRE_UNLOAD] = new AjxCallback(this, this._preUnloadCallback);
	callbacks[ZmAppViewMgr.CB_POST_SHOW] = new AjxCallback(this, this._postShowCallback);
	callbacks[ZmAppViewMgr.CB_PRE_SHOW] = new AjxCallback(this, this._preShowCallback);
	callbacks[ZmAppViewMgr.CB_POST_HIDE] = new AjxCallback(this, this._postHideCallback);
	var elements = {};
	this._initializeToolBar();
	elements[ZmAppViewMgr.C_TOOLBAR_TOP] = this._toolbar;
	elements[ZmAppViewMgr.C_APP_CONTENT] = this._composeView;
	this._app.createView({viewId:this.viewId, elements:elements, callbacks:callbacks, tabParams:this._getTabParams()});
	if (initHide) {
		this._composeView.setLocation(Dwt.LOC_NOWHERE, Dwt.LOC_NOWHERE);
		this._composeView.enableInputs(false);
	}

	if (this._composeView.identitySelect) {
		this._composeView.identitySelect.addChangeListener(new AjxListener(this, this._identityChangeListener, [true]));
	}
};

ZmComposeController.prototype._getTabParams =
function() {
	return {id:this.tabId, image:"NewMessage", text:ZmComposeController.DEFAULT_TAB_TEXT, textPrecedence:75,
			tooltip:ZmComposeController.DEFAULT_TAB_TEXT};
};

ZmComposeController.prototype._identityChangeListener =
function(setSignature, event) {

	var signatureId = this._composeView._getSignatureIdForAction(null, this._action);
	var resetBody = this._composeView.isDirty();

	// don't do anything if signature is same
	if (signatureId == this._currentSignatureId) { return; }

	// apply settings
	this._applyIdentityToBody(setSignature, resetBody);
	this._currentSignatureId = signatureId;
};

ZmComposeController.prototype._applyIdentityToBody =
function(setSignature, resetBody) {
	var identity = this._composeView.getIdentity();
	if (setSignature) {
		var sigId = this._composeView._getSignatureIdForAction(identity);
		this.setSelectedSignature(sigId);
	}
	var newMode = this._getComposeMode(this._msg, identity);
	if (newMode != this._composeView.getComposeMode()) {
		this._composeView.setComposeMode(newMode);
	}
	this._composeView.applySignature(this._getBodyContent(), this._currentSignatureId);
	this._setAddSignatureVisibility();
};

ZmComposeController.prototype._handleSelectSignature =
function(ev) {
	var sigId = ev.item.getData(ZmComposeController.SIGNATURE_KEY);
	this.setSelectedSignature(sigId);
	this.resetSignature(sigId);
};

/**
 * Sets the tab stops for the compose form. All address fields are added; they're
 * not actual tab stops unless they're visible. The textarea for plain text and
 * the HTML editor for HTML compose are swapped in and out depending on the mode.
 * 
 * @private
 */
ZmComposeController.prototype._setComposeTabGroup =
function() {
	var tg = this._createTabGroup();
	var rootTg = appCtxt.getRootTabGroup();
	tg.newParent(rootTg);
	tg.addMember(this._toolbar);
	for (var i = 0; i < ZmMailMsg.COMPOSE_ADDRS.length; i++) {
		tg.addMember(this._composeView._field[ZmMailMsg.COMPOSE_ADDRS[i]]);
	}
	tg.addMember(this._composeView._subjectField);
	var mode = this._composeView.getComposeMode();
	var member = (mode == DwtHtmlEditor.TEXT) ? this._composeView._bodyField : this._composeView.getHtmlEditor();
    if(this._composeView.isTinyMCEEnabled()) {
        var htmlEditor = this._composeView.getHtmlEditor();
        member = htmlEditor.getEditorContainer();
    }
	tg.addMember(member);
};

ZmComposeController.prototype.getKeyMapName =
function() {
	return "ZmComposeController";
};

ZmComposeController.prototype.handleKeyAction =
function(actionCode) {
	switch (actionCode) {
		case ZmKeyMap.CANCEL:
			DBG.println("aif1", "Compose ctlr: CANCEL");
			AjxDebug.println(AjxDebug.REPLY, "Reset compose view: handleKeyAction");
			this._cancelCompose();
			break;

		case ZmKeyMap.SAVE: // Save to draft
			if (this._canSaveDraft()) {
				this.saveDraft();
			}
			break;

		case ZmKeyMap.SEND: // Send message
			this._send();
			break;

		case ZmKeyMap.ATTACHMENT:
			this._attachmentListener();
			break;

		case ZmKeyMap.SPELLCHECK:
			if (!appCtxt.get(ZmSetting.SPELL_CHECK_ENABLED)) break;
			this.toggleSpellCheckButton(true);
			this._spellCheckListener();
			break;

		case ZmKeyMap.HTML_FORMAT:
			if (appCtxt.get(ZmSetting.HTML_COMPOSE_ENABLED)) {
				var mode = this._composeView.getComposeMode();
				var newMode = (mode == DwtHtmlEditor.TEXT) ? DwtHtmlEditor.HTML : DwtHtmlEditor.TEXT;
				this._setFormat(newMode);
				this._setOptionsMenu(newMode);
			}
			break;

		case ZmKeyMap.ADDRESS_PICKER:
			this._composeView._addressButtonListener(null, AjxEmailAddress.TO);
			break;

		case ZmKeyMap.NEW_WINDOW:
			if (!appCtxt.isChildWindow) {
				this._detachListener();
			}
			break;

		default:
			return ZmMailListController.prototype.handleKeyAction.call(this, actionCode);
			break;
	}
	return true;
};

ZmComposeController.prototype.mapSupported =
function(map) {
	return (map == "editor");
};

/**
 * Gets the selected signature.
 * 
 * @return	{String}	the selected signature key or <code>null</code> if none selected
 */
ZmComposeController.prototype.getSelectedSignature =
function() {
	var button = this._toolbar.getButton(ZmOperation.ADD_SIGNATURE);
	var menu = button ? button.getMenu() : null;
	if (menu) {
		var menuitem = menu.getSelectedItem(DwtMenuItem.RADIO_STYLE);
		return menuitem ? menuitem.getData(ZmComposeController.SIGNATURE_KEY) : null;
	}
};

/**
 * Gets the selected signature.
 * 
 * @param	{String}	value 	the selected signature key
 */
ZmComposeController.prototype.setSelectedSignature =
function(value) {
	var button = this._toolbar.getButton(ZmOperation.ADD_SIGNATURE);
	var menu = button ? button.getMenu() : null;
	if (menu) {
		menu.checkItem(ZmComposeController.SIGNATURE_KEY, value, true);
	}
};

ZmComposeController.prototype.resetSignature =
function(sigId, account) {
	this._composeView.applySignature(this._getBodyContent(), this._currentSignatureId, account);
	this._currentSignatureId = sigId;
};

//
// Protected methods
//

ZmComposeController.prototype._deleteDraft =
function(delMsg) {
    var ac = window.parentAppCtxt || window.appCtxt;

    if (delMsg && delMsg.isSent) {
      var folder = delMsg.folderId ? ac.getById(delMsg.folderId) : null;
	  if (folder && folder.isRemote() && !folder.isPermAllowed(ZmOrganizer.PERM_DELETE)) {
         return;   //remote folder no permission to delete, exit
	  }
    }

	var list = delMsg.list;
	var mailItem, request;

	if (list && list.type == ZmItem.CONV) {
		mailItem = list.getById(delMsg.cid);
		request = "ConvActionRequest";
	} else {
		mailItem = delMsg;
		request = "MsgActionRequest";
	}

	// manually delete "virtual conv" or msg created but never added to internal list model
	var soapDoc = AjxSoapDoc.create(request, "urn:zimbraMail");
	var actionNode = soapDoc.set("action");
	actionNode.setAttribute("id", mailItem.id);
	actionNode.setAttribute("op", "delete");

    appCtxt.getAppController().sendRequest({soapDoc:soapDoc, asyncMode:true});
};

/**
 * Creates the compose view based on the mode we're in. Lazily creates the
 * compose toolbar, a contact picker, and the compose view itself.
 *
 * @param action		[constant]		new message, reply, forward, or an invite action
 * @param msg			[ZmMailMsg]*	the original message (reply/forward), or address (new message)
 * @param toOverride 	[string]*		initial value for To: field
 * @param subjOverride 	[string]*		initial value for Subject: field
 * @param extraBodyText [string]*		canned text to prepend to body (invites)
 * @param composeMode	[constant]*		HTML or text compose
 * @param accountName	[string]*		on-behalf-of From address
 * @param msgIds		[Array]*		list of msg Id's to be added as attachments
 * @param readReceipt   [boolean]       true/false read receipt setting
 */
ZmComposeController.prototype._setView =
function(params) {

	if (this._autoSaveTimer) {
		this._autoSaveTimer.kill();
	}

	// save args in case we need to re-display (eg go from Reply to Reply All)
	var action = this._action = params.action;
	var msg = this._msg = params.msg;
	this._toOverride = params.toOverride;
	this._subjOverride = params.subjOverride;
	this._extraBodyText = params.extraBodyText;
	this._msgIds = params.msgIds;
	this._accountName = params.accountName;

	var account = (appCtxt.multiAccounts && appCtxt.getActiveAccount().isMain)
		? appCtxt.accountList.defaultAccount : null;
	var identityCollection = appCtxt.getIdentityCollection(account);
	var identity = (msg && msg.identity) ? msg.identity : identityCollection.selectIdentity(msg);
	params.identity = identity;

	this._composeMode = params.composeMode || this._getComposeMode(msg, identity);
	AjxDebug.println(AjxDebug.REPLY, "ZmComposeController::_setView - Compose mode: " + this._composeMode);

	if (this._needComposeViewRefresh || (this._composeView && appCtxt.get(ZmSetting.SHOW_MAIL_CONFIRM))) {
		this._composeView.dispose();
		this._composeView = null;
		this._needComposeViewRefresh = false;
	}

	var cv = this._composeView;
	if (!cv) {
		this.initComposeView(null, this._composeMode);
		cv = this._composeView;
	} else {
		cv.setComposeMode(this._composeMode, false, true);
	}

	if (identity) {
		this._currentSignatureId = cv._getSignatureIdForAction(identity, action);
	}

	this._initializeToolBar();
	this.resetToolbarOperations();

	this._setAddSignatureVisibility();

	this._curIncOptions = null;
	cv.set(params);
	this._setOptionsMenu();
	appCtxt.notifyZimlets("initializeToolbar", [this._app, this._toolbar, this, this.viewId], {waitUntilLoaded:true});

    if (params.readReceipt) {
        var menu = this._optionsMenu[this._action];
        var mi = menu && menu.getOp(ZmOperation.REQUEST_READ_RECEIPT);
        if (mi && this.isReadReceiptEnabled()) {
            mi.setChecked(true, true);
        }
    }

	this._setComposeTabGroup();
	this._app.pushView(this.viewId);
	if (!appCtxt.isChildWindow) {
		cv.updateTabTitle();
	}
	cv.reEnableDesignMode();

	if (msg && (action == ZmOperation.DRAFT)) {
		this._draftType = ZmComposeController.DRAFT_TYPE_MANUAL;
		if (msg.autoSendTime) {
			this.saveDraft(ZmComposeController.DRAFT_TYPE_MANUAL, null, null, new AjxCallback(msg, msg.setAutoSendTime, null));
			if (!this._autoSendHaltedDialog) {
				this._autoSendHaltedDialog = new DwtMessageDialog({parent:this._shell, buttons:[DwtDialog.OK_BUTTON]});
				this._autoSendHaltedDialog.setMessage(ZmMsg.messageAutoSaveAborted, DwtMessageDialog.WARNING_STYLE);
			}
			this._autoSendHaltedDialog.popup(cv._getDialogXY());
		}
	} else {
		this._draftType = ZmComposeController.DRAFT_TYPE_NONE;
	}

    this.sendMsgCallback = params.sendMsgCallback;

	if (params.callback) {
		params.callback.run(this);
	}
    
};

ZmComposeController.prototype._initializeToolBar =
function() {
	if (this._toolbar) { return; }

	var buttons = [];
	if (this._canSaveDraft() && appCtxt.get(ZmSetting.MAIL_SEND_LATER_ENABLED)) {
		buttons.push(ZmOperation.SEND_MENU);
	} else {
		buttons.push(ZmOperation.SEND);
	}

	buttons.push(ZmOperation.CANCEL, ZmOperation.SEP, ZmOperation.SAVE_DRAFT);

	if (appCtxt.get(ZmSetting.ATTACHMENT_ENABLED)) {
		buttons.push(ZmOperation.ATTACHMENT);
	}

	if (!appCtxt.isOffline) {
		buttons.push(ZmOperation.SPELL_CHECK);
	}
	if (appCtxt.get(ZmSetting.SIGNATURES_ENABLED)) {
		buttons.push(ZmOperation.ADD_SIGNATURE);
	}
	buttons.push(ZmOperation.COMPOSE_OPTIONS, ZmOperation.FILLER);

	if (appCtxt.get(ZmSetting.DETACH_COMPOSE_ENABLED) && !appCtxt.isChildWindow) {
		buttons.push(ZmOperation.DETACH_COMPOSE);
	}

	var tb = this._toolbar = new ZmButtonToolBar({
		parent: this._container,
		buttons: buttons,
		className: (appCtxt.isChildWindow ? "ZmAppToolBar_cw" : "ZmAppToolBar") + " ImgSkin_Toolbar",
		context: this.viewId
	});

	for (var i = 0; i < tb.opList.length; i++) {
		var button = tb.opList[i];
		if (this._listeners[button]) {
			tb.addSelectionListener(button, this._listeners[button]);
		}
	}

	this._setAddSignatureVisibility();

	if (appCtxt.get(ZmSetting.SIGNATURES_ENABLED) || appCtxt.multiAccounts) {
		var sc = appCtxt.getSignatureCollection();
		sc.addChangeListener(new AjxListener(this, this._signatureChangeListener));

		var button = tb.getButton(ZmOperation.ADD_SIGNATURE);
		if (button) {
			button.setMenu(new AjxCallback(this, this._createSignatureMenu));
		}
	}

	var actions = [ZmOperation.NEW_MESSAGE, ZmOperation.REPLY, ZmOperation.FORWARD_ATT, ZmOperation.DECLINE_PROPOSAL, ZmOperation.CAL_REPLY];
	this._optionsMenu = {};
	for (var i = 0; i < actions.length; i++) {
		this._optionsMenu[actions[i]] = this._createOptionsMenu(actions[i]);
	}
	this._optionsMenu[ZmOperation.REPLY_ALL] = this._optionsMenu[ZmOperation.REPLY];
    this._optionsMenu[ZmOperation.CAL_REPLY_ALL] = this._optionsMenu[ZmOperation.CAL_REPLY];
	this._optionsMenu[ZmOperation.FORWARD_INLINE] = this._optionsMenu[ZmOperation.FORWARD_ATT];
	this._optionsMenu[ZmOperation.REPLY_CANCEL] = this._optionsMenu[ZmOperation.REPLY_ACCEPT] =
		this._optionsMenu[ZmOperation.DECLINE_PROPOSAL] = this._optionsMenu[ZmOperation.REPLY_DECLINE] = this._optionsMenu[ZmOperation.REPLY_TENTATIVE] =
		this._optionsMenu[ZmOperation.SHARE] = this._optionsMenu[ZmOperation.DRAFT] =
		this._optionsMenu[ZmOperation.NEW_MESSAGE];

	// change default button style to select for spell check button
	var spellCheckButton = tb.getButton(ZmOperation.SPELL_CHECK);
	if (spellCheckButton) {
		spellCheckButton.setAlign(DwtLabel.IMAGE_LEFT | DwtButton.TOGGLE_STYLE);
	}

	var button = tb.getButton(ZmOperation.NEW_MENU);
	if (button) {
		var listener = new AjxListener(tb, ZmListController._newDropDownListener);
		button.addDropDownSelectionListener(listener);
		tb._ZmListController_this = this;
		tb._ZmListController_newDropDownListener = listener;
	}

	var button = tb.getButton(ZmOperation.SEND_MENU);
	if (button) {
		var menu = new ZmPopupMenu(button, null, null, this);
		var sendItem = menu.createMenuItem(ZmOperation.SEND, ZmOperation.defineOperation(ZmOperation.SEND));
		sendItem.addSelectionListener(this._listeners[ZmOperation.SEND]);
		var sendLaterItem = menu.createMenuItem(ZmOperation.SEND_LATER, ZmOperation.defineOperation(ZmOperation.SEND_LATER));
		sendLaterItem.addSelectionListener(this._listeners[ZmOperation.SEND_LATER]);
		button.setMenu(menu);
	}

};

ZmComposeController.prototype._initAutoSave =
function() {
	if (!this._canSaveDraft()) { return; }

    var defaultAutoSaveInterval = appCtxt.get(ZmSetting.AUTO_SAVE_DRAFT_INTERVAL);
	if (defaultAutoSaveInterval) {
        var interval = this._autoSaveInterval ? this._autoSaveInterval: defaultAutoSaveInterval
        if (!this._autoSaveTimer) {
            this._autoSaveTimer = new DwtIdleTimer(interval * 1000, new AjxCallback(this, this._autoSaveCallback, true));
        }
        else {
            this._autoSaveTimer.resurrect(interval * 1000);
        }
	}
};

ZmComposeController.prototype._setAddSignatureVisibility =
function(account) {
	var ac = window.parentAppCtxt || window.appCtxt;
	var button = ac.get(ZmSetting.SIGNATURES_ENABLED, null, account) &&
				 this._toolbar.getButton(ZmOperation.ADD_SIGNATURE);
	if (button) {
		button.setVisible(appCtxt.getSignatureCollection(account).getSize() > 0);
	}
};

ZmComposeController.prototype._createOptionsMenu =
function(action) {

	var isReply = this._composeView._isReply(action);
	var isCalReply = this._composeView._isCalReply(action);
	var isInviteReply = this._composeView._isInviteReply(action);
	var isForward = this._composeView._isForward(action);
	var list = [];
    var ac = window.parentAppCtxt || window.appCtxt;
	if (isReply || isCalReply) {
		list.push(ZmOperation.REPLY, ZmOperation.REPLY_ALL, ZmOperation.SEP);
	}
	if (ac.get(ZmSetting.HTML_COMPOSE_ENABLED)) {
		list.push(ZmOperation.FORMAT_HTML, ZmOperation.FORMAT_TEXT, ZmOperation.SEP);
	}
	if (isInviteReply) { // Accept/decline/etc... an appointment invitation
		list.push(ZmOperation.SEP, ZmOperation.INC_NONE, ZmOperation.INC_BODY, ZmOperation.INC_SMART);
	} else if (isCalReply) { // Regular reply to an appointment
		list.push(ZmOperation.SEP, ZmOperation.INC_NONE, ZmOperation.INC_BODY, ZmOperation.INC_SMART);
	} else if (isReply) { // Message reply
        list.push(ZmOperation.SEP, ZmOperation.INC_NONE, ZmOperation.INC_BODY, ZmOperation.INC_SMART, ZmOperation.INC_ATTACHMENT);
	} else if (isForward) { // Message forwarding
        list.push(ZmOperation.SEP, ZmOperation.INC_BODY, ZmOperation.INC_ATTACHMENT);
	}
    if (isReply || isForward || isCalReply) {
        list.push(ZmOperation.SEP, ZmOperation.USE_PREFIX, ZmOperation.INCLUDE_HEADERS);
    }

	// add read receipt
	if (ac.get(ZmSetting.MAIL_READ_RECEIPT_ENABLED, null, ac.getActiveAccount())) {
		var fid = this._msg && this._msg.folderId;
		var folder = fid ? ac.getById(fid) : null;
		if (!folder || (folder && !folder.isRemote())) {
			list.push(ZmOperation.SEP, ZmOperation.REQUEST_READ_RECEIPT);
		}
	}

	var button = this._toolbar.getButton(ZmOperation.COMPOSE_OPTIONS);

	var overrides = {};
	for (var i = 0; i < list.length; i++) {
		var op = list[i];
		if (op == ZmOperation.SEP) { continue; }
		overrides[op] = {};
		if (ZmComposeController.OP_CHECK[op]) {
			overrides[op].style = DwtMenuItem.CHECK_STYLE;
		} else {
			overrides[op].style = DwtMenuItem.RADIO_STYLE;
			overrides[op].radioGroupId = ZmComposeController.RADIO_GROUP[op];
		}
		if (op == ZmOperation.REPLY || op == ZmOperation.CAL_REPLY) {
			overrides[op].text = ZmMsg.replySender;
		}
	}

	var menu = new ZmActionMenu({parent:button, menuItems:list, overrides:overrides,
								 context:[this.viewId, action].join("_")});

	for (var i = 0; i < list.length; i++) {
		var op = list[i];
		var mi = menu.getOp(op);
		if (!mi) { continue; }
		if (op == ZmOperation.FORMAT_HTML) {
			mi.setData(ZmHtmlEditor._VALUE, DwtHtmlEditor.HTML);
		} else if (op == ZmOperation.FORMAT_TEXT) {
			mi.setData(ZmHtmlEditor._VALUE, DwtHtmlEditor.TEXT);
		}
		mi.setData(ZmOperation.KEY_ID, op);
		mi.addSelectionListener(this._listeners[ZmOperation.COMPOSE_OPTIONS]);
	}

	return menu;
};

ZmComposeController.prototype._setOptionsMenu =
function(composeMode, incOptions) {

	composeMode = composeMode || this._composeMode;
	incOptions = incOptions || this._curIncOptions || {};
    var ac = window.parentAppCtxt || window.appCtxt;

	var button = this._toolbar.getButton(ZmOperation.COMPOSE_OPTIONS);
	button.setToolTipContent(ZmMsg[ZmComposeController.OPTIONS_TT[this._action]]);
	var menu = this._optionsMenu[this._action];
	if (!menu) { return; }

	if (ac.get(ZmSetting.HTML_COMPOSE_ENABLED)) {
		menu.checkItem(ZmHtmlEditor._VALUE, composeMode, true);
	}

	if (ac.get(ZmSetting.MAIL_READ_RECEIPT_ENABLED) || ac.multiAccounts) {
		var mi = menu.getOp(ZmOperation.REQUEST_READ_RECEIPT);
		if (mi) {
			// did this draft have "request read receipt" option set?
			if (this._msg && this._msg.isDraft) {
				mi.setChecked(this._msg.readReceiptRequested);
			} else {
				// bug: 41329 - always re-init read-receipt option to be off
				mi.setChecked(false, true);
			}

			if (ac.multiAccounts) {
                mi.setEnabled(ac.get(ZmSetting.MAIL_READ_RECEIPT_ENABLED, null, this._composeView.getFromAccount()));
			}
		}
	}

	if (this._action == ZmOperation.REPLY || this._action == ZmOperation.REPLY_ALL  ||
        this._action == ZmOperation.CAL_REPLY || this._action == ZmOperation.CAL_REPLY_ALL) {
		menu.checkItem(ZmOperation.KEY_ID, this._action, true);
	}

	this._setDependentOptions(incOptions);

	button.setMenu(menu);
};

ZmComposeController.prototype._setDependentOptions =
function(incOptions) {

	incOptions = incOptions || this._curIncOptions || {};

	var menu = this._optionsMenu[this._action];
	if (!menu) { return; }

	// handle options for what's included
	var what = incOptions.what;
	menu.checkItem(ZmOperation.KEY_ID, ZmComposeController.INC_OP[what], true);
	var allowOptions = (what == ZmSetting.INC_BODY || what == ZmSetting.INC_SMART);
	var mi = menu.getOp(ZmOperation.USE_PREFIX);
	if (mi) {
		mi.setEnabled(allowOptions);
		mi.setChecked(incOptions.prefix, true);
	}
	mi = menu.getOp(ZmOperation.INCLUDE_HEADERS);
	if (mi) {
		mi.setEnabled(allowOptions);
		mi.setChecked(incOptions.headers, true);
	}
};

/**
 * Called in multi-account mode, when an account has been changed
 */
ZmComposeController.prototype._resetReadReceipt =
function(newAccount) {
	var menu = this._optionsMenu[this._action];
	var mi = menu && menu.getOp(ZmOperation.REQUEST_READ_RECEIPT);
	if (mi) {
		var isEnabled = appCtxt.get(ZmSetting.MAIL_READ_RECEIPT_ENABLED, null, newAccount);
		if (!isEnabled) {
			mi.setChecked(false, true);
		}
		mi.setEnabled(isEnabled);
	}
};

ZmComposeController.prototype._getComposeMode =
function(msg, identity) {

	// depending on COS/user preference set compose format
	var composeMode = DwtHtmlEditor.TEXT;
    var ac = window.parentAppCtxt || window.appCtxt;
	if (ac.get(ZmSetting.HTML_COMPOSE_ENABLED)) {
        if (this._action == ZmOperation.NEW_MESSAGE) {
            if (ac.get(ZmSetting.COMPOSE_AS_FORMAT) == ZmSetting.COMPOSE_HTML) {
                composeMode = DwtHtmlEditor.HTML;
            }
        } 
		else if (this._action == ZmOperation.DRAFT) {
			if (msg && msg.isHtmlMail()) {
				composeMode = DwtHtmlEditor.HTML;
			}
		}
		else if (identity) {
			var sameFormat = ac.get(ZmSetting.COMPOSE_SAME_FORMAT);
			var asFormat = ac.get(ZmSetting.COMPOSE_AS_FORMAT);
			if ((!sameFormat && asFormat == ZmSetting.COMPOSE_HTML) ||  (sameFormat && msg && msg.isHtmlMail())) {
				composeMode = DwtHtmlEditor.HTML;
			}
		}
	}

	return composeMode;
};

ZmComposeController.prototype._getBodyContent =
function() {
	return this._composeView.getHtmlEditor().getContent();
};

ZmComposeController.prototype._setFormat =
function(mode) {

	var curMode = this._composeView.getComposeMode();
	if (mode == curMode) { return; }

	var cv = this._composeView;
	var dirty = cv.isDirty();
	if (!AjxUtil.isEmpty(this._getBodyContent())) {
		if (!this._formatWarningDialog) {
			this._formatWarningDialog = new DwtMessageDialog({parent:this._shell, buttons:[DwtDialog.OK_BUTTON, DwtDialog.CANCEL_BUTTON]});
		}
		var fwDlg = this._formatWarningDialog;
		fwDlg.registerCallback(DwtDialog.OK_BUTTON, this._formatOkCallback, this, [mode]);
		fwDlg.registerCallback(DwtDialog.CANCEL_BUTTON, this._formatCancelCallback, this, [curMode]);
		var msg  = (mode == DwtHtmlEditor.TEXT) ? ZmMsg.switchToText : ZmMsg.switchToHtml;
		fwDlg.setMessage(msg, DwtMessageDialog.WARNING_STYLE);
		fwDlg.popup(cv._getDialogXY());
	} else {
		// bug 26658: remove the signature before changing mode, and
		//            add it back after
		var tmp = this._currentSignatureId;
		if (tmp) {
			this.setSelectedSignature("");
			this._composeView.applySignature(this._getBodyContent(), tmp);
		}
		cv.setComposeMode(mode, true);
		if (tmp) {
			this.setSelectedSignature(tmp);
			cv.applySignature(this._getBodyContent(), tmp);
		}
		return true;
	}

	return false;
};

/**
 *
 * @return {Boolean} needToPop - whether we need to pop the view
 */
ZmComposeController.prototype._processSendMsg =
function(draftType, msg, resp) {

	this._msgSent = true;
	var isScheduled = draftType == ZmComposeController.DRAFT_TYPE_DELAYSEND;
	var isDraft = (draftType != ZmComposeController.DRAFT_TYPE_NONE && !isScheduled);
	var needToPop = false;
	if (!isDraft) {
		needToPop = true;
		if (appCtxt.get(ZmSetting.SHOW_MAIL_CONFIRM)) {
			var confirmController = AjxDispatcher.run("GetMailConfirmController");
			confirmController.showConfirmation(msg, this.viewId, this.tabId);
			needToPop = false;	// don't pop confirm page
		} else {
			if (appCtxt.isChildWindow && window.parentController) {
				window.onbeforeunload = null;
				if (draftType == ZmComposeController.DRAFT_TYPE_DELAYSEND) {
                    window.parentController.setStatusMsg(ZmMsg.messageScheduledSent);
                }
                else if (!appCtxt.isOffline) { // see bug #29372
					window.parentController.setStatusMsg(ZmMsg.messageSent);
				}
			} else {
				if (draftType == ZmComposeController.DRAFT_TYPE_DELAYSEND) {
					appCtxt.setStatusMsg(ZmMsg.messageScheduledSent);
				} else if (!appCtxt.isOffline) { // see bug #29372
					appCtxt.setStatusMsg(ZmMsg.messageSent);
				}
			}
		}

		if (resp || !appCtxt.get(ZmSetting.SAVE_TO_SENT)) {
			AjxDebug.println(AjxDebug.REPLY, "Reset compose view: _processSendMsg");
			this._composeView.reset(false);

			// if the original message was a draft and we're not autosending, we need to nuke it
			var origMsg = msg._origMsg;
			if (origMsg && origMsg.isDraft && !isScheduled)
				this._deleteDraft(origMsg);

			// bug 36341
			if (!appCtxt.isOffline && resp && appCtxt.get(ZmSetting.SAVE_TO_IMAP_SENT) && msg.identity) {
				var datasources = appCtxt.getDataSourceCollection();
				var datasource = datasources && datasources.getById(msg.identity.id);
				if (datasource && datasource.type == ZmAccount.TYPE_IMAP) {
					var parent = appCtxt.getById(datasource.folderId);
					var folder;
					if (parent) {
						// try to find the sent folder from list of possible choices
						var prefix = parent.getName(false, null, true, true) + "/";
						var folderNames = [
							appCtxt.get(ZmSetting.SENT_FOLDER_NAME) || "Sent",
							ZmMsg.sent, "Sent Messages", "[Gmail]/Sent Mail"
						];
						for (var i = 0; i < folderNames.length; i++) {
							folder = parent.getByPath(prefix+folderNames[i]);
							if (folder) break;
						}
					}
					if (folder) {
						var jsonObj = {
							ItemActionRequest: {
								_jsns:  "urn:zimbraMail",
								action: {
									id:     resp.m[0].id,
									op:     "move",
									l:      folder.id
								}
							}
						};
						var params = {
							jsonObj: jsonObj,
							asyncMode: true,
							noBusyOverlay: true
						};
						appCtxt.getAppController().sendRequest(params);
					}
				}
			}
		}
	} else {
		if (draftType != ZmComposeController.DRAFT_TYPE_AUTO) {
			var transitions = [ ZmToast.FADE_IN, ZmToast.IDLE, ZmToast.PAUSE, ZmToast.FADE_OUT ];
			appCtxt.setStatusMsg(ZmMsg.draftSaved, ZmStatusView.LEVEL_INFO, null, transitions);
		}
		this._composeView.processMsgDraft(msg);
		// TODO - disable save draft button indicating a draft was saved

		var listController = this._listController; // non child window case
		if (appCtxt.isChildWindow) {
			//Check if Mail App view has been created and then update the MailListController
			if (window.parentAppCtxt.getAppViewMgr().getAppView(ZmApp.MAIL)) {
				listController = window.parentAppCtxt.getApp(ZmApp.MAIL).getMailListController();
			}
		}
		//listController is available only when editing an existing draft.
		if (listController && listController._draftSaved) {
			var savedMsg = appCtxt.isChildWindow ? null : msg;
			var savedResp = appCtxt.isChildWindow ? resp.m[0] : null; //Pass the mail response to the parent window such that the ZmMailMsg obj is created in the parent window.
			listController._draftSaved(savedMsg, savedResp);
		}
	}

	if (isScheduled) {
		if (appCtxt.isChildWindow) {
			var pAppCtxt = window.parentAppCtxt;
			if(pAppCtxt.getAppViewMgr().getAppView(ZmApp.MAIL)) {
				var listController = pAppCtxt.getApp(ZmApp.MAIL).getMailListController();
				if (listController && listController._draftSaved) {
					//Pass the mail response to the parent window such that the ZmMailMsg obj is created in the parent window.
					listController._draftSaved(null, resp.m[0]);
				}
			}
		} else {
			if (this._listController && this._listController._draftSaved) {
				this._listController._draftSaved(msg);
			}
		}
	}
	return needToPop;
};


// Listeners

// Send button was pressed
ZmComposeController.prototype._sendListener =
function(ev) {
	this._send();
};

ZmComposeController.prototype._send =
function() {
	this._toolbar.enableAll(false); // thwart multiple clicks on Send button
	this._resetDelayTime();
	this.sendMsg();
};

ZmComposeController.prototype._sendLaterListener =
function(ev) {
	this.showDelayDialog();
};

// Cancel button was pressed
ZmComposeController.prototype._cancelListener =
function(ev) {
	AjxDebug.println(AjxDebug.REPLY, "ZmComposeController: _cancelListener");
	this._cancelCompose();
};

ZmComposeController.prototype._cancelCompose =
function() {
	var dirty = this._composeView.isDirty();
	var needPrompt = dirty || (this._draftType == ZmComposeController.DRAFT_TYPE_AUTO);
	if (!needPrompt) {
		AjxDebug.println(AjxDebug.REPLY, "Reset compose view: _cancelCompose");
		this._composeView.reset(true);
	} else {
		this._composeView.enableInputs(false);
	}
	this._composeView.reEnableDesignMode();
	this._app.popView(!needPrompt);
};

// Attachment button was pressed
ZmComposeController.prototype._attachmentListener =
function(ev) {

	if (!this._detachOkCancel) {
		// detach ok/cancel dialog is only necessary if user clicked on the add attachments button
		this._detachOkCancel = new DwtMessageDialog({parent:this._shell, buttons:[DwtDialog.OK_BUTTON, DwtDialog.CANCEL_BUTTON]});
		this._detachOkCancel.setMessage(ZmMsg.detachAnyway, DwtMessageDialog.WARNING_STYLE);
		this._detachOkCancel.registerCallback(DwtDialog.OK_BUTTON, this._detachCallback, this);
	}
	this._composeView.showAttachmentDialog();
};

ZmComposeController.prototype._optionsListener =
function(ev) {

	var op = ev.item.getData(ZmOperation.KEY_ID);
	if (op == ZmOperation.REQUEST_READ_RECEIPT) { return; }

	// Click on "Options" button.
	if (op == ZmOperation.COMPOSE_OPTIONS && this._optionsMenu[this._action]) {
		var button = this._toolbar.getButton(ZmOperation.COMPOSE_OPTIONS);
		var bounds = button.getBounds();
		this._optionsMenu[this._action].popup(0, bounds.x, bounds.y + bounds.height, false);
		return;
	}

	// ignore UNCHECKED for radio buttons
	if (ev.detail != DwtMenuItem.CHECKED && !ZmComposeController.OP_CHECK[op]) { return; }

	if (op == ZmOperation.REPLY || op == ZmOperation.REPLY_ALL || op == ZmOperation.CAL_REPLY || op == ZmOperation.CAL_REPLY_ALL) {
		var cv = this._composeView;
		cv.setAddress(AjxEmailAddress.TO, "");
		cv.setAddress(AjxEmailAddress.CC, "");
		cv._setAddresses(op, AjxEmailAddress.TO, this._toOverride);
	} else if (op == ZmOperation.FORMAT_HTML || op == ZmOperation.FORMAT_TEXT) {
		if (this._setFormat(ev.item.getData(ZmHtmlEditor._VALUE))) {
			this._switchInclude(op);
		}
	} else {
		if (this._setInclude(op)) {
			this._switchInclude(op);
			this._setDependentOptions();
		}
	}
};

ZmComposeController.prototype._setInclude =
function(op) {

	var what = this._curIncOptions.what;
	var canInclude = (what == ZmSetting.INC_BODY || what == ZmSetting.INC_SMART);
	if (this._composeView.isDirty() && canInclude) {
		// warn user of possible lost content
		if (!this._switchIncludeDialog) {
			this._switchIncludeDialog = new DwtMessageDialog({parent:this._shell, buttons:[DwtDialog.OK_BUTTON, DwtDialog.CANCEL_BUTTON]});
			this._switchIncludeDialog.setMessage(ZmMsg.switchIncludeWarning, DwtMessageDialog.WARNING_STYLE);
		}
		this._switchIncludeDialog.registerCallback(DwtDialog.OK_BUTTON, this._switchIncludeOkCallback, this, [op]);
		var origIncOptions = AjxUtil.hashCopy(this._curIncOptions);
		this._switchIncludeDialog.registerCallback(DwtDialog.CANCEL_BUTTON, this._switchIncludeCancelCallback, this, [origIncOptions]);
		this._switchIncludeDialog.popup(this._composeView._getDialogXY());
		return false;
	} else {
		return true;
	}
};

ZmComposeController.prototype._switchInclude =
function(op) {

	var origWhat = this._curIncOptions.what;
	var menu = this._optionsMenu[this._action];
	if (op == ZmOperation.USE_PREFIX || op == ZmOperation.INCLUDE_HEADERS) {
		var mi = menu.getOp(op);
		if (mi) {
			if (op == ZmOperation.USE_PREFIX) {
				this._curIncOptions.prefix = mi.getChecked();
			} else {
				this._curIncOptions.headers = mi.getChecked();
			}
		}
	} else if (ZmComposeController.INC_MAP[op]) {
		this._curIncOptions.what = ZmComposeController.INC_MAP[op];
	}

	var cv = this._composeView;
	if (op != ZmOperation.FORMAT_HTML && op != ZmOperation.FORMAT_TEXT) {
		if (cv._composeMode == DwtHtmlEditor.TEXT) {
			AjxTimedAction.scheduleAction(new AjxTimedAction(this, function() { cv.getHtmlEditor().moveCaretToTop(); }), 200);
		}
	}    

	// forwarding actions are tied to inc option
	var what = this._curIncOptions.what;
	if (this._action == ZmOperation.FORWARD_INLINE && what == ZmSetting.INC_ATTACH) {
		this._action = ZmOperation.FORWARD_ATT;
	}
	if (this._action == ZmOperation.FORWARD_ATT && what != ZmSetting.INC_ATTACH) {
		this._action = ZmOperation.FORWARD_INLINE;
	}

	var unquotedContent = cv.getUnQuotedContent();

	cv.resetBody(this._action, this._msg, unquotedContent);
};

ZmComposeController.prototype._detachListener =
function(ev) {
	var atts = this._composeView.getAttFieldValues();
	if (atts.length) {
		this._detachOkCancel.popup(this._composeView._getDialogXY());
	} else {
		this.detach();
	}
};

// Save Draft button was pressed
ZmComposeController.prototype._saveDraftListener =
function(ev) {
	this.saveDraft();
};

ZmComposeController.prototype._autoSaveCallback =
function(idle) {
	if (idle && !DwtBaseDialog.getActiveDialog() && !this._composeView.getHtmlEditor().isSpellCheckMode() && this._composeView.isDirty()) {
		this.saveDraft(ZmComposeController.DRAFT_TYPE_AUTO);
	}
};


ZmComposeController.prototype.saveDraft =
function(draftType, attId, docIds, callback, contactId) {

	if (!this._canSaveDraft()) { return; }
	draftType = draftType || ZmComposeController.DRAFT_TYPE_MANUAL;
	
	var addrs = this._composeView._collectAddrs();
	if (addrs && addrs._bad_addrs_ && addrs._bad_addrs_.size()) {
		if (draftType == ZmComposeController.DRAFT_TYPE_MANUAL) {
			var dlg = appCtxt.getMsgDialog();
			dlg.reset();
			var badAddrs = [];
			for (var i=0; i<addrs._bad_addrs_.size(); i++) {
				badAddrs.push(AjxStringUtil.htmlEncode(addrs._bad_addrs_.get(i)));	
			}
			dlg.setMessage(AjxMessageFormat.format(ZmMsg.errorSavingDraftInvalidEmails, [badAddrs.join("<br/>")]));
			dlg.popup();
		}
		return;
	}

	var respCallback = new AjxCallback(this, this._handleResponseSaveDraftListener, [draftType, callback]);
	this._resetDelayTime();
	if (!docIds) {
		this.sendMsg(attId, draftType, respCallback, contactId);
	} else {
		this.sendDocs(docIds, draftType, respCallback, contactId);
	}
};

ZmComposeController.prototype._handleResponseSaveDraftListener =
function(draftType, callback) {
	if (draftType == ZmComposeController.DRAFT_TYPE_AUTO &&
		this._draftType == ZmComposeController.DRAFT_TYPE_NONE) {
		this._draftType = ZmComposeController.DRAFT_TYPE_AUTO;
	} else if (draftType == ZmComposeController.DRAFT_TYPE_MANUAL) {
		this._draftType = ZmComposeController.DRAFT_TYPE_MANUAL;
	}
	this._action = ZmOperation.DRAFT;

	this._composeView._isDirty = false;

	if (callback) {
		callback.run();
	}
};

ZmComposeController.prototype._spellCheckListener =
function(ev) {
	var spellCheckButton = this._toolbar.getButton(ZmOperation.SPELL_CHECK);
	var htmlEditor = this._composeView.getHtmlEditor();

	if (spellCheckButton.isToggled()) {
		var callback = new AjxCallback(this, this.toggleSpellCheckButton);
		if (!htmlEditor.spellCheck(callback))
			this.toggleSpellCheckButton(false);
	} else {
		htmlEditor.discardMisspelledWords();
	}
};

ZmComposeController.prototype._handleSettingChange =
function(ev) {
	if (ev.type != ZmEvent.S_SETTING) return;

	var id = ev.source.id;
	if (id == ZmSetting.SHOW_BCC) {
		//Handle, if SHOW_BCC setting is changed, need to do when we come up with a COS Preference.
	}
	else if (id == ZmSetting.USE_ADDR_BUBBLES) {
		this._needComposeViewRefresh = true;
	}
};

ZmComposeController.prototype.showDelayDialog =
function() {
	if (!this._delayDialog) {
		this._delayDialog = new ZmTimeDialog({parent:this._shell, buttons:[DwtDialog.OK_BUTTON, DwtDialog.CANCEL_BUTTON]});
		this._delayDialog.setButtonListener(DwtDialog.OK_BUTTON, new AjxListener(this, this._handleDelayDialog));
	}
	this._delayDialog.popup(this._composeView._getDialogXY());
};

ZmComposeController.prototype._handleDelayDialog =
function() {
	this._delayDialog.popdown();
	var time = this._delayDialog.getValue(); //Returns {date: Date, timezone: String (see AjxTimezone)}

	var date = time.date;
	var dateOffset = AjxTimezone.getOffset(AjxTimezone.getClientId(time.timezone), date);
	var utcDate = new Date(date.getTime() - dateOffset*60*1000);

	var now = new Date();
	var nowOffset = AjxTimezone.getOffset(AjxTimezone.DEFAULT_RULE, now);
	var utcNow = new Date(now.getTime() - nowOffset*60*1000);

	if (utcDate < utcNow) {
		this.showDelayPastDialog();
	} else {
		this._toolbar.enableAll(false); // thwart multiple clicks on Send button
		this._sendTime = time;
		this.sendMsg(null, ZmComposeController.DRAFT_TYPE_DELAYSEND, null);
	}
};

ZmComposeController.prototype.showDelayPastDialog =
function() {
	if (!this._delayPastDialog) {
		this._delayPastDialog = new DwtMessageDialog({parent:this._shell, buttons:[DwtDialog.OK_BUTTON, DwtDialog.CANCEL_BUTTON], id: "ShowDelayPastDialog"});
		this._delayPastDialog.setMessage(ZmMsg.sendLaterPastError, DwtMessageDialog.WARNING_STYLE);
		this._delayPastDialog.registerCallback(DwtDialog.OK_BUTTON, this._handleDelayPastDialog, this, []);
	}
	this._delayPastDialog.popup(this._composeView._getDialogXY());
};

ZmComposeController.prototype._handleDelayPastDialog =
function() {
	this._delayPastDialog.popdown();
	this._send();
};

ZmComposeController.prototype._resetDelayTime =
function() {
	this._sendTime = null;
};

// Callbacks

ZmComposeController.prototype._detachCallback =
function() {
	// get rid of any lingering attachments since they cannot be detached
	this._composeView.cleanupAttachments();
	this._detachOkCancel.popdown();
	this.detach();
};

ZmComposeController.prototype._formatOkCallback =
function(mode) {
	this._formatWarningDialog.popdown();
	this._composeView.setComposeMode(mode, true);
	this._composeView._isDirty = true;
};

ZmComposeController.prototype._formatCancelCallback =
function(mode) {
	this._formatWarningDialog.popdown();

	// reset the radio button for the format button menu
	var menu = this._toolbar.getButton(ZmOperation.COMPOSE_OPTIONS).getMenu();
	menu.checkItem(ZmHtmlEditor._VALUE, mode, true);

	this._composeView.reEnableDesignMode();
};

/**
 * Called as: Yes, save as draft
 * 			  Yes, go ahead and cancel
 *
 * @param mailtoParams		[Object]*	Used by offline client to pass on mailto handler params
 */
ZmComposeController.prototype._popShieldYesCallback =
function(mailtoParams) {
	this._popShield.removePopdownListener(this._dialogPopdownListener);
	this._popShield.popdown();
	this._composeView.enableInputs(true);
	if (this._canSaveDraft()) {
		// save as draft
		var callback = mailtoParams ? new AjxCallback(this, this.doAction, mailtoParams) :
					   				  new AjxCallback(this, this._popShieldYesDraftSaved);
		this._resetDelayTime();
		this.sendMsg(null, ZmComposeController.DRAFT_TYPE_MANUAL, callback);
	} else {
		// cancel
		if (appCtxt.isChildWindow && window.parentController) {
			window.onbeforeunload = null;
		} else {
			AjxDebug.println(AjxDebug.REPLY, "Reset compose view: _popShieldYesCallback");
			this._composeView.reset(false);
		}
		if (mailtoParams) {
			this.doAction(mailtoParams);
		} else {
			appCtxt.getAppViewMgr().showPendingView(true);
		}
	}
};

ZmComposeController.prototype._popShieldYesDraftSaved =
function() {
	appCtxt.getAppViewMgr().showPendingView(true);
};

// Called as: No, don't save as draft
//			  No, don't cancel
ZmComposeController.prototype._popShieldNoCallback =
function(mailtoParams) {
	this._popShield.removePopdownListener(this._dialogPopdownListener);
	this._popShield.popdown();
	this._composeView.enableInputs(true);
	if (this._canSaveDraft()) {
		if (appCtxt.isChildWindow && window.parentController) {
			window.onbeforeunload = null;
		}

		AjxDebug.println(AjxDebug.REPLY, "Reset compose view: _popShieldNoCallback");
        this._composeView.reset(false);

		if (!mailtoParams) {
			appCtxt.getAppViewMgr().showPendingView(true);
		}
	} else {
		if (!mailtoParams) {
			appCtxt.getAppViewMgr().showPendingView(false);
		}
		this._composeView.reEnableDesignMode();
	}

	if (mailtoParams) {
		this.doAction(mailtoParams);
	}
};

ZmComposeController.prototype._popShieldDiscardCallback =
function() {
	this._deleteDraft(this._composeView._msg);
	this._popShieldNoCallback();
};

// Called as: Don't save as draft or cancel
ZmComposeController.prototype._popShieldDismissCallback =
function() {
	this._popShield.removePopdownListener(this._dialogPopdownListener);
	this._popShield.popdown();
	this._cancelViewPop();
	this._initAutoSave(); //re-init autosave since it was killed in preHide

};

ZmComposeController.prototype._switchIncludeOkCallback =
function(op) {
	this._switchIncludeDialog.popdown();
	this._switchInclude(op);
	this._setDependentOptions();
};

ZmComposeController.prototype._switchIncludeCancelCallback =
function(origIncOptions) {
	this._switchIncludeDialog.popdown();
	this._setOptionsMenu(null, origIncOptions);
};

/**
 * Handles re-enabling inputs if the pop shield is dismissed via
 * Esc. Otherwise, the handling is done explicitly by a callback.
 */
ZmComposeController.prototype._dialogPopdownActionListener =
function() {
	this._cancelViewPop();
};

ZmComposeController.prototype._cancelViewPop =
function() {
	this._composeView.enableInputs(true);
	appCtxt.getAppViewMgr().showPendingView(false);
	this._composeView.reEnableDesignMode();
};

ZmComposeController.prototype._getDefaultFocusItem =
function() {
	if (this._action == ZmOperation.NEW_MESSAGE ||
		this._action == ZmOperation.FORWARD_INLINE ||
		this._action == ZmOperation.FORWARD_ATT)
	{
		return this._composeView._field[AjxEmailAddress.TO];
	}

	return (this._composeView.getComposeMode() == DwtHtmlEditor.TEXT)
		? this._composeView._bodyField
		: this._composeView._htmlEditor;
};

ZmComposeController.prototype._createSignatureMenu =
function(button, account) {
	if (!this._composeView) { return null; }

	var button = this._toolbar.getButton(ZmOperation.ADD_SIGNATURE);
	if (!button) { return null; }

	var menu;
	var options = appCtxt.getSignatureCollection(account).getSignatureOptions();
	if (options.length > 0) {
		menu = new DwtMenu({parent:button});
		var listener = new AjxListener(this, this._handleSelectSignature);
		var radioId = this._composeView._htmlElId + "_sig";
		for (var i = 0; i < options.length; i++) {
			var option = options[i];
			var menuitem = new DwtMenuItem({parent:menu, style:DwtMenuItem.RADIO_STYLE, radioGroupId:radioId});
			menuitem.setText(AjxStringUtil.htmlEncode(option.displayValue));
			menuitem.setData(ZmComposeController.SIGNATURE_KEY, option.value);
			menuitem.addSelectionListener(listener);
			menu.checkItem(ZmComposeController.SIGNATURE_KEY, option.value, option.selected);
		}
	}
	return menu;
};

ZmComposeController.prototype._signatureChangeListener =
function(ev) {
	this.resetSignatureToolbar(this.getSelectedSignature());
};

/**
 * Resets the signature dropdown based on the given account and selects the
 * given signature if provided.
 *
 * @param selected	[String]*			ID of the signature to select
 * @param account	[ZmZimbraAccount]*	account for which to load signatures
 */
ZmComposeController.prototype.resetSignatureToolbar =
function(selected, account) {
	var button = this._toolbar.getButton(ZmOperation.ADD_SIGNATURE);
	var menu = button && this._createSignatureMenu(null, account);
	if (menu) {
		button.setMenu(menu);
		this.setSelectedSignature(selected || "");
	}

	this._setAddSignatureVisibility(account);
};

ZmComposeController.prototype.resetToolbarOperations =
function() {
	this._toolbar.enableAll(true);
	if (this._composeView._isInviteReply(this._action)) {
		var ops = [ ZmOperation.SAVE_DRAFT, ZmOperation.ATTACHMENT ];
		this._toolbar.enable(ops, false);
	}
	var op = this._toolbar.getOp(ZmOperation.COMPOSE_OPTIONS);
	if (op) {
		op.setVisible(appCtxt.get(ZmSetting.HTML_COMPOSE_ENABLED));
	}
	appCtxt.notifyZimlets("resetToolbarOperations", [this._toolbar, 1]);
};

ZmComposeController.prototype._canSaveDraft =
function() {
	return appCtxt.get(ZmSetting.SAVE_DRAFT_ENABLED) && !this._composeView._isInviteReply(this._action);
};

/*
 * Return true/false if read receipt is being requested
 */
ZmComposeController.prototype.isRequestReadReceipt =
function(){

  	// check for read receipt
	var requestReadReceipt = false;
    var isEnabled = this.isReadReceiptEnabled();
	if (isEnabled) {
		var menu = this._toolbar.getButton(ZmOperation.COMPOSE_OPTIONS).getMenu();
		if (menu) {
			var mi = menu.getItemById(ZmOperation.KEY_ID, ZmOperation.REQUEST_READ_RECEIPT);
			requestReadReceipt = (!!(mi && mi.getChecked()));
		}
	}

    return requestReadReceipt;
};

/*
 * Return true/false if read receipt is enabled
 */
ZmComposeController.prototype.isReadReceiptEnabled =
function(){
    var acctName = appCtxt.multiAccounts
		? this._composeView.getFromAccount().name : this._accountName;
    var acct = acctName && appCtxt.accountList.getAccountByName(acctName);
    if (appCtxt.get(ZmSetting.MAIL_READ_RECEIPT_ENABLED, null, acct)){
        return true;
    }

    return false;
};

/*
 * Return ZmMailMsg object
 * @return {ZmMailMsg} message object
 */
ZmComposeController.prototype.getMsg =
function(){
    return this._msg;
};

ZmComposeController.prototype._pasteHandler = function( ev ){
    var items,blob,controller,responseHandler;
    if (ev.clipboardData){
        items = ev.clipboardData.items;
        if( items ){
            blob = items[0].getAsFile();
            if( blob ){
                controller = this;
                blob.name = blob.name || ev.timeStamp;//For paste from clipboard filename is undefined so we are using timestamp
                responseHandler = function( response ){
                    if( response ){
                        response.clipboardPaste = true;
                        controller.saveDraft(ZmComposeController.DRAFT_TYPE_AUTO, response);
                    }
                };
                controller._uploadImage(blob, new AjxCallback(this, responseHandler));
            }
        }
    }
};

ZmComposeController.prototype._processDataURIImages = function(idoc, callback){
    if (!idoc || !window.atob) {
        return;
    }
    var BlobBuilder = window.MozBlobBuilder || window.WebKitBlobBuilder || window.MSBlobBuilder || window.BlobBuilder;
    if (typeof (BlobBuilder || window.Blob) !== "function") {
        return;
    }
    var imgArray = idoc.getElementsByTagName("img");
    var length = imgArray.length;
    if(length === 0){//No image elements in the editor document
        return;
    }
    //converts datauri string to blob object used for uploading the image
    //dataURI format  data:image/png;base64,iVBORw0
    var dataURItoBlob = function( dataURI ){
        if( dataURI ){
            var dataURIArray = dataURI.split(",");
            if( dataURIArray.length === 2 ){
                // convert base64 to raw binary data held in a string
                // doesn't handle URLEncoded DataURIs
                var byteString = window.atob( dataURIArray[1] );
                if( !byteString ){
                    return;
                }
                // separate out the mime component
                var mimeString = dataURIArray[0].split(':');
                if( !mimeString[1] ){
                    return;
                }
                mimeString = mimeString[1].split(';')[0];
                if(mimeString){
                    // write the bytes of the string to an ArrayBuffer
                    var byteStringLength = byteString.length;
                    var ab = new ArrayBuffer( byteStringLength );
                    var ia = new Uint8Array(ab);
                    for (var i = 0; i < byteStringLength; i++) {
                        ia[i] = byteString.charCodeAt(i);
                    }
                    // write the ArrayBuffer to a blob, and you're done
                    var blob;
                    if (typeof window.Blob === "function") {
                        blob = new Blob([ab], {"type" : mimeString});
                    }
                    else {
                        var blobbuilder = new BlobBuilder();
                        blobbuilder.append(ab);

                        blob = blobbuilder.getBlob(mimeString);
                        blob.type = mimeString;
                        blob.name = blob.name || new Date().getTime();
                    }
                    return blob;
                }
            }
        }
    };

    var blobArray = [];//Array containing blob objects used for uploading images
    for(var i = 0; i < length; i++ ){
        var imgSrc = imgArray[i].src;
        if(imgSrc && imgSrc.indexOf("data:") !== -1){
            var blob = dataURItoBlob(imgSrc);
            if(blob){
                //setting data-zim-uri attribute for image replacement in callback
                var id = Dwt.getNextId();
                imgArray[i].setAttribute("id", id);
                imgArray[i].setAttribute("data-zim-uri", id);
                blob.id = id;
                blobArray.push(blob);
            }
        }
    }
    length = blobArray.length;
    if(length === 0){
        return;
    }

    this._uploadedImageArray = [];
    this._dataURIImagesLength = length;
    for( i = 0; i < length ; i++ ){
        var blob = blobArray[i];
        var uploadImageCallback = new AjxCallback(this, this._handleUploadImage, [callback, blob.id]);
        this._uploadImage( blob, uploadImageCallback);
    }
    return true;
};

ZmComposeController.prototype._uploadImage = function(blob, callback){
    var req = new XMLHttpRequest();
    req.open("POST", appCtxt.get(ZmSetting.CSFE_ATTACHMENT_UPLOAD_URI)+"?fmt=extended,raw", true);
    req.setRequestHeader("Cache-Control", "no-cache");
    req.setRequestHeader("X-Requested-With", "XMLHttpRequest");
    req.setRequestHeader("Content-Type", blob.type);
    req.setRequestHeader("Content-Disposition", 'attachment; filename="' + blob.name + '"');
    req.onreadystatechange = function(){
        if(req.readyState === 4 && req.status === 200) {
            var resp = eval("["+req.responseText+"]");
            callback.run(resp[2]);
        }
    }
    req.send(blob);
};

ZmComposeController.prototype._handleUploadImage = function(callback, id, response){
    this._dataURIImagesLength--;
    var uploadedImageArray = this._uploadedImageArray;
    if( response && id ){
        response[0]["id"] = id;
        uploadedImageArray.push(response[0]);
    }
    if( this._dataURIImagesLength === 0 && callback ){
        if( uploadedImageArray.length > 0 ){
            uploadedImageArray.clipboardPaste = true;
            if(callback.args[0]){  //attid argument of _sendMsg method
                callback.args[0] = callback.args[0].concat(uploadedImageArray);
            }
            else{
                callback.args[0] = uploadedImageArray;
            }
        }
        callback.run();
        delete this._uploadedImageArray;
        delete this._dataURIImagesLength;
    }
};
}
if (AjxPackage.define("zimbraMail.mail.controller.ZmMsgController")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * Creates an empty message controller.
 * @constructor
 * @class
 * This class controls the display and management of a single message in the content area. Since it
 * needs to handle pretty much the same operations as a list, it extends ZmMailListController.
 *
 * @author Parag Shah
 * @author Conrad Damon
 * 
 * @param {ZmComposite}	container	the containing shell
 * @param {ZmMailApp}	mailApp			the containing app
 * 
 * @extends		ZmMailListController
 */
ZmMsgController = function(container, mailApp) {

	ZmMailListController.call(this, container, mailApp);
};

ZmMsgController.prototype = new ZmMailListController;
ZmMsgController.prototype.constructor = ZmMsgController;

ZmMsgController.MODE_TO_CONTROLLER = {};
ZmMsgController.MODE_TO_CONTROLLER[ZmId.VIEW_TRAD]		= "GetTradController";
ZmMsgController.MODE_TO_CONTROLLER[ZmId.VIEW_CONV]		= "GetConvController";
ZmMsgController.MODE_TO_CONTROLLER[ZmId.VIEW_CONVLIST]	= "GetConvListController";

ZmMsgController.DEFAULT_TAB_TEXT = ZmMsg.message;

ZmMsgController.viewToTab = {};

// Public methods

ZmMsgController.prototype.toString = 
function() {
	return "ZmMsgController";
};

/**
 * Displays a message in the single-pane view.
 *
 * @param {ZmMailMsg}	msg		the message to display
 * @param {constant}	mode		the owning view ID
 * @param {AjxCallback}	callback	the client callback
 * @param {Boolean}	markRead	if <code>true</code>, mark msg read
 */
ZmMsgController.prototype.show = 
function(msg, mode, callback, markRead) {
	this.setMsg(msg);
	this._mode = mode;
	this._currentView = this._getViewType();
	this._list = msg.list;
	if (!msg._loaded) {
		var respCallback = new AjxCallback(this, this._handleResponseShow, [callback]);
		if (msg._loadPending) {
			// override any local callback if we're being launched by double-pane view,
			// so that multiple GetMsgRequest's aren't made
			msg._loadCallback = respCallback;
		} else {
			markRead = markRead || (appCtxt.get(ZmSetting.MARK_MSG_READ) == ZmSetting.MARK_READ_NOW);
			msg.load({callback:respCallback, markRead:markRead});
		}
	} else {
		this._handleResponseShow(callback);
	}
};

ZmMsgController.prototype._handleResponseShow = 
function(callback, result) {
	this._showMsg();
	if (callback instanceof AjxCallback) {
		callback.run();
	}
};


/**
 * Called by ZmNewWindow.unload to remove tag list listener (which resides in 
 * the parent window). Otherwise, after the child window is closed, the parent 
 * window is still referencing the child window's msg controller, which has
 * been unloaded!!
 * 
 * @private
 */
ZmMsgController.prototype.dispose = 
function() {
	this._tagList.removeChangeListener(this._tagChangeLstnr);
};

ZmMsgController.prototype._showMsg = 
function() {
	var avm = appCtxt.getAppViewMgr();
	this._setup(this._currentView);
	var elements = {};
	elements[ZmAppViewMgr.C_TOOLBAR_TOP] = this._toolbar[this._currentView];
	elements[ZmAppViewMgr.C_APP_CONTENT] = this._listView[this._currentView];
	var curView = avm.getCurrentViewId();
	var tabId = (curView && curView.indexOf(ZmId.VIEW_MSG) == 0) ? ZmMsgController.viewToTab[curView] : Dwt.getNextId();
	ZmMsgController.viewToTab[this.viewId] = tabId;
	var viewParams = {view:this._currentView, elements:elements, clear:appCtxt.isChildWindow, tabParams:this._getTabParams(tabId, new AjxCallback(this, this._tabCallback))};
	var buttonText = (this._msg && this._msg.subject) ? AjxStringUtil.htmlEncode(this._msg.subject.substr(0, ZmAppViewMgr.TAB_BUTTON_MAX_TEXT)) : ZmMsgController.DEFAULT_TAB_TEXT;
	this._setView(viewParams);
	avm.setTabTitle(this.viewId, buttonText);
	this._resetOperations(this._toolbar[this._currentView], 1); // enable all buttons
	this._resetNavToolBarButtons(this._currentView);
	this._toolbar[this._currentView].adjustSize();
};

ZmMsgController.prototype._getTabParams =
function(tabId, tabCallback) {
	return {id:tabId, image:"MessageView", textPrecedence:85, tooltip:ZmMsgController.DEFAULT_TAB_TEXT, tabCallback: tabCallback};
};

ZmMsgController.prototype.getKeyMapName =
function() {
	return "ZmMsgController";
};

ZmMsgController.prototype.handleKeyAction =
function(actionCode) {
	DBG.println(AjxDebug.DBG3, "ZmMsgController.handleKeyAction");
	
	switch (actionCode) {
		case ZmKeyMap.CANCEL:
			this._backListener();
			break;

		case ZmKeyMap.NEXT_PAGE:
			this._goToMsg(this._currentView, true);
			break;

		case ZmKeyMap.PREV_PAGE:
			this._goToMsg(this._currentView, false);
			break;

		default:
			return ZmMailListController.prototype.handleKeyAction.call(this, actionCode);
			break;
	}
	return true;
};

ZmMsgController.prototype.mapSupported =
function(map) {
	return false;
};

// Private methods (mostly overrides of ZmListController protected methods)

ZmMsgController.prototype._getToolBarOps = 
function() {
	var list;
	if (appCtxt.isChildWindow) {
		list = [ZmOperation.CLOSE, ZmOperation.SEP, ZmOperation.PRINT, ZmOperation.DELETE];
		list.push(ZmOperation.SEP);
		list = list.concat(this._msgOps());
		list.push(ZmOperation.SEP, ZmOperation.SPAM, ZmOperation.SEP, ZmOperation.TAG_MENU);
	}
	else {
		list = this._standardToolBarOps();
		list.push(ZmOperation.SEP);
		list = list.concat(this._msgOps());
		list.push(ZmOperation.SEP,
					ZmOperation.SPAM,
					ZmOperation.SEP,
					ZmOperation.TAG_MENU,
					ZmOperation.SEP);
		if (appCtxt.get(ZmSetting.DETACH_MAILVIEW_ENABLED)) {
			list.push(ZmOperation.DETACH);
		}
	}
	return list;
};

ZmMsgController.prototype._initializeToolBar =
function(view) {
	if (!appCtxt.isChildWindow) {
		ZmMailListController.prototype._initializeToolBar.call(this, view);
	} else {
		var buttons = this._getToolBarOps();
		if (!buttons) return;
		var params = {
			parent:this._container,
			buttons:buttons,
			className:"ZmMsgViewToolBar_cw",
			context:this._getViewType(),
			controller:this
		};
		var tb = this._toolbar[view] = new ZmButtonToolBar(params);

		buttons = tb.opList;
		for (var i = 0; i < buttons.length; i++) {
			var button = buttons[i];
			if (this._listeners[button]) {
				tb.addSelectionListener(button, this._listeners[button]);
			}
		}

		this._setupSpamButton(tb);
		button = tb.getButton(ZmOperation.TAG_MENU);
		if (button) {
			button.noMenuBar = true;
			this._setupTagMenu(tb);
		}
	}
};

ZmMsgController.prototype._navBarListener =
function(ev) {
	var op = ev.item.getData(ZmOperation.KEY_ID);
	if (op == ZmOperation.PAGE_BACK || op == ZmOperation.PAGE_FORWARD) {
		this._goToMsg(this._currentView, (op == ZmOperation.PAGE_FORWARD));
	}
};

// message view has no view menu button
ZmMsgController.prototype._setupViewMenu = function(view, firstTime) {};

ZmMsgController.prototype._getActionMenuOps =
function() {
	return null;
};

ZmMsgController.prototype._getViewType =
function() {
	return this.viewId;
};

ZmMsgController.prototype._initializeListView =
function(view) {
	if (!this._listView[view]) {
		var params = {
			parent:		this._container,
			id:			ZmId.getViewId(ZmId.VIEW_MSG, null, view),
			posStyle:	Dwt.ABSOLUTE_STYLE,
			mode:		ZmId.VIEW_MSG,
			controller:	this
		};
		this._listView[view] = new ZmMailMsgView(params);
		this._listView[view].addInviteReplyListener(this._inviteReplyListener);
		this._listView[view].addShareListener(this._shareListener);
	}
};

ZmMsgController.prototype.getReferenceView =
function () {
	return this._listView[this._currentView];
};

ZmMsgController.prototype._getSearchFolderId =
function() {
	return this._msg.folderId ? this._msg.folderId : (this._msg.list && this._msg.list.search) ?
		this._msg.list.search.folderId : null;
};

ZmMsgController.prototype._getTagMenuMsg =
function() {
	return ZmMsg.tagMessage;
};

ZmMsgController.prototype._getMoveDialogTitle =
function() {
	return ZmMsg.moveMessage;
};

ZmMsgController.prototype._setViewContents =
function(view) {
	this._listView[view].set(this._msg);
};

ZmMsgController.prototype._resetNavToolBarButtons =
function(view) {
	if (!this._navToolBar[view]) { return; }
	// NOTE: we purposely do not call base class here!
	if (!appCtxt.isChildWindow) {
		var list = this._msg.list && this._msg.list.getVector();

		this._navToolBar[view].enable(ZmOperation.PAGE_BACK, (list && (list.get(0) != this._msg)));

		var bEnableForw = list && (this._msg.list.hasMore() || (list.getLast() != this._msg));
		this._navToolBar[view].enable(ZmOperation.PAGE_FORWARD, bEnableForw);

		this._navToolBar[view].setToolTip(ZmOperation.PAGE_BACK, ZmMsg.previousMessage);
		this._navToolBar[view].setToolTip(ZmOperation.PAGE_FORWARD, ZmMsg.nextMessage);
	}
};

ZmMsgController.prototype._goToMsg =
function(view, next) {
	var controller = this._getParentListController();
	if (controller) {
		controller.pageItemSilently(this._msg, next, this);
	}
};

ZmMsgController.prototype._selectNextItemInParentListView =
function() {
	var controller = this._getParentListController();
	if (controller) {
		controller._listView[controller._currentView]._itemToSelect = controller._getNextItemToSelect();
	}
};

ZmMsgController.prototype._doDelete =
function() {
	this._selectNextItemInParentListView();
	ZmMailListController.prototype._doDelete.apply(this, arguments);
};

ZmMsgController.prototype._doMove =
function() {
	this._selectNextItemInParentListView();
	ZmMailListController.prototype._doMove.apply(this, arguments);
};

ZmMsgController.prototype._doSpam =
function() {
	this._selectNextItemInParentListView();
	ZmMailListController.prototype._doSpam.apply(this, arguments);
};

ZmMsgController.prototype._menuPopdownActionListener =
function(ev) {
	// dont do anything since msg view has no action menus
};

// Miscellaneous

ZmMsgController.prototype.getMsg =
function(params) {
	return this._msg;
};

ZmMsgController.prototype._getLoadedMsg =
function(params, callback) {
	callback.run(this._msg);
};

ZmMsgController.prototype._getSelectedMsg =
function() {
	return this._msg;
};

ZmMsgController.prototype.setMsg =
function (msg) {
	this._msg = msg;
};

// No-op replenishment
ZmMsgController.prototype._checkReplenish =
function(params) {
	// XXX: remove this when replenishment is fixed for msg controller!
	DBG.println(AjxDebug.DBG1, "SORRY. NO REPLENISHMENT FOR YOU.");
};

ZmMsgController.prototype._checkItemCount =
function() {
	this._backListener();
};

ZmMsgController.prototype._getDefaultFocusItem = 
function() {
	return this._toolbar[this._currentView];
};

ZmMsgController.prototype._backListener =
function(ev) {
	var isChildWindow = appCtxt.isChildWindow;
	if (!this._app.popView() && !isChildWindow) {
		this._app.mailSearch();
	}
};

ZmMsgController.prototype.isTransient =
function(oldView, newView) {
	return (newView && newView.indexOf(ZmId.VIEW_COMPOSE) != 0);
};

ZmMsgController.prototype._tabCallback =
function(oldView, newView) {
	return (oldView && oldView.indexOf(ZmId.VIEW_MSG) == 0);
};

ZmMsgController.prototype._printListener =
function(ev) {
    var ids = [];
    var item = this._msg;
    var id;
    var showImages;
    // always extract out the msg ids from the conv
    if (item.toString() == "ZmConv") {
        // get msg ID in case of virtual conv.
        // item.msgIds.length is inconsistent, so checking if conv id is negative.
        if (appCtxt.isOffline && item.id.split(":")[1]<0) {
            id = item.msgIds[0];
        } else {
            id = "C:" + item.id;
        }
        var msgList = item.getMsgList();
        for(var j=0; j<msgList.length; j++) {
            if(msgList[j].showImages) {
                showImages = true;
                break;
            }
        }
    } else {
        id = item.id;
        if (item._part) { id+= "&part=" + item._part; }
        if (item.showImages) {
            showImages = true;
        }
    }
    var url = "/h/printmessage?id=" + id;
    if (appCtxt.get(ZmSetting.DISPLAY_EXTERNAL_IMAGES) || showImages) {
        url += "&xim=1";
    }
    if (appCtxt.isOffline) {
        var acctName = item.getAccount().name;
        url+="&acct=" + acctName ;
    }
    window.open(appContextPath+url, "_blank");
};

/**
 * Returns the parent list controller (TV, CLV, or CV)
 *
 * @private
 */
ZmMsgController.prototype._getParentListController =
function() {
	var ac = appCtxt.isChildWindow ? parentAppCtxt : appCtxt;
	var mailApp = ac.getApp(ZmApp.MAIL);
	if (this._mode == ZmId.VIEW_TRAD) {
		return mailApp.getTradController();
	} else if (this._mode == ZmId.VIEW_CONV) {
		return mailApp.getConvController();
	} else if (this._mode == ZmId.VIEW_CONVLIST) {
		return mailApp.getConvListController();
	}
};

ZmMsgController.prototype._acceptShareHandler =
function(ev) {
    ZmMailListController.prototype._acceptShareHandler.call(this, ev);
    //Close View
    appCtxt.getAppViewMgr().popView();
};

ZmMsgController.prototype._handleResponseDoAction =
function(params, msg) {

    var action = params.action;
    if ( !appCtxt.isChildWindow &&
       (  action == ZmOperation.REPLY
       || action == ZmOperation.REPLY_ALL
       || action == ZmOperation.FORWARD_INLINE
       || action == ZmOperation.FORWARD_ATT
       || action == ZmOperation.FORWARD)) {

        this._backListener(); //close the message view
    }

    //complete action to open compose window
    ZmMailListController.prototype._handleResponseDoAction.call(this, params, msg);
};
}
if (AjxPackage.define("zimbraMail.mail.controller.ZmConvController")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * Creates a new, empty conversation controller.
 * @constructor
 * @class
 * This class manages the two-pane conversation view. The top pane contains a list
 * view of the messages in the conversation, and the bottom pane contains the current
 * message.
 *
 * @author Conrad Damon
 * 
 * @param {ZmComposite}	container	the containing shell
 * @param {ZmMailApp}	mailApp			the containing app
 * 
 * @extends		ZmDoublePaneController
 */
ZmConvController = function(container, mailApp) {

	ZmDoublePaneController.call(this, container, mailApp);

	this._convDeleteListener = new AjxListener(this, this._deleteListener);
	this._listeners[ZmOperation.DELETE_MENU] = this._convDeleteListener;
	this._msgControllerMode = ZmId.VIEW_CONV;
};

ZmConvController.prototype = new ZmDoublePaneController;
ZmConvController.prototype.constructor = ZmConvController;

ZmMailListController.GROUP_BY_ICON[ZmId.VIEW_CONV]			= "ConversationView";

// Public methods

ZmConvController.prototype.toString = 
function() {
	return "ZmConvController";
};

/**
 * Displays the given conversation in a two-pane view. The view is actually
 * created in _loadConv(), since it is a scheduled method and must execute
 * last.
 *
 * @param {ZmSearch}	activeSearch		the current search results
 * @param {ZmConv}	conv				a conversation
 * @param {ZmMailController}	parentController	the controller that called this method
 * @param {AjxCallback}	callback			the client callback
 * @param {Boolean}	markRead		if <code>true</code>, mark msg read
 */
ZmConvController.prototype.show =
function(activeSearch, conv, parentController, callback, markRead) {
	this._conv = conv;
	var lv = this._listView[this._currentView];
	// always reset offset & sortby to asc.
	if (lv) {
		lv.offset = 0;
		lv.setSortByAsc(ZmItem.F_DATE, false);
	}
	this._parentController = parentController;

	// this._list will be set when conv is loaded
	ZmDoublePaneController.prototype.show.call(this, activeSearch, conv, callback, markRead);
};

ZmConvController.prototype.getConv =
function() {
	return this._conv;
};


// Private and protected methods

ZmConvController.prototype._getReadingPanePref =
function() {
	return appCtxt.get(ZmSetting.READING_PANE_LOCATION_CV);
};

ZmConvController.prototype._setReadingPanePref =
function(value) {
	appCtxt.set(ZmSetting.READING_PANE_LOCATION_CV, value);
};

ZmConvController.prototype._createDoublePaneView =
function() {
	return (new ZmConvView({parent:this._container, controller:this, dropTgt:this._dropTgt}));
};

// Creates the conv view, which is not a standard list view (it's a two-pane sort of thing).
ZmConvController.prototype._initialize =
function(view) {
	ZmDoublePaneController.prototype._initialize.call(this, view);
	
	// set up custom listeners for this view 
	if (this._doublePaneView) {
		this._doublePaneView.addTagClickListener(new AjxListener(this, ZmConvController.prototype._convTagClicked));
	}
};

ZmConvController.prototype._initializeToolBar = 
function(view) {
	if (!this._toolbar[view]) {
		ZmDoublePaneController.prototype._initializeToolBar.call(this, view);
	}
	this._setupDeleteMenu(view);	// ALWAYS call setup to turn delete menu on/off
	this._setupSpamButton(this._toolbar[view]);
	this._setupCheckMailButton(this._toolbar[view]);
};

// conv view has arrows to go to prev/next conv, so needs regular nav toolbar
ZmConvController.prototype._initializeNavToolBar =
function(view) {
	ZmMailListController.prototype._initializeNavToolBar.apply(this, arguments);
	this._itemCountText[ZmSetting.RP_BOTTOM] = this._navToolBar[view]._textButton;
};

ZmConvController.prototype._navBarListener =
function(ev) {
	var op = ev.item.getData(ZmOperation.KEY_ID);
	if (op == ZmOperation.PAGE_BACK || op == ZmOperation.PAGE_FORWARD) {
		this._goToConv(op == ZmOperation.PAGE_FORWARD);
	}
};

ZmConvController.prototype._setupViewMenuItems =
function(view, btn) {

	var menu = new ZmPopupMenu(btn);
	btn.setMenu(menu);

	this._setupReadingPaneMenuItems(view, menu, this.isReadingPaneOn());

	return menu;
};

ZmConvController.prototype._setupDeleteMenu =
function(view) {
	var delButton = this._toolbar[view].getButton(ZmOperation.DELETE_MENU);
	if (this._conv.numMsgs > 1) {
		var menu = new ZmPopupMenu(delButton);
		delButton.setMenu(menu);
		delButton.noMenuBar = true;

		var id = ZmOperation.DELETE_MSG;
		var mi = menu.createMenuItem(id, {image:ZmOperation.getProp(id, "image"), text:ZmMsg[ZmOperation.getProp(id, "textKey")]});
		mi.setData(ZmOperation.MENUITEM_ID, ZmOperation.DELETE_MSG);
		mi.addSelectionListener(this._listeners[ZmOperation.DELETE]);

		id = ZmOperation.DELETE_CONV;
		mi = menu.createMenuItem(id, {image:ZmOperation.getProp(id, "image"), text:ZmMsg[ZmOperation.getProp(id, "textKey")]});
		mi.setData(ZmOperation.MENUITEM_ID, ZmOperation.DELETE_CONV);
		mi.addSelectionListener(this._listeners[ZmOperation.DELETE]);
	}
	else if (delButton.getMenu()) {
		delButton.setMenu(null);
	}
};

/**
 * Override to replace DELETE with DELETE_MENU
 */
ZmConvController.prototype._standardToolBarOps =
function() {
	return [ZmOperation.NEW_MENU,
			ZmOperation.SEP,
			ZmOperation.CHECK_MAIL,
			ZmOperation.SEP,
			ZmOperation.DELETE_MENU, ZmOperation.MOVE, ZmOperation.PRINT];
};

ZmConvController.prototype._getViewType =
function() {
	return ZmId.VIEW_CONV;
};

ZmConvController.prototype._setActiveSearch =
function(view) {
	// bug fix #7389 - do nothing!
};

// Operation listeners

// Delete one or more items.
ZmConvController.prototype._deleteListener =
function(ev) {
	if (ev.item.getData(ZmOperation.MENUITEM_ID) == ZmOperation.DELETE_CONV) {
		// use conv list controller to delete conv
		var clc = AjxDispatcher.run("GetConvListController");
		clc._doDelete([this._conv]);
		this._app.popView();
	}
	else if (ev.item.getMenu() == null) {
		var items = this._listView[this._currentView].getSelection();
		var delItems = [];
		for (var i = 0; i < items.length; i++) {
			var item = items[i];
			var folder = item.folderId ? appCtxt.getById(item.folderId) : null;
			var canDelete = (!folder || (folder && !folder.isHardDelete()));
			if (canDelete) {
				delItems.push(item);
			}
		}
		if (delItems.length) {
			this._doDelete(delItems);
		}
	} else {
		ev.item.popup();
	}
};

// Tag in the summary area clicked, do a tag search.
ZmConvController.prototype._convTagClicked =
function(tagId) {
	var tag = appCtxt.getById(tagId);
	appCtxt.getSearchController().search({query: tag.createQuery()});
};

// Handle DnD tagging (can only add a tag to a single item) - if a tag got dropped onto
// a msg, we need to update its conv
ZmConvController.prototype._dropListener =
function(ev) {
	ZmListController.prototype._dropListener.call(this, ev);
	// need to check to make sure tagging actually happened
	if (ev.action == DwtDropEvent.DRAG_DROP) {
		var div = this._listView[this._currentView].getTargetItemDiv(ev.uiEvent);
		if (div) {
			var tag = ev.srcData;
			if (!this._conv.hasTag(tag.id)) {
				this._doublePaneView._setTags(this._conv); 	// update tag summary
			}
		}
	}
};

// same as for ZmTradController
ZmConvController.prototype._listSelectionListener =
function(ev) {
	var item = ev.item;
	if (!item) { return; }
	var handled = ZmDoublePaneController.prototype._listSelectionListener.apply(this, arguments);
	if (!handled && ev.detail == DwtListView.ITEM_DBL_CLICKED) {
		var respCallback = new AjxCallback(this, this._handleResponseListSelectionListener, item);
		AjxDispatcher.run("GetMsgController", item && item.nId).show(item, this._msgControllerMode, respCallback, true);
	}
};

// Miscellaneous

// Called after a delete/move notification has been received.
// Return value indicates whether view was popped as a result of a delete.
ZmConvController.prototype.handleDelete =
function() {

	var popView = true;

	if (this._conv.numMsgs > 1) {
		popView = !this._conv.hasMatchingMsg(AjxDispatcher.run("GetConvListController").getList().search, true);
	}

	// Don't pop unless we're currently visible!
	var currViewId = appCtxt.getCurrentViewId();

	// bug fix #4356 - if currViewId is compose (among other restrictions) then still pop
	var popAnyway = false;
	if (currViewId == ZmId.VIEW_COMPOSE && this._conv.numMsgs == 1 && this._conv.msgs) {
		var msg = this._conv.msgs.getArray()[0];
		popAnyway = (msg.isInvite() && msg.folderId == ZmFolder.ID_TRASH);
	}

	popView = popView && ((currViewId == this._currentView) || popAnyway);

	if (popView) {
		this._app.popView();
	} else {
		var delButton = this._toolbar[this._currentView].getButton(ZmOperation.DELETE_MENU);
		var delMenu = delButton ? delButton.getMenu() : null;
		if (delMenu) {
			delMenu.enable(ZmOperation.DELETE_MSG, false);
		}
	}

	return popView;
};

ZmConvController.prototype.getKeyMapName =
function() {
	return "ZmConvController";
};

ZmConvController.prototype.handleKeyAction =
function(actionCode) {
	DBG.println(AjxDebug.DBG3, "ZmConvController.handleKeyAction");

	switch (actionCode) {
		case ZmKeyMap.CANCEL:
			this._backListener();
			break;

		case ZmKeyMap.NEXT_CONV:
			if (this._navToolBar[this._currentView].getButton(ZmOperation.PAGE_FORWARD).getEnabled()) {
				this._goToConv(true);
			}
			break;

		case ZmKeyMap.PREV_CONV:
			if (this._navToolBar[this._currentView].getButton(ZmOperation.PAGE_BACK).getEnabled()) {
				this._goToConv(false);
			}
			break;

		default:
			return ZmDoublePaneController.prototype.handleKeyAction.call(this, actionCode);
	}
	return true;
};


ZmConvController.prototype._resetOperations =
function(parent, num) {
	ZmDoublePaneController.prototype._resetOperations.call(this, parent, num);

	var canDelete = false;
	var items = this._doublePaneView.getSelection();
	for (var i = 0; i < items.length; i++) {
		var item = items[i];
		var folder = item.folderId ? appCtxt.getById(item.folderId) : null;
		canDelete = (!folder || (folder && !folder.isInTrash()));
		if (canDelete) {
			break;
		}
	}

	parent.enable(ZmOperation.DELETE, canDelete);

	if (parent instanceof ZmButtonToolBar) {
		parent.enable(ZmOperation.DELETE_MENU, true);
		var delButton = parent.getButton(ZmOperation.DELETE_MENU);
		var delMenu = delButton ? delButton.getMenu() : null;
		if (delMenu) {
			delMenu.enable(ZmOperation.DELETE_MSG, canDelete);
		}
	}
};

ZmConvController.prototype._resetNavToolBarButtons =
function(view) {

	ZmDoublePaneController.prototype._resetNavToolBarButtons.call(this, view);
	if (!this._navToolBar[view]) { return; }

	var list = this._conv.list.getVector();

	// enable/disable arrows per conversation index
	var first = list.get(0);
	this._navToolBar[view].enable(ZmOperation.PAGE_BACK, (first && first != this._conv));
	var enablePgDn = this._conv.list.hasMore() || (list.getLast() != this._conv);
	this._navToolBar[view].enable(ZmOperation.PAGE_FORWARD, enablePgDn);

	this._navToolBar[view].setToolTip(ZmOperation.PAGE_BACK, ZmMsg.previousConversation);
	this._navToolBar[view].setToolTip(ZmOperation.PAGE_FORWARD, ZmMsg.nextConversation);
};

ZmConvController.prototype._getNumTotal =
function() {
	return this._conv.numMsgs;
};

// overloaded...
ZmConvController.prototype._search = 
function(view, offset, limit, callback) {
	var params = {
		sortBy: appCtxt.get(ZmSetting.SORTING_PREF, view),
		offset: offset,
		limit: limit,
		getFirstMsg: this.isReadingPaneOn()
	};
	this._conv.load(params, callback);
};

ZmConvController.prototype._goToConv =
function(next) {
	var ctlr = this._parentController || AjxDispatcher.run("GetConvListController");
	if (ctlr) {
		ctlr.pageItemSilently(this._conv, next);
	}
};

ZmConvController.prototype._getSearchFolderId = 
function() {
	return this._conv.list.search.folderId;
};

// top level view means this view is allowed to get shown when user clicks on 
// app icon in app toolbar - we dont want conv view to be top level (always show CLV)
ZmConvController.prototype._isTopLevelView = 
function() {
	return false;
};

// don't preserve selection in CV, just select first hot msg as usual
ZmConvController.prototype._resetSelection = function() {};

ZmConvController.prototype._selectNextItemInParentListView =
function() {
	var controller = this._parentController || AjxDispatcher.run("GetConvListController");
	if (controller) {
		controller._listView[controller._currentView]._itemToSelect = controller._getNextItemToSelect();
	}
};

ZmConvController.prototype._doDelete =
function() {
	this._selectNextItemInParentListView();
	ZmDoublePaneController.prototype._doDelete.apply(this, arguments);
};

ZmConvController.prototype._doMove =
function() {
	this._selectNextItemInParentListView();
	ZmDoublePaneController.prototype._doMove.apply(this, arguments);
};

ZmConvController.prototype._doSpam =
function() {
	this._selectNextItemInParentListView();
	ZmDoublePaneController.prototype._doSpam.apply(this, arguments);
};

ZmConvController.prototype._msgViewCurrent =
function() {
	return true;
};
}
if (AjxPackage.define("zimbraMail.mail.controller.ZmAttachmentsController")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * Creates the attachments controller.
 * @class
 * This class represents an attachments controller.
 * 
 * @param	{ZmComposite}	container		the container
 * @param	{ZmMailApp}		app				the application
 * 
 * @extends		ZmController
 * 
 */
ZmAttachmentsController = function(container, app) {

    if (arguments.length == 0) { return; }

    ZmController.call(this, container, app);

    this._trees = {};

    this._listeners = {};

};

ZmAttachmentsController.prototype = new ZmController();
ZmAttachmentsController.prototype.constructor = ZmAttachmentsController;

// Main Categories

ZmAttachmentsController.CATEGORY_BYTYPE = "BY_TYPE";
ZmAttachmentsController.CATEGORY_BYTIME = "BY_TIME";
ZmAttachmentsController.CATEGORY_BYTAG = "BY_SENDER";
ZmAttachmentsController.CATEGORY_BYSENDER = "BY_TAG";
ZmAttachmentsController.CATEGORY_BYFOLDER = "BY_FOLDER";

ZmAttachmentsController.CATEGORIES = [];

ZmAttachmentsController.CATEGORIES.push(ZmAttachmentsController.CATEGORY_BYTYPE);
ZmAttachmentsController.CATEGORIES.push(ZmAttachmentsController.CATEGORY_BYTIME);
ZmAttachmentsController.CATEGORIES.push(ZmAttachmentsController.CATEGORY_BYTAG);
ZmAttachmentsController.CATEGORIES.push(ZmAttachmentsController.CATEGORY_BYSENDER);
ZmAttachmentsController.CATEGORIES.push(ZmAttachmentsController.CATEGORY_BYFOLDER);



ZmAttachmentsController.CATEGORY_MSGKEY = {};
ZmAttachmentsController.CATEGORY_MSGKEY[ZmAttachmentsController.CATEGORY_BYTYPE]   = "byType";
ZmAttachmentsController.CATEGORY_MSGKEY[ZmAttachmentsController.CATEGORY_BYTIME]   = "byTime";
ZmAttachmentsController.CATEGORY_MSGKEY[ZmAttachmentsController.CATEGORY_BYTAG]    = "byTag";
ZmAttachmentsController.CATEGORY_MSGKEY[ZmAttachmentsController.CATEGORY_BYSENDER] = "bySender";
ZmAttachmentsController.CATEGORY_MSGKEY[ZmAttachmentsController.CATEGORY_BYFOLDER] = "byFolder";

ZmAttachmentsController.CATEGORY_ICON = {};
ZmAttachmentsController.CATEGORY_ICON[ZmAttachmentsController.CATEGORY_BYTYPE]   = "NodeCollapsed";
ZmAttachmentsController.CATEGORY_ICON[ZmAttachmentsController.CATEGORY_BYTIME]   = "NodeCollapsed";
ZmAttachmentsController.CATEGORY_ICON[ZmAttachmentsController.CATEGORY_BYTAG]    = "NodeCollapsed";
ZmAttachmentsController.CATEGORY_ICON[ZmAttachmentsController.CATEGORY_BYSENDER] = "NodeCollapsed";
ZmAttachmentsController.CATEGORY_ICON[ZmAttachmentsController.CATEGORY_BYFOLDER] = "NodeCollapsed";

//Query Commands

ZmAttachmentsController.ATTACH_QUERY_CMD = "attachment:";
ZmAttachmentsController.AFTER_QUERY_CMD  = "after:";
ZmAttachmentsController.BEFORE_QUERY_CMD = "before:";
ZmAttachmentsController.FROM_QUERY_CMD   = "from:";
ZmAttachmentsController.TAG_QUERY_CMD    = "tag:";
ZmAttachmentsController.FOLDER_QUERY_CMD = "in:";

ZmAttachmentsController.DEFAULT_QUERY = ZmAttachmentsController.ATTACH_QUERY_CMD + "any";

//Sub Categories

ZmAttachmentsController.SUBCATEGORY_BYTYPE_ALL              = "BY_TYPE_ALL";
ZmAttachmentsController.SUBCATEGORY_BYTYPE_IMAGE            = "BY_TYPE_IMAGE";
ZmAttachmentsController.SUBCATEGORY_BYTYPE_DOCS             = "BY_TYPE_DOCS";
ZmAttachmentsController.SUBCATEGORY_BYTYPE_PRESENTATIONS    = "BY_TYPE_PPT";
ZmAttachmentsController.SUBCATEGORY_BYTYPE_SPREADSHEETS     = "BY_TYPE_SHEET";
ZmAttachmentsController.SUBCATEGORY_BYTYPE_MEDIA            = "BY_TYPE_MEDIA";


ZmAttachmentsController.CATEGORY_ITEMS = {};
ZmAttachmentsController.CATEGORY_ITEMS[ZmAttachmentsController.CATEGORY_BYTYPE] = [
    {
        id     : ZmAttachmentsController.SUBCATEGORY_BYTYPE_ALL,
        msgKey : "all",
        image  : "Attachment",
        ct     : ["any"],
        cmd    : ZmAttachmentsController.ATTACH_QUERY_CMD
    },
    {
        id     : ZmAttachmentsController.SUBCATEGORY_BYTYPE_IMAGE,
        msgKey : "images",
        image  : "ImageDoc",
        ct     : ["image"],
        cmd    : ZmAttachmentsController.ATTACH_QUERY_CMD
    },
    {
        id     : ZmAttachmentsController.SUBCATEGORY_BYTYPE_DOCS,
        msgKey : "documents",
        image  : "MSWordDoc",
        ct     : ["text/plain", "application/msword", "application/pdf"],
        cmd    : ZmAttachmentsController.ATTACH_QUERY_CMD
    },
    {
        id     : ZmAttachmentsController.SUBCATEGORY_BYTYPE_PRESENTATIONS,
        msgKey : "presentations",
        image  : "MSPowerpointDoc",
        ct     : ["application/vnd.ms-powerpoint"],
        cmd    : ZmAttachmentsController.ATTACH_QUERY_CMD
    },
    {
        id     : ZmAttachmentsController.SUBCATEGORY_BYTYPE_SPREADSHEETS,
        msgKey : "sheet",
        image  : "MSExcelDoc",
        ct     : ["application/vnd.ms-excel"],
        cmd    : ZmAttachmentsController.ATTACH_QUERY_CMD
    },
    {
        id     : ZmAttachmentsController.SUBCATEGORY_BYTYPE_MEDIA,
        msgKey : "audioVideo",
        image  : "AudioDoc",
        ct     : ["audio", "audio/x-wav", "audio/mpeg", "video", "video/x-ms-wmv"],
        cmd    : ZmAttachmentsController.ATTACH_QUERY_CMD
    }
];

ZmAttachmentsController.SUBCATEORY_BYTIME_TODAY         = "BY_TIME_TODAY";
ZmAttachmentsController.SUBCATEORY_BYTIME_WEEK          = "BY_TIME_WEEK";
ZmAttachmentsController.SUBCATEORY_BYTIME_MONTH         = "BY_TIME_MONTH";
ZmAttachmentsController.SUBCATEORY_BYTIME_YEAR          = "BY_TIME_YEAR";

ZmAttachmentsController.CATEGORY_ITEMS[ZmAttachmentsController.CATEGORY_BYTIME] = [
    {
        id      :   ZmAttachmentsController.SUBCATEORY_BYTIME_TODAY,
        msgKey  :   "today",
        image    :  "DayView",
        cmd     :   ZmAttachmentsController.AFTER_QUERY_CMD
    },
    {
        id      :   ZmAttachmentsController.SUBCATEORY_BYTIME_WEEK,
        msgKey  :   "lastWeek",
        image    :   "WeekView",
        cmd     :   ZmAttachmentsController.AFTER_QUERY_CMD
    },
    {
        id      :   ZmAttachmentsController.SUBCATEORY_BYTIME_MONTH,
        msgKey  :   "lastMonth",
        image    :   "MonthView",
        cmd     :   ZmAttachmentsController.AFTER_QUERY_CMD
    },
    {
        id      :   ZmAttachmentsController.SUBCATEORY_BYTIME_YEAR,
        msgKey  :   "lastYear",
        image    :   "CalendarApp",
        cmd     :   ZmAttachmentsController.AFTER_QUERY_CMD
    },
    {
        id      :   ZmAttachmentsController.SUBCATEORY_BYTIME_ANYTIME,
        msgKey  :   "anyTime",
        image    :   "Date"
    }
];

ZmAttachmentsController.prototype._initToolBar =
function(){

    if (this._toolbar) return;

    this._listeners[ZmOperation.RESET] = new AjxListener(this, this._reset);
    this._listeners[ZmOperation.CANCEL] = new AjxListener(this, this._cancel);
    //this._listeners[ZmOperation.SEND] = new AjxListener(this, this._sendMail);

    var buttons = [ZmOperation.RESET, /*ZmOperation.SEND,*/ ZmOperation.CANCEL];


	var className = appCtxt.isChildWindow ? "ZmAppToolBar_cw" : "ZmAppToolBar";
	this._toolbar = new ZmButtonToolBar({parent:this._container, buttons:buttons, className:className+" ImgSkin_Toolbar",
										 context:ZmId.VIEW_ATTACHMENTS});

	for (var i = 0; i < this._toolbar.opList.length; i++) {
		var button = this._toolbar.opList[i];
		if (this._listeners[button]) {
			this._toolbar.addSelectionListener(button, this._listeners[button]);
		}
	}	
};

ZmAttachmentsController.prototype._reset = function(ev){
     this._attachmentsView._resetFilterTrees();
     this.search({query:this._getQuery()});
};

ZmAttachmentsController.prototype._cancel = function(){
    this._app.popView();
};

ZmAttachmentsController.prototype.initAttachmentsView = function(){

    if (this._attachmentsView) return;

    this._attachmentsView =  new ZmAttachmentsView(this._container, this);
	this._initToolBar();

    var elements = {};
    elements[ZmAppViewMgr.C_TOOLBAR_TOP] = this._toolbar;
	elements[ZmAppViewMgr.C_APP_CONTENT] = this._attachmentsView;

    this._app.createView({viewId:ZmId.VIEW_ATTACHMENTS, elements:elements, isTransient:true});

};

ZmAttachmentsController.prototype.show = function(){

    this.initAttachmentsView();

    this._app.pushView(ZmId.VIEW_ATTACHMENTS);

    this._reset();
    
};

ZmAttachmentsController.prototype.search = function(params){

    params.query  = params.query || ZmAttachmentsController.DEFAULT_QUERY;
    params.offset = 0;
    params.limit  = 50;
    params.sortBy = ZmSearch.DATE_DESC;
    params.fetch  = "all";
    params.searchFor = ZmItem.MSG;

    var types = new AjxVector();
    types.add(ZmItem.MSG);
    params.types  = types;

    params.noBusyOverlay = true;
    params.noRender = true;
    params.callback = new AjxCallback(this, this._handleSearchResponse);

    var sc = appCtxt.getSearchController();
    sc.search(params);

};

ZmAttachmentsController.prototype._handleSearchResponse = function(results){

    results = results.getResponse().getResults(ZmItem.MSG);

    this._attachmentsView.set(results);

};

ZmAttachmentsController.prototype._filterTreeListener = function(ev){

    if(ev.detail == DwtTree.ITEM_SELECTED){
        this.search({query:this._getQuery()});
    }
};


ZmAttachmentsController.prototype._getQuery = function(){

    var query = "";

    var queryString = "";

    var byTypeItem = this._attachmentsView.getSelectedItem(ZmAttachmentsController.CATEGORY_BYTYPE);
    if(byTypeItem){

        var subQuery = [];
        var ct = byTypeItem.ct;
        if(byTypeItem.id == ZmAttachmentsController.SUBCATEGORY_BYTYPE_ALL){
            subQuery.push("any");
        }else{
            for(var i=0; i<ct.length; i++){
                subQuery.push('"'+ct[i]+'"');
            }
        }        
        subQuery = subQuery.join(" OR ");

        query += byTypeItem.cmd + "("+ subQuery+")";

        queryString += '<strong>' + ZmMsg[byTypeItem.msgKey] + '</strong> attachments <br>';

    }else{
        queryString += '<strong>' + ZmMsg['all'] + '</strong> attachments <br>';
    }

    var byFolderItem = this._attachmentsView.getSelection(ZmAttachmentsController.CATEGORY_BYFOLDER);
    if(byFolderItem){
         var folder = byFolderItem.getData(Dwt.KEY_OBJECT);
         query += ' ' + folder.createQuery();

         queryString += 'in <strong>' + folder.getName() + '</strong>'+' folder<br>';
    }

    var byTagItem = this._attachmentsView.getSelection(ZmAttachmentsController.CATEGORY_BYTAG);
    if(byTagItem){
        var tag = byTagItem.getData(Dwt.KEY_OBJECT);
        query += ' ' + ZmAttachmentsController.TAG_QUERY_CMD + '"' + tag.name + '"';

        queryString += 'with tag <strong>' + tag.getName() + '</strong><br>';
    }

    var byTimeItem = this._attachmentsView.getSelectedItem(ZmAttachmentsController.CATEGORY_BYTIME);
    if(byTimeItem){

        var date = new Date();
        var offset = -1;
        if(byTimeItem.id == ZmAttachmentsController.SUBCATEORY_BYTIME_TODAY){
            date.setDate(date.getDate() + offset);
        }else if(byTimeItem.id == ZmAttachmentsController.SUBCATEORY_BYTIME_WEEK){
            date.setDate(date.getDate()+7*offset);
        }else if(byTimeItem.id == ZmAttachmentsController.SUBCATEORY_BYTIME_MONTH){
            d = date.getDate();
            date.setDate(1);
            date.setMonth(date.getMonth() + offset);
            var max = AjxDateUtil.daysInMonth(date.getFullYear(), date.getMonth());
		    date.setDate(Math.min(d, max));
        }else if(byTimeItem.id == ZmAttachmentsController.SUBCATEORY_BYTIME_YEAR){
            d = date.getDate();
            date.setDate(1);
            date.setFullYear(date.getFullYear() + offset);
		    var max = AjxDateUtil.daysInMonth(date.getFullYear(), date.getMonth());
		    date.setDate(Math.min(d, max));
        }

        var formatter = AjxDateFormat.getDateInstance(AjxDateFormat.SHORT);
        var afterDate = formatter.format(date);

        if(byTimeItem.cmd) {
            query += ' ' + byTimeItem.cmd + '"' +  afterDate  +'"';
        }

        queryString += 'during <strong>' + ZmMsg[byTimeItem.msgKey] + '</strong><br>';

    }

    //if(queryString == "") queryString = '<strong>' + ZmMsg['all'] + '</strong> attachments<br>';

    this._attachmentsView.setQueryString(queryString);

    return (query != "") ? query : ZmAttachmentsController.DEFAULT_QUERY;
};

ZmAttachmentsController.prototype._sendMail = function(ev){
  //Need Implmentation
};

}
if (AjxPackage.define("zimbraMail.mail.controller.ZmMailConfirmController")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * Creates a new controller to show mail send confirmation.
 * @constructor
 * @class
 * This class represents the mail confirmation controller.
 * 
 * @param {ZmComposite}		container		the containing shell
 * @param {ZmMailApp}		mailApp			the containing app
 * 
 * @extends		ZmController
 */
ZmMailConfirmController = function(container, mailApp) {

	ZmController.call(this, container, mailApp);
};

ZmMailConfirmController.prototype = new ZmController();
ZmMailConfirmController.prototype.constructor = ZmMailConfirmController;

ZmMailConfirmController.prototype.toString =
function() {
	return "ZmMailConfirmController";
};

/**
 * Shows the confirmation that the message was sent.
 *
 * @param {ZmMailMsg}	msg			the message that was sent
 * @param	{constant}	composeViewId		the compose view id
 * @param	{constant}	composeTabId		the compose tab id
 */
ZmMailConfirmController.prototype.showConfirmation =
function(msg, composeViewId, composeTabId) {
	this._composeViewId = composeViewId;
	this._composeTabId = composeTabId;

	if (!this._view) {
		this._initView();
	}

    this._initializeToolBar();
	this.resetToolbarOperations(this._toolbar);
	this._view.showConfirmation(msg);

	if (appCtxt.isChildWindow) {
		appCtxt.getAppViewMgr()._setViewVisible(ZmId.VIEW_LOADING, false);
	}

	appCtxt.getAppViewMgr().replaceView(this._composeViewId, this.viewId);
};

ZmMailConfirmController.prototype.resetToolbarOperations =
function() {
	this._toolbar.enableAll(true);
};

ZmMailConfirmController.prototype.getKeyMapName =
function() {
	return "Global";
};

ZmMailConfirmController.prototype.handleKeyAction =
function(actionCode) {
	switch (actionCode) {
		case ZmKeyMap.CANCEL:
			this._closeListener();
			break;

		default:
			return ZmController.prototype.handleKeyAction.call(this, actionCode);
			break;
	}
	return true;
};

ZmMailConfirmController.prototype._initView =
function() {
	this._view = new ZmMailConfirmView(this._container, this);
	this._view.addNewContactsListener(new AjxListener(this, this._addNewContactsListener));

	var tg = this._createTabGroup();
	var rootTg = appCtxt.getRootTabGroup();
	tg.newParent(rootTg);
	tg.addMember(this._view.getTabGroupMember());

	var elements = {};
	this._initializeToolBar();
	elements[ZmAppViewMgr.C_TOOLBAR_TOP] = this._toolbar;
	elements[ZmAppViewMgr.C_APP_CONTENT] = this._view;
	var callbacks = {};
	callbacks[ZmAppViewMgr.CB_PRE_HIDE] = new AjxCallback(this, this._preHideCallback);
	callbacks[ZmAppViewMgr.CB_POST_SHOW] = new AjxCallback(this, this._postShowCallback);
    this._app.createView({viewId:this.viewId, elements:elements, callbacks:callbacks,
						  tabParams: { id:this._composeTabId }});
};

ZmMailConfirmController.prototype._initializeToolBar =
function() {
	if (this._toolbar) return;

	var buttons = [ZmOperation.CLOSE];

	var className = appCtxt.isChildWindow ? "ZmAppToolBar_cw" : "ZmAppToolBar";
	this._toolbar = new ZmButtonToolBar({parent:this._container, buttons:buttons, className:className+" ImgSkin_Toolbar",
										 context:ZmId.VIEW_MAIL_CONFIRM});
	this._toolbar.addSelectionListener(ZmOperation.CLOSE, new AjxListener(this, this._closeListener));
};

ZmMailConfirmController.prototype._getDefaultFocusItem =
function() {
	return this._view.getDefaultFocusItem();
};

ZmMailConfirmController.prototype._closeListener =
function() {
	this._doClose();
};

ZmMailConfirmController.prototype._addNewContactsListener =
function(attrs) {
	if (!attrs.length) {
		this._doClose();
		return;
	}
	
	var batchCommand = new ZmBatchCommand(false, null, true);
	for (var i = 0, count = attrs.length; i < count; i++) {
		var contact = new ZmContact();
		batchCommand.add(new AjxCallback(contact, contact.create, [attrs[i]]));
	}
	batchCommand.run(new AjxCallback(this, this._handleResponseCreateContacts));
};

ZmMailConfirmController.prototype._handleResponseCreateContacts =
function() {
	this._doClose();
};

ZmMailConfirmController.prototype._doClose =
function() {
	appCtxt.getAppViewMgr().popView(true);
};
}
}
