if (AjxPackage.define("PreferencesCore")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2006, 2007, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */
/*
 * Package: PreferencesCore
 * 
 * Supports: Loading of identities and data sources
 * 
 * Loaded: When identities and/or data sources arrive in a GetInfoResponse
 */
if (AjxPackage.define("zimbraMail.prefs.model.ZmPref")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

ZmPref = function(id, name, dataType) {

	ZmSetting.call(this, id, name, ZmSetting.T_PREF, dataType);

	this.origValue = null;
	this.isDirty = false;
};

ZmPref.prototype = new ZmSetting;
ZmPref.prototype.constructor = ZmPref;

ZmPref.KEY_ID				= "prefId_";

ZmPref.TYPE_STATIC			= "STATIC"; // static text
ZmPref.TYPE_INPUT			= "INPUT";
ZmPref.TYPE_CHECKBOX		= "CHECKBOX";
ZmPref.TYPE_COLOR			= "COLOR";
ZmPref.TYPE_RADIO_GROUP		= "RADIO_GROUP";
ZmPref.TYPE_SELECT			= "SELECT";
ZmPref.TYPE_COMBOBOX		= "COMBOBOX";
ZmPref.TYPE_TEXTAREA		= "TEXTAREA";
ZmPref.TYPE_PASSWORD		= "PASSWORD";
ZmPref.TYPE_IMPORT			= "IMPORT";
ZmPref.TYPE_EXPORT			= "EXPORT";
ZmPref.TYPE_SHORTCUTS		= "SHORTCUTS";
ZmPref.TYPE_CUSTOM			= "CUSTOM";
ZmPref.TYPE_LOCALES			= "LOCALES";

ZmPref.ORIENT_VERTICAL		= "vertical";
ZmPref.ORIENT_HORIZONTAL	= "horizontal";

ZmPref.MAX_ROWS	= 7;

// custom functions for loading and validation

ZmPref.loadSkins =
function(setup) {
	var skins = appCtxt.get(ZmSetting.AVAILABLE_SKINS);
	setup.options = []; // re-init otherwise we could possibly have dupes.
	for (var i = 0; i < skins.length; i++) {
		var skin = skins[i];
		setup.options.push(skin);
		var text = ZmMsg['theme-' + skin] || skin.substr(0, 1).toUpperCase() + skin.substr(1);
		setup.displayOptions.push(text);
	}
};

ZmPref.loadCsvFormats =
function(setup){
    var formats = appCtxt.get(ZmSetting.AVAILABLE_CSVFORMATS);
	if (!formats._options) {
		var options = formats._options = [];
		var displayOptions = formats._displayOptions = [];
		for(var i=0; i<formats.length; i++){
			options.push(formats[i]);
		}
		options.sort(ZmPref.__BY_CSVFORMAT);
		for(var i=0; i < options.length; i++){
			displayOptions.push((ZmMsg[options[i]] || options[i]));
		}
	}
	setup.options = formats._options;
	setup.displayOptions = formats._displayOptions;
};
ZmPref.__BY_CSVFORMAT = function(a, b) {
	if (a.match(/^zimbra/)) return -1;
	if (b.match(/^zimbra/)) return  1;
	if (a.match(/^yahoo/))  return -1;
	if (b.match(/^yahoo/))  return  1;
	return a.localeCompare(b);
};

ZmPref.loadPageSizes =
function(setup) {
	var max = (setup.maxSetting && appCtxt.get(setup.maxSetting)) || 100;
	var list = [];
	for (var i = 0; i < ZmPref.PAGE_SIZES.length; i++) {
		var num = parseInt(ZmPref.PAGE_SIZES[i]);
		if (num <= max) {
			list.push(ZmPref.PAGE_SIZES[i]);
		}
	}
	if (max > ZmPref.PAGE_SIZES[ZmPref.PAGE_SIZES.length - 1]) {
		list.push(String(max));
	}
	setup.displayOptions = setup.options = list;
};
ZmPref.PAGE_SIZES = ["10", "25", "50", "100", "250", "500", "1000"];

ZmPref.validateEmail =
function(emailStr) {
	if (emailStr) {
		// NOTE: Handle localhost for development purposes
		return emailStr.match(/\@localhost$/i) || AjxEmailAddress.parse(emailStr) != null;
	}
	return true;
};

ZmPref.validateEmailList =
function(emailStrArray) {
    for(var i in emailStrArray) {
        if(!ZmPref.validateEmail(emailStrArray[i])) return false;
    }
    return true;
};

ZmPref.downloadSinceDisplay =
function(dateStr) {
	if (dateStr == "") return 0;
	if (dateStr == appCtxt.get(ZmSetting.POP_DOWNLOAD_SINCE)) return 1;
	return 2;
};
ZmPref.downloadSinceValue =
function(value) {
	if (value == 0) return "";
	if (value == 1) return appCtxt.get(ZmSetting.POP_DOWNLOAD_SINCE);
	var date = new Date();
	date.setMinutes(date.getMinutes() + date.getTimezoneOffset());
	return AjxDateFormat.format("yyyyMMddHHmmss'Z'", date);
};

ZmPref.validatePollingInterval =
function(interval) {
	var minimum = appCtxt.get(ZmSetting.MIN_POLLING_INTERVAL);
	if (interval && minimum && interval >= minimum) {
		return true;
	} else {
		var min = minimum / 60;
		ZmPref.SETUP[ZmSetting.POLLING_INTERVAL].errorMessage = AjxMessageFormat.format(ZmMsg.invalidPollingInterval, min);
		return false;
	}
};

ZmPref.pollingIntervalDisplay =
function(seconds) {
    if (appCtxt.get(ZmSetting.INSTANT_NOTIFY) && seconds == appCtxt.get(ZmSetting.INSTANT_NOTIFY_INTERVAL))
        return seconds;
    else
	    return seconds / 60;
};

ZmPref.pollingIntervalValue =
function(minutes) {
    if (appCtxt.get(ZmSetting.INSTANT_NOTIFY) && minutes == appCtxt.get(ZmSetting.INSTANT_NOTIFY_INTERVAL))
        return minutes;
    else
	    return minutes * 60;
};

ZmPref.dateLocal2GMT =
function(value) {
	if (!value) { return ""; }

	var yr, mo, da, hr, mi, se; // really smart parsing.
	yr = parseInt(value.substr(0,  4), 10);
	mo = parseInt(value.substr(4,  2), 10);
	da = parseInt(value.substr(6,  2), 10);
	hr = parseInt(value.substr(8,  2), 10);
	mi = parseInt(value.substr(10, 2), 10);
	se = parseInt(value.substr(12, 2), 10);
	var date = new Date(yr, mo - 1, da, hr, mi, se, 0);
	yr = date.getUTCFullYear();
	mo = date.getUTCMonth() + 1;
	da = date.getUTCDate();
	hr = date.getUTCHours();
	mi = date.getUTCMinutes();
	se = date.getUTCSeconds();
	var a = [ yr, mo, da, hr, mi, se ];
	for (var i = a.length; --i > 0;) {
		var n = a[i];
		if (n < 10)
			a[i] = "0" + n;
	}
	return (a.join("") + "Z");
};

ZmPref.dateGMT2Local =
function(value) {
	if (!value) { return ""; }

	var yr, mo, da, hr, mi, se; // really smart parsing.
	yr = parseInt(value.substr(0,  4), 10);
	mo = parseInt(value.substr(4,  2), 10);
	da = parseInt(value.substr(6,  2), 10);
	hr = parseInt(value.substr(8,  2), 10);
	mi = parseInt(value.substr(10, 2), 10);
	se = parseInt(value.substr(12, 2), 10);
	var date = new Date();
	date.setUTCMilliseconds(0);
	date.setUTCSeconds(se);
	date.setUTCMinutes(mi);
	date.setUTCHours(hr);
	date.setUTCDate(da);
	date.setUTCMonth(mo - 1);
	date.setUTCFullYear(yr);
	yr = date.getFullYear();
	mo = date.getMonth() + 1;
	da = date.getDate();
	hr = date.getHours();
	mi = date.getMinutes();
	se = date.getSeconds();
	var a = [yr, mo, da, hr, mi, se];
	for (var i = a.length; --i > 0;) {
		var n = a[i];
		if (n < 10)
			a[i] = "0" + n;
	}
	return (a.join("") + "Z");
};

ZmPref.int2DurationDay =
function(intValue) {
	return intValue != null && intValue != 0 ? intValue + "d" : intValue;
};

ZmPref.string2EmailList =
function(value) {
    var emailList = [];
    var addr,addrs = AjxEmailAddress.split(value);
    if (addrs && addrs.length) {
        for (var i = 0; i < addrs.length; i++) {
            addr = addrs[i];
            var email = AjxEmailAddress.parse(addr);
            if(email) {
                addr = email.getAddress();
            }
            if(addr) emailList.push(AjxStringUtil.htmlEncode(addr));
        }
    }
	return emailList;
};

ZmPref.durationDay2Int =
function(durValue) {
	return parseInt(durValue, 10); // NOTE: parseInt ignores non-digits
};

ZmPref.approximateInterval =
function(value) {
	var values = [].concat(ZmPref.SETUP["POLLING_INTERVAL"].options);
	values.sort(ZmPref.__BY_NUMBER);
	return ZmPref.approximateValue(values, value);
};

ZmPref.approximateValue =
function(sortedValues, value) {
	// find closest value
	for (var i = 0; i < sortedValues.length + 1; i++) {
		var a = sortedValues[i];
		var b = sortedValues[i+1];
		if (value < b) {
			var da = value - a;
			var db = b - value;
			return da < db ? a : b;
		}
	}
	return sortedValues[sortedValues.length - 1];
};

ZmPref.validateLifetime =
function(value) {
	var globalValue = appCtxt.get(ZmSetting.MAIL_LIFETIME_GLOBAL);
	if (globalValue == "0") return true;
	return ZmPref.__BY_DURATION(value, globalValue) <= 0;
};

ZmPref.validateLifetimeJunk =
function(value) {
	var globalValue = appCtxt.get(ZmSetting.MAIL_LIFETIME_JUNK_GLOBAL);
	if (globalValue == "0") return true;
	return ZmPref.__BY_DURATION(value, globalValue) <= 0 && ZmPref.validateLifetime(value);
};

ZmPref.validateLifetimeTrash =
function(value) {
	var globalValue = appCtxt.get(ZmSetting.MAIL_LIFETIME_TRASH_GLOBAL);
	if (globalValue == "0") return true;
	return ZmPref.__BY_DURATION(value, globalValue) <= 0 && ZmPref.validateLifetime(value);
};

ZmPref.approximateLifetimeInboxRead =
function(value) {
	return ZmPref.approximateLifetime("MAIL_LIFETIME_INBOX_READ", value, ZmPref.validateLifetime);
};

ZmPref.approximateLifetimeInboxUnread =
function(value) {
	return ZmPref.approximateLifetime("MAIL_LIFETIME_INBOX_UNREAD", value, ZmPref.validateLifetime);
};

ZmPref.approximateLifetimeJunk =
function(value) {
	return ZmPref.approximateLifetime("MAIL_LIFETIME_JUNK", value, ZmPref.validateLifetimeJunk, ZmPref.validateLifetime);
};

ZmPref.approximateLifetimeSent =
function(value) {
	return ZmPref.approximateLifetime("MAIL_LIFETIME_SENT", value, ZmPref.validateLifetime);
};

ZmPref.approximateLifetimeTrash =
function(value) {
	return ZmPref.approximateLifetime("MAIL_LIFETIME_TRASH", value, ZmPref.validateLifetimeTrash, ZmPref.validateLifetime);
};

ZmPref.approximateLifetime =
function(prefId, duration, validateFunc1/*, ..., validateFuncN*/) {
	// convert durations to seconds
	var values = [].concat(ZmPref.SETUP[prefId].options);
	for (var i = 0; i < values.length; i++) {
		var value = values[i];
		values[i] = ZmPref.__DUR2SECS(value != "0" ? value+"d" : value);
	}
	values.sort(ZmPref.__BY_NUMBER);

	// remove invalid options
	valuesLoop: for (var i = values.length - 1; i >= 0; i--) {
		for (var j = 2; j < arguments.length; j++) {
			var validateFunc = arguments[j];
			var value = ZmPref.__SECS2DUR(values[i]);
			if (!validateFunc(value)) {
				values.pop();
				continue valuesLoop;
			}
		}
		break;
	}

	// if zero, the closest match is the greatest
	var seconds;
	if (duration == "0") {
		seconds = values[values.length - 1];
	}

	// approximate to closest number of seconds
	else {
		seconds = ZmPref.approximateValue(values, ZmPref.__DUR2SECS(duration+"d"));
	}

	// convert back to duration
	duration = ZmPref.__SECS2DUR(seconds);
	return duration != "0"  ? parseInt(duration, 10) : 0;
};

ZmPref.markMsgReadDisplay =
function(value) {
	return (value > 0) ? 1 : value;
};

ZmPref.markMsgReadValue =
function(value) {
	if (value == ZmSetting.MARK_READ_TIME) {
		var input = Dwt.byId(DwtId._makeId(ZmId.WIDGET_INPUT, ZmId.OP_MARK_READ));
		if (input) {
			return input.value;
		}
	}
	return value;
};

ZmPref.setFormValue =
function(pref, value) {
	var app = appCtxt.getApp(ZmApp.PREFERENCES);
	var section = ZmPref.getPrefSectionWithPref(pref);
	if (app && section) {
		var page = app.getPreferencesPage(section.id);
		if (page) page.setFormValue(pref, value);
	}
};

ZmPref.getFormValue =
function(pref) {
	var app = appCtxt.getApp(ZmApp.PREFERENCES);
	var section = ZmPref.getPrefSectionWithPref(pref);
	if (app && section) {
		var page = app.getPreferencesPage(section.id);
		if (page) return page.getFormValue(pref);
	}
};

ZmPref.setIncludeOrig =
function(pref, value, list) {

	pref.setValue(value);
    pref.origValue = pref.copyValue();
	var settings = [ZmSetting.REPLY_INCLUDE_WHAT, ZmSetting.REPLY_USE_PREFIX, ZmSetting.REPLY_INCLUDE_HEADERS];
	var settingsHash = AjxUtil.arrayAsHash(settings);
	var mainSetting = ZmSetting.REPLY_INCLUDE_ORIG;
	if (!settingsHash[pref.id]) {
		settings = [ZmSetting.FORWARD_INCLUDE_WHAT, ZmSetting.FORWARD_USE_PREFIX, ZmSetting.FORWARD_INCLUDE_HEADERS];
		mainSetting = ZmSetting.FORWARD_INCLUDE_ORIG;
	}

	var values = AjxUtil.map(settings, function(setting) { return appCtxt.get(setting); });
	var key = (values[0] == ZmSetting.INC_NONE || values[0] == ZmSetting.INC_ATTACH) ? values[0] : values.join("|");
	var newValue = ZmMailApp.INC_MAP_REV[key];
	var prefToChange = appCtxt.getSettings().getSetting(mainSetting);
	prefToChange.setValue(newValue);
	list.push(prefToChange);
};

ZmPref.initIncludeWhat =
function(select, value) {
	// wait until prefix/headers checkboxes have been created
	AjxTimedAction.scheduleAction(new AjxTimedAction(this, function() {
		ZmPref._showIncludeOptions(select, (value == ZmSetting.INC_BODY || value == ZmSetting.INC_SMART));
	}), 100);
};

ZmPref.onChangeIncludeWhat =
function(ev) {
	var nv = ev._args.newValue;
	var ov = ev._args.oldValue;
	var newAllowOptions = (nv == ZmSetting.INC_BODY || nv == ZmSetting.INC_SMART);
	var oldAllowOptions = (ov == ZmSetting.INC_BODY || ov == ZmSetting.INC_SMART);
	if (newAllowOptions != oldAllowOptions) {
		ZmPref._showIncludeOptions(ev._args.selectObj, newAllowOptions);
	}
};

ZmPref._showIncludeOptions =
function(select, show) {
	var optionIds = (select._name == ZmSetting.REPLY_INCLUDE_WHAT) ?
						[ZmSetting.REPLY_USE_PREFIX, ZmSetting.REPLY_INCLUDE_HEADERS] :
						[ZmSetting.FORWARD_USE_PREFIX, ZmSetting.FORWARD_INCLUDE_HEADERS];
	for (var i = 0; i < optionIds.length; i++) {
		var cbox = select.parent._dwtObjects[optionIds[i]];
		if (cbox) {
			cbox.setVisible(show);
		}
	}
};

ZmPref.getSendToFiltersActive =
function(ev, callback) {
	if (ev.target.checked) {
		if (callback)
			callback.run(false);
		return false;
	}
	AjxDispatcher.run("GetFilterController").hasOutgoingFiltersActive(callback);
};

ZmPref.onChangeConfirm =
function(confirmMsg, showIfCallback, useCallback, revertCallback, ev) {
	var show = false;
	var callback = useCallback ? new AjxCallback(this, ZmPref._showOnChangeConfirm, [confirmMsg, revertCallback]) : null;
	if (AjxUtil.isFunction(showIfCallback))
		show = showIfCallback(ev, callback);
	else if (AjxUtil.isInstance(showIfCallback, AjxCallback))
		show = showIfCallback.run(ev, callback);
	else
		show = showIfCallback;
	ZmPref._showOnChangeConfirm(confirmMsg, revertCallback, show);
};

ZmPref._showOnChangeConfirm =
function(confirmMsg, revertCallback, show) {
	if (show) {
		if (show) {
			var dialog = appCtxt.getYesNoMsgDialog();
			dialog.reset();
			dialog.setMessage(confirmMsg);
			dialog.setButtonListener(DwtDialog.NO_BUTTON, new AjxListener(null, ZmPref._handleOnChangeConfirmNo, [revertCallback]));
			dialog.popup();
		}
	}
};

ZmPref._handleOnChangeConfirmNo =
function(revertCallback) {
	if (revertCallback)
		revertCallback.run();
	appCtxt.getYesNoMsgDialog().popdown();
};

// Comparators

ZmPref.__BY_NUMBER =
function(a, b) {
	if (a == b) return 0;
	if (a == Math.POSITIVE_INFINITY || b == Math.NEGATIVE_INFINITY) return 1;
	if (b == Math.POSITIVE_INFINITY || a == Math.NEGATIVE_INFINITY) return -1;
	return Number(a) - Number(b);
};

ZmPref.__BY_DURATION =
function(a, b) {
	if (a == b) return 0;
	if (a == "0") return 1;
	if (b == "0") return -1;
	var asecs = ZmPref.__DUR2SECS(a);
	var bsecs = ZmPref.__DUR2SECS(b);
	return asecs - bsecs;
};

// Converters

ZmPref.__DURMULT = { "s": 1, "m": 60, "h": 3600, "d": 86400/*, "w": 604800*/ };
ZmPref.__DURDIV = { /*604800: "w",*/ 86400: "d", 3600: "h", 60: "m", 1: "s" };

ZmPref.__DUR2SECS =
function(duration) {
	if (duration == "0") return Number.POSITIVE_INFINITY;

	var type = duration.substring(duration.length - 1).toLowerCase();
	return parseInt(duration, 10) * ZmPref.__DURMULT[type];
};

ZmPref.__SECS2DUR =
function(seconds, type) {
	if (seconds == Number.POSITIVE_INFINITY) return "0";

	var divisors = ZmPref.__DURDIV;
	if (type) {
		type = {};
		type[ ZmPref.__DURMULT[type] ] = type;
	}
	for (var divisor in divisors) {
		var result = Math.floor(seconds / divisor);
		if (result > 0) {
			return [ result, divisors[divisor] ].join("");
		}
	}

	return "0"+type;
};

// maximum value lengths
ZmPref.MAX_LENGTH = {};
ZmPref.MAX_LENGTH[ZmSetting.INITIAL_SEARCH]	= 512;
ZmPref.MAX_LENGTH[ZmSetting.SIGNATURE]		= 1024;
ZmPref.MAX_LENGTH[ZmSetting.AWAY_MESSAGE]	= 8192;

ZmPref.setPrefList =
function(prefsId, list) {
	ZmPref[prefsId] = list;
};

/**
 * The SETUP object for a pref gets translated into a form input.
 * Available properties are:
 *
 * displayName			descriptive text
 * displayFunc			A function that returns the descriptive text. Only implemented for checkboxes.
 * displayContainer		type of form input: checkbox, select, input, or textarea
 * options				values for a select input
 * displayOptions		text for the select input's values
 * validationFunction	function to validate the value
 * errorMessage			message to show if validation fails
 * precondition			pref will not be displayed unless precondition is true
 * inputId              array of unique ids to be applied to input
 */
ZmPref.SETUP = {};

ZmPref.registerPref =
function(id, params) {
	ZmPref.SETUP[id] = params;
};

/** Clears all of the preference sections. */
ZmPref.clearPrefSections =
function() {
	ZmPref._prefSectionMap = {};
	ZmPref._prefSectionArray = null;
};

/**
 * Registers a preferences section ("tab").
 * <p>
 * The <code>params</code> argument can have the following properties:
 *
 * @param title         [string]    The section title.
 * @param templateId    [string]    (Optional) The template associated to this
 *                                  section. If not specified, the id is used.
 *                                  Note: The default template base is:
 *                                  "prefs.Pages#".
 * @param priority      [int]       The section priority used when determining
 *                                  the order of the sections.
 * @param precondition  [any]       (Optional) Specifies the precondition
 *                                  under which this section is shown. If this
 *                                  parameter is a function, it returns a
 *                                  boolean signifying that this section is
 *                                  present -- true to show this section. If
 *                                  this parameter is <em>not</em> a function,
 *                                  then it is considered a setting identifier
 *                                  that determines the presence of this
 *                                  section.
 * @param prefs         [Array]     List of preferences that appear in this
 *                                  section.
 * @param manageChanges [boolean]   Determines whether this section manages
 *                                  its own pref changes. If true, then
 *                                  ZmPrefView#getChangedPrefs will not query
 *                                  the section for changes.
 * @param manageDirty   [boolean]   Determines whether this section manages
 *                                  its "dirty" state (i.e. whether any prefs
 *                                  have changed values). If true, then
 *                                  ZmPrefView#getChangedPrefs will call
 *                                  isDirty() on the section view.
 */
ZmPref.registerPrefSection =
function(id, params) {
	if (!id || !params) { return; }

	// set template for section
	var templateId = params.templateId || id;
	if (!templateId.match(/#/)) {
		templateId = ["prefs.Pages",templateId].join("#");
	}
	params.templateId = templateId;
	params.id = id;

	// save section
	appCtxt.set(ZmSetting.PREF_SECTIONS, params, id);
	ZmPref._prefSectionArray = null;
};

ZmPref.unregisterPrefSection =
function(id) {
	appCtxt.set(ZmSetting.PREF_SECTIONS, null, id);
	ZmPref._prefSectionArray = null;
};

/** Returns the pref sections map. */
ZmPref.getPrefSectionMap =
function() {
	return appCtxt.get(ZmSetting.PREF_SECTIONS);
};

/** Returns a sorted array of pref sections (based on priority). */
ZmPref.getPrefSectionArray =
function() {
	if (!ZmPref._prefSectionArray) {
		ZmPref._prefSectionArray = [];
		var prefSectionMap = appCtxt.get(ZmSetting.PREF_SECTIONS);
		for (var id in prefSectionMap) {
			ZmPref._prefSectionArray.push(prefSectionMap[id]);
		}
		ZmPref._prefSectionArray.sort(ZmPref.__BY_PRIORITY);
	}
	return ZmPref._prefSectionArray;
};

/** Returns the section that contains the specified pref. */
ZmPref.getPrefSectionWithPref =
function(prefId) {
	var prefSectionMap = appCtxt.get(ZmSetting.PREF_SECTIONS);
	for (var sectionId in prefSectionMap) {
		var section = prefSectionMap[sectionId];
		if (section.prefs == null) continue;

		for (var i = 0; i < section.prefs.length; i++) {
			if (section.prefs[i] == prefId) {
				return section;
			}
		}
	}
	return null;
};

/** Returns true if <em>all</em> of the preconditions pass. */
ZmPref.requireAllPreConditions =
function(pre1 /* ..., preN */) {
	var app = appCtxt.getApp(ZmApp.PREFERENCES);
	var controller = app.getPrefController();

	for (var i = 0; i < arguments.length; i++) {
		if (!controller.checkPreCondition( null, arguments[i])) {
			return false;
		}
	}
	return true;
};

// Make sure the pref sections are init'd
ZmPref.clearPrefSections();

//
// Private functions
//

ZmPref.__BY_PRIORITY =
function(a, b) {
	return Number(a.priority) - Number(b.priority);
};
}
if (AjxPackage.define("zimbraMail.prefs.model.ZmPrefPage")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * Creates a preferences section. This is a "pseudo" organizer for
 * the preferences application tree view.
 * @constructor
 * @class
 * This class represents the preference page in the preferences application.
 * 
 * @param {Hash}	params    a hash of parameters
 * @param	{int}	     params.id			the numeric ID
 * @param	{String}	params.name		the name
 * @param	{ZmOrganizer}	params.parent		the parent folder
 * @param	{ZmTree}	params.tree		the tree model that contains this folder
 * @param	{String}	params.pageId		the ID of pref page
 * @param	{String}	params.icon		the icon name
 * @param	{String}	params.tooltip		the tool tip text
 * 
 * @extends		ZmOrganizer
 */
ZmPrefPage = function(params) {
	if (arguments.length == 0) { return; }
	params.type = params.type || ZmOrganizer.PREF_PAGE;
	ZmOrganizer.call(this, params);
	this.pageId = params.pageId;
	this.icon = params.icon;
	this.tooltip = params.tooltip;
};

ZmPrefPage.prototype = new ZmOrganizer;
ZmPrefPage.prototype.constructor = ZmPrefPage;

ZmPrefPage.prototype.toString = function() {
	return "ZmPrefPage";
};

//
// Constants
//

ZmOrganizer.ORG_CLASS[ZmId.ORG_PREF_PAGE] = "ZmPrefPage";

//
// Static functions
//

ZmPrefPage.createFromSection = function(section) {
	var overviewController = appCtxt.getOverviewController(); 
	var treeController = overviewController.getTreeController(ZmOrganizer.PREF_PAGE);
	var params = {
		id: ZmId.getPrefPageId(section.id),
		name: section.title,
		parent: null,
		tree: treeController.getDataTree(),
		icon: section.icon,
		tooltip: section.description
	};
	return new ZmPrefPage(params);
};

//
// Public methods
//

// ZmOrganizer methods

ZmPrefPage.prototype.getIcon = function() {
	return this.icon || "Preferences";
};

ZmPrefPage.prototype.getToolTip = function(force) {
	return this.tooltip || "";
};

}
if (AjxPackage.define("zimbraMail.prefs.model.ZmPersona")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * Creates a personna.
 * @class
 * This class represents a personna.
 * 
 * @param	{ZmIdentity}		identity		the identity
 * @param	{Object}		list		the list
 * @extends		ZmAccount
 */
ZmPersona = function(identity, list) {
	if (arguments.length == 0) { return; }
	
	ZmAccount.call(this, ZmAccount.TYPE_PERSONA, identity.id, null, list);

	identity.sendFromDisplay = identity.sendFromDisplay || appCtxt.get(ZmSetting.DISPLAY_NAME);
	identity.sendFromAddress = identity.sendFromAddress || appCtxt.get(ZmSetting.USERNAME);
	this.identity = identity;
};
ZmPersona.prototype = new ZmAccount;
ZmPersona.prototype.constructor = ZmPersona;

ZmPersona.prototype.toString =
function() {
	return "ZmPersona";
};


//
// Public methods
//

ZmPersona.prototype.setName =
function(name) {
	this.getIdentity().name = AjxStringUtil.htmlEncode(name);
};

ZmPersona.prototype.getName =
function() {
	return AjxStringUtil.htmlDecode(this.getIdentity().name);
};

ZmPersona.prototype.setEmail =
function(email) {
	this.getIdentity().sendFromAddress = email;
};

ZmPersona.prototype.getEmail =
function() {
	return this.getIdentity().sendFromAddress;
};

ZmPersona.prototype.getIdentity =
function() {
	return this.identity;
};

ZmPersona.prototype.create =
function(callback, errorCallback, batchCmd) {
	return this.getIdentity().create(callback, errorCallback, batchCmd);
};

ZmPersona.prototype.save =
function(callback, errorCallback, batchCmd) {
	return this.getIdentity().save(callback, errorCallback, batchCmd);
};

ZmPersona.prototype.doDelete = 
function(callback, errorCallback, batchCmd) {
	return this.getIdentity().doDelete(callback, errorCallback, batchCmd);
};
}

if (AjxPackage.define("zimbraMail.prefs.controller.ZmPrefPageTreeController")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2009, 2010, 2011, 2012, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * Creates a preferences page tree controller.
 * @class
 * This class represents the preferences page tree controller.
 * 
 * @extends		ZmTreeController
 */
ZmPrefPageTreeController = function() {
	ZmTreeController.apply(this, arguments);
};
ZmPrefPageTreeController.prototype = new ZmTreeController;
ZmPrefPageTreeController.prototype.constructor = ZmPrefPageTreeController;

ZmPrefPageTreeController.prototype.toString =
function() {
	return "ZmPrefPageTreeController";
};

//
// Public methods
//

ZmPrefPageTreeController.prototype.show =
function(params) {
	// populate tree
	var app = appCtxt.getApp(ZmApp.PREFERENCES);
	var view = app.getPrefController().getPrefsView();
	var account = params.account;

	if (appCtxt.multiAccounts && !this._currentAccount) {
		this._currentAccount = account;
	}

	var tree = new ZmTree(ZmOrganizer.PREF_PAGE);
	var root = tree.root = new ZmPrefPage({id:ZmId.getPrefPageId(0), name:"", tree:tree});
	appCtxt.cacheSet(root.id, root);

	// create pseudo-organizers
	var organizers = [];
	var count = view.getNumTabs();
	for (var i = 0; i < count; i++) {
		var tabKey = i+1;
		var name = view.getTabTitle(tabKey);
		var section = view.getSectionForTab(tabKey);
		if (!account || this._showSection(account, section.id)) {
			// for multi-account mbox, child accounts only show a select few pref options
			var organizer = ZmPrefPage.createFromSection(section);
			organizer.pageId = tabKey;
			organizer.account = account;
			organizers.push(organizer);
		}
	}

	// order pages
	for (var i = 0; i < organizers.length; i++) {
		var organizer = organizers[i];
		var section = view.getSectionForTab(organizer.pageId);
		var parentId = section.parentId;
		if (appCtxt.isOffline &&
			(section.id == "SIGNATURES" ||
			 section.id == "ACCOUNTS" ||
			 section.id == "COMPOSING" ||
			 section.id == "FILTERS"))
		{
			parentId = null;
		}
		var parent = (parentId && tree.getById(ZmId.getPrefPageId(parentId))) || root;
		parent.children.add(organizer);

		organizer.parent = parent;
		organizer.icon = section.icon || parent.getIcon();
	}

	appCtxt.setTree(tree.type, tree, account);

	// setup tree view
	var treeView = ZmTreeController.prototype.show.apply(this, arguments);

	if (!appCtxt.multiAccounts || (appCtxt.multiAccounts && account.isMain)) {
		var page1 = root.children.get(0);
		if (page1) {
			treeView.setSelected(page1, true);
		}
	}
	if (!appCtxt.isOffline) {
		var hi = treeView.getHeaderItem();
		if (hi) {
			hi.setExpanded(true, true);
		}
	}
	treeView.addSelectionListener(this._handleTreeItemSelection.bind(this, view));
	treeView.addPreSelectionListener(this._handleTreeItemPreSelection.bind(this, view));

	return treeView;
};

ZmPrefPageTreeController.prototype._showSection =
function(account, sectionId) {

	if (appCtxt.isOffline) {
		if (sectionId == "MOBILE") {
			return false;
		}

		if (account.isMain) {
			if (sectionId == "FILTERS" ||
				sectionId == "SHARING" ||
				sectionId == "SIGNATURES" ||
				sectionId == "ACCOUNTS" ||
                sectionId == "NOTIFICATIONS" ||
                sectionId == "TRUSTED_ADDR")
			{
				return false;
			}
		}
		else {
			if (sectionId == "COMPOSING") {
				return false;
			}
			if (!account.isZimbraAccount &&
				(sectionId == "MAIL" ||
				 sectionId == "SHARING" ||
				 sectionId == "CALENDAR" ||
                 sectionId == "NOTIFICATIONS" ||
                 sectionId == "TRUSTED_ADDR" ))
			{
				return false;
			}
		}

	}

	return (account.isMain ||
			(!account.isMain && (sectionId != "GENERAL" &&
								 sectionId != "SHORTCUTS" &&
								 sectionId != "PREF_ZIMLETS" &&
                                 sectionId != "BACKUP" &&
								 sectionId != "COMPOSING")
			));
};

//
// Protected methods
//

// ZmTreeController methods

ZmPrefPageTreeController.prototype._dragListener =
function(ev) {
	ev.operation = Dwt.DND_DROP_NONE;
};

ZmPrefPageTreeController.prototype._dropListener =
function(ev) {
	ev.doIt = false;
};

// handlers

ZmPrefPageTreeController.prototype._handleTreeItemSelection =
function(tabView, ev) {
	if (ev.detail != DwtTree.ITEM_SELECTED || ev.handled) { return; }

	var organizer = ev.item.getData(Dwt.KEY_OBJECT);
	tabView.switchToTab(organizer && organizer.pageId);
};

ZmPrefPageTreeController.prototype._handleTreeItemPreSelection =
function(tabView, ev) {

	var organizer = ev.item.getData(Dwt.KEY_OBJECT);

	if (organizer.nId != "PREF_PAGE_FILTERS") {
		return true;
	}

	var serverVersion = appCtxt.get(ZmSetting.OFFLINE_REMOTE_SERVER_VERSION, null, organizer.account);
	if (serverVersion && serverVersion.substr(0, 1) > 7) {
		var dlg = appCtxt.getMsgDialog();
		dlg.reset();
		dlg.setMessage(ZmMsg.filtersDisabled, DwtMessageDialog.INFO_STYLE);
		dlg.popup();
		return false;
	}

	return true;

};


ZmPrefPageTreeController.prototype._handleMultiAccountItemSelection =
function(ev, overview, treeItem, item) {
	if (this._currentAccount != item.account) {
		var prefsController = appCtxt.getApp(ZmApp.PREFERENCES).getPrefController();
		var prefsView = prefsController.getPrefsView();

		this._currentAccount = prefsController._activeAccount = item.account;

		if (prefsView.getChangedPrefs(true, true)) {
			ev.handled = true;

			var dialog = appCtxt.getYesNoCancelMsgDialog();
			var args = [ev, overview, treeItem, item, prefsController, dialog];
			var yesCallback = new AjxCallback(this, this._savePrefsYes, args);
			var noCallback = new AjxCallback(this, this._savePrefsNo, args);
			var cancelCallback = new AjxCallback(this, this._savePrefsCancel, dialog);

			dialog.reset();
			dialog.setMessage(ZmMsg.confirmExitPreferencesChangeAcct, DwtMessageDialog.WARNING_STYLE);
			dialog.registerCallback(DwtDialog.YES_BUTTON, yesCallback, this);
			dialog.registerCallback(DwtDialog.NO_BUTTON, noCallback, this);
			dialog.registerCallback(DwtDialog.CANCEL_BUTTON, cancelCallback, this);
			dialog.popup();
			return;
		}
		else {
			prefsView.resetOnAccountChange();
		}
	}

	ev.handled = false;
	this._handleItemSelection(ev, overview, treeItem, item);
};

ZmPrefPageTreeController.prototype._savePrefsYes =
function(ev, overview, treeItem, item, prefsController, dialog) {
	dialog.popdown();

	var callback = new AjxCallback(this, this._continueTreeItemSelection, [ev, overview, treeItem, item, prefsController]);
	prefsController.save(callback, true);
};

ZmPrefPageTreeController.prototype._savePrefsNo =
function(ev, overview, treeItem, item, prefsController, dialog) {
	dialog.popdown();

	prefsController.getPrefsView().reset();
	this._continueTreeItemSelection(ev, overview, treeItem, item, prefsController);
};

ZmPrefPageTreeController.prototype._savePrefsCancel =
function(dialog) {
	dialog.popdown();
};

ZmPrefPageTreeController.prototype._continueTreeItemSelection =
function(ev, overview, treeItem, item, prefsController) {
	prefsController.getPrefsView().resetOnAccountChange();

	this._handleItemSelection(ev, overview, treeItem, item);
	prefsController.getPrefsView().switchToTab(item.pageId);
};
}

if (AjxPackage.define("ajax.dwt.widgets.DwtCalendar")) {
/*
 * ***** BEGIN LICENSE BLOCK *****
 * Zimbra Collaboration Suite Web Client
 * Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2013 Zimbra Software, LLC.
 * 
 * The contents of this file are subject to the Zimbra Public License
 * Version 1.4 ("License"); you may not use this file except in
 * compliance with the License.  You may obtain a copy of the License at
 * http://www.zimbra.com/license.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * ***** END LICENSE BLOCK *****
 */

/**
 * Creates a calendar widget
 * @constructor
 * @class
 * This class provides a calendar view.
 *
 * @author Ross Dargahi
 * @author Roland Schemers
 *
 * @param {hash}		params			a hash of parameters
 * @param {DwtComposite}      params.parent			the parent widget
 * @param {string}      params.className			the CSS class
 * @param {constant}      params.posStyle			the positioning style (see {@link Dwt})
 * @param {constant}     [params.firstDayOfWeek=DwtCalendar.SUN]		the first day of the week
 * @param {boolean}	[params.forceRollOver=true] 	if <code>true</code>, then clicking on (or setting) the widget to a 
 *												date that is not part of the current month (i.e. one of 
 *												the grey prev or next month days) will result in the 
 *												widget rolling 	the date to that month.
 * @param {array}      params.workingDays		a list of days that are work days. This array assumes that
 * 												index 0 is Sunday. Defaults to Mon-Fri being work days.
 * @param {boolean}      params.hidePrevNextMo 	a flag indicating whether widget should hide days of the 
 *												previous/next month
 * @param {boolean}      params.readOnly 		a flag indicating that this widget is read-only (should not 
 *												process events such as mouse clicks)
 * @param {boolean}      params.showWeekNumber	a flag indicating whether widget should show week number
 *        
 * @extends		DwtComposite
 */
DwtCalendar = function(params) {
	if (arguments.length == 0) { return; }
	params = Dwt.getParams(arguments, DwtCalendar.PARAMS);
	params.className = params.className || "DwtCalendar";
	DwtComposite.call(this, params);

	this._skipNotifyOnPage = false;
	this._hidePrevNextMo = params.hidePrevNextMo;
	this._readOnly = params.readOnly;
	this._showWeekNumber = params.showWeekNumber;
	this._uuid = Dwt.getNextId();
	var cn = this._origDayClassName = params.className + "Day";
	this._todayClassName = " " + params.className + "Day-today";
	this._selectedDayClassName = " " + cn + "-" + DwtCssStyle.SELECTED;
	this._hoveredDayClassName = " " + cn + "-" + DwtCssStyle.HOVER;
	this._activeDayClassName = " " + cn + "-" + DwtCssStyle.ACTIVE;
	this._hiliteClassName = " " + cn + "-hilited";
	this._greyClassName = " " + cn + "-grey";
	
	if (!this._readOnly) {
		this._installListeners();
	}

	this._selectionMode = DwtCalendar.DAY;
	
	this._init();

	this._weekDays = new Array(7);
	this._workingDays = params.workingDays || DwtCalendar._DEF_WORKING_DAYS;
    this._useISO8601WeekNo = params.useISO8601WeekNo;
	this.setFirstDayOfWeek(params.firstDayOfWeek || DwtCalendar.SUN);
	
	this._forceRollOver = (params.forceRollOver !== false);
};

DwtCalendar.PARAMS = ["parent", "className", "posStyle", "firstDayOfWeek", "forceRollOver",
					  "workingDaysArray", "hidePrevNextMo", "readOnly"];

DwtCalendar.prototype = new DwtComposite;
DwtCalendar.prototype.constructor = DwtCalendar;

/**
 * Sunday.
 */
DwtCalendar.SUN = 0;
/**
 * Monday.
 */
DwtCalendar.MON = 1;
/**
 * Tuesday.
 */
DwtCalendar.TUE = 2;
/**
 * Wednesday.
 */
DwtCalendar.WED = 3;
/**
 * Thursday.
 */
DwtCalendar.THU = 4;
/**
 * Friday.
 */
DwtCalendar.FRI = 5;
/**
 * Saturday.
 */
DwtCalendar.SAT = 6;

// Selection modes
/**
 * Defines the "day" selection mode.
 */
DwtCalendar.DAY = 1;
/**
 * Defines the "week" selection mode.
 */
DwtCalendar.WEEK = 2;
/**
 * Defines the "work week" selection mode.
 */
DwtCalendar.WORK_WEEK = 3;
/**
 * Defines the "month" selection mode.
 */
DwtCalendar.MONTH = 4;

DwtCalendar.RANGE_CHANGE = "DwtCalendar.RANGE_CHANGE";

DwtCalendar._FULL_WEEK = [1, 1, 1, 1, 1, 1, 1];
DwtCalendar._DEF_WORKING_DAYS = [0, 1, 1, 1, 1, 1, 0];
DwtCalendar._DAYS_IN_MONTH = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];

DwtCalendar._NO_MONTH = -2;
DwtCalendar._PREV_MONTH = -1;
DwtCalendar._THIS_MONTH = 0;
DwtCalendar._NEXT_MONTH = 1;

DwtCalendar._NORMAL = 1;
DwtCalendar._HOVERED = 2;
DwtCalendar._ACTIVE = 3;
DwtCalendar._SELECTED = 4;
DwtCalendar._DESELECTED = 5;

DwtCalendar.DATE_SELECTED 		= 1;
DwtCalendar.DATE_DESELECTED 	= 2;
DwtCalendar.DATE_DBL_CLICKED 	= 3;

DwtCalendar._LAST_DAY_CELL_IDX = 41;

DwtCalendar._BUTTON_CLASS = "DwtCalendarButton";
DwtCalendar._BUTTON_HOVERED_CLASS = DwtCalendar._BUTTON_CLASS + "-" + DwtCssStyle.HOVER;
DwtCalendar._BUTTON_ACTIVE_CLASS = DwtCalendar._BUTTON_CLASS + "-" + DwtCssStyle.ACTIVE;

DwtCalendar._TITLE_CLASS = "DwtCalendarTitle";
DwtCalendar._TITLE_HOVERED_CLASS = DwtCalendar._TITLE_CLASS + "-" + DwtCssStyle.HOVER;
DwtCalendar._TITLE_ACTIVE_CLASS = DwtCalendar._TITLE_CLASS + "-" + DwtCssStyle.ACTIVE;

/**
 * Returns a string representation of the object.
 * 
 * @return		{string}		a string representation of the object
 */
DwtCalendar.prototype.toString = 
function() {
	return "DwtCalendar";
};

/**
 * Adds a selection listener.
 * 
 * @param	{AjxListener}	listener		the listener
 */
DwtCalendar.prototype.addSelectionListener = 
function(listener) {
	this.addListener(DwtEvent.SELECTION, listener);
};

/**
 * Removes a selection listener.
 * 
 * @param	{AjxListener}	listener		the listener
 */
DwtCalendar.prototype.removeSelectionListener = 
function(listener) { 
	this.removeListener(DwtEvent.SELECTION, listener);
};

/**
 * Adds an action listener.
 * 
 * @param	{AjxListener}	listener		the listener
 */
DwtCalendar.prototype.addActionListener = 
function(listener) {
	this.addListener(DwtEvent.ACTION, listener);
};

/**
 * Removes an action listener.
 * 
 * @param	{AjxListener}	listener		the listener
 */
DwtCalendar.prototype.removeActionListener = 
function(listener) { 
	this.removeListener(DwtEvent.ACTION, listener);
};

/**
 * Adds a date range listener. Date range listeners are called whenever the date range of the calendar
 * changes (i.e. when it rolls over due to a programatic action via {@link #setDate} or
 * via user selection).
 *
 * @param 	{AjxListener}		listener		the listener
 */
DwtCalendar.prototype.addDateRangeListener = 
function(listener) {
	this.addListener(DwtEvent.DATE_RANGE, listener);
};

/**
 * Removes a date range listener.
 * 
 * @param 	{AjxListener}		listener		the listener
 */
DwtCalendar.prototype.removeDateRangeListener = 
function(listener) { 
	this.removeListener(DwtEvent.DATE_RANGE, listener);
};

/**
 * Sets the skip notify on page. This method notify (or not) selection when paging arrow buttons
 * are clicked.
 *
 * @param	{boolean}	skip		if <code>true</code>, do not notify selection
 */
DwtCalendar.prototype.setSkipNotifyOnPage = 
function(skip) {
	this._skipNotifyOnPage = skip;
};

/**
 * Gets the skip notify on page setting.
 * 
 * @return	{boolean}	<code>true</code>, do not notify selection
 */
DwtCalendar.prototype.getSkipNotifyOnPage = 
function() {
	return this._skipNotifyOnPage;
};

/**
 * Sets the date.
 * 
 * @param	{Date}	date	the date
 * @param	{boolean}	skipNotify		if <code>true</code>, do not notify selection
 * @param {boolean}	forceRollOver 	if <code>true</code>, then clicking on (or setting) the widget to a 
 *												date that is not part of the current month (i.e. one of 
 *												the grey prev or next month days) will result in the 
 *												widget rolling 	the date to that month.
 * @param	{boolean}	dblClick		if <code>true</code>, require a double click
 */
DwtCalendar.prototype.setDate =
function(date, skipNotify, forceRollOver, dblClick) {

	forceRollOver = (forceRollOver == null) ? this._forceRollOver : forceRollOver;
	
	// Check if the date is available for selection. Basically it is unless we are in
	// work week selection mode and <date> is not a working day
	//if (this._selectionMode == DwtCalendar.WORK_WEEK && !this._currWorkingDays[date.getDay()])
	//	return false;

	if(!date) {
		date = new Date();
	}
	var newDate = new Date(date.getTime());
	var oldDate = this._date;

	var layout = false;
	var notify = false;
	var cellId;

	if (this._date2CellId != null) {
		var idx = (newDate.getFullYear() * 10000) + (newDate.getMonth() * 100) + newDate.getDate();
		var cellId = this._date2CellId[idx];
		
		if (cellId) {
		 	if (cellId == this._selectedCellId)
		 		notify = true;

			var cell = document.getElementById(cellId);
			if (cell._dayType == DwtCalendar._THIS_MONTH)
				notify = true;
			else if (forceRollOver)
				notify = layout = true;
			else
				notify = true;
		} else {
			 notify = layout = true;
		}
	} else {
		notify = layout = true;
	}

	// update before layout, notify after layout
	if (notify) {
		if (this._date){
			// 5/13/2005 EMC -- I'm not sure why this was setting the hours to 0.
			// I think it should respect what the user passed in, and only change
			// the parts of the date that it is responsible for.
			//newDate.setHours(0,0,0,0);
			//handle daylight saving
			if(AjxDateUtil.isDayShifted(newDate)) {
				AjxDateUtil.rollToNextDay(newDate);
			}
			newDate.setHours(this._date.getHours(), this._date.getMinutes(), this._date.getSeconds(), 0);            
		}

		this._date = newDate;
		if (!layout && !this._readOnly) {
			this._setSelectedDate();
			this._setToday();
		}
	}

	if (layout) {
		this._layout();
	}

	if (notify && !skipNotify) {
		var type = dblClick ? DwtCalendar.DATE_DBL_CLICKED : DwtCalendar.DATE_SELECTED;
		this._notifyListeners(DwtEvent.SELECTION, type, this._date);
	}
	
	return true;
};

/**
 * Checks if the cell is selected.
 * 
 * @param	{string}	cellId			the cell id	
 * @return	{boolean}	<code>true</code> if the cell is the selected day
 */
DwtCalendar.prototype.isSelected =
function(cellId) {
	// if cellId is the selected day, then return true, else if we are NOT in
	// day selection mode (i.e. week/work week) then compute the row and index
	// of cellId and look it up in the week array to see if it is a selectable day
	if (cellId == this._selectedDayElId) {
		return true;
	} else if (this._selectionMode != DwtCalendar.DAY) {
		// If the cell is in the same row as the currently selected cell and it
		// is a selectable day (i.e. a working day in the case of work week),
		// then say it is selected
		var cellIdx = this._getDayCellIndex(cellId);
		if (Math.floor(cellIdx / 7) == Math.floor(this._getDayCellIndex(this._selectedDayElId) / 7)
			&& this._currWorkingDays[cellIdx % 7])
			return true;
	}
	return false;
};

/**
 * Gets the force roll over setting. Force roll over is occurs when a date that
 * is not part of the current month (i.e. one of the grey prev or next month
 * days) will result in the widget rolling 	the date to that month.
 * 
 * @return	{boolean}	<code>true</code> if force roll over is set
 */
DwtCalendar.prototype.getForceRollOver =
function() {
	return this._forceRollOver;
};

/**
 * Sets the force roll over setting. Force roll over is occurs when a date that
 * is not part of the current month (i.e. one of the grey prev or next month
 * days) will result in the widget rolling 	the date to that month.
 * 
 * @param	{boolean}	force		if <code>true</code>, force roll over
 */
DwtCalendar.prototype.setForceRollOver =
function(force) {
	if (force == null) { return; }
	
	if (this._forceRollOver != force) {
		this._forceRollOver = force;
		this._layout();
	}
};

/**
 * Gets the selection mode.
 * 
 * @return	{constant}		the selection mode
 */
DwtCalendar.prototype.getSelectionMode =
function() {
	return this._selectionMode;
};

/**
 * Sets the selection mode.
 * 
 * @return	{constant}		selectionMode		the selection mode
 */
DwtCalendar.prototype.setSelectionMode =
function(selectionMode) {
	if (this._selectionMode == selectionMode) { return; }

	this._selectionMode = selectionMode;
	if (selectionMode == DwtCalendar.WEEK) {
		this._currWorkingDays = DwtCalendar._FULL_WEEK;
	} else if (selectionMode == DwtCalendar.WORK_WEEK) {
		this._currWorkingDays = this._workingDays;
	}

	this._layout();
};

/**
 * Sets the working week.
 * 
 * @param	{array}	workingDaysArray		an array of days
 */
DwtCalendar.prototype.setWorkingWeek =
function(workingDaysArray) {
	// TODO Should really create a copy of workingDaysArray
	this._workingDays = this._currWorkingDays = workingDaysArray;
	
	if (this._selectionMode == DwtCalendar.WORK_WEEK) {
		DBG.println("FOO!!!");
		this._layout();
	}
};

/**
 * Enables/disables the highlight (i.e. "bolding") on the dates in <code>&lt;dates&gt;</code>.
 *
 * @param {array}	dates	an array of {@link Date} objects for which to enable/disable highlighting
 * @param {boolean}	enable 	if <code>true</code>, enable highlighting
 * @param {boolean}	clear 	if <code>true</code>, clear current highlighting
 */
DwtCalendar.prototype.setHilite =
function(dates, enable, clear) {
	if (this._date2CellId == null) { return; }

	var cell;
	var aDate;
	if (clear) {
		for (aDate in this._date2CellId) {
			cell = document.getElementById(this._date2CellId[aDate]);
			if (cell._isHilited) {
				cell._isHilited = false;
				this._setClassName(cell, DwtCalendar._NORMAL);
			}	
		}
	}

	var cellId;
	for (var i in dates) {
		aDate = dates[i];
		cellId = this._date2CellId[aDate.getFullYear() * 10000 + aDate.getMonth() * 100 + aDate.getDate()];

		if (cellId) {
			cell = document.getElementById(cellId);
			if (cell._isHilited != enable) {
				cell._isHilited = enable;
				this._setClassName(cell, DwtCalendar._NORMAL);
			}
		}
	}
};

/**
 * Gets the date.
 * 
 * @return	{Date}	the date
 */
DwtCalendar.prototype.getDate =
function() {
	return this._date;
};

/**
 * Sets the first date of week.
 * 
 * @param	{constant}		firstDayOfWeek		the first day of week
 */
DwtCalendar.prototype.setFirstDayOfWeek =
function(firstDayOfWeek) {
	for (var i = 0; i < 7; i++) {
		this._weekDays[i] = (i < firstDayOfWeek)
			? (6 - (firstDayOfWeek -i - 1))
			: (i - firstDayOfWeek);

		var dowCell = document.getElementById(this._getDOWCellId(i));
		dowCell.innerHTML = AjxDateUtil.WEEKDAY_SHORT[(firstDayOfWeek + i) % 7];
	}
    this._firstDayOfWeek = firstDayOfWeek
	this._layout();
};

/**
 * Gets the date range.
 * 
 * @return	{Object}		the range (<code>range.start</code> and <code>range.end</code>)
 */
DwtCalendar.prototype.getDateRange =
function () {
	return this._range;
};

DwtCalendar.prototype._getDayCellId =
function(cellId) {
	return ("c:" + cellId + ":" + this._uuid);
};

DwtCalendar.prototype._getDayCellIndex =
function(cellId) {
	return cellId.substring(2, cellId.indexOf(":", 3));
};

DwtCalendar.prototype._getDOWCellId =
function(cellId) {
	return ("w:" + cellId + ":" + this._uuid);
};

DwtCalendar.prototype._getWeekNumberCellId =
function(cellId) {
	return ("k:" + cellId + ":" + this._uuid);
};

DwtCalendar.prototype._getDaysInMonth =
function(mo, yr) {
	/* If we are not dealing with Feb, then simple lookup
	 * Leap year rules
	 *  1. Every year divisible by 4 is a leap year.
	 *  2. But every year divisible by 100 is NOT a leap year
	 *  3. Unless the year is also divisible by 400, then it is still a leap year.*/
	if (mo != 1) {
		return DwtCalendar._DAYS_IN_MONTH[mo];
	}

	if (yr % 4 != 0 || (yr % 100 == 0 && yr % 400 != 0)) {
		return 28;
	}

	return 29;
};

DwtCalendar.prototype._installListeners =
function() {
	this._setMouseEventHdlrs();
	this.addListener(DwtEvent.ONMOUSEOVER, new AjxListener(this, this._mouseOverListener));
	this.addListener(DwtEvent.ONMOUSEOUT, new AjxListener(this, this._mouseOutListener));
	this.addListener(DwtEvent.ONMOUSEDOWN, new AjxListener(this, this._mouseDownListener));
	this.addListener(DwtEvent.ONMOUSEUP, new AjxListener(this, this._mouseUpListener));
	this.addListener(DwtEvent.ONDBLCLICK, new AjxListener(this, this._doubleClickListener));
};

DwtCalendar.prototype._notifyListeners =
function(eventType, type, detail, ev) {
	if (!this.isListenerRegistered(eventType)) { return; }

	var selEv = DwtShell.selectionEvent;
	if (ev) {
		DwtUiEvent.copy(selEv, ev);
	} else {
		selEv.reset();
	}
	selEv.item = this;
	selEv.detail = detail;
	selEv.type = type;
	this.notifyListeners(eventType, selEv);
};

DwtCalendar.prototype._layout =
function() {
	if (this._date == null) { this._date = new Date(); }

	if (!this._calWidgetInited) {
		this._init();
	}

	var date = new Date(this._date.getTime());
	date.setDate(1);
	var year = date.getFullYear();
	var month  = date.getMonth();
	var firstDay = date.getDay();
	var daysInMonth = this._getDaysInMonth(month, year);
	var day = 1;
	var nextMoDay = 1;

	this._date2CellId = new Object();
	this._selectedDayElId = null;

	// Figure out how many days from the previous month we have to fill in
	// (see comment below)
	var lastMoDay, lastMoYear, lastMoMonth, nextMoMonth, nextMoYear;
	if (!this._hidePrevNextMo) {
		if (month != 0) {
			lastMoDay = this._getDaysInMonth(month - 1, year) - this._weekDays[firstDay] + 1;
			lastMoYear = year;
			lastMoMonth = month - 1;
			if (month != 11) {
				nextMoMonth = month + 1;
				nextMoYear = year;
			} else {
				nextMoMonth = 0;
				nextMoYear = year + 1;
			}
		} else {
			lastMoDay = this._getDaysInMonth(11, year - 1) - this._weekDays[firstDay] + 1;
			lastMoYear = year - 1;
			lastMoMonth = 11;
			nextMoMonth = 1;
			nextMoYear = year;
		}
	}

	for (var i = 0; i < 6; i++) {
		for (var j = 0; j < 7; j++) {
			var dayCell = document.getElementById(this._getDayCellId(i * 7 + j));

			if (dayCell._isHilited == null) {
				dayCell._isHilited = false;
			}

			if (day <= daysInMonth) {
				/* The following if statement deals with the first day of this month not being
				 * the first day of the week. In this case we must fill the preceding days with
				 * the final days of the previous month */
				if (i != 0 || j >= this._weekDays[firstDay]) {
					this._date2CellId[(year * 10000) + (month * 100) + day] = dayCell.id;
					dayCell._day = day;
					dayCell._month = month;
					dayCell._year = year;
					dayCell.innerHTML = day++;
					dayCell._dayType = DwtCalendar._THIS_MONTH;
					if (this._readOnly) {
						dayCell.style.fontFamily = "Arial";
						dayCell.style.fontSize = "10px";
					}
				} else {
					if (this._hidePrevNextMo) {
						dayCell.innerHTML = "";
					} else {
						this._date2CellId[(lastMoYear * 10000) + (lastMoMonth * 100) + lastMoDay] = dayCell.id;
						dayCell._day = lastMoDay;
						dayCell._month = lastMoMonth;
						dayCell._year = lastMoYear;
						dayCell.innerHTML = lastMoDay++;
						dayCell._dayType = DwtCalendar._PREV_MONTH;
					}
				}
			} else if (!this._hidePrevNextMo) {
				// Fill any remaining slots with days from next month
				this._date2CellId[(nextMoYear * 10000) + (nextMoMonth * 100) + nextMoDay] = dayCell.id;
				dayCell._day = nextMoDay;
				dayCell._month = nextMoMonth;
				dayCell._year = nextMoYear;
				dayCell.innerHTML = nextMoDay++;
				dayCell._dayType = DwtCalendar._NEXT_MONTH;
			}
			this._setClassName(dayCell, DwtCalendar._NORMAL);
		}

		if (this._showWeekNumber) {
			var kwCellId = this._getWeekNumberCellId('kw' + i * 7);
			var kwCell = document.getElementById(kwCellId);
			if (kwCell) {
				var firstDayCell = document.getElementById(this._getDayCellId(i * 7));
				kwCell.innerHTML = AjxDateUtil.getWeekNumber(new Date(firstDayCell._year, firstDayCell._month, firstDayCell._day), this._firstDayOfWeek, null, this._useISO8601WeekNo);
			}
		}
	}

	this._setTitle(month, year);

	// Compute the currently selected day
	if (!this._readOnly) {
		this._setSelectedDate();
		this._setToday();
	}
	
	this._setRange();
};

DwtCalendar.prototype._setRange =
function() {
	var cell = document.getElementById(this._getDayCellId(0));
	var start = new Date(cell._year, cell._month, cell._day, 0, 0, 0, 0);

	cell = document.getElementById(this._getDayCellId(DwtCalendar._LAST_DAY_CELL_IDX));
	
	var daysInMo = this._getDaysInMonth(cell._month, cell._year);
	var end;
	if (cell._day < daysInMo) {
		end = new Date(cell._year, cell._month, cell._day + 1, 0, 0, 0, 0);
	} else if (cell._month < 11) {
		end = new Date(cell._year, cell._month + 1, 1, 0, 0, 0, 0);
	} else {
		end = new Date(cell._year + 1, 0, 1, 0, 0, 0, 0);
	}

	if (this._range == null) {
		this._range = {};
	} else if (this._range.start.getTime() == start.getTime() && this._range.end.getTime() == end.getTime()) {
		return false;
	}

	this._range.start = start;
	this._range.end = end;

	// Notify any listeners
	if (!this.isListenerRegistered(DwtEvent.DATE_RANGE)) { return; }

	if (!this._dateRangeEvent) {
		this._dateRangeEvent = new DwtDateRangeEvent(true);
	}

	this._dateRangeEvent.item = this;
	this._dateRangeEvent.start = start;
	this._dateRangeEvent.end = end;
	this.notifyListeners(DwtEvent.DATE_RANGE, this._dateRangeEvent);
};

DwtCalendar.prototype._setToday =
function() {
	var cell;
	var today = new Date();
	var todayDay = today.getDate();

	if (!this._todayDay || this._todayDay != todayDay) {
		if (this._todayCellId != null) {
			cell = document.getElementById(this._todayCellId);
			cell._isToday = false;
			this._setClassName(cell, DwtCalendar._NORMAL);
		}

		this._todayCellId = this._date2CellId[(today.getFullYear() * 10000) + (today.getMonth() * 100) + todayDay];
		if (this._todayCellId != null) {
			cell = document.getElementById(this._todayCellId);
			cell._isToday = true;
			this._setClassName(cell, DwtCalendar._NORMAL);
		}
	}
};

DwtCalendar.prototype._setSelectedDate =
function() {
	var day = this._date.getDate();
	var month = this._date.getMonth();
	var year = this._date.getFullYear();
	var cell;

	if (this._selectedDayElId) {
		cell = document.getElementById(this._selectedDayElId);
		this._setClassName(cell, DwtCalendar._DESELECTED);
	}

	var cellId = this._date2CellId[(year * 10000) + (month * 100) + day];
	cell = document.getElementById(cellId);
	this._selectedDayElId = cellId;
	this._setClassName(cell, DwtCalendar._SELECTED);
};

DwtCalendar.prototype._setCellClassName = 
function(cell, className, mode) {
	if (cell._dayType != DwtCalendar._THIS_MONTH) {
		className += this._greyClassName;
	}

	if (this._selectionMode == DwtCalendar.DAY &&
		cell.id == this._selectedDayElId &&
		mode != DwtCalendar._DESELECTED)
	{
		className += this._selectedDayClassName;
	}
	else if (this._selectionMode != DwtCalendar.DAY &&
			 mode != DwtCalendar._DESELECTED &&
			 this._selectedDayElId != null)
	{
		var idx = this._getDayCellIndex(cell.id);
		if (Math.floor(this._getDayCellIndex(this._selectedDayElId) / 7) == Math.floor(idx / 7) &&
			this._currWorkingDays[idx % 7])
		{
			className += this._selectedDayClassName;
		}
	}

	if (cell._isHilited) {
		className += this._hiliteClassName;
	}

	if (cell._isToday) {
		className += this._todayClassName;
	}

	return className;
};

DwtCalendar.prototype._setClassName = 
function(cell, mode) {
	var className = "";
	
	if (mode == DwtCalendar._NORMAL) {
		className = this._origDayClassName;
	} else if (mode == DwtCalendar._HOVERED) {
		className = this._hoveredDayClassName;
	} else if (mode == DwtCalendar._ACTIVE) {
		className = this._activeDayClassName;
	} else if (mode == DwtCalendar._DESELECTED && this._selectionMode == DwtCalendar.DAY) {
		className = this._origDayClassName;
	} else if (this._selectionMode != DwtCalendar.DAY &&
			(mode == DwtCalendar._SELECTED || mode == DwtCalendar._DESELECTED))
	{
		// If we are not in day mode, then we need to highlite multiple cells
		// e.g. the whole week if we are in week mode
		var firstCellIdx = Math.floor(this._getDayCellIndex(this._selectedDayElId) / 7) * 7;

		for (var i = 0; i < 7; i++) {
			className = this._origDayClassName;
			var aCell = document.getElementById(this._getDayCellId(firstCellIdx++));
			aCell.className = this._setCellClassName(aCell, className, mode);
		}
		return;
	}

	cell.className = this._setCellClassName(cell, className, mode);
};

DwtCalendar.prototype._setTitle =
function(month, year) {
	var cell = document.getElementById(this._monthCell);
	var formatter = DwtCalendar.getMonthFormatter();
	var date = new Date(year, month);
	cell.innerHTML = formatter.format(date);
};

DwtCalendar.prototype._init =
function() {
	var html = new Array(100);
	var idx = 0;
	this._monthCell = "t:" + this._uuid;

	// Construct the header row with the prev/next year and prev/next month
	// icons as well as the month/year title cell
	html[idx++] =	"<table width=100% cellspacing='0' cellpadding='0' style='border-collapse:collapse;'>";
	html[idx++] =		"<tr><td class=DwtCalendarTitlebar>";
	html[idx++] =			"<table width='100%' cellspacing='0' cellpadding='0'>";
	html[idx++] =				"<tr>";
	html[idx++] =					"<td class='";
	html[idx++] =						DwtCalendar._BUTTON_CLASS;
	html[idx++] =						"' id='b:py:";
	html[idx++] =						this._uuid;
	html[idx++] =						"'>";
	html[idx++] =						AjxImg.getImageHtml("FastRevArrowSmall", null, ["id='b:py:img:", this._uuid, "'"].join(""));
	html[idx++] =					"</td>";
	html[idx++] =					"<td class='";
	html[idx++] =						DwtCalendar._BUTTON_CLASS;
	html[idx++] =						"' id='b:pm:";
	html[idx++] =						this._uuid;
	html[idx++] =						"'>";
	html[idx++] =						AjxImg.getImageHtml("RevArrowSmall", null, ["id='b:pm:img:", this._uuid, "'"].join(""));
	html[idx++] =					"</td>";
	html[idx++] =					"<td align='center' class='DwtCalendarTitleCell' nowrap'><span class='";
	html[idx++] =						DwtCalendar._TITLE_CLASS;
	html[idx++] = 						"' id='";
	html[idx++] =						this._monthCell;
	html[idx++] =					"'>&nbsp;</span></td>";
	html[idx++] =					"<td class='";
	html[idx++] =						DwtCalendar._BUTTON_CLASS;
	html[idx++] =						"' id='b:nm:";
	html[idx++] =						this._uuid;
	html[idx++] =						"'>";
	html[idx++] =						AjxImg.getImageHtml("FwdArrowSmall", null, ["id='b:nm:img:", this._uuid, "'"].join(""));
	html[idx++] =					"</td>";
	html[idx++] =					"<td class='";
	html[idx++] =						DwtCalendar._BUTTON_CLASS;
	html[idx++] =						"' id='b:ny:";
	html[idx++] =						this._uuid;
	html[idx++] =						"'>";
	html[idx++] =						AjxImg.getImageHtml("FastFwdArrowSmall", null, ["id='b:ny:img:", this._uuid, "'"].join(""));
	html[idx++] =					"</td>";
	html[idx++] =				"</tr>";
	html[idx++] =			"</table>";
	html[idx++] =		"</td></tr>";
	html[idx++] =	"<tr><td>";
	html[idx++] = "<table cellspacing='0' cellpadding='1' width='100%'>";
	html[idx++] = "<tr>";

	if (this._showWeekNumber) {
		html[idx++] = "<td class='DwtCalendarWeekNoTitle' width='14%' id='";
		html[idx++] = this._getWeekNumberCellId('kw');
		html[idx++] = "'>";
		html[idx++] = AjxMsg.calendarWeekTitle;
		html[idx++] = "</td>";
	}

	for (var i = 0; i < 7; i++) {
		html[idx++] = "<td class='DwtCalendarDow' width='";
		html[idx++] = (i < 5 ? "14%" : "15%");
		html[idx++] = "' id='";
		html[idx++] = this._getDOWCellId(i);
		html[idx++] = "'>&nbsp;</td>";
	}
	html[idx++] = "</tr>";

	for (var i = 0; i < 6; i++) {
		// bug fix #3355 - linux fix and bug fix #25564 for Mac
		html[idx++] = (AjxEnv.isLinux  || AjxEnv.isMac) ? "<tr style='line-height:12px'>" : "<tr>";
		if (this._showWeekNumber) {
			html[idx++] = "<td class='DwtCalendarWeekNo' id='" + this._getWeekNumberCellId('kw' + i * 7) + "'>&nbsp;</td>";
		}
		for (var j = 0; j < 7; j++) {
			html[idx++] = "<td id='";
			html[idx++] = this._getDayCellId(i * 7 + j);
			html[idx++] = "'>&nbsp;</td>";
		}
		html[idx++] ="</tr>";
	}

	html[idx++] = "</td></tr></table></table>";

	this.getHtmlElement().innerHTML = html.join("");
	if (!this._readOnly) {
		document.getElementById("b:py:img:" + this._uuid)._origClassName = AjxImg.getClassForImage("FastRevArrowSmall");
		document.getElementById("b:pm:img:" + this._uuid)._origClassName = AjxImg.getClassForImage("RevArrowSmall");
		document.getElementById("b:nm:img:" + this._uuid)._origClassName = AjxImg.getClassForImage("FwdArrowSmall");
		document.getElementById("b:ny:img:" + this._uuid)._origClassName = AjxImg.getClassForImage("FastFwdArrowSmall");
	}

	this._calWidgetInited = true;
};

/**
 * Sets the mouse over day callback.
 * 
 * @param	{AjxCallback}		callback		the callback
 */
DwtCalendar.prototype.setMouseOverDayCallback =
function(callback) {
	this._mouseOverDayCB = callback;
};

/**
 * Sets the mouse out day callback.
 * 
 * @param	{AjxCallback}		callback		the callback
 */
DwtCalendar.prototype.setMouseOutDayCallback =
function(callback) {
	this._mouseOutDayCB = callback;
};

/**
 * Gets the date value for the last cell that the most recent
 * Drag-and-drop operation occurred over. Typically it will be called by a DwtDropTarget
 * listener when an item is dropped onto the mini calendar
 * 
 * @return	{Date}		the date or <code>null</code> for none
 */
DwtCalendar.prototype.getDndDate =
function() {
	var dayCell = this._lastDndCell;
	if (dayCell) {
		return new Date(dayCell._year, dayCell._month, dayCell._day);
	}

	return null;
};

// Temp date used for callback in mouseOverListener
DwtCalendar._tmpDate = new Date();
DwtCalendar._tmpDate.setHours(0, 0, 0, 0);

DwtCalendar.prototype._mouseOverListener = 
function(ev) {
	var target = ev.target;
	if (target.id.charAt(0) == 'c') {
		this._setClassName(target, DwtCalendar._HOVERED);
		// If a mouse over callback has been registered, then call it to give it
		// chance do work like setting the tooltip content
		if (this._mouseOverDayCB) {
			DwtCalendar._tmpDate.setFullYear(target._year, target._month, target._day);
			this._mouseOverDayCB.run(this, DwtCalendar._tmpDate);
		}
	} else if (target.id.charAt(0) == 't') {
		// Dont activate title for now
		return;
	} else if (target.id.charAt(0) == 'b') {
		var img;
		if (target.firstChild == null) {
			img = target;
			AjxImg.getParentElement(target).className = DwtCalendar._BUTTON_HOVERED_CLASS;
		} else {
			target.className = DwtCalendar._BUTTON_HOVERED_CLASS;
			img = AjxImg.getImageElement(target);
		}
		img.className = img._origClassName;
	}

	ev._stopPropagation = true;
};

DwtCalendar.prototype._mouseOutListener = 
function(ev) {
	this.setToolTipContent(null);
	var target = ev.target;
	if (target.id.charAt(0) == 'c') {
		this._setClassName(target, DwtCalendar._NORMAL);
		if (this._mouseOutDayCB) {
			this._mouseOutDayCB.run(this);
		}
	} else if (target.id.charAt(0) == 'b') {
		var img;
		target.className = DwtCalendar._BUTTON_CLASS;
		if (target.firstChild == null) {
			img = target;
			AjxImg.getParentElement(target).className = DwtCalendar._BUTTON_CLASS;
		} else {
			target.className = DwtCalendar._BUTTON_CLASS;
			img = AjxImg.getImageElement(target);
		}
		img.className = img._origClassName;
	}
};

DwtCalendar.prototype._mouseDownListener = 
function(ev) {
	if (ev.button == DwtMouseEvent.LEFT) {
		var target = ev.target;
		if (target.id.charAt(0) == 'c') {
			this._setClassName(target, DwtCalendar._ACTIVE);
		} else if (target.id.charAt(0) == 't') {
			target.className = DwtCalendar._TITLE_ACTIVE_CLASS;
		} else if (target.id.charAt(0) == 'b') {
			var img;
			if (target.firstChild == null) {
				img = target;
				AjxImg.getParentElement(target).className = DwtCalendar._BUTTON_ACTIVE_CLASS;
			} else {
				target.className = DwtCalendar._BUTTON_ACTIVE_CLASS;
				img = AjxImg.getImageElement(target);
			}
			img.className = img._origClassName;
		} else if (target.id.charAt(0) == 'w') {
		}
	}
};

DwtCalendar.prototype._mouseUpListener = 
function(ev) {
	var target = ev.target;
	if (ev.button == DwtMouseEvent.LEFT) {
		if (target.id.charAt(0) == 'c') {
			// If our parent is a menu then we need to have it close
			if (this.parent instanceof DwtMenu)
				DwtMenu.closeActiveMenu();

            var sDate = new Date(target._year, target._month, target._day);
            if(sDate.getDate() != target._day) {
                sDate.setDate(target._day);                 
            }
			if (this.setDate(sDate)) { return; }

			this._setClassName(target, DwtCalendar._HOVERED);
		} else if (target.id.charAt(0) == 'b') {
			var img;
			if (target.firstChild == null) {
				img = target;
				AjxImg.getParentElement(target).className = DwtCalendar._BUTTON_HOVERED_CLASS;
			} else {
				target.className = DwtCalendar._BUTTON_HOVERED_CLASS;
				img = AjxImg.getImageElement(target);
			}
			img.className = img._origClassName;
			
			if (img.id.indexOf("py") != -1) {
				this._prevYear();
			} else if (img.id.indexOf("pm") != -1) {
				this._prevMonth();
			} else if (img.id.indexOf("nm") != -1) {
				this._nextMonth();
			} else {
				this._nextYear();
			}
		} else if (target.id.charAt(0) == 't') {
			// TODO POPUP MENU
			target.className = DwtCalendar._TITLE_HOVERED_CLASS;
			this.setDate(new Date(), this._skipNotifyOnPage);
			// If our parent is a menu then we need to have it close
			if (this.parent instanceof DwtMenu) {
				DwtMenu.closeActiveMenu();
			}
		}
	} else if (ev.button == DwtMouseEvent.RIGHT && target.id.charAt(0) == 'c') {
		this._notifyListeners(DwtEvent.ACTION, 0, new Date(target._year, target._month, target._day), ev);
	}
};

DwtCalendar.prototype._doubleClickListener =
function(ev) {
	var target = ev.target;
	if (this._selectionEvent) {
		this._selectionEvent.type = DwtCalendar.DATE_DBL_CLICKED;
	}
	if (target.id.charAt(0) == 'c') {
		// If our parent is a menu then we need to have it close
		if (this.parent instanceof DwtMenu) {
			DwtMenu.closeActiveMenu();
		}
		this.setDate(new Date(target._year, target._month, target._day), false, false, true)
	}
};

DwtCalendar.prototype._prevMonth = 
function(ev) {
	var d = new Date(this._date.getTime());
	this.setDate(AjxDateUtil.roll(d, AjxDateUtil.MONTH, -1), this._skipNotifyOnPage);
};

DwtCalendar.prototype._nextMonth = 
function(ev) {
	var d = new Date(this._date.getTime());
	this.setDate(AjxDateUtil.roll(d, AjxDateUtil.MONTH, 1), this._skipNotifyOnPage);
};

DwtCalendar.prototype._prevYear = 
function(ev) {
	var d = new Date(this._date.getTime());
	this.setDate(AjxDateUtil.roll(d, AjxDateUtil.YEAR, -1), this._skipNotifyOnPage);
};

DwtCalendar.prototype._nextYear = 
function(ev) {
	var d = new Date(this._date.getTime());
	this.setDate(AjxDateUtil.roll(d, AjxDateUtil.YEAR, 1), this._skipNotifyOnPage);
};

/**
 * Gets the date formatter.
 * 
 * @return	{AjxDateFormat}		the date formatter
 * 
 * @private
 */
DwtCalendar.getDateFormatter =
function() {
	if (!DwtCalendar._dateFormatter) {
		DwtCalendar._dateFormatter = new AjxDateFormat(AjxMsg.formatCalDate);
	}
	return DwtCalendar._dateFormatter;
};

/**
 * Gets the date long formatter.
 * 
 * @return	{AjxDateFormat}		the date formatter
 * 
 * @private
 */
DwtCalendar.getDateLongFormatter =
function() {
	if (!DwtCalendar._dateLongFormatter) {
		DwtCalendar._dateLongFormatter = new AjxDateFormat(AjxMsg.formatCalDateLong);
	}
	return DwtCalendar._dateLongFormatter;
};

/**
 * Gets the date full formatter.
 * 
 * @return	{AjxDateFormat}		the date formatter
 * 
 * @private
 */
DwtCalendar.getDateFullFormatter =
function() {
	if (!DwtCalendar._dateFullFormatter) {
		DwtCalendar._dateFullFormatter = new AjxDateFormat(AjxMsg.formatCalDateFull);
	}
	return DwtCalendar._dateFullFormatter;
};

/**
 * Gets the hour formatter.
 * 
 * @return	{AjxDateFormat}		the date formatter
 * 
 * @private
 */
DwtCalendar.getHourFormatter =
function() {
	if (!DwtCalendar._hourFormatter) {
		DwtCalendar._hourFormatter = new AjxMessageFormat(AjxMsg.formatCalHour);
	}
	return DwtCalendar._hourFormatter;
};

/**
 * Gets the day formatter.
 * 
 * @return	{AjxDateFormat}		the date formatter
 * 
 * @private
 */
DwtCalendar.getDayFormatter =
function() {
	if (!DwtCalendar._dayFormatter) {
		DwtCalendar._dayFormatter = new AjxDateFormat(AjxMsg.formatCalDay);
	}
	return DwtCalendar._dayFormatter;
};

/**
 * Gets the month formatter.
 * 
 * @return	{AjxDateFormat}		the date formatter
 * 
 * @private
 */
DwtCalendar.getMonthFormatter =
function() {
	if (!DwtCalendar._monthFormatter) {
		DwtCalendar._monthFormatter = new AjxDateFormat(AjxMsg.formatCalMonth);
	}
	return DwtCalendar._monthFormatter;
};

/**
 * Gets the short month formatter.
 * 
 * @return	{AjxDateFormat}		the date formatter
 * 
 * @private
 */
DwtCalendar.getShortMonthFormatter =
function() {
	if (!DwtCalendar._shortMonthFormatter) {
		DwtCalendar._shortMonthFormatter = new AjxDateFormat(AjxMsg.formatShortCalMonth);
	}
	return DwtCalendar._shortMonthFormatter;
};

DwtCalendar.prototype._dragEnter =
function(ev) {
};

DwtCalendar.prototype._dragHover =
function(ev) {
};

DwtCalendar.prototype._dragOver =
function(ev) {
	var target = ev.target;
	if (target.id.charAt(0) == 'c') {
		this._setClassName(target, DwtCalendar._HOVERED);
		this._lastDndCell = target;
	} else {
		this._lastDndCell = null;
	}
};

DwtCalendar.prototype._dragLeave =
function(ev) {
};
}
}
if (AjxPackage.define("prefs.Options")) {
AjxTemplate.register("prefs.Options#OptionsOuter", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table id='' class='fullSize' cellspacing=0 cellpadding=0><tr><td><ul id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_outer_tabs\" class='ZTabList'><li id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_outer_tabs_general' class='ZTab' onclick='controller.showPage(\"general\")'>General</li><li id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_outer_tabs_mail' class='ZTab'>Mail</li><li id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_outer_tabs_identity' class='ZTab' onclick='controller.showPage(\"identity\")'>Mail Identities</li><li id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_outer_tabs_pop' class='ZTab' onclick='controller.showPage(\"pop\")'>Pop Accounts</li><li id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_outer_tabs_filters' class='ZTab'>Mail Filters</li><li id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_outer_tabs_addressBook' class='ZTab'>Address Book</li><li id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_outer_tabs_calendar' class='ZTab'>Calendar</li><li id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_outer_tabs_shortcuts' class='ZTab'>Shortcuts</li></ul></td></tr><tr><td colspan=2 style='padding:0px'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_page_title' class='ZmPanelHead'></div></td></tr><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_page_container' style='height:100%' valign=top ><!--\t\t<div class='ZScrollContainerOuter'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_page_container' class='ZScrollContainerInner'></div></div>\n";
	buffer[_i++] = "--></td><tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Options#OptionsOuter"
}, false);
AjxTemplate.register("prefs.Options", AjxTemplate.getTemplate("prefs.Options#OptionsOuter"), AjxTemplate.getParams("prefs.Options#OptionsOuter"));

AjxTemplate.register("prefs.Options#PopForm", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<!--<div class='ZScrollContainerInner'>\t--><table class=\"ZFormTable ZFixedTable\" cellspacing=0 cellpadding=2 border=0><colgroup><col width=30%><col width=70%></colgroup><tr><td><div class='ZSmallSpacer'></div></td></tr><tr><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.popAccountNameLabel ;
	buffer[_i++] = "</td><td><input id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_name\" type=text class='ZFieldSizeMedium'></td></tr><tr><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.popAccountFolderLabel ;
	buffer[_i++] = "</td><td><button id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_location\" type=text  class='DwtButtton' style='width:200px'>Inbox</button></td></tr><tr><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.popAccountDownloadLabel ;
	buffer[_i++] = "</td><td><select id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_download\" class='ZFieldSizeMedium'><option>";
	buffer[_i++] =  ZmMsg.popAccountDownloadLeave ;
	buffer[_i++] = "</option><option>";
	buffer[_i++] =  ZmMsg.popAccountDownloadRemove ;
	buffer[_i++] = "</option></select></td></tr><tr><td colspan=2><div class=ZLineDivider><span class=ZLineDividerTitle>";
	buffer[_i++] =  ZmMsg.accountSettings ;
	buffer[_i++] = "</span></div></td></tr><tr><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.popAccountServerLabel ;
	buffer[_i++] = "</td><td><input id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_server\" type=text class='ZFieldSizeHusky'></td></tr><tr><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.usernameLabel ;
	buffer[_i++] = "</td><td><input id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_username\" type=text class='ZFieldSizeHusky'></td></tr><tr><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.passwordLabel ;
	buffer[_i++] = "</td><td><table cellspacing=0 cellpadding=0><tr><td><input id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_password\" type='password' class='ZFieldSizeSmall'></td><td>&nbsp;<input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_show_password' type='checkbox' class='ZCheckbox'></td><td><label for='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_show_password' class='ZCheckboxLabelInline'>";
	buffer[_i++] =  ZmMsg.showPassword ;
	buffer[_i++] = "</label></td></tr></table></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_ssl_row'><td class='ZCheckboxCell'><input id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_ssl\" type=checkbox></td><td><label for='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_ssl' class='ZCheckboxLabel'>";
	buffer[_i++] =  ZmMsg.popAccountUseSSL ;
	buffer[_i++] = "</label></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_port_row'><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.portLabel ;
	buffer[_i++] = "</td><td><table cellspacing=0 cellpadding=0><tr><td><input id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_port\" type=text style='width:6em'></td><td>&nbsp;<span class='ZHelpLabel' id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_port_label\">";
	buffer[_i++] =  ZmMsg.defaultPort ;
	buffer[_i++] = "</span></td></tr></table></td></tr><tr><td><div class='ZSmallSpacer'></div></td></tr><tr><td></td><td><div class='ZFieldSizeHusky'><button id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_testButton'>";
	buffer[_i++] =  ZmMsg.popAccountTest ;
	buffer[_i++] = "</button></div></td></tr><tr><td colspan=2><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_identity_title_row' class=ZLineDivider><span class=ZLineDividerTitle>";
	buffer[_i++] =  ZmMsg.identitiesTab ;
	buffer[_i++] = "</span></div></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_identity_help_row'><td colspan=2><div class='ZHelpLabelIndent'>";
	buffer[_i++] =  ZmMsg.popAccountIdentityHelp ;
	buffer[_i++] = "<div class='ZSmallSpacer'></div></div></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_identity_create_row'><td class='ZCheckboxLabelCell' colspan=2 style='padding-left:50px;'><table><tr><td><input id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_create_identity\" type=checkbox onchange='pageController.toggleIdentityFields(this.checked)'></td><td width=100%><label for='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_create_identity' class='ZCheckboxLabelInline'>";
	buffer[_i++] =  ZmMsg.popAccountCreateNewIdentity ;
	buffer[_i++] = "</label></td></tr></table></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_identity_spacer_row'><td><div class='ZSmallSpacer'></div></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_identity_name_row'><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.identityNameLabel ;
	buffer[_i++] = "</td><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_identity_name\" class='ZLabelCellLeft'>New Account 1</td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_identity_email_row'><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.emailAddrLabel ;
	buffer[_i++] = "</td><td><input id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_email\" type=text  class='ZFieldSizeHusky'></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_identity_use_address_row'><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.linkToNewIdentity ;
	buffer[_i++] = "</td><td><table cellspacing=0 cellpadding=0><tr><td><input id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_identity_use_address\" type=checkbox></td><td><label for=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_identity_use_address\" class='ZCheckboxLabel'>";
	buffer[_i++] =  ZmMsg.whenReplyingToAddress ;
	buffer[_i++] = "</label></td></tr></table></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_identity_use_folder_row'><td></td><td><table cellspacing=0 cellpadding=0 border=0><tr><td><input id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_identity_use_folder\" type=checkbox></td><td><label for=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_identity_use_folder\" class='ZCheckboxLabel'>";
	buffer[_i++] =  ZmMsg.whenReplyingToFolder ;
	buffer[_i++] = "</label></td></tr></table></td></tr><tr><td><div class='ZSmallSpacer'></div></td></tr></table></div>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Options#PopForm"
}, false);

AjxTemplate.register("prefs.Options#IdentityForm", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table class=\"ZFormTable ZFixedTable\" cellspacing=0 cellpadding=2 border=0 style='height:100%'><colgroup><col width=103><col width='100%'></colgroup><tr><td><div class='ZSmallSpacer'></div></td></tr><tr><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.identityNameLabel ;
	buffer[_i++] = "</td><td><input id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_name\" type=text class='ZFieldSizeMedium'></td></tr><tr><td><div class='ZSmallSpacer'></div></td></tr><tr><td colspan=2><ul id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_tabs\" class='ZTabList'><li id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_tabs_options' class='ZTab' onclick='pageController.showPage(\"options\")'>Identity Options</li><li id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_tabs_signature' class='ZTab' onclick='pageController.showPage(\"signature\")'>Signature</li><li id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_tabs_advanced' class='ZTab' onclick='pageController.showPage(\"advanced\")'>Advanced</li></ul></td></tr><tr><td style='height:100%' valign=top colspan=2><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_subFormContainer' style='width:95%;position:relative;'></div></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Options#IdentityForm"
}, false);

AjxTemplate.register("prefs.Options#IdentityForm_options", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<fieldset class='ZFieldset' style='width:auto'><legend class='ZLegend'>";
	buffer[_i++] =  ZmMsg.sendWithIdentity ;
	buffer[_i++] = "</legend><table class='ZFormTable XZFixedTable' cellspacing=0 cellpadding=2 width='99%'><colgroup><col width=20><col width=60><col width='50%'><col width='50%'></colgroup><tr><td class='ZLabelCell' colspan=2>";
	buffer[_i++] =  ZmMsg.sendFrom ;
	buffer[_i++] = "</td><td><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_sendFromName' style='width:100%'></td><td><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_sendFromAddress' style='width:90%'><option>bob@zimbra.com</option></select></td></tr><tr><td class='ZCheckboxCell'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_setReplyToCheckbox' type='checkbox'></td><td class='ZCheckboxLabelCell'><nobr>";
	buffer[_i++] =  ZmMsg.setReplyTo ;
	buffer[_i++] = "</nobr></td><td><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_setReplyToName' style='width:100%'></td><td><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_setReplyToAddress' style='width:90%'><option>bob@zimbra.com</option></select></td></tr></table></fieldset>";
	 if (AjxEnv.isIE) { 
	buffer[_i++] = "<br>";
	 } 
	buffer[_i++] = "<fieldset class='ZFieldset' style='width:auto'><legend class='ZLegend'>";
	buffer[_i++] =  ZmMsg.selectIdentityWhen ;
	buffer[_i++] = "</legend><table class='ZFormTable ZFixedTable' cellspacing=0 cellpadding=2 width='99%' border=0><colgroup><col width=20><col width='100%'><col width=40></colgroup><tr><td class='ZCheckboxCell'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_whenSentToCheckbox' type='checkbox'></td><td class='ZCheckboxLabelCell'>";
	buffer[_i++] =  ZmMsg.whenSentTo ;
	buffer[_i++] = "</td></tr><tr><td></td><td><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_whenSentToInput' style='width:100%'></td></tr><tr><td></td><td class='ZHelpLabel'>";
	buffer[_i++] =  ZmMsg.whenSentToHint ;
	buffer[_i++] = "</td></tr><tr><td><div class='ZSmallSpacer'></div></td></tr><tr><td class='ZCheckboxCell'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_whenInFolderCheckbox' type='checkbox'></td><td class='ZCheckboxLabelCell'>";
	buffer[_i++] =  ZmMsg.whenInFolder ;
	buffer[_i++] = "</td></tr><tr><td></td><td><table cellspacing=0 cellpadding=0 style='width:100%'><tr><td style='width:100%'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_whenInFolderInput' style='width:100%'></td><td style='padding-left:10px;'><button id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_folderBrowseButton' style='width:30px'><div class='ImgFolder'></div></button></td></tr></table></td></tr><tr><td></td><td class='ZHelpLabel'>";
	buffer[_i++] =  ZmMsg.whenInFolderHint ;
	buffer[_i++] = "</td></tr></table></fieldset>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Options#IdentityForm_options"
}, false);

AjxTemplate.register("prefs.Options#IdentityForm_signature", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table class='ZFormTable ZFixedTable' cellspacing=0 cellpadding=2 style='width:95%;' border=0><colgroup><col width=103><col width='100%'><col width=20></colgroup><tr><td><div class='ZSmallSpacer'></div></td></tr><tr><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.placeSignature ;
	buffer[_i++] = "</td><td colspan=2><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_signatureStyleSelect'><option>";
	buffer[_i++] =  ZmMsg.aboveQuotedText ;
	buffer[_i++] = "</option><option>";
	buffer[_i++] =  ZmMsg.atBottomOfMessage ;
	buffer[_i++] = "</option></select></td></tr><tr><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.applySignature ;
	buffer[_i++] = "</td><td colspan=2><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_signatureEnabledSelect'><option>";
	buffer[_i++] =  ZmMsg.automaticSignature ;
	buffer[_i++] = "</option><option>";
	buffer[_i++] =  ZmMsg.manualSignature ;
	buffer[_i++] = "</option></select></td></tr><tr><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.mailSignature ;
	buffer[_i++] = "</td><td><textarea id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_signature' style='width:100%;height:120px'></textarea></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Options#IdentityForm_signature"
}, false);

AjxTemplate.register("prefs.Options#IdentityForm_advanced", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table class='ZFormTable ZFixedTable' cellspacing=0 cellpadding=2 width='99%' border=0><colgroup><col width='200'><col width='100%'</colgroup><tr><td><div class='ZSmallSpacer'></div></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_useDefaultsRadios'><td class='ZLabelCell'>";
	buffer[_i++] =  ZmMsg.replyWithIdentity ;
	buffer[_i++] = "</td><td style='padding:0px'><table><tr><td class='ZCheckboxCell'><input  name='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_group' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_useDefaultsCheckbox_default' class='ZRadioButton' type='radio' onclick='";
	buffer[_i++] = data["controller"];
	buffer[_i++] = "._toggleAdvancedSettings(false)'></td><td class='ZCheckboxLabelCell' colspan='2'><label for='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_useDefaultsCheckbox_default' class='ZCheckboxLabel'>";
	buffer[_i++] =  ZmMsg.identitiesUseDefault ;
	buffer[_i++] = "</label></td></tr><tr><td class='ZCheckboxCell'><input  name='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_group' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_useDefaultsCheckbox_custom' class='ZRadioButton' type='radio' onclick='";
	buffer[_i++] = data["controller"];
	buffer[_i++] = "._toggleAdvancedSettings(true)'></td><td class='ZCheckboxLabelCell' colspan='2'><label for='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_useDefaultsCheckbox_custom' class='ZCheckboxLabel'>";
	buffer[_i++] =  ZmMsg.customizeSettings ;
	buffer[_i++] = "</label></td></tr></table></td></tr><tr><td><div class='ZSmallSpacer'></div></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_replyForwardSelect_row'><td class='ZLabelCell' ><label id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_replyForwardSelect_label' for='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_replyForwardSelect'>";
	buffer[_i++] =  ZmMsg.replyForwardFormat ;
	buffer[_i++] = "</label></td><td><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_replyForwardSelect' class='ZFieldSizeLarge'><option>";
	buffer[_i++] =  ZmMsg.originalFormat ;
	buffer[_i++] = "</option><option>";
	buffer[_i++] =  ZmMsg.text ;
	buffer[_i++] = "</option><option>";
	buffer[_i++] =  ZmMsg.htmlDocument ;
	buffer[_i++] = "</option></select></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_replyIncludeSelect_row'><td class='ZLabelCell' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_replyIncludeSelect_label' >";
	buffer[_i++] =  ZmMsg.replyInclude ;
	buffer[_i++] = "</td><td><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_replyIncludeSelect' class='ZFieldSizeLarge'><option>";
	buffer[_i++] =  ZmMsg.dontIncludeMessage ;
	buffer[_i++] = "</option><option>";
	buffer[_i++] =  ZmMsg.includeInBody ;
	buffer[_i++] = "</option><option>";
	buffer[_i++] =  ZmMsg.includePrefix ;
	buffer[_i++] = "</option><option>";
	buffer[_i++] =  ZmMsg.includeOriginalAsAttach ;
	buffer[_i++] = "</option><option>";
	buffer[_i++] =  ZmMsg.smartInclude ;
	buffer[_i++] = "</option></select></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_forwardIncludeSelect_row'><td class='ZLabelCell' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_forwardIncludeSelect_label' >";
	buffer[_i++] =  ZmMsg.forwardInclude ;
	buffer[_i++] = "</td><td><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_forwardIncludeSelect' class='ZFieldSizeLarge'><option>";
	buffer[_i++] =  ZmMsg.includeOriginalInBody ;
	buffer[_i++] = "</option><option>";
	buffer[_i++] =  ZmMsg.includePrefix ;
	buffer[_i++] = "</option><option>";
	buffer[_i++] =  ZmMsg.includeOriginalAsAttach ;
	buffer[_i++] = "</option></select></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_prefixSelect_row'><td class='ZLabelCell' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_prefixSelect_label' >";
	buffer[_i++] =  ZmMsg.prefixTextWith ;
	buffer[_i++] = "</td><td><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_prefixSelect' class='ZFieldSizeSmall'><option>&gt;</option><option>|</option></select></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Options#IdentityForm_advanced"
}, false);

AjxTemplate.register("prefs.Options#GeneralForm", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "General Options";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Options#GeneralForm"
}, false);

}
if (AjxPackage.define("prefs.Pages")) {
AjxTemplate.register("prefs.Pages#General", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div align=right style='margin-right: 20px; margin-bottom: 10px;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REVERT_PAGE' tabindex='10' style='margin-right: 25px;'></div></div><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.loginOptions;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%>";
	 if (!appCtxt.isOffline) { 
	 if (data.isEnabled(ZmSetting.CHANGE_PASSWORD_ENABLED)) { 
	buffer[_i++] = "<tr id='prefs.Pages#General_PASSWORD_LABEL'><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.password;
	buffer[_i++] = ":</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PASSWORD' tabindex='50'></div></td></tr>";
	 } 
	buffer[_i++] = "<tr><td class='ZOptionsLabelTop'>";
	buffer[_i++] = ZmMsg.loginLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CLIENT_TYPE' tabindex='20'></div></td></tr>";
	 } 
	 if (!appCtxt.isOffline) { 
	buffer[_i++] = "<tr><td colspan=2><hr></td></tr>";
	 } 
	 if (data.isEnabled(ZmSetting.SKIN_CHANGE_ENABLED)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.themeLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SKIN_NAME' tabindex='30'></div></td></tr>";
	 } 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.languageLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_LOCALE_NAME' tabindex='40'></div></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.defaultTimezone;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEFAULT_TIMEZONE' tabindex='45'></div></td></tr></table></td></tr></table><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.searches;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%>";
	 if (data.isEnabled(ZmSetting.SEARCH_INCLUDES_SPAM, ZmSetting.SEARCH_INCLUDES_TRASH)) { 
	 if (data.isEnabled(ZmSetting.SEARCH_INCLUDES_SPAM)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.searchSettingsLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SEARCH_INCLUDES_SPAM' tabindex='60'></div></td></tr>";
	 } 
	 if (data.isEnabled(ZmSetting.SEARCH_INCLUDES_TRASH)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	 if (!data.isEnabled(ZmSetting.SEARCH_INCLUDES_SPAM)) { 
	buffer[_i++] = ZmMsg.searchSettingsLabel;
	 } else { 
	buffer[_i++] = "\n";
	buffer[_i++] = "\t\t\t\t\t\t\t\t\t\t&nbsp;\n";
	buffer[_i++] = "\t\t\t\t\t\t\t\t\t";
	 } 
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SEARCH_INCLUDES_TRASH' tabindex='70'></div></td></tr>";
	 } 
	buffer[_i++] = "<tr><td colspan=2><hr></td></tr>";
	 } 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.searchLanguageLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SHOW_SEARCH_STRING' tabindex='80'></div></td></tr></table></td></tr></table><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.other;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.pageSizeLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><table cellspacing=0 cellpadding=0 border=0><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PAGE_SIZE' tabindex='90'></div></td><td style=\"padding-left:4px;\">";
	buffer[_i++] = ZmMsg.pageSizeDesc;
	buffer[_i++] = "</td></tr></table></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.selection;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SHOW_SELECTION_CHECKBOX' tabindex='95'></div></td></tr>";
	 if (appCtxt.isOffline && appCtxt.accountList.size() > 2) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'></td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_OFFLINE_SHOW_ALL_MAILBOXES' tabindex='100'></div></td></tr>";
	 } 
	 if (data.isEnabled(ZmSetting.OFFLINE_SUPPORTS_MAILTO)) { 
	buffer[_i++] = "<tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.systemDefaults;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_OFFLINE_IS_MAILTO_HANDLER' tabindex='110'></div></td></tr>";
	 } 
	 if (appCtxt.isOffline && appCtxt.accountList.accountTypeExists(ZmAccount.TYPE_ZIMBRA)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'></td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_OFFLINE_NOTEBOOK_SYNC_ENABLED' tabindex='120'></div></td></tr>";
	 } 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.printFontSizeLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEFAULT_PRINTFONTSIZE' tabindex='130'></select></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.emailAddrs;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SHORT_ADDRESS' tabindex='140'></div></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.bubbles;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_USE_ADDR_BUBBLES' tabindex='150'></div></td></tr>";
	 if (appCtxt.isOffline) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.offlineUpdateNotifyLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_OFFLINE_UPDATE_NOTIFY' tabindex='160'></select></td></tr>";
	 } 
	buffer[_i++] = "</table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#General"
}, false);
AjxTemplate.register("prefs.Pages", AjxTemplate.getTemplate("prefs.Pages#General"), AjxTemplate.getParams("prefs.Pages#General"));

AjxTemplate.register("prefs.Pages#Mail", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div align=right style='margin-right: 20px; margin-bottom: 10px;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REVERT_PAGE' tabindex='10' style='margin-right: 25px;'></div></div>";
	 if (!appCtxt.multiAccounts || data.activeAccount.isMain) { 
	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.displayMessages;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%>";
	 if (!appCtxt.isOffline && data.isEnabled(ZmSetting.POLLING_INTERVAL_ENABLED)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.pollingIntervalLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_POLLING_INTERVAL' tabindex='40' size='3' /></td></tr>";
	 } 
	buffer[_i++] = "<tr><td class='ZOptionsLabelTop'>";
	buffer[_i++] =  appCtxt.isOffline ? ZmMsg.checkMailPrefLabelOffline : ZmMsg.checkMailPrefLabel ;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_GET_MAIL_ACTION' tabindex='45'></div></td></tr><tr><td class='ZOptionsLabelTop'>";
	buffer[_i++] = ZmMsg.displayMail;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_VIEW_AS_HTML' tabindex='50'></div></td></tr><tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.messagePreviewLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SHOW_FRAGMENTS' tabindex='70'></div></td></tr><tr><td class='ZOptionsLabel'></td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_OPEN_MAIL_IN_NEW_WIN' tabindex='80'></div></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.imagesLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DISPLAY_EXTERNAL_IMAGES' tabindex='90'></div></td></tr><tr><td class='ZOptionsLabelTop'>";
	buffer[_i++] = ZmMsg.messageReadLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MARK_MSG_READ' tabindex='95'></div></td></tr><tr><td class='ZOptionsLabelTop'>";
	buffer[_i++] = ZmMsg.selectAfterDeleteLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SELECT_AFTER_DELETE' tabindex='97'></div></td></tr>";
	 if (!appCtxt.isOffline) { 
	buffer[_i++] = "<tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.initialMailSearchLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_INITIAL_SEARCH' tabindex='100'></div></td></tr>";
	 } 
	buffer[_i++] = "</table></td></tr></table>";
	 } 
	
			if (appCtxt.isOffline && data.activeAccount == appCtxt.accountList.defaultAccount) {
				var size = appCtxt.accountList.size();
				if ((size > 2 && !appCtxt.get(ZmSetting.OFFLINE_SHOW_ALL_MAILBOXES)) || size == 2) {
		
	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.displayMessages;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.initialMailSearchLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_INITIAL_SEARCH' tabindex='100'></div></td></tr></table></td></tr></table>";
	 } 
	 } 
	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.messagesReceiving;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%>";
	 if (data.isEnabled(ZmSetting.MAIL_NOTIFY_SOUNDS)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.messageArrivalLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MAIL_NOTIFY_SOUNDS' tabindex='105' type=checkbox /></td></tr>";
	 } 
	 if (data.isEnabled(ZmSetting.MAIL_NOTIFY_APP)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MAIL_NOTIFY_APP' tabindex='106' type=checkbox /></td></tr>";
	 } 
	 if (data.isEnabled(ZmSetting.MAIL_NOTIFY_BROWSER)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MAIL_NOTIFY_BROWSER' tabindex='107' type=checkbox /></td></tr>";
	 } 
	 if (data.isEnabled(ZmSetting.MAIL_NOTIFY_TOASTER)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><table cellspacing=0 cellpadding=0><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MAIL_NOTIFY_TOASTER' tabindex='108'></div></td><td>";
	buffer[_i++] = ZmDesktopAlert.getInstance().getLinkText() ;
	buffer[_i++] = "</td></tr></table></td></tr>";
	 } 
	 if (appCtxt.isOffline) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_OFFLINE_NOTIFY_NEWMAIL_ON_INBOX' tabindex='109' /></td></tr>";
	 } 
	 if (!appCtxt.multiAccounts || appCtxt.isFamilyMbox || (!data.activeAccount.isMain && data.activeAccount.isZimbraAccount)) { 
	 if (data.isEnabled(ZmSetting.MAIL_FORWARDING_ENABLED)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	 if (!data.isEnabled(ZmSetting.MAIL_NOTIFY_SOUNDS)) { 
	buffer[_i++] = ZmMsg.messageArrivalLabel;
	 } else { 
	buffer[_i++] = "\n";
	buffer[_i++] = "\t\t\t\t\t\t\t\t\t\t&nbsp;\n";
	buffer[_i++] = "\t\t\t\t\t\t\t\t\t";
	 } 
	buffer[_i++] = "</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.forwardCopyTo;
	buffer[_i++] = "</td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField' style='padding-left:2em;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MAIL_FORWARDING_ADDRESS' tabindex='110' /></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField' style='padding-left:2em;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MAIL_LOCAL_DELIVERY_DISABLED' tabindex='120'></td></tr>";
	 } 
	 if (data.isEnabled(ZmSetting.NOTIF_FEATURE_ENABLED)) { 
	 if (data.isEnabled(ZmSetting.MAIL_FORWARDING_ENABLED)) { 
	buffer[_i++] = "<tr><td>&nbsp;</td><td><hr></td></tr>";
	 } 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	 if (!data.isEnabled(ZmSetting.MAIL_FORWARDING_ENABLED)) { 
	buffer[_i++] = ZmMsg.messageArrivalLabel;
	 } else { 
	buffer[_i++] = "\n";
	buffer[_i++] = "\t\t\t\t\t\t\t\t&nbsp;\n";
	buffer[_i++] = "\t\t\t\t\t\t\t";
	 } 
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_NOTIF_ENABLED' tabindex='130' type=checkbox></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField' style='padding-left:2em;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_NOTIF_ADDRESS' tabindex='140' /></td></tr>";
	 } 
	 if (data.isEnabled(ZmSetting.VACATION_MSG_FEATURE_ENABLED)) { 
	 if (data.isEnabled(ZmSetting.MAIL_FORWARDING_ENABLED,ZmSetting.NOTIF_FEATURE_ENABLED)) { 
	buffer[_i++] = "<tr><td>&nbsp;</td><td><hr></td></tr>";
	 } 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	 if (!data.isEnabled(ZmSetting.MAIL_FORWARDING_ENABLED) && !data.isEnabled(ZmSetting.NOTIF_FEATURE_ENABLED)) { 
	buffer[_i++] = ZmMsg.messageArrivalLabel;
	 } else { 
	buffer[_i++] = "\n";
	buffer[_i++] = "\t\t\t\t\t\t\t\t&nbsp;\n";
	buffer[_i++] = "\t\t\t\t\t\t\t";
	 } 
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_VACATION_MSG_ENABLED' tabindex='150' type=checkbox></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField' style='padding-left:2em;'><table border=\"0\"><tr><td><textarea id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_VACATION_MSG' tabindex='160' rows=3 cols=70></textarea><span style=\"display:none\"><input type=\"hidden\" name=\"st_dt\" id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_VACATION_FROM\"/><input type=\"hidden\" name=\"ed_dt\" id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_VACATION_UNTIL\"/></span></td></tr><tr><td><table><tr><td><input type=\"checkbox\" id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_START_DATE_ENABLED\"></td><td><input style='height:22px;' type='text' autocomplete='off' size=14 maxlength=10 id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_VACATION_FROM1'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_startMiniCal'></td></tr><tr><td><input type=\"checkbox\" id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_END_DATE_ENABLED\"></td><td><input style='height:22px;' type='text' autocomplete='off' size=14 maxlength=10 id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_VACATION_UNTIL1'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_endMiniCal'></td></tr></table></td></tr></table></td></tr>";
	 } 
	 if (data.isEnabled(ZmSetting.MAIL_READ_RECEIPT_ENABLED)) { 
	buffer[_i++] = "<tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.readReceipt;
	buffer[_i++] = "</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.readReceiptPref;
	buffer[_i++] = "</td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MAIL_SEND_READ_RECEIPTS' tabindex='170'></div></td></tr><tr><td colspan=2><hr></td></tr>";
	 } 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.removeDupesToSelfLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.removeDupesToSelf;
	buffer[_i++] = "</td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEDUPE_MSG_TO_SELF' tabindex='180'></div></td></tr>";
	 } 
	buffer[_i++] = "</table></td></tr></table>";
	 if (!appCtxt.multiAccounts || appCtxt.isFamilyMbox || (!data.activeAccount.isMain && data.activeAccount.isZimbraAccount)) { 
	 if (appCtxt.get(ZmSetting.MAIL_BLACKLIST_MAX_NUM_ENTRIES) > 0 || appCtxt.get(ZmSetting.MAIL_WHITELIST_MAX_NUM_ENTRIES) > 0) { 
	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.junkMailOptions;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td class='ZOptionsLabel' style='text-align:left'>";
	buffer[_i++] = ZmMsg.blackListLabel;
	buffer[_i++] = "</td><td class='ZOptionsLabel' style='text-align:left'>";
	buffer[_i++] = ZmMsg.whiteListLabel;
	buffer[_i++] = "</td></tr><tr><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MAIL_BLACKLIST'></div></td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MAIL_WHITELIST'></div></td></tr></table></td></tr></table>";
	 } 
	 } 
	 if (data.isEnabled(ZmSetting.POP_ENABLED)) { 
	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.externalAccess;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td class='ZOptionsLabelTop'>";
	buffer[_i++] = ZmMsg.externalAccessPop;
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_POP_DOWNLOAD_SINCE' tabindex='200' type=radio></td></tr><tr><td></td><td class='ZOptionsInfo'><span id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_POP_DOWNLOAD_SINCE_VALUE'></span></td></tr></table></td></tr></table>";
	 } 

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#Mail"
}, false);

AjxTemplate.register("prefs.Pages#BlackList", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EMAIL_ADDRESS' tabindex='182'></div></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_ADD_BUTTON' valign=top tabindex='184'></td></tr><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_LISTVIEW' tabindex='186'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REMOVE_BUTTON' valign=top tabindex='188'></td></tr></table><div class='ZOptionsInfo' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_NUM_USED'></div>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#BlackList"
}, false);

AjxTemplate.register("prefs.Pages#WhiteList", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EMAIL_ADDRESS' tabindex='190'></div></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_ADD_BUTTON' valign=top tabindex='192'></td></tr><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_LISTVIEW' tabindex='194'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REMOVE_BUTTON' valign=top tabindex='196'></td></tr></table><div class='ZOptionsInfo' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_NUM_USED'></div>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#WhiteList"
}, false);

AjxTemplate.register("prefs.Pages#TrustedList", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EMAIL_ADDRESS' tabindex='190'></div></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_ADD_BUTTON' valign=top tabindex='192'></td></tr><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_LISTVIEW' tabindex='194'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REMOVE_BUTTON' valign=top tabindex='196'></td></tr></table><div class='ZOptionsInfo' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_NUM_USED'></div>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#TrustedList"
}, false);

AjxTemplate.register("prefs.Pages#Trusted", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div align=right style='margin-right: 20px; margin-bottom: 10px;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REVERT_PAGE' tabindex='10' style='margin-right: 25px;'></div></div><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><!-- TODO: What about intersection with notebook page compose? --><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.trustedAddrsDomains;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><div>";
	buffer[_i++] = ZmMsg.trustedAddrHeaderMsg;
	buffer[_i++] = "</div><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_TRUSTED_ADDR_LIST' tabindex='65' /></td></td></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#Trusted"
}, false);

AjxTemplate.register("prefs.Pages#Composing", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div align=right style='margin-right: 20px; margin-bottom: 10px;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REVERT_PAGE' tabindex='10' style='margin-right: 25px;'></div></div><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><!-- TODO: What about intersection with notebook page compose? --><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.composingMessages;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%>";
	 var isHtmlCompose = data.isEnabled(ZmSetting.HTML_COMPOSE_ENABLED); 
	 if (isHtmlCompose) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel' valign=top>";
	buffer[_i++] = ZmMsg.composeLabel;
	buffer[_i++] = "</td><td class='ZOptionsNestedTable'><table><colgroup><col width=1><col width=1><col> <!-- family --><col width=1><col> <!-- size --><col width=1><col> <!-- color --></colgroup><tr valign=top><td class='ZOptionsFieldTop'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_COMPOSE_AS_FORMAT' tabindex='20'></div></td><td class='ZOptionsLabelTop ZOptionsLabelNarrow'>";
	buffer[_i++] = ZmMsg.fFamilyLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_COMPOSE_INIT_FONT_FAMILY' tabindex='30'></select></td><td class='ZOptionsLabelTop ZOptionsLabelNarrow'>";
	buffer[_i++] = ZmMsg.fSizeLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_COMPOSE_INIT_FONT_SIZE' tabindex='40'></select></td><td class='ZOptionsLabelTop ZOptionsLabelNarrow'>";
	buffer[_i++] = ZmMsg.fColorLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_COMPOSE_INIT_FONT_COLOR' tabindex='50'></select></td></tr></table></td></tr>";
	 } 
	buffer[_i++] = "<tr><td class='ZOptionsLabel' valign=top>";
	buffer[_i++] = ZmMsg.settingsLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_AUTO_SAVE_DRAFT_INTERVAL' tabindex='65' /></td></tr>";
	 if (isHtmlCompose) { 
	buffer[_i++] = "<tr><td>&nbsp;</td><td class='ZOptionsField'><div type=checkbox id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_COMPOSE_SAME_FORMAT\" /></td></tr>";
	 } 
	 var isNewWindow = data.isEnabled(ZmSetting.NEW_WINDOW_COMPOSE); 
	 if (isNewWindow) { 
	buffer[_i++] = "<tr><td>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_NEW_WINDOW_COMPOSE' tabindex='60' /></td></tr>";
	 } 
	 if (appCtxt.get(ZmSetting.SPELL_CHECK_ENABLED)) { 
	buffer[_i++] = "<tr><td>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_MAIL_MANDATORY_SPELLCHECK' tabindex='62' /></td></tr>";
	 } 
	 if (data.isEnabled(ZmSetting.SAVE_TO_SENT)) { 
	buffer[_i++] = "<tr><td>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SAVE_TO_SENT' tabindex='67' /></td></tr>";
	 } 
	 if (data.isEnabled(ZmSetting.MAIL_ENABLED)) { 
	 if (isHtmlCompose || isNewWindow ) { 
	buffer[_i++] = "<tr><td colspan=2><hr></td></tr>";
	 } 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.composeReplyLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.composeReplyEmail;
	buffer[_i++] = "</td></tr><tr><td>&nbsp;</td><td class='ZOptionsField'><table><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REPLY_INCLUDE_WHAT' tabindex='70' /></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REPLY_USE_PREFIX' tabindex='72' /></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REPLY_INCLUDE_HEADERS' tabindex='74' /></td></tr></table></td></tr><tr><td colspan=2></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.forwardingLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.forwardingEmail;
	buffer[_i++] = "</td></tr><tr><td>&nbsp;</td><td class='ZOptionsField'><table><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_FORWARD_INCLUDE_WHAT' tabindex='80' /></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_FORWARD_USE_PREFIX' tabindex='82' /></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_FORWARD_INCLUDE_HEADERS' tabindex='84' /></td></tr></table></td></tr><tr><td colspan=2></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.prefixLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.prefixTextWith;
	buffer[_i++] = "</td></tr><tr><td>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REPLY_PREFIX' tabindex='90' /></td></tr><tr><td>&nbsp;</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.prefixNote;
	buffer[_i++] = "</td></tr>";
	 } 
	 if (!appCtxt.isOffline) { 
	buffer[_i++] = "<tr><td colspan=2><hr></td></tr><tr><td colspan=2><b>";
	buffer[_i++] = ZmMsg.composeMoreOptions;
	buffer[_i++] = "</b></td></tr>";
	 } 
	buffer[_i++] = "</table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#Composing"
}, false);

AjxTemplate.register("prefs.Pages#Signatures", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div align=right style='margin-right: 20px; margin-bottom: 10px;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REVERT_PAGE' tabindex='10' style='margin-right: 25px;'></div></div><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.signatures;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100% cellspacing=0><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SIGNATURES' tabindex='20'></div></td></tr></table></td></tr></table><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.signaturesUsing;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SIG_TABLE' border=0 cellpadding=0 cellspacing=10><tr><td>&nbsp;</td><td>";
	buffer[_i++] = ZmMsg.newMessages;
	buffer[_i++] = "</td><td>";
	buffer[_i++] = ZmMsg.repliesForwards;
	buffer[_i++] = "</td></tr></table><hr><table width=100% cellspacing=0><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.placeSignature;
	buffer[_i++] = "</td><td class='ZOptionsInfo'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SIGNATURE_STYLE' tabindex='40'></div></td></tr></table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#Signatures"
}, false);

AjxTemplate.register("prefs.Pages#SignatureSplitView", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellpadding=0 cellspacing=0 width='100%'><tr><td>&nbsp;</td><td><table border=0 cellpadding=0 cellspacing=0 width=\"100%\"><tr><td class=\"Label\">";
	buffer[_i++] = ZmMsg.nameLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SIG_NAME' tabindex='50' size=30></td><td width=\"100%\">&nbsp;</td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SIG_FORMAT' tabIndex='60'></div></td></tr></table></td></tr><tr><td width=180 height=150 valign='top'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SIG_LIST' width='100%' tabindex='40'></div></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SIG_EDITOR' tabindex='80'></div></td></tr><tr><td><table><tr><td class=\"ZOptionsField\"><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SIG_NEW' tabIndex='86'></div></td><td class='ZOptionsField'>&nbsp;</td><td class=\"ZOptionsField\" align='right'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SIG_DELETE' tabIndex='87'></div></td></tr></table></td><td align='right'><table border=0 cellpadding=0 cellspacing=0><tr><td class=\"Label\">";
	buffer[_i++] = ZmMsg.signatureVcardLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SIG_VCARD' tabindex='85' size=30></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SIG_VCARD_BROWSE' style='padding-right:5px; padding-left:5px;' tabIndex='86'></div></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SIG_VCARD_CLEAR' tabIndex='87'></div></td></tr></table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#SignatureSplitView"
}, false);

AjxTemplate.register("prefs.Pages#Contacts", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<!-- REVERT_PAGE (button) --><div align=right style='margin-right: 20px; margin-bottom: 10px;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REVERT_PAGE' tabindex='10' style='margin-right: 25px;'></div></div>";
	 if (data.activeAccount.isMain) { 
	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.options;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%>";
	 if (!appCtxt.multiAccounts || data.activeAccount.isMain) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.settingsLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_AUTO_ADD_ADDRESS' tabindex='40' type=checkbox></td></tr>";
	 } 
	 if (!appCtxt.multiAccounts || (appCtxt.isFamilyMbox && data.activeAccount.isMain) || (!data.activeAccount.isMain && data.activeAccount.isZimbraAccount)) { 
	 if (data.isEnabled(ZmSetting.INITIALLY_SEARCH_GAL)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_INITIALLY_SEARCH_GAL' tabindex='50' type=checkbox></td></tr>";
	 } 
	 } 
	buffer[_i++] = "</table></td></tr></table>";
	 } 
	 if (!appCtxt.multiAccounts || (appCtxt.isFamilyMbox && data.activeAccount.isMain) || !data.activeAccount.isMain) {
	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.autocomplete;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%>";
	 if (appCtxt.isOffline && !data.activeAccount.isMain) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_AUTOCOMPLETE_SHARE' tabindex='55' type=checkbox></td></tr>";
	 } 
	 if (data.activeAccount.isZimbraAccount) { 
	 if (data.isEnabled(ZmSetting.GAL_ENABLED) && data.isEnabled(ZmSetting.GAL_AUTOCOMPLETE_ENABLED)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_GAL_AUTOCOMPLETE' tabindex='60' type=checkbox></td></tr>";
	 } 
	 if (data.isEnabled(ZmSetting.SHARING_ENABLED)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_AUTOCOMPLETE_SHARED_ADDR_BOOKS' tabindex='70'></div></td></tr>";
	 } 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_AUTOCOMPLETE_ON_COMMA' tabindex='80'></div></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_AUTOCOMPLETE_NO_GROUP_MATCH' tabindex='90'></div></td></tr>";
	 } 
	buffer[_i++] = "</table></td></tr></table>";
	 } 

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#Contacts"
}, false);

AjxTemplate.register("prefs.Pages#Accounts", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div align=right style='margin-right: 20px; margin-bottom: 10px;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REVERT_PAGE' tabindex='10' style='margin-right: 25px;'></div></div><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.accounts;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td style='padding:0;margin:0;border:solid 1px silver'> <!-- TODO: correct border color --><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_ACCOUNTS' tabindex='20'></div></td></tr><tr><td><table width=100%><colgroup><col width=1><col width=*><col width=1><col width=1></colgroup><tr><td align=left><button id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DELETE' tabindex='30'>";
	buffer[_i++] = ZmMsg.del;
	buffer[_i++] = "</button></td><td>&nbsp;</td><td align=right>";
	 if (!appCtxt.isOffline && (data.isEnabled(ZmSetting.POP_ACCOUNTS_ENABLED) || data.isEnabled(ZmSetting.IMAP_ACCOUNTS_ENABLED))) { 
	buffer[_i++] = "<button id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_ADD_EXTERNAL' tabindex='40'>";
	buffer[_i++] = ZmMsg.addExternalAccount;
	buffer[_i++] = "</button>";
	 } 
	buffer[_i++] = "</td><td align=right>";
	 if (data.isEnabled(ZmSetting.IDENTITIES_ENABLED)) { 
	buffer[_i++] = "<button id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_ADD_PERSONA' tabindex='50'>";
	buffer[_i++] = ZmMsg.addPersona;
	buffer[_i++] = "</button>";
	 } 
	buffer[_i++] = "</td></tr></table></td></tr></table></td></tr></table><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PRIMARY\" class='ZAccountSettings'>";
	buffer[_i++] =  AjxTemplate.expand("#PrimaryAccount", data) ;
	buffer[_i++] = "</div>";
	 if (data.isEnabled(ZmSetting.POP_ACCOUNTS_ENABLED) || data.isEnabled(ZmSetting.IMAP_ACCOUNTS_ENABLED)) { 
	buffer[_i++] = "<div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL\" class='ZAccountSettings' style='display:none;'>";
	buffer[_i++] =  AjxTemplate.expand("#ExternalAccount", data) ;
	buffer[_i++] = "</div>";
	
				var providers = ZmDataSource.getProviders();
				for (var pid in providers) {
					if (!AjxTemplate.getTemplate("prefs.Pages#ExternalAccount-"+pid)) { continue; }
			
	buffer[_i++] = "<div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_";
	buffer[_i++] = pid;
	buffer[_i++] = "\" class='ZAccountSettings' style='display:none;'>";
	buffer[_i++] =  AjxTemplate.expand("#ExternalAccount-"+pid, data) ;
	buffer[_i++] = "</div>";
	 } 
	 } 
	 if (data.isEnabled(ZmSetting.IDENTITIES_ENABLED)) { 
	buffer[_i++] = "<div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PERSONA\" class='ZAccountSettings' style='display:none;'>";
	buffer[_i++] =  AjxTemplate.expand("#Persona", data) ;
	buffer[_i++] = "</div>";
	 } 

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#Accounts"
}, false);

AjxTemplate.register("prefs.Pages#PrimaryAccount", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.accountHeaderPrimary;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.emailAddrLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><span id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PRIMARY_EMAIL' tabindex='1010'></span></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.accountNameLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PRIMARY_NAME' tabindex='1020'></td></tr>";
	 if (appCtxt.isFamilyMbox) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.isVisible;
	buffer[_i++] = ":</td><td class='ZOptionsField'><span id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PRIMARY_VISIBLE' tabindex='1021'></span></td></tr>";
	 } 
	buffer[_i++] = "<tr><td colspan=2><hr></td></tr><tr><td style='text-align:left' colspan=2><b>";
	buffer[_i++] = ZmMsg.accountFromPrompt;
	buffer[_i++] = "</b></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.fromLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.fromDetail;
	buffer[_i++] = "</td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><table><tr><td style='padding-right:.5em;'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PRIMARY_FROM_NAME' tabindex='1030' size=30></td><td><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PRIMARY_FROM_EMAIL' tabindex='1035' size=40></select></td></tr></table></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.setReplyTo;
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PRIMARY_REPLY_TO' tabindex='1040' type=checkbox></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><table cellpadding='0' cellspacing='0' border='0'><tr><td style='padding-right:.5em;'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PRIMARY_REPLY_TO_NAME' tabindex='1050' size=30></td><td><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PRIMARY_REPLY_TO_EMAIL' tabindex='1060'></select></td></tr></table></td></tr><tr valign=top><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.signatureLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.manageSignatures;
	buffer[_i++] = "</td></tr></table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#PrimaryAccount"
}, false);

AjxTemplate.register("prefs.Pages#ExternalAccount", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.accountHeaderExternal;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td colspan=2><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_ALERT'></div></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.emailAddrLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_EMAIL' tabindex='2010' size=30></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.accountNameLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_NAME' tabindex='2020' size=30></div></td></tr><tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.accountTypeLabel;
	buffer[_i++] = "</td>";
	 if (data.isEnabled(ZmSetting.POP_ACCOUNTS_ENABLED) && data.isEnabled(ZmSetting.IMAP_ACCOUNTS_ENABLED)) { 
	buffer[_i++] = "<td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_ACCOUNT_TYPE' tabindex='2030'></div></td>";
	 } else if (data.isEnabled(ZmSetting.POP_ACCOUNTS_ENABLED)) { 
	buffer[_i++] = "<td class='ZOptionsField'>POP3</td>";
	 } else { 
	buffer[_i++] = "<td class='ZOptionsField'>IMAP</td>";
	 } 
	buffer[_i++] = "</tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.accountUsernameLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_USERNAME' tabindex='2040' size=30></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.accountServerLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_HOST' tabindex='2050' size=30></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.passwordLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><!-- NOTE: impossible to set a hint because it will just be turned into ****** --><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_PASSWORD' tabindex='2060' type=password></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.advancedSettingsLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><table cellpadding='0' cellspacing='0' border='0'><tr><td><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_CHANGE_PORT' tabindex='2070' type=checkbox></td><td><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_PORT' tabindex='2080' size=4></td><td class='ZOptionsInfo'><span id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_PORT_DEFAULT'></span></td></tr></table></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_SSL' tabindex='2090' type=checkbox></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><table><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_TEST' tabindex='2100'></div></td></tr></table></td></tr><tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabelTop'>";
	buffer[_i++] = ZmMsg.accountDownloadToLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_DOWNLOAD_TO' tabindex='2110'></div></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><hr></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_DELETE_AFTER_DOWNLOAD' tabindex='2120' type=checkbox></td></tr><tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel' style='text-align:left' colspan=2>";
	buffer[_i++] = ZmMsg.accountFromPrompt;
	buffer[_i++] = "</td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.fromLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.fromDetail;
	buffer[_i++] = "</td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><table><tr><td style='padding-right:.5em;'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_FROM_NAME' tabindex='2130' size=30></td><td><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_FROM_EMAIL' tabindex='2135' size=40></select></td></tr></table></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.setReplyTo;
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_REPLY_TO' tabindex='2140' type=checkbox></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><table cellpadding='0' cellspacing='0' border='0'><tr><td style='padding-right:.5em;'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_REPLY_TO_NAME' tabindex='2150' size=30></td><td><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EXTERNAL_REPLY_TO_EMAIL' tabindex='2160'></select></td></tr></table></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.signatureLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.manageSignatures;
	buffer[_i++] = "</td></tr></table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#ExternalAccount"
}, false);

AjxTemplate.register("prefs.Pages#Persona", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.personaSettings;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.accountPersonaInstructions;
	buffer[_i++] = "</td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.accountPersonaLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PERSONA_NAME' tabindex='3010'></div></td></tr><tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel' style='text-align:left' colspan=2>";
	buffer[_i++] = ZmMsg.accountFromPrompt;
	buffer[_i++] = "</td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.fromLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.fromDetail;
	buffer[_i++] = "</td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField' style='padding-left:2em;'><table><tr><td style='padding-right:.5em;'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PERSONA_FROM_NAME' tabindex='3020' size=30></td><td><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PERSONA_FROM_EMAIL' tabindex='3030' size=40></select></td></tr></table></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.setReplyTo;
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PERSONA_REPLY_TO' tabindex='3040' type=checkbox></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField' style='padding-left:2em;'><table><tr><td style='padding-right:.5em;'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PERSONA_REPLY_TO_NAME' tabindex='3050' size=30></td><td><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PERSONA_REPLY_TO_EMAIL' tabindex='3060' size=40></select></td></tr></table></td></tr><tr valign=top><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.signatureLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.manageSignatures;
	buffer[_i++] = "</td></tr><tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.accountPersonaUseLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PERSONA_WHEN_SENT_TO' tabindex='3080' type=checkbox></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsInfo' style='padding-left:2em;'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PERSONA_WHEN_SENT_TO_LIST' tabindex='3085' size=80></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsInfo' style='padding-left:2em;'>";
	buffer[_i++] = ZmMsg.whenSentToHint;
	buffer[_i++] = "</td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PERSONA_WHEN_IN_FOLDER' tabindex='3090' type=checkbox></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField' style='padding-left:2em;'><table><tr><td><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PERSONA_WHEN_IN_FOLDER_LIST' tabindex='3100' size=60></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_PERSONA_WHEN_IN_FOLDER_BUTTON' tabindex='3110'></div></td></tr></table></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsInfo' style='padding-left:2em;'>";
	buffer[_i++] = ZmMsg.whenInFolderHint;
	buffer[_i++] = "</td></tr></table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#Persona"
}, false);

AjxTemplate.register("prefs.Pages#MailFilters", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div class=\"rightAlign\">";
	buffer[_i++] = ZmMsg.filterRuleChangesSavedImmediately;
	buffer[_i++] = "</div><table border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionTabView'><table width=100% cellspacing=0 cellpadding=0><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_tabview' tabindex='20'></div></td></tr></table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#MailFilters"
}, false);

AjxTemplate.register("prefs.Pages#MailFilter", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100% cellpadding=0><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_toolbar' tabindex='20'></div></td></tr><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_list' tabindex='30'></div></td></tr></table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#MailFilter"
}, false);

AjxTemplate.register("prefs.Pages#MailFilterRule", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table width='100%'><tr><td width='1%' nowrap>";
	buffer[_i++] = ZmMsg.filterNameLabel;
	buffer[_i++] = "</td><td><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_name' type='text' width=100 tabindex='10'></td><td width='1%' style='text-align:right; white-space:nowrap;'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_active' type='checkbox' name='cbActive' checked tabindex='20'>";
	buffer[_i++] = ZmMsg.active;
	buffer[_i++] = "</td></tr></table><br><div style='color:#555555'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_condition' tabindex='30'></div></div><fieldset class='FilterRuleSection'><div style='overflow-x:hidden;overflow-y:scroll;height:145px;'><table id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_conditions' width='100%' border=0 cellpadding=0 cellspacing=0 tabindex='40'><tbody></tbody></table></div></fieldset><br><div style='color:#555555'>";
	buffer[_i++] = ZmMsg.filterActions;
	buffer[_i++] = "</div><fieldset class='FilterRuleSection'><div style='overflow-x:hidden;overflow-y:scroll;height:82px'><table id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_actions' width='100%' border=0 cellpadding=0 cellspacing=0 tabindex='50'><tbody></tbody></table></div></fieldset><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_stop' type='checkbox' name='cbStop' checked tabindex='60'>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#MailFilterRule"
}, false);

AjxTemplate.register("prefs.Pages#Calendar", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div align=right style='margin-right: 20px; margin-bottom: 10px;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REVERT_PAGE' tabindex='10' style='margin-right: 25px;'></div></div>";
	 if (data.activeAccount.isMain) { 
	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.general;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.defaultViewLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CALENDAR_INITIAL_VIEW' tabindex='20'></select></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.calendarStartWeekLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_FIRST_DAY_OF_WEEK' tabindex='30'></select></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.calendarInitialApptVisibility ;
	buffer[_i++] = "</td><td class='ZOptionsField'><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_APPT_VISIBILITY' tabindex='35'></select></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_ALWAYS_SHOW_MINI_CAL' tabindex='40' type=checkbox></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_SHOW_CALENDAR_WEEK' tabindex='40' type=checkbox></td></tr>";
	 if (!appCtxt.multiAccounts) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_AUTO_ADD_INVITES' tabindex='40' type=checkbox></td></tr>";
	 } 
	buffer[_i++] = "<tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.deleteInviteOnReplyLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DELETE_INVITE_ON_REPLY' tabindex='50' type=checkbox></td></tr><tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.forwardInvitesToLabel;
	buffer[_i++] = "</td><td class='ZOptionsField' style='padding-left:2em;'>";
	buffer[_i++] = ZmMsg.forwardInvitesTo;
	buffer[_i++] = " <input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_INV_FORWARDING_ADDRESS' tabindex='110'></td></tr><tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.apptReminderLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_REMINDER_WARNING_TIME' tabindex='60'></select></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_SHOW_PAST_DUE_REMINDERS' tabindex='59' type=checkbox></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_REMINDER_NOTIFY_SOUNDS' tabindex='65' type=checkbox></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_REMINDER_NOTIFY_BROWSER' tabindex='68' type=checkbox></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><table cellspacing=0 cellpadding=0><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_REMINDER_NOTIFY_TOASTER' tabindex='69'></div></td><td>";
	buffer[_i++] = ZmDesktopAlert.getInstance().getLinkText() ;
	buffer[_i++] = "</td></tr></table></td></tr></table></td></tr></table><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.calendarWorkHoursHeader;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain' id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_HOURS'></td></tr></table><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.apptCreating;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_USE_QUICK_ADD' tabindex='70' type=checkbox></td></tr><tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_SHOW_TIMEZONE' tabindex='80' type=checkbox></td></tr></table></td></tr></table>";
	
		}
		else if (appCtxt.multiAccounts &&
				data.activeAccount.isZimbraAccount &&
				appCtxt.getSettings(data.activeAccount).attrExists(ZmSetting.CAL_AUTO_ADD_INVITES))
		{
		
	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.general;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_AUTO_ADD_INVITES' tabindex='81' type=checkbox></td></tr></table></td></tr></table>";
	 } 
	 if (data.isAclSupported) { 
	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.calPerms;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td colspan=2>";
	buffer[_i++] =  AjxMessageFormat.format(ZmMsg.calPermsNote, data.domain) ;
	buffer[_i++] = "</td></tr><tr><td colspan=2>&nbsp;</td></tr><tr><td class='ZOptionsLabelTop'>";
	buffer[_i++] = ZmMsg.freeBusyLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_FREE_BUSY_ACL' tabindex='82'></div></td></tr><tr><td>&nbsp;</td><td><textarea id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_FREE_BUSY_ACL_USERS' tabindex='84' rows=3 cols=70></textarea></td></tr><tr><td class='ZOptionsLabelTop'>";
	buffer[_i++] = ZmMsg.invitesLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_INVITE_ACL' tabindex='86'></div></td></tr><tr><td>&nbsp;</td><td><textarea id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_INVITE_ACL_USERS' tabindex='88' rows=3 cols=70></textarea></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_SEND_INV_DENIED_REPLY' tabindex='40' type=checkbox></td></tr></table></td></tr></table>";
	 } 
	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>Apple iCal</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td colspan=2>";
	buffer[_i++] = ZmMsg.calAppleICalNote ;
	buffer[_i++] = "</td></tr><tr><td colspan=2>&nbsp;</td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_ENABLE_APPL_ICAL_DELEGATION' tabindex='50' type=checkbox></td></tr></table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#Calendar"
}, false);

AjxTemplate.register("prefs.Pages#WorkHours", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.calendarWorkWeek ;
	buffer[_i++] = "</td><td class='ZOptionsField'><table><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_0' tabindex='41'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_1' tabindex='42'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_2' tabindex='43'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_3' tabindex='44'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_4' tabindex='45'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_5' tabindex='46'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_6' tabindex='47'></td></tr></table></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.calendarWorkHours ;
	buffer[_i++] = "</td><td class='ZOptionsField'><table><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_HOURS_NORMAL'><td><td> ";
	buffer[_i++] = ZmMsg.calendarWorkHoursNormal ;
	buffer[_i++] = " <td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_START_TIME' tabindex='48'></td><td> ";
	buffer[_i++] = ZmMsg.calendarWorkHoursDelimiter ;
	buffer[_i++] = " </td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_END_TIME' tabindex='49'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_HOURS_CUSTOM'><td><td> ";
	buffer[_i++] = ZmMsg.calendarWorkHoursCustom ;
	buffer[_i++] = " <td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_CUSTOM_WORK_HOURS'></td></tr></table></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'>";
	buffer[_i++] = ZmMsg.calendarWorkHoursDisclaimer ;
	buffer[_i++] = " ";
	buffer[_i++] = AjxTimezone.getMediumName(appCtxt.get(ZmSetting.DEFAULT_TIMEZONE)) ;
	buffer[_i++] = "</td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#WorkHours"
}, false);

AjxTemplate.register("prefs.Pages#CustomWorkHoursDlg", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellspacing=2 cellpadding=2 width=300><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_0' tabindex='10'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_START_TIME_0' tabindex='20'></td><td> ";
	buffer[_i++] = ZmMsg.calendarWorkHoursDelimiter ;
	buffer[_i++] = " </td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_END_TIME_0' tabindex='30'></td></tr><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_1' tabindex='40'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_START_TIME_1' tabindex='50'></td><td> ";
	buffer[_i++] = ZmMsg.calendarWorkHoursDelimiter ;
	buffer[_i++] = " </td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_END_TIME_1' tabindex='60'></td></tr><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_2' tabindex='70'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_START_TIME_2' tabindex='80'></td><td> ";
	buffer[_i++] = ZmMsg.calendarWorkHoursDelimiter ;
	buffer[_i++] = " </td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_END_TIME_2' tabindex='90'></td></tr><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_3' tabindex='100'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_START_TIME_3' tabindex='110'></td><td> ";
	buffer[_i++] = ZmMsg.calendarWorkHoursDelimiter ;
	buffer[_i++] = " </td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_END_TIME_3' tabindex='120'></td></tr><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_4' tabindex='130'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_START_TIME_4' tabindex='140'></td><td> ";
	buffer[_i++] = ZmMsg.calendarWorkHoursDelimiter ;
	buffer[_i++] = " </td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_END_TIME_4' tabindex='150'></td></tr><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_5' tabindex='160'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_START_TIME_5' tabindex='170'></td><td> ";
	buffer[_i++] = ZmMsg.calendarWorkHoursDelimiter ;
	buffer[_i++] = " </td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_END_TIME_5' tabindex='180'></td></tr><tr><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_DAY_6' tabindex='190'></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_START_TIME_6' tabindex='200'></td><td> ";
	buffer[_i++] = ZmMsg.calendarWorkHoursDelimiter ;
	buffer[_i++] = " </td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CAL_WORKING_END_TIME_6' tabindex='210'></td></tr><tr><td colspan=\"4\">";
	buffer[_i++] = ZmMsg.calendarWorkHoursDisclaimer ;
	buffer[_i++] = " ";
	buffer[_i++] = AjxTimezone.getMediumName(appCtxt.get(ZmSetting.DEFAULT_TIMEZONE)) ;
	buffer[_i++] = "</td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#CustomWorkHoursDlg"
}, false);

AjxTemplate.register("prefs.Pages#IM", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div align=right style='margin-right: 20px; margin-bottom: 10px;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REVERT_PAGE' tabindex='10' style='margin-right: 25px;'></div></div><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.options;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%>";
	 if (data.isEnabled(ZmSetting.IM_PREF_AUTO_LOGIN)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.imLoginLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_IM_PREF_AUTO_LOGIN' tabindex='20'></div></td></tr><tr><td colspan=2><hr></td></tr>";
	 } 
	 if (data.isEnabled(ZmSetting.IM_PREF_INSTANT_NOTIFY)) { 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.imNotificationsLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_IM_PREF_INSTANT_NOTIFY' tabindex='30'></div></td></tr>";
	 } 
	buffer[_i++] = "<tr><td class='ZOptionsLabel'>";
	buffer[_i++] =  data.isEnabled(ZmSetting.IM_PREF_INSTANT_NOTIFY) ? "&nbsp;" : ZmMsg.imNotificationsLabel ;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_IM_PREF_NOTIFY_PRESENCE' tabindex='50'></div></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_IM_PREF_NOTIFY_STATUS' tabindex='60'></div></td></tr><tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.imArrivalLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_IM_PREF_NOTIFY_SOUNDS' tabindex='35'></div></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_IM_PREF_FLASH_BROWSER' tabindex='40'></div></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><table cellspacing=0 cellpadding=0><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_IM_PREF_DESKTOP_ALERT' tabindex='40'></div></td><td>";
	buffer[_i++] = ZmMsg.showPopupBrowserPlusLink ;
	buffer[_i++] = "</td></tr></table></td></tr><tr><td colspan=2><hr></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.imChat ;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_IM_PREF_LOGCHATS_ENABLED' tabindex='70'></div></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsField'><table cellspacing=\"0\" cellpadding=\"0\"><tr><td colspan=\"2\"><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_IM_PREF_REPORT_IDLE' tabindex='70'></div></td></tr><tr><td style=\"padding-left: 10px; text-align: right;\">";
	buffer[_i++] = ZmMsg.imPrefIdleTimeoutLabel ;
	buffer[_i++] = "</td><td style=\"padding: 3px\"><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_IM_PREF_IDLE_TIMEOUT' tabindex='80'></div></td></tr><tr><td style=\"padding-left: 10px; text-align: right;\">";
	buffer[_i++] = ZmMsg.imPrefIdleStatusLabel ;
	buffer[_i++] = "</td><td style=\"padding: 3px\"><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_IM_PREF_IDLE_STATUS' tabindex='90'></div></td></tr></table></td></tr></table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#IM"
}, false);

AjxTemplate.register("prefs.Pages#ImAccountLabel", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table class=\"ImAccountLabel\"><tr><td>";
	buffer[_i++] = data["name"];
	buffer[_i++] = "</td><td style='width:1%;'><img src=\"";
	buffer[_i++] = appContextPath;
	buffer[_i++] = "/public/tmp/";
	buffer[_i++] = data["nameLower"];
	buffer[_i++] = ".png\"/></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#ImAccountLabel"
}, false);

AjxTemplate.register("prefs.Pages#ZmImAccountRow", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table><tr><td class=\"ZOptionsLabel\" style=\"vertical-align:top;\"><table class=\"ImAccountLabel\"><tr><td>";
	buffer[_i++] = data["name"];
	buffer[_i++] = "</td><td style='width:1%;'><img src=\"";
	buffer[_i++] = appContextPath;
	buffer[_i++] = "/public/tmp/";
	buffer[_i++] = data["nameLower"];
	buffer[_i++] = ".png\"/></td></tr></table></td><td id = \"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_valueCell\" class=\"ZOptionsField\" width=\"100%\"></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#ZmImAccountRow"
}, false);

AjxTemplate.register("prefs.Pages#ZmImAccountSeparator", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<hr>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#ZmImAccountSeparator"
}, false);

AjxTemplate.register("prefs.Pages#ZmImGatewayControl", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table><tr><td class=\"ImAccountGatewayLabel\">";
	buffer[_i++] = data["nameLabel"];
	buffer[_i++] = "</td><td class=\"ImAccountGatewayValue\"><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_nameField'/></td></tr><tr><td class=\"ImAccountGatewayLabel\">";
	buffer[_i++] = ZmMsg.passwordLabel;
	buffer[_i++] = "</td><td class=\"ImAccountGatewayValue\"><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_passwordField'/></td></tr><tr><td class=\"ImAccountLabel\">&nbsp</td><td class=\"ImAccountGatewayValue\"><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_clearButton'/></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#ZmImGatewayControl"
}, false);

AjxTemplate.register("prefs.Pages#Shortcuts", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.keyboardShortcuts;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><div align=\"right\"><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SHORTCUT_PRINT' style='width:80px;'></div></div><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_SHORTCUT_LIST'></div></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#Shortcuts"
}, false);

AjxTemplate.register("prefs.Pages#AccountTestContent", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_accts' class='ZmDataSourceTestTable' border=0 cellspacing=0 cellpadding=3><tr><th>";
	buffer[_i++] = ZmMsg.account;
	buffer[_i++] = "</th><th>";
	buffer[_i++] = ZmMsg.status;
	buffer[_i++] = "</th></tr>";
	 for (var i = 0; i < data.accounts.length; i++) { 
	buffer[_i++] =  AjxTemplate.expand("#AccountTestItem", { id: data.id, account: data.accounts[i] }) ;
	 } 
	buffer[_i++] = "</table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#AccountTestContent"
}, false);

AjxTemplate.register("prefs.Pages#AccountTestItem", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<tr><td class='ZmTestItem'>";
	buffer[_i++] = AjxStringUtil.htmlEncode(data.account.name);
	buffer[_i++] = "</td><td id='";
	buffer[_i++] = data["account"]["id"];
	buffer[_i++] = "_test_status' class='ZmTestStatus'></td></tr><tr id='";
	buffer[_i++] = data["account"]["id"];
	buffer[_i++] = "_test_details' style='display:none'><td><table border='0'><tr valign='top'><td class='ZmTestError'>";
	buffer[_i++] = ZmMsg.errorLabel;
	buffer[_i++] = "</td><td id='";
	buffer[_i++] = data["account"]["id"];
	buffer[_i++] = "_test_error' class='ZmTestError'></td></tr><tr valign='top'><td class='ZmTestNote'>";
	buffer[_i++] = ZmMsg.noteLabel;
	buffer[_i++] = "</td><td class='ZmTestNote'>";
	buffer[_i++] = ZmMsg.popAccountTestNote;
	buffer[_i++] = "</td></tr></table></td></tr>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#AccountTestItem"
}, false);

AjxTemplate.register("prefs.Pages#Import", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellpadding=3 cellspacing=3><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = data["label"];
	buffer[_i++] = "</td><td class='ZOptionsInfo'><form id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_form'\n";
	buffer[_i++] = "\t\t\t\t  action='";
	buffer[_i++] = data["action"];
	buffer[_i++] = "' method='POST' enctype='multipart/form-data'\n";
	buffer[_i++] = "\t\t\t\t  style='margin: 0px; padding: 0px;'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_input' tabindex='10'\n";
	buffer[_i++] = "\t\t\t\t\t   type='file' name='";
	buffer[_i++] = data["name"];
	buffer[_i++] = "'\n";
	buffer[_i++] = "\t\t\t\t\t   style='font-family:Tahoma; font-size:10px'></form></td></tr><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsInfo'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_button' tabindex='20'></div></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#Import"
}, false);

AjxTemplate.register("prefs.Pages#Export", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellpadding=3 cellspacing=3><tr><td class='ZOptionsLabel'>&nbsp;</td><td class='ZOptionsInfo'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_options' tabindex=\"10\"></div></td><td class='ZOptionsInfo'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_button' tabindex=\"20\"></div></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#Export"
}, false);

AjxTemplate.register("prefs.Pages#Zimlets", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div align=right style='margin-right: 20px; margin-bottom: 10px;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REVERT_PAGE' tabindex='10' style='margin-right: 25px;'></div></div><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.zimlets;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td><div>";
	buffer[_i++] = ZmMsg.zimletsPrefsMsg;
	buffer[_i++] = "</div><br/></td></tr><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_CHECKED_ZIMLETS' tabindex='30'></div></td></tr></table></td></tr></table>";
	 if (appCtxt.isOffline) { 
	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.zimletInstall;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td><div>";
	buffer[_i++] = ZmMsg.zimletsPrefsUpload;
	buffer[_i++] = "</div><br/></td></tr><tr><td><table border=0><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.fileLabel;
	buffer[_i++] = "</td><td><form id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_form' action='";
	buffer[_i++] = data["action"];
	buffer[_i++] = "' method='POST' enctype='multipart/form-data' style='margin:0px; padding:0px;'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_input' tabindex='40' type='file' name='zimletUpload' style='font-family:Tahoma; font-size:10px'></form></td></tr><tr><td></td><td id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_button'></td></tr></table></td></tr></table></td></tr></table><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.zimletSyncData;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td><select id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_OFFLINE_ZIMLET_SYNC_ACCOUNT_ID' tabindex='130'></select></td></tr></table></td></tr></table>";
	 } 

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#Zimlets"
}, false);

AjxTemplate.register("prefs.Pages#BackUp", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.offlineBackupHeader;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100% cellpadding=5><tr><td class='ZOptionsLabel'></td><td class='ZOptionsField' colspan=2><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_button' tabindex='200'></div></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.offlineBackUpAccounts;
	buffer[_i++] = "</td><td class='ZOptionsField' colspan=2><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_OFFLINE_BACKUP_ACCOUNT_ID' tabindex='200'></div></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.offlineBackUpInterval;
	buffer[_i++] = "</td><td class='ZOptionsField' colspan=2><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_OFFLINE_BACKUP_INTERVAL' tabindex='200'></div></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.offlineBackUpPath;
	buffer[_i++] = "</td><td class='ZOptionsField' colspan=2><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_OFFLINE_BACKUP_PATH' tabindex='210'></div></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.offlineBackUpRetention;
	buffer[_i++] = "</td><td class='ZOptionsField' width=\"2%\"><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_OFFLINE_BACKUP_KEEP' tabindex='230'></div></td><td>";
	buffer[_i++] = ZmMsg.offlineBackUpKeep;
	buffer[_i++] = "</td></tr></table></td></tr></table><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.offlineBackUpRestore;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100% cellpadding=5><tr><td class='ZOptionsField' colspan=2><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_OFFLINE_BACKUP_RESTORE' tabindex='300'></div></td></tr><tr><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_restore_button' tabindex='320'></div></td><td>&nbsp;</td></tr></table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#BackUp"
}, false);

AjxTemplate.register("prefs.Pages#SharingPrefPage", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.pendingSharesTitle;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_shareForm'></div></td></tr></table></td></tr><tr><td class='ZOptionsSectionMain'><div><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_pendingShares'></div></div></td></tr></table><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.mountedSharesTitle;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><div><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_mountedShares'></div></div></td></tr></table><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.sharingByMe;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_sharesBy'></div></td></tr><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td><div class=\"sharingPageListTitle\">";
	buffer[_i++] = ZmMsg.shareAFolder;
	buffer[_i++] = "</div></td></tr><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_grantForm'></div></td></tr></table></td></tr></table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#SharingPrefPage"
}, false);

AjxTemplate.register("prefs.Pages#ShareForm", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table><tr><td colspan=3><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_group\" class=\"ZOptionsInfo\" tabindex='5'></div></td></tr><tr><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_user\" class=\"ZOptionsInfo\" tabindex='10'></div></td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_owner\" class=\"ZOptionsInfo\" tabindex='20'></div></td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_findButton\" tabindex='30'></div></td></tr></table><br/>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#ShareForm"
}, false);

AjxTemplate.register("prefs.Pages#GrantForm", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div>";
	buffer[_i++] = ZmMsg.grantFormText;
	buffer[_i++] = "</div><br/><table cellspacing=5><tr><td>";
	buffer[_i++] = ZmMsg.folderType;
	buffer[_i++] = "</td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_folderType' tabindex='40'></div></td><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_shareButton' tabindex='50'></div></td></tr></table><br/>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#GrantForm"
}, false);

AjxTemplate.register("prefs.Pages#MobileDevices", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.mobileDevices;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_deviceToolbar' tabindex='20'></div></td></tr><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_deviceList' tabindex='30'></div></td></tr></table></td></tr></table>";
	buffer[_i++] = ZmMsg.mobileDeviceChangesSavedImmediately;
	buffer[_i++] = "<br>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#MobileDevices"
}, false);

AjxTemplate.register("prefs.Pages#MobileDeviceInfo", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<table border=0><tr><td>";
	buffer[_i++] = ZmMsg.mobileDevice;
	buffer[_i++] = ":</td><td>";
	buffer[_i++] = data.device.type;
	buffer[_i++] = "</td></tr><tr><td>";
	buffer[_i++] = ZmMsg.mobileUserAgent;
	buffer[_i++] = ":</td><td>";
	buffer[_i++] = data.device.ua;
	buffer[_i++] = "</td></tr><tr><td>";
	buffer[_i++] = ZmMsg.mobileDeviceId;
	buffer[_i++] = ":</td><td>";
	buffer[_i++] = data.device.id;
	buffer[_i++] = "</td></tr><tr><td>";
	buffer[_i++] = ZmMsg.mobileProtocolVersion;
	buffer[_i++] = ":</td><td>";
	buffer[_i++] = data.device.protocol;
	buffer[_i++] = "</td></tr><tr><td>";
	buffer[_i++] = ZmMsg.mobileProvisionable;
	buffer[_i++] = ":</td><td>";
	buffer[_i++] = data.device.provisionable ? AjxMsg.yes : AjxMsg.no ;
	buffer[_i++] = "</td></tr><tr><td>";
	buffer[_i++] = ZmMsg.status;
	buffer[_i++] = ":</td><td>";
	buffer[_i++] = data.device.getStatusString();
	buffer[_i++] = "</td></tr>";
	 if (data.device.firstReqReceived) { 
	buffer[_i++] = "<tr><td>";
	buffer[_i++] = ZmMsg.mobileFirstReqReceived;
	buffer[_i++] = ":</td><td>";
	buffer[_i++] = data.device.getFirstReqReceivedString();
	buffer[_i++] = "</td></tr>";
	 } 
	 if (data.device.lastPolicyUpdate) { 
	buffer[_i++] = "<tr><td>";
	buffer[_i++] = ZmMsg.mobileLastPolicyUpdate;
	buffer[_i++] = ":</td><td>";
	buffer[_i++] = data.device.getLastPolicyUpdateString();
	buffer[_i++] = "</td></tr>";
	 } 
	 if (data.device.remoteWipeReqTime) { 
	buffer[_i++] = "<tr><td>";
	buffer[_i++] = ZmMsg.mobileRemoteWipeReq;
	buffer[_i++] = ":</td><td>";
	buffer[_i++] = data.device.getRemoteWipeReqTimeString();
	buffer[_i++] = "</td></tr>";
	 } 
	 if (data.device.remoteWipeAckTime) { 
	buffer[_i++] = "<tr><td>";
	buffer[_i++] = ZmMsg.mobileRemoteWipeAck;
	buffer[_i++] = ":</td><td>";
	buffer[_i++] = data.device.getRemoteWipeAckTimeString();
	buffer[_i++] = "</td></tr>";
	 } 
	buffer[_i++] = "</table>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#MobileDeviceInfo"
}, false);

AjxTemplate.register("prefs.Pages#Notifications", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<!-- REVERT_PAGE (button) --><div align=right style='margin-right: 20px; margin-bottom: 10px;'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_REVERT_PAGE' tabindex='10' style='margin-right: 25px;'></div></div><table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.notificationsEmail;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0 width=100%><tr><td class='ZOptionsSectionMain'><table width=100%><!--<tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.emailNotificationsLabel;
	buffer[_i++] = "</td>--><!--<td class='ZOptionsField'><input id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EMAIL_ENABLED' tabindex='100' type='checkbox'></td>--><!--</tr>--><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.emailNotificationsDescription;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_EMAIL' tabindex='200'></div></td></tr></table></td></tr></table>";
	 if (appCtxt.get(ZmSetting.CAL_DEVICE_EMAIL_REMINDERS_ENABLED)) { 
	buffer[_i++] = "<table border=0 cellspacing=0 cellpadding=0 width=100%><tr class='ZOptionsHeaderRow'><td class='ZOptionsHeaderL'><div class='ImgPrefsHeader_L'></div></td><td class='ZOptionsHeader ImgPrefsHeader'>";
	buffer[_i++] = ZmMsg.notificationsDeviceEmail;
	buffer[_i++] = "</td><td class='ZOptionsHeaderR'><div class='ImgPrefsHeader_R'></div></td></tr></table><div class='ZOptionsSectionMain' style=\"width:100%;padding:0px;\"><table class='ZOptionsSectionTable' border=0 cellspacing=0 cellpadding=0><tr><td class='ZOptionsSectionMain'><table width=100%><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.deviceEmailNotificationsRegionLabel;
	buffer[_i++] = "</td><td class='ZOptionsField' colspan=\"4\"><table border=\"0\" cellpadding=\"0\" cellspacing=\"0\"><tr><td><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_REGION' tabindex='400'></div></td></tr></table></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.deviceEmailNotificationsCarrierLabel;
	buffer[_i++] = "</td><td class='ZOptionsField' colspan=\"4\"><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_CARRIER' tabindex='500'></div></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_PHONE_row'><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.deviceEmailNotificationsPhoneNumberLabel;
	buffer[_i++] = "</td><td class='ZOptionsField' colspan=\"3\"><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_PHONE' tabindex='600'></div></td><td class=\"ZOptionsField\"><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_PHONE_SEND_CODE\" tabindex='610'></div></td></tr><tr id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_CUSTOM_NUMBER_row\"><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.deviceEmailNotificationsCustomeEmailLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_CUSTOM_NUMBER' tabindex='700'></div></td><td class=\"ZOptionsField\">@</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_CUSTOM_ADDRESS' tabindex='710'></div></td><td class=\"ZOptionsField\"><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_CUSTOM_SEND_CODE\" tabindex='720'></div></td></tr><tr id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_PHONE_HINT_row'><td></td><td class=\"ZOptionsInfo\" colspan=\"4\"><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_PHONE_HINT\"></div></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.deviceEmailNotificationsVerificationCodeLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_CODE' tabindex='800'></div></td><td class=\"ZOptionsField\" colspan=\"3\"><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_CODE_VALIDATE\" tabindex='810'></div></td></tr><tr><td class='ZOptionsLabel'>";
	buffer[_i++] = ZmMsg.deviceEmailNotificationsVerificationStatusLabel;
	buffer[_i++] = "</td><td class='ZOptionsField'><div id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_CODE_STATUS' tabindex='900'></div></td><td class=\"ZOptionsField\" colspan=\"3\"><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_DEVICE_EMAIL_CODE_INVALIDATE\" tabindex='910'></div></td></tr></table></td></tr></table></div>";
	 } 

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Pages#Notifications"
}, false);

}
if (AjxPackage.define("prefs.Widgets")) {
AjxTemplate.register("prefs.Widgets#infoBox", 
function(name, params, data, buffer) {
	var _hasBuffer = Boolean(buffer);
	data = (typeof data == "string" ? { id: data } : data) || {};
	buffer = buffer || [];
	var _i = buffer.length;

	buffer[_i++] = "<div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_infoBox\" class='ZInfoBox'><table id='";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_infoBox_outer' width='100%' cellspacing=0 cellpadding=0><tr><td rowspan=2 style='vertical-align:top'><div class='ImgInformation_32'></div></td><td><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_infoBox_close\" class=\"ZInfoTitleClose\" onclick='Dwt.toggle(\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_infoBox_container\")'>";
	buffer[_i++] =  ZmMsg.close ;
	buffer[_i++] = "</div><div id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_title\" class='ZInfoTitle'>";
	buffer[_i++] = data["_labels"]["infoTitle"];
	buffer[_i++] = "</div></td></tr><tr><td id=\"";
	buffer[_i++] = data["id"];
	buffer[_i++] = "_body\">";
	buffer[_i++] = data["_labels"]["infoContents"];
	buffer[_i++] = "</td></tr></table></div>";

	return _hasBuffer ? buffer.length : buffer.join("");
},
{
	"id": "prefs.Widgets#infoBox",
	"xml:space": "strip"
}, false);
AjxTemplate.register("prefs.Widgets", AjxTemplate.getTemplate("prefs.Widgets#infoBox"), AjxTemplate.getParams("prefs.Widgets#infoBox"));

}
